// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
package applications.exerciseapplet;


import javax.swing.*;
import javax.swing.border.LineBorder;
import javax.swing.border.TitledBorder;

import java.awt.*;
import java.awt.event.*;
import java.lang.reflect.Constructor;
import java.util.Properties;
import java.util.StringTokenizer;

import prototype.ui.*;

import content.ExerciseProperties;
import content.interfaces.Exercise;
import content.interfaces.SimulationExercise;
import content.interfaces.SimulationExerciseModel;
import content.interfaces.ComparableExercise;
import content.interfaces.SwapBehaviour;

import content.assessment.*;

import matrix.animation.*;
import matrix.util.*;
import matrix.util.configuration.MatrixConfiguration;


public class ExerciseApplet extends JApplet  {

    protected SimulationExerciseModel exercise = null;

    protected ExercisePanel exercisePanel;

    protected JButton gradeButton;
    protected JButton resetButton;
    protected JButton modelAnswerButton;
//    protected JButton compareButton;

    protected WrappingLabel descriptionTextLabel ;


    protected JComboBox fontSizeChoice = null;

    protected boolean gradingEnabled;  // true if grading is allowed in the beginning
    protected boolean modelAnswerEnabled;
    protected boolean resetEnabled;
    protected boolean modelAnswerAfterGrading;
    protected boolean resetAfterGrading;
    protected boolean noExercise; // the applet can be configured to show only the model answer
    protected boolean gradeButtonUpdatesDisabled;

    protected long seed = -1;


    /* Trakla2ExerciseApplet-specific attributes*/
    protected JTextArea statusbar;
    private boolean hasStatusbar = false;
    private String userId = null;
    private int currentSubmissions, maxSubmissions, currentPoints, maxPoints;
    /* END Trakla2ExerciseApplet-specific attributes */

    public void init() {

        if (!applications.lib.VMCheck.isJava2Enabled()) {
            getContentPane().add(new JLabel("Your Java version is not Java2 compatible."));
            return;
        }

        MatrixConfiguration.setCodebaseURL(getCodeBase().toString());
        MatrixConfiguration.getInstance();
        
        ExerciseProperties exerciseProperties = ExerciseProperties.getInstance();
        
        String exStr = getParameter("exercise_class");
        if (exStr == null) {
            exStr = getParameter("ex");
        }
        if (exStr != null) {
            try {
                exercise = createInstance(exStr);
                fetchLanguageOverrides(exStr, exerciseProperties); // attempt to fetch the language localization strings
                
            } catch (java.lang.Exception e) {
                Note.err(this, "Error while initializing an exercise: " + e);
                System.exit(1);
            }
        }

        if (exercise == null) {
            System.out.println("cannot load exercise: " + exStr);
            System.exit(0);
        }

        seed = getLongParameter("seed");
        if (seed >= 0) {
            exercise.setSeed(seed);            
        } else {
            exercise.setSeed(System.currentTimeMillis());
        }

        noExercise = false; //default
        int val = getIntParameter("noexercise");
        if (val == 1) { 
            noExercise = true;
        } 

        commonInit();

        validate();

        if (fontSizeChoice != null) {
            fontSizeChoice.setSelectedItem(exercisePanel.getFont().getSize() + "");
        }

    }

    protected void commonInit() {
        Insets ins = new Insets(2, 2, 2, 2);
        Animator animator = new Animator();
        ExerciseProperties lang = ExerciseProperties.getInstance();
        
        if (noExercise) { // no exercise, but model answer only
            exercisePanel = new ExercisePanel(animator, exercise);
            MyModelAnswerFrame mf = new MyModelAnswerFrame((SimulationExerciseModel)exercise, false);
            
            mf.getStructurePanel().setFontSize(getExercisePanel().getFontSize());
            
            /*
            exercisePanel.addActionListener( new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    validate();
                }
            });
            */
            
            ApplicationAdapter.setApplication(mf.getStructurePanel());
            setContentPane(mf.getContentPane());
            
            return; 
        }

        // this is the default behaviour
        exercisePanel = new ExercisePanel(animator, exercise);
        exercisePanel.setExerciseApplet(this);

        exercisePanel.addActionListener( new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                validate();
            }
        });

        ApplicationAdapter.setApplication(exercisePanel);
        setBackground(exercisePanel.getBackground());
        
        VisualAnimator visualAnimator = new ExerciseVisualAnimator(animator);
        visualAnimator.setResetButtonVisible(false);
        visualAnimator.setPlayButtonVisible(false);
        visualAnimator.setResetButtonVisible(false);
        visualAnimator.setBorder(new TitledBorder(lang.get("ANIMATOR_BORDER_TITLE")));
        exercisePanel.setVisualAnimator(visualAnimator);
        getContentPane().setLayout(new GridBagLayout());
        GridBagConstraints gbc = new GridBagConstraints();

        descriptionTextLabel = makeDescriptionTextLabel(exercise);
        //setting the font of exercise description area (in Courier all the characters have same width)
        descriptionTextLabel.setFont(new Font("monospaced", 0, 12));
        //setting border for text area
        descriptionTextLabel.setBorder(new LineBorder(Color.black));
        //component which is finally added to exercise applet
        JComponent descriptionComponentToAdd = descriptionTextLabel;
        if (descriptionTextLabel.getRows() > 10) {
            //if row count is bigger than 10, we put text area in JScrollPane
            JScrollPane descriptionScrollPane = new JScrollPane();
            descriptionScrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
            descriptionScrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
            descriptionScrollPane.setViewportView(descriptionTextLabel);
            /* This is needed to set the minimun height for the text area.
             * Otherwise the description area can be only one line if the
             * applet is too small for the exercise.
             */
            descriptionScrollPane.setMinimumSize(new Dimension(100,150));
            //In this case, the scrollpane is that component which is added to the exercise applet 
            descriptionComponentToAdd = descriptionScrollPane;
        }

        boolean isDescriptionAreaVertical = "vertical".equals(getParameter("text_area_direction"));
        
        if (!descriptionTextLabel.getText().equals("")) {
            if (isDescriptionAreaVertical) {
                /* Added later
                //lay out the description area vertically beside the StructurePanel
                descriptionTextLabel.setColumns(0);
                // add description text
                gbc.gridx = 0;
                gbc.gridy = 3;
                gbc.gridwidth = 1;
                gbc.weightx = 1.0;
                gbc.weighty = 0.0;
                gbc.fill = GridBagConstraints.VERTICAL;
                gbc.anchor =  GridBagConstraints.NORTHWEST;
                gbc.insets = ins;
                getContentPane().add(descriptionComponentToAdd, gbc);
                */                
            } else { //add description text to the top of the applet horizontally
                // add description text
                gbc.gridx = 0;
                gbc.gridy = 0;
                gbc.gridwidth = 3;
                gbc.weightx = 1.0;
                gbc.weighty = 0.0;
                gbc.fill = GridBagConstraints.HORIZONTAL;
                gbc.anchor =  GridBagConstraints.NORTHWEST;
                gbc.insets = ins;
                getContentPane().add(descriptionComponentToAdd, gbc);
            }
        }

        // add font size choice
        gbc.gridx = 0;
        gbc.gridy = 1;
        gbc.gridwidth = 1;
        gbc.weightx = 0.0;
        gbc.weighty = 0.0;
        gbc.fill = GridBagConstraints.VERTICAL;
        gbc.anchor =  GridBagConstraints.NORTHWEST;
        gbc.insets = ins;
        getContentPane().add(makeFontSizeChoice(), gbc);


        // add animator panel
        gbc.gridx = 1;
        gbc.gridy = 1;
        gbc.gridwidth = 1;
        gbc.weightx = 0.0;
        gbc.weighty = 0.0;
        gbc.fill = GridBagConstraints.VERTICAL;
        gbc.anchor =  GridBagConstraints.NORTHWEST;
        gbc.insets = ins;
        getContentPane().add(exercisePanel.getVisualAnimator(), gbc);

        // add task panel
        gbc.gridx = 2;
        gbc.gridy = 1;
        gbc.gridwidth = 1;
        gbc.weightx = 0.0;
        gbc.weighty = 0.0;
        gbc.fill = GridBagConstraints.VERTICAL;
        gbc.anchor =  GridBagConstraints.NORTHWEST;
        gbc.insets = ins;
        getContentPane().add(makeTaskPanel(), gbc);

        gbc.gridx = 0;
        gbc.gridy = 3;
        gbc.gridwidth = 3;
        gbc.weightx = 1.0;
        gbc.weighty = 1.0;
        gbc.fill = GridBagConstraints.BOTH;
        gbc.anchor =  GridBagConstraints.NORTHWEST;
        gbc.insets = ins;

        JScrollPane sp = new JScrollPane();

        sp.setViewportView(exercisePanel);
        if (isDescriptionAreaVertical && !descriptionTextLabel.getText().equals("")) {
            JSplitPane splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,
                                                  descriptionComponentToAdd, sp);
            splitPane.setOneTouchExpandable(true);
            splitPane.setDividerSize(8);
            this.getContentPane().add(splitPane, gbc);
        } else {
            this.getContentPane().add(sp, gbc);
        }


        /* The statusbar for Trakla2-applets. 
         * Exercise name, current/max points current/max submissions and current/deadline time */
        if(hasStatusbar) {
            gbc.gridx = 0;
            gbc.gridy = 4;
            gbc.gridwidth = 3;
            gbc.weightx = 0.0;
            gbc.weighty = 0.0;
            gbc.fill = GridBagConstraints.BOTH;
            gbc.anchor =  GridBagConstraints.NORTHWEST;
            gbc.insets = ins;
            initializeStatusbar();
            getContentPane().add(statusbar, gbc);
        }

        /*        String fontSize = getParameter("font_size");
        if (fontSize != null) {
            setFontSize(fontSize);
        }*/

        if (exercise instanceof SwapBehaviour){
            exercisePanel.setSwapEnabled(((SwapBehaviour)exercise).getSwapBehaviour());
        }
        else{
            String swap = getParameter("swap");
            if (swap != null && swap.equals("on")) {
                exercisePanel.setSwapEnabled(true);
            }  else {
                exercisePanel.setSwapEnabled(false);
            }
        }

        // initialize buttons
        gradingEnabled = false;
        resetEnabled = false;
        modelAnswerEnabled = true;
        modelAnswerAfterGrading = false;
        resetAfterGrading = false;
        gradeButtonUpdatesDisabled = false;
        int val = getIntParameter("grading");
        if (val == 0) { //default
            gradingEnabled=false;
        } else if (val == 1) {
            gradingEnabled = true;
        }
        gradeButton.setEnabled(gradingEnabled);
        val = getIntParameter("reset_bg");
        if(val == 1) { 
            resetEnabled = false;
            resetAfterGrading = true;
        } else {
            val = getIntParameter("reset");
            if (val == 1) {
                resetEnabled=true;
            } else { //default
                resetEnabled = false;
                resetAfterGrading = true;
            }
        }
        resetButton.setEnabled(resetEnabled);
        val = getIntParameter("modelanswer_bg");
        if(val == 1) {
            modelAnswerEnabled = false;
            modelAnswerAfterGrading = true;
        } else {
            val = getIntParameter("modelanswer");
            if (val == 1) { //default
                modelAnswerEnabled = true;
            } else if (val == 0) {
                modelAnswerEnabled = false;
            }
        }
        modelAnswerButton.setEnabled(modelAnswerEnabled);

    }

    /**
     * 
     * @param name
     * @return the integer value of param name, -1 if not integer or not defined
     */
    public int getIntParameter(String name) {
        String str = getParameter(name);
        try {
            if (str != null) {
                return Integer.valueOf(str).intValue();
            }
            return -1;
        } catch (Exception e) {
            Note.err(this, "cannot get parameter: " + name
                    + ". Exception: " + e.getMessage());
            return -1;
        }
    }


    /**
     * 
     * @param name
     * @return the long value of param name, -1 if not integer or not defined
     */
    public long getLongParameter(String name) {
        String str = getParameter(name);
        try {
            if (str != null) {
                return Long.valueOf(str).longValue();
            }
            return -1;
        } catch (Exception e) {
            Note.err(this, "cannot get parameter: " + name
                    + ". Exception: " + e.getMessage());
            return -1;
        }
    }

    protected JPanel makeFontSizeChoice() {
        ExerciseProperties lang = ExerciseProperties.getInstance();
        
        fontSizeChoice = new JComboBox();
        fontSizeChoice.addItem("6");
        fontSizeChoice.addItem("8");
        fontSizeChoice.addItem("10");
        fontSizeChoice.addItem("12");
        fontSizeChoice.addItem("14");
        fontSizeChoice.addItem("16");
        fontSizeChoice.addItem("18");
        fontSizeChoice.addItem("20");
        fontSizeChoice.addItem("24");
        fontSizeChoice.addItem("28");
        fontSizeChoice.addItem("32");
        fontSizeChoice.addItemListener(new ItemListener() {
            public void itemStateChanged(ItemEvent evt) {
                String newSizeStr = (String)evt.getItem();
                setFontSize(newSizeStr);
//              fontSizeChoice.select(newSizeStr);
                validate();
            }
        });

        JPanel panel = new JPanel();
        panel.setBorder(new TitledBorder(lang.get("FONT_BORDER_TITLE")));
        panel.setLayout(new GridLayout(1,1));
        panel.add(fontSizeChoice);
        return panel;
    }

    protected void setStatusbarParameters(String userId, 
            int currentSubmissions, int maxSubmissions, int currentPoints, int maxPoints) {
        hasStatusbar = true;
        this.userId = userId;
        this.currentSubmissions = currentSubmissions;
        this.maxSubmissions = maxSubmissions;
        this.currentPoints = currentPoints;
        this.maxPoints = maxPoints;

    }

    protected void refreshStatusbar(int grade) {
        if(currentPoints < grade)
            currentPoints = grade;
        currentSubmissions++;
        String statusbarText = "Points: " + currentPoints + "/" + maxPoints + "   Submissions: " + currentSubmissions;
        if(maxSubmissions < 100)
            statusbarText = statusbarText + "/" + maxSubmissions;
        statusbar.setText(statusbarText);
        statusbar.validate();
    }

    protected void initializeStatusbar() {
        String statusbarText = "Points: " + currentPoints + "/" + maxPoints + "   Submissions: " + currentSubmissions;
        if(maxSubmissions < 100)
            statusbarText = statusbarText + "/" + maxSubmissions;
        statusbar = new JTextArea(statusbarText, 1, statusbarText.length() + 1);
        statusbar.setOpaque( false );
        statusbar.setBackground(new Color(255,255,255,100));
    }

    protected WrappingLabel makeDescriptionTextLabel(Exercise ex) {
        String text = ex.getDescription();
        return new WrappingLabel(text);
    }


    protected JPanel makeTaskPanel() {
        ExerciseProperties lang = ExerciseProperties.getInstance();
        
        JPanel p = new JPanel(new GridBagLayout());
        GridBagConstraints c = new GridBagConstraints();

        p.setBorder(new TitledBorder(lang.get("EXERCISE_BORDER_TITLE")));
        // By default, we have reset, model answer, and grade/submit
        int buttonCount = 3;
        
        c.gridx = 0;
        c.gridwidth = 1;
        c.weightx = 1;
        c.gridy = 0;
        c.weighty = 1;

        resetButton = new JButton(lang.get("RESET_BUTTON"));
        resetButton.setToolTipText(lang.get("RESET_TOOLTIP"));
        resetButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                reset();
            }
        });
        p.add(resetButton, c);
        
        c.gridx = 2;
        c.gridwidth = 1;
        c.weightx = 1;
        c.gridy = 0;
        c.weighty = 1;


        gradeButton = getGradeButton();
        gradeButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                grade();
                exercisePanel.validate();
                exercisePanel.validateAnimator();
            }
        });
        p.add(gradeButton, c);
        
        c.gridx = 1;
        c.gridwidth = 1;
        c.weightx = 1;
        c.gridy = 0;
        c.weighty = 1;


        modelAnswerButton = new JButton(lang.get("MODEL_ANSWER_BUTTON"));
        modelAnswerButton.setToolTipText(lang.get("MODEL_ANSWER_TOOLTIP"));
        modelAnswerButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                modelAnswer();
            }
        });
        p.add(modelAnswerButton, c);
        
//        p.setLayout(new GridLayout(1, buttonCount));
        return p;
    }

    protected JButton getGradeButton() {
        ExerciseProperties lang = ExerciseProperties.getInstance();        
        JButton gradeButton = new JButton(lang.get("GRADE_BUTTON"));
        gradeButton.setToolTipText(lang.get("GRADE_TOOLTIP"));
        return gradeButton;
    }



    protected void reset() {
        try {
            exercise = (SimulationExerciseModel)exercise.getClass().newInstance();
            if (seed < 0) {
                exercise.setSeed(System.currentTimeMillis());
            } else {
                exercise.setSeed(seed);
            }
        } catch (Exception e) {
            Note.err(this, e.getMessage());
            Note.printStackTrace();
        }
        getExercisePanel().setExercise(exercise);
        getExercisePanel().reset();
        descriptionTextLabel.setText(exercise.getDescription());
        //gradeButton.setEnabled(true);
        validate();
    }


    protected void modelAnswer() {
        if(exercise instanceof SimulationExerciseModel) {
            MyModelAnswerFrame mf = new MyModelAnswerFrame((SimulationExerciseModel)exercise);
            mf.getStructurePanel().setFontSize(getExercisePanel().getFontSize());
            
            //gradeButton.setEnabled(false);
        } else {
            Note.show(this, "Cannot show model answer, none given.");
        }
    }

    protected void compare() {
        if (exercise instanceof ComparableExercise) {
            ComparisonFrame cf = new ComparisonFrame(getAnimator(), (ComparableExercise) exercise);
            cf.setVisible(true);
        } else {
            Note.show(this, "This exercise is not comparable.");
        }
    }


    protected void grade() {
        // exercise is defined to be a SimulationExerciseModel -vkaravir
        //        if(exercise instanceof SimulationExerciseModel) {
        Judgement j = Jury.getInstance().judge(exercise.getAnswer(), getAnimator(), exercise);

        DeprecatedJudgement dj = DeprecatedJury.judge(((SimulationExercise)exercise).getAnswer(), getAnimator(), (SimulationExerciseModel)exercise);

        if (dj.equals(j) == false)
            Note.show(this, "Refactored Judgement does not match old Judgement");


        new GradeDialog(j).setVisible(true);
        //gradeButton.setEnabled(false);
        //} else {
        //Note.show(this, "Cannot judge this exercise. No method of judging given.");
        //}
    }


    protected Animator getAnimator() {
        return exercisePanel.getAnimator();
    }


    public ExercisePanel getExercisePanel() {
        return exercisePanel;
    }

    public SimulationExerciseModel getExercise() {
        return exercise;
    }

    protected void setFontSize(String sizeStr) {
        int size = 0;
        try {
            size = Integer.parseInt(sizeStr);
        } catch (NumberFormatException e) {
            System.err.println("Invalid font size: " + sizeStr);
        }
        getExercisePanel().setFontSize(size);
    }

    public SimulationExerciseModel createInstance(String exerciseClassName) {
        Properties appletProperties = fetchProperties(); 

        try {
            Class exerciseClass = Class.forName(exerciseClassName);

            //If the exercise class has a constructur with a properties parameter
            //give the properties. Otherwise use the parameterless constructor.

            try{
                Constructor c = exerciseClass.getConstructor(new Class[]{Properties.class});                        
                return (SimulationExerciseModel)c.newInstance(new Object[]{appletProperties});
            }catch(NoSuchMethodException f){
                Constructor c = exerciseClass.getConstructor(new Class[0]);                        
                return (SimulationExerciseModel)c.newInstance(new Object[0]);                
            }

        } catch (Exception e) {
            System.err.println("Can't get object of class " + exerciseClassName);
            e.printStackTrace(System.err);
            return null;
        }
    }

    /* Resizes the applet. Used in the Trakla2-environment (with javascript-calls).
     * @see java.awt.Component#setSize(int, int)
     */
    public void setSize(int x, int y) {
        super.setSize(x,y);
        validate();
    }

    public Properties fetchProperties(){
        Properties appletProperties = new Properties();

        //Adding the standard properties
        String knownProperties[] = {
                "seed","grading","reset_bg","reset","modelanswer_bg",
                "modelanswer","swap","font_size", "courseid",
                "trakla2Host","trakla2Port","points","fingerprint",
                "exercise", "currentpoints", "submissions", "currentsubmissions",
        "noexercise" };

        for (int i=0; i<knownProperties.length; i++){
            String propertyValue = getParameter(knownProperties[i]);
            if (propertyValue != null){
                appletProperties.put(knownProperties[i],propertyValue.trim());
            }
        }

        String exStr          = getParameter("exercise_class");
        if (exStr == null) {
            exStr = getParameter("ex");
        }

        String studentid      = getParameter("userid");
        //if (studentid != null)
        //    studentid = studentid.toLowerCase();

        if (exStr != null)
            appletProperties.put("exercise_class",exStr);
        if (studentid != null)
            appletProperties.put("student_id"    ,studentid);

        // Additional parameters available for the applet must be defined
        // in a separate parameter called additional_parameters
        //
        // The value of this parameter is the names of all the additional
        // parameters separated by commas.

        String additionalParameters = getParameter("additional_parameters");
        if (additionalParameters != null){
            StringTokenizer st = new StringTokenizer(additionalParameters,",");
            while (st.hasMoreTokens()){
                String paramName = st.nextToken().trim();
                String paramValue = getParameter(paramName).trim();
                if (paramValue != null){
                    appletProperties.put(paramName, paramValue);
                }
            }
        }

        return appletProperties;
    }

    public void fetchLanguageOverrides(String exerciseClassName, ExerciseProperties languageProperties) {
    
        String propertyList = languageProperties.get(exerciseClassName);
        StringTokenizer st =  new StringTokenizer(propertyList, ",");
        
        while (st.hasMoreTokens()){
            String paramId = st.nextToken().trim();
            
            String currentParam = getParameter(paramId);
            if (currentParam != null)
                languageProperties.setString(paramId, currentParam.trim());
        }
        
        String globalPropertyList = languageProperties.get("GLOBAL_STRINGS");
        st =  new StringTokenizer(globalPropertyList, ",");
        
        while (st.hasMoreTokens()){
            String paramId = st.nextToken().trim();
            
            String currentParam = getParameter(paramId);
            if (currentParam != null)
                languageProperties.setString(paramId, currentParam.trim());
        }

        
    }
    
    /**
     * Enables gradebutton if submission is allowed and animator has steps
     * Disables gradebutton animator doesn't have steps
     *
     */
    public void updateGradeButton(){
    	
    	if (!gradeButtonUpdatesDisabled){
	    	int state = getAnimator().getState();
	    	if (state == 0 && !gradingEnabled){
	    		gradeButton.setEnabled(false);
	    	}
	    	else if (state > 0){
	    		gradeButton.setEnabled(true);
	    	}
    	}
    }
}

