package applications.exerciseapplet;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JPanel;
import javax.swing.JScrollPane;

import matrix.animation.Animator;
import matrix.animation.VisualAnimator;
import matrix.simulation.VisualTypeConf;
import matrix.structures.FDT.FDT;
import matrix.uitools.StructurePanel;
import matrix.visual.VisualArray;
import matrix.visual.VisualContainer;
import matrix.visual.VisualType;
import content.interfaces.ConfigureVisualType;
import content.interfaces.HasRotatedContainers;
import content.interfaces.LayoutExercise;
import content.interfaces.ModelAnswerNames;
import content.interfaces.SimulationExerciseModel;
import content.interfaces.StyledExercise;

/**
 * ModelAnswerPanel used to show the model answer in the grade dialog
 * @author lhakulin
 *
 */

public class ModelAnswerPanel extends JPanel {
	
	private VisualAnimator visualAnimator;
	private StructurePanel modelAnswerPanel;

    public ModelAnswerPanel(SimulationExerciseModel mod) {
    	
    	Animator a = new Animator();
    	
    	visualAnimator = new MyVisualAnimator(a);
    	visualAnimator.setResetButtonVisible(false);
    	visualAnimator.setPlayButtonVisible(false);

    	modelAnswerPanel = new StructurePanel(a);
        modelAnswerPanel.setVisualAnimator(visualAnimator);

        // Set the structurepanel and visualanimator to listen
        // to each other, so that when one changes, the other
        // is also updated.
        modelAnswerPanel.addActionListener(new ActionListener() {
        	public void actionPerformed(ActionEvent evt) {
        		validate();
        	}
        });
        visualAnimator.addActionListener(new ActionListener() {
        	public void actionPerformed(ActionEvent evt) {
        		validate();
        	}
        });
        
        layoutComponents();

        Animator.setActiveAnimator(a);
        FDT[] answer = mod.makeModelAnswer();
        ((MyVisualAnimator)visualAnimator).initIndices();

        VisualType[] vt = null;
        if(mod instanceof LayoutExercise) {
            if(mod instanceof StyledExercise) {
                vt = modelAnswerPanel.addStructures(answer, ((StyledExercise)mod).getModelAnswerVisualisations(), ((LayoutExercise)mod).getModelAnswerConstraints());    		   
            } else {
                vt = modelAnswerPanel.addStructures(answer, ((LayoutExercise)mod).getModelAnswerConstraints());    		  
            }
        } else if(mod instanceof StyledExercise) {
            vt = modelAnswerPanel.addStructures(answer, ((StyledExercise)mod).getModelAnswerVisualisations());
        } else {
            vt = modelAnswerPanel.addStructures(answer);
        }

        if (mod instanceof ModelAnswerNames) {
            String[] names = ((ModelAnswerNames)mod).getModelAnswerNames();
            for (int i = 0; i < vt.length; i++) {
                vt[i].setName(names[i]);
            }
        }

        if (mod instanceof HasRotatedContainers) {
            boolean[] rotateInfo = ((HasRotatedContainers)mod).getModelAnswerRotations();
            for (int i = 0; i < vt.length; i++) {
                if (vt[i] instanceof matrix.visual.VisualContainer)
                    ((matrix.visual.VisualContainer)vt[i]).setRotated(rotateInfo[i]);
            }	  
        }

        for (int i = 0; i < vt.length; i++) {
            if (vt[i] instanceof VisualContainer) {
                ((VisualContainer)vt[i]).setHotSpotEnabled(false);
                ((VisualContainer)vt[i]).setNullHotSpotEnabled(false);
                ((VisualContainer)vt[i]).setSearchHotSpotEnabled(false);
            }
            if (vt[i] instanceof VisualArray) {
                ((VisualArray)vt[i]).setResizeHotSpotEnabled(false);
            }
        }

        if (mod instanceof ConfigureVisualType) {
            VisualTypeConf[] conf = ((ConfigureVisualType)mod).conf();
            for (int i = 0; i < vt.length; i++) {
                vt[i].configure(conf[i]);
            }
        }

        modelAnswerPanel.disableVisualComponents();
        visualAnimator.validate();
    }
    
    protected void layoutComponents() {

        // See java.awt.GridBagLayout for details.
        GridBagLayout grid = new  GridBagLayout();
        setLayout(grid);
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.insets = new Insets(10,10,10,10);
        gbc.fill = GridBagConstraints.NONE;

        gbc.gridy = 0;
        gbc.gridx = 0;
        add(visualAnimator, gbc);

        gbc.gridy = 1;
        gbc.gridx = 0;
        
        gbc.weightx = 1.0;
        gbc.weighty = 1.0;
        gbc.anchor = GridBagConstraints.NORTH;
        gbc.fill = GridBagConstraints.BOTH;
        
        JScrollPane scrollPane = new JScrollPane();
        scrollPane.setViewportView(modelAnswerPanel);
        add(scrollPane, gbc);
      }
    
    public StructurePanel getStructurePanel(){
    	return modelAnswerPanel;
    }
}
