// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
package applications.exerciseapplet;

import java.awt.Dimension;

import javax.swing.JMenuBar;

import content.interfaces.ConfigureVisualType;
import content.interfaces.HasRotatedContainers;
import content.interfaces.ModelAnswerNames;
import content.interfaces.SimulationExerciseModel;
import content.interfaces.StyledExercise;
import content.interfaces.LayoutExercise;

import matrix.animation.Animator;
import matrix.simulation.VisualTypeConf;
import matrix.structures.FDT.FDT;
import matrix.uitools.*;
import matrix.visual.VisualArray;
import matrix.visual.VisualType;
import matrix.visual.VisualContainer;

public class MyModelAnswerFrame extends MatrixFrame {

    public MyModelAnswerFrame(SimulationExerciseModel mod) {
        this(new Animator(), mod);
    }
    
    public MyModelAnswerFrame(SimulationExerciseModel mod, boolean visibility) {
        this(new Animator(), mod, visibility);
    }
    
    public MyModelAnswerFrame(Animator a, SimulationExerciseModel mod) {
        this(a, mod, true);
    }
    
    public MyModelAnswerFrame(Animator a, SimulationExerciseModel mod, boolean visibility) {
        super(new MyVisualAnimator(a), a, visibility);
        setTitle("Model answer");
        //System.out.println(this + " init");
        getVisualAnimator().setResetButtonVisible(false);
        getVisualAnimator().setPlayButtonVisible(false);
        StructurePanel p = getStructurePanel();
        
        Animator.setActiveAnimator(getAnimator());
        FDT[] answer = mod.makeModelAnswer();
        ((MyVisualAnimator)getVisualAnimator()).initIndices();

        VisualType[] vt = null;
        if(mod instanceof LayoutExercise) {
            if(mod instanceof StyledExercise) {
                vt = p.addStructures(answer, ((StyledExercise)mod).getModelAnswerVisualisations(), ((LayoutExercise)mod).getModelAnswerConstraints());    		   
            } else {
                vt = p.addStructures(answer, ((LayoutExercise)mod).getModelAnswerConstraints());    		  
            }
        } else if(mod instanceof StyledExercise) {
            vt = p.addStructures(answer, ((StyledExercise)mod).getModelAnswerVisualisations());
        } else {
            vt = p.addStructures(answer);
        }

        if (mod instanceof ModelAnswerNames) {
            String[] names = ((ModelAnswerNames)mod).getModelAnswerNames();
            for (int i = 0; i < vt.length; i++) {
                vt[i].setName(names[i]);
            }
        }

        if (mod instanceof HasRotatedContainers) {
            boolean[] rotateInfo = ((HasRotatedContainers)mod).getModelAnswerRotations();
            for (int i = 0; i < vt.length; i++) {
                if (vt[i] instanceof matrix.visual.VisualContainer)
                    ((matrix.visual.VisualContainer)vt[i]).setRotated(rotateInfo[i]);
            }	  
        }

        for (int i = 0; i < vt.length; i++) {
            if (vt[i] instanceof VisualContainer) {
                ((VisualContainer)vt[i]).setHotSpotEnabled(false);
                ((VisualContainer)vt[i]).setNullHotSpotEnabled(false);
                ((VisualContainer)vt[i]).setSearchHotSpotEnabled(false);
            }
            if (vt[i] instanceof VisualArray) {
                ((VisualArray)vt[i]).setResizeHotSpotEnabled(false);
            }
        }

        if (mod instanceof ConfigureVisualType) {
            VisualTypeConf[] conf = ((ConfigureVisualType)mod).conf();
            for (int i = 0; i < vt.length; i++) {
                vt[i].configure(conf[i]);
            }
        }
        
        // Increase the size of the frame if the panel doesn't fit without scrollbars
        Dimension prefSize = p.getPreferredSize();
        int visualAnimatorPrefHeight = getVisualAnimator().getComponentsPrefHeightSum();
        int prefWidth = prefSize.width;
        int prefHeight = prefSize.height + visualAnimatorPrefHeight;
        int width = getWidth();
        int height = getHeight();
        if (prefWidth > width){
        	int maxWidth = (int)(1.5*width);
        	width = Math.min(maxWidth, prefWidth);
        }
        if (prefHeight > height){
        	int maxHeight = (int)(1.5*height);
        	height = Math.min(maxHeight, prefHeight);
        }
        setSize(width, height);

        getStructurePanel().disableVisualComponents();
        getVisualAnimator().validate();
    }


    protected JMenuBar createMenuBar() {
        JMenuBar mb = new JMenuBar();
        return mb;
    }


    public void quit() {
        dispose();
    }


    public int[] getAnimationStepIndices() {
        return ((MyVisualAnimator)getVisualAnimator()).getSteps();
    }
}
