// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
package applications.trakla2.ui;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JEditorPane;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextArea;
import javax.swing.border.BevelBorder;

public class MCQDialog extends JDialog implements ActionListener{

    private static final long serialVersionUID = 1672868523502880584L;

    //WindowListener which disallows closing the window before a choice has been made
    private WindowListener windowListener;
    
    //Constraints for the GridBagLayout
    private GridBagConstraints gbc;    

    //Closing button
    private JButton closingButton; 
    
    //Counter for added elements for GridBagLayout
    private int elements;

    //Counter for question
    private int questions;
    
    //Array storing the current selections for each of the MCQs
    private int[] selections;
    
    // -1 in the selections array marks that nothing has been selected
    public static final int NOTHING_SELECTED = -1;

    //Has the filler been added
    private boolean fillerAdded;
    
    private JTextArea freeInput;
    
    private static String HTML_BEGIN = "<html><body><font face=\"Arial, Helvetica\">";
    private static String HTML_END   = "</font></body></html>";
    
    public MCQDialog(String title, String intro){
        super(new JFrame(), title);

        elements    = 0;
        questions   = 0;

        fillerAdded = false;

        getContentPane().setLayout(new GridBagLayout());

        JEditorPane text = null;
        if (intro != null)
            text = makeTextArea(intro);

        gbc = new GridBagConstraints();

        gbc.insets = new Insets(10,10,10,10);

        gbc.fill   =GridBagConstraints.HORIZONTAL;
        gbc.anchor =GridBagConstraints.NORTH;

        // If there is an introductory text element, add it

        if (intro != null){
            gbc.gridy = elements;
            gbc.weighty=0.0;
            gbc.weightx=1.0;

            elements++;
            getContentPane().add(text, gbc);
        }

        setModal(true);
        setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
        
        closingButton = new JButton("Ok");
        closingButton.addActionListener(
                new ActionListener(){
                    public void actionPerformed(ActionEvent e){
                        if (isAnswered()){
                            setVisible(false);
                            dispose();
                        }else{
                            JOptionPane.showMessageDialog(null, "Please choose one of the available options", "Please choose", JOptionPane.INFORMATION_MESSAGE);
                        }
                        
                    }
                }        
        );
        
        
        windowListener = new WindowAdapter(){
            public void windowClosing(WindowEvent e){
                if (isAnswered()){
                    setVisible(false);
                    dispose();
                }else{
                    JOptionPane.showMessageDialog(null, "Please choose one of the available options", "Please choose", JOptionPane.INFORMATION_MESSAGE);
                }
                
            }
        };
        addWindowListener(windowListener);
    }

    public void addMCQ (String question, String[] options){
        if (!fillerAdded){
            JPanel buttonPanel=makeButtonPanel(questions, question, options);

            gbc.gridx = 0;          // First in the row
            gbc.gridy = elements;   // y position from elements-field
            gbc.weighty=0.0;        // Don't scale the panel

            elements++;        
            questions++;

            selections = new int[questions]; //Clears the current selections, but nothing should be visible yet
            for (int i=0; i<questions; i++){
                selections[i] = NOTHING_SELECTED;
            }

            getContentPane().add(buttonPanel, gbc);
        }else{
            throw new IllegalStateException("Question will not be added - Dialog is already visible");
        }
    }

    public void addMCQ (String question, String[] options, ImageIcon picture){
        if (!fillerAdded){
            JPanel buttonPanel=makeButtonPanel(questions, question, options);

            gbc.gridx = 0;          // First in the row
            gbc.gridy = elements;   // y position from elements-field
            gbc.weighty=0.0;        // Don't scale the panel

            elements++;        
            questions++;

            selections = new int[questions]; //Clears the current selections, but nothing should be visible yet
            for (int i=0; i<questions; i++){
                selections[i] = NOTHING_SELECTED;
            }

            JPanel jp = new JPanel();
            jp.setLayout(new BoxLayout(jp, BoxLayout.X_AXIS));
            jp.add(buttonPanel);
            jp.add(new JLabel(picture));
                        
            getContentPane().add(jp, gbc);
                        
        }else{
            throw new IllegalStateException("Question will not be added - Dialog is already visible");
        }
    }
    
    
    public void setVisible(boolean value){
        if (value && fillerAdded == false){

            gbc.fill    = GridBagConstraints.NONE;
            gbc.weightx = 0;
            gbc.gridy   = elements++;
            
            getContentPane().add(closingButton, gbc);            

            //Add a filler that fixes the gridbaglayout
            
            gbc.fill    = GridBagConstraints.BOTH;        
            gbc.weighty = 0.001;
            gbc.gridy   = elements;

            JPanel freeWord = new JPanel();
            freeWord.setBorder(new BevelBorder(1));
            freeInput = new JTextArea();            
            
            freeWord.setLayout(new BoxLayout(freeWord, BoxLayout.Y_AXIS));
            
            JPanel labelWrap = new JPanel();
            labelWrap.add(new JLabel("Extra comments here : "));
            freeWord.add(labelWrap);
            freeWord.add(freeInput);
            
            getContentPane().add(freeWord, gbc);
            pack();
            setSize(900, 700);

            fillerAdded = true;
        }

        super.setVisible(value);
    }


    private JPanel makeButtonPanel(int questionNumber, String question, String[] options) {

        JPanel panel = new JPanel();

        panel.setBorder(BorderFactory.createEtchedBorder());        
        panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));

        JEditorPane jep = new JEditorPane();
        jep.setContentType("text/html");
        jep.setText(HTML_BEGIN + question + HTML_END);
        jep.setEditable(false);
        panel.add(jep);

        ButtonGroup bg = new ButtonGroup();


        for (int i=0; i<options.length; i++){
            JPanel innerPanel = new JPanel();
            innerPanel.setLayout(new BoxLayout(innerPanel, BoxLayout.X_AXIS));
            innerPanel.setBorder(BorderFactory.createEtchedBorder());

            NumberedRadioButton jrb = new NumberedRadioButton(questionNumber, i);
            
            bg.add(jrb);

            jrb.addActionListener(this);            

            innerPanel.add(jrb);

            JEditorPane jtp = new JEditorPane();
            jtp.setContentType("text/html");
            jtp.setText(HTML_BEGIN + options[i] + HTML_END);

            jrb.setBackground(jtp.getBackground());            
            
            innerPanel.add(jtp);
            panel.add(innerPanel);
        }

        return panel;
    }

    private static JEditorPane makeTextArea(String text) {
        JEditorPane jep = new JEditorPane();
        jep.setContentType("text/html");
        jep.setText(HTML_BEGIN + text + HTML_END);
        jep.setEditable(false);
        return jep;
    }

    private static class NumberedRadioButton extends JRadioButton{
        private static final long serialVersionUID = 8125772861489889586L;
        private int question;
        private int option;

        public NumberedRadioButton(int question, int option){
            super();
            this.question = question;
            this.option   = option;
        }

        public int getQuestion(){
            return question;
        }

        public int getOption(){
            return option;
        }
    }

    public void actionPerformed(ActionEvent e) {
        NumberedRadioButton nrb = (NumberedRadioButton)(e.getSource());

        selections[nrb.getQuestion()] = nrb.getOption();        
    }

    /**
     * Returns the selections for each question.
     * 
     * @return an array containing the numbers of the choices made or NOTHING_SELECTED
     */
    public int[] getSelections(){
        return selections;
    }

    /**
     * Checks whether all the MCQ:s in the form have been properly answered
     * 
     * @return are the questions answered
     */
    
    public boolean isAnswered(){
        for (int i=0; i<selections.length; i++){
            if (selections[i] == NOTHING_SELECTED)
                return false;
        }
        return true;
    }
    
    public String getFreeInput(){
        return freeInput.getText().trim();
    }
}
