// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
package applications.trakla2.ui;

import javax.swing.*;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectOutputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.util.*;

import applications.exerciseapplet.ExerciseApplet;
import applications.trakla2.datalogging.*;

import content.ExerciseProperties;
import content.exercises.Heap_Buildheap_Study;
import content.interfaces.KnownMisconceptions;
import content.interfaces.SimulationExercise;
import content.interfaces.SimulationExerciseModel;

import content.assessment.*;
import content.interfaces.*;

import matrix.animation.AnimatedStructures;
import matrix.animation.Animator;
import matrix.structures.FDT.FDT;
import matrix.util.*;
import matrix.util.configuration.MatrixConfiguration;
import prototype.ui.ComparisonFrame;




public class Trakla2ExerciseApplet extends ExerciseApplet {

    public static final int CONNECTION_TIMEOUT = 15;
    public static final int MINIMUM_ADVANCE    = 2;


    private boolean logModelAnswer = true;

    private boolean logGrade = true;

	/** A variable that is true if the exercise has been 
		submitted or submission is no longer possible. */
    private boolean submitted = false;

    String studentid;

    int maxPoints = 0;

    int currentPoints = 0;

    int maxSubmissions = 0;

    int currentSubmissions = 0;

    Date operationTime = new Date();

    private boolean miscStudy = false;
    private ArrayList offeredMisconceptions;
    private ArrayList offeredNumbers;
    private Integer   selectedMisconception;
    private String    misconceptionFreeText;
	private String exerciseId;
	private String serverUrl;
	private String submitChecksum;

    public void init() {
        studentid = getParameter("user_id");
        
        exerciseId = getParameter("exercise_id");
        seed = getLongParameter("random_seed");
        
        serverUrl = getParameter("server_url");
        maxPoints = getIntParameter("maximum_grade");
        currentPoints = getIntParameter("current_grade");
        maxSubmissions = getIntParameter("maximum_submissions");
        currentSubmissions = getIntParameter("current_submissions");
        submitChecksum = getParameter("submit_checksum");

        submitted = false;

        if (!applications.lib.VMCheck.isJava2Enabled()) {
            getContentPane().add(new JLabel("Your Java version is not Java2 compatible."));
            return;
        }

        MatrixConfiguration.setCodebaseURL(getCodeBase().toString());
        // force MatrixConfiguration read configuration file
        MatrixConfiguration.getInstance();

        ExerciseProperties exerciseProperties = ExerciseProperties.getInstance();
        
        // XXX ex is not part of the protocol specification?
        String exStr = getParameter("exercise_class");
        if (exStr == null) {
            exStr = getParameter("ex");
        }
        if (exStr != null) {
            exercise = createInstance(exStr);
            fetchLanguageOverrides(exStr, exerciseProperties); // attempt to fetch the language localization strings
        }

        if (exercise == null) {
            System.out.println("cannot load exercise: " + exStr);
            System.exit(0);
        }

        init2();
    }




    protected void init2() {

        // add applet components

        getContentPane().removeAll();

        // pass statusbar-parameters to ExerciseApplet
        setStatusbarParameters(studentid, currentSubmissions, maxSubmissions, currentPoints, maxPoints);

        // shared initialization method implemented in ExerciseApplet
        commonInit();

        // font unknown until validate is called
        if (fontSizeChoice != null) {
            String fontSize = getParameter("font_size");
            if(fontSize != null)
                fontSizeChoice.setSelectedItem(fontSize);
            else fontSizeChoice.setSelectedItem(exercisePanel.getFont().getSize()+"");
            
        }
        validate();
    }

    protected JButton getGradeButton() {
        ExerciseProperties lang = ExerciseProperties.getInstance();
        JButton submitButton = new JButton(lang.get("SUBMIT_BUTTON"));
        submitButton.setToolTipText(lang.get("SUBMIT_TOOLTIP"));
        return submitButton;
    }

    protected void grade() {
        if(exercise instanceof SimulationExerciseModel) {
            Judgement j =
                Jury.getInstance().judge(((SimulationExercise)exercise).getAnswer(),
                        getAnimator(),
                        (SimulationExerciseModel)exercise);

            DeprecatedJudgement dj = DeprecatedJury.judge(((SimulationExercise)exercise).getAnswer(), getAnimator(), (SimulationExerciseModel)exercise);

            if (dj.equals(j) == false)
                Note.show(this, "Refactored Judgement does not match old Judgement");

            if (exercise instanceof KnownMisconceptions && 
                    exercise instanceof Heap_Buildheap_Study){

                final ArrayList collectedExplanations = new ArrayList();
                final ArrayList collectedNumbers      = new ArrayList();
                final boolean[] assessmentSucceeded  = new boolean[1];
                final Judgement normalJudgement       = j;
                
                // The following thread magic is a protection mechanism against possible
                // mistakes in the misconception assessment code which would result
                // in endless loops

                Thread misconceptionJudging = new Thread(){

                    public void run(){
                        judgeMisconceptions(exercise, getAnimator(), collectedExplanations, collectedNumbers, normalJudgement);
                        assessmentSucceeded[0] = true;
                        synchronized(Trakla2ExerciseApplet.this){
                            Trakla2ExerciseApplet.this.notify();
                        }
                    }
                };

                try{
                    misconceptionJudging.start();
                    synchronized(this){
                        wait(10000); //Misconception judging should not take more than 10 secs
                    }
                }catch(InterruptedException e){

                }

                if (assessmentSucceeded[0] == true){
                    MCQDialog mcqd = new MCQDialog("Research questionnaire",                            
                            "<b>Automaattisen virheentulkinnan tutkimus</b>. " +
                            "<small>Alla on listattu joukko selityksi palauttamallesi ratkaisulle. "+
                            "Selitykset on rakennettu automaattisesti ja haluamme nyt mitata "+
                            "kuinka hyvin tunnistus toimii. Valitse allaolevista vaihtoehdoista "+
                            "sopivin ja kirjoita tarvittaessa tarkennus tekstikenttn. "+
                            "Valitse sen mukaan mit ajattelit tehtv tehdess, vaikka tietisitkin nyt "+
                            "ettei vastauksesi ollut oikein. Vastaukset ksitelln anonyymisti ja niit "+
                            "kytetn kurssin tykalujen kehittmiseen tulevia kursseja varten. </small>"+
                            "</font><hr><font face=\"Arial, Helvetica\"> <b>Research questionnaire</b> <small>Below are listed a number of automatically generated possible explanations "+
                            "for your solution sequence. Choose the one that best explains "+
                            "how you thought about the algorithm <em>while solving</em> the exercise. The answers "+
                            "are used anonymously and are part of a study aiming to improve automatic feedback.</small>");
                            
                    String[] possibilities = new String[collectedExplanations.size() + 1];

                    collectedExplanations.toArray(possibilities);
                    possibilities[possibilities.length - 1] = "Mikn tarjotuista vaihtoehdoista ei ole riittvn lhell. <b>Kirjoitan tarkennuksen ruudun alalaidassa olevaan tekstikenttn.</b></p>None of the options is near enough, <b>I will write an explanation</b> in the text field below.";

//                    URL imUrl = Trakla2ExerciseApplet.class.getResource("/" + "pics/cases.jpg");
                    mcqd.addMCQ("Valitse jokin allaolevista vaihtoehdoista / Choose one of the options below", possibilities/*, new ImageIcon(imUrl)*/);
                    mcqd.setVisible(true);

                    //TODO: Add stuff for the rest of the mcqs

                    // These go to the submissionData-object
                    miscStudy = true;

                    offeredMisconceptions = collectedExplanations;
                    selectedMisconception = new Integer(mcqd.getSelections()[0]);
                    misconceptionFreeText = mcqd.getFreeInput();
                    offeredNumbers        = collectedNumbers;      
                    // end submissiondata
                }
            }


            int grade = resolveGrade(j);

            // show dialog
            Trakla2GradeDialog gd = new Trakla2GradeDialog(j, grade, this);
            gd.setVisible(true);
            if(gd.submitOk())
                super.refreshStatusbar(grade); //refresh the points and submissions in the applet's statusbar
            gradeButton.setEnabled(false);
            gradeButtonUpdatesDisabled = true;
            if(modelAnswerAfterGrading) {
                modelAnswerButton.setEnabled(true);
            }
            if(resetAfterGrading) {
                resetButton.setEnabled(true);
            }
        } else {
            Note.show(this, "Cannot judge this exercise. No method of judging given.");
        }
    }

    protected void modelAnswer() {
        ExerciseProperties lang = ExerciseProperties.getInstance();
        if (!submitted) {
            int showModelAnswer = JOptionPane.showConfirmDialog(this, lang.get("SHOW_MODEL_ANSWER_TEXT"), lang.get("SHOW_MODEL_ANSWER_TITLE"), JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE);
            if (showModelAnswer == JOptionPane.NO_OPTION) {
                return;
            } else {
                submitted = true;
            }
        }

        if(exercise instanceof SimulationExerciseModel) {
            Trakla2ModelAnswerFrame mf = new Trakla2ModelAnswerFrame(this, (SimulationExerciseModel)exercise);
            mf.getStructurePanel().setFontSize(getExercisePanel().getFontSize());
            gradeButton.setEnabled(false);

        } else {
            Note.show(this, "Cannot show model answer, none given.");
        }
        gradeButtonUpdatesDisabled = true;
        resetButton.setEnabled(true);
    }
    
    protected void compare() {
        ExerciseProperties lang = ExerciseProperties.getInstance();
        if (!submitted) {
            int showCompare = JOptionPane.showConfirmDialog(this, lang.get("SHOW_COMPARISON_TEXT"), lang.get("SHOW_COMPARISON_TITLE"), JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE);
            if (showCompare == JOptionPane.NO_OPTION) {
                return;
            } else {
                submitted = true;
            }
        }
    	
    	if (exercise instanceof ComparableExercise) {
    		ComparisonFrame cf = new ComparisonFrame(getAnimator(), (ComparableExercise)exercise);
    		cf.setVisible(true);
    		gradeButton.setEnabled(false);
    	} else {
    		Note.show(this, "This exercise is not comparable.");
    	}
        resetButton.setEnabled(true);
    }


    protected void reset() {
        long seed = 0;
        submitted = false;

        // XXX We don't use the server-provided seed after reset
        // XXX We should implement the reset-protocol at some point!
        seed = new Date().getTime();

        try {
            exercise = (SimulationExerciseModel)exercise.getClass().newInstance();
        } catch (Exception e) {
            Note.show(this, "ERROR: " + e.getMessage());
            Note.err(this, e.getMessage());
            Note.printStackTrace();
        }
        ((SimulationExercise)exercise).setSeed(seed);
        getExercisePanel().setExercise(exercise);
        getExercisePanel().reset();
        descriptionTextLabel.setText(exercise.getDescription());
        if(gradingEnabled) {
            gradeButton.setEnabled(true);
        }
        if(modelAnswerAfterGrading) {
            modelAnswerButton.setEnabled(false);
        }
        if(resetAfterGrading) {
            resetButton.setEnabled(false);
        }
        gradeButtonUpdatesDisabled = false;
        validate();
    }



    protected int getMaxPoints() {
        return maxPoints;
    }



    protected Map submit(int grade) {
    	SubmissionData data = new SubmissionData();
        data.setGrade(grade);
        data.setFingerprint(submitChecksum);
        data.setMaxPoints(getMaxPoints());
        StructureOperations so = new StructureOperations();
        so.setStructures(getStructures());
        so.setAnimator(getAnimator());
        data.setStudentSolution(so);
        
        /*StructureOperations so = new StructureOperations();
        so.setStructures(getStructures());
        so.setAnimator(getAnimator());*/
        
		try {
			URLConnection con = new URL(serverUrl).openConnection();
			ClientHttpRequest req = new ClientHttpRequest(con);
			req.setParameter("user_id", studentid);
			req.setParameter("points", "" + grade);
			req.setParameter("maximum_points", "" + getMaxPoints());
			req.setParameter("random_seed", "" + seed);
			req.setParameter("submit_checksum", submitChecksum);
			req.setParameter("exercise_id", exerciseId);
			req.setParameter("page", "Submit");
			req.setParameter("service", "direct");
			req.setParameter("component", "form");
			req.setParameter("exercise_class", exercise.getClass().getName());
			req.setParameter("formids", "user_id,exercise_id,points,maximum_points,random_seed,submit_checksum,exercise_class,file");

	        /*
	        if (miscStudy){
            	data.addPayload("offeredMisconceptions", offeredMisconceptions);
            	data.addPayload("offeredNumbers",        offeredNumbers);
            	data.addPayload("selectedMisconception", selectedMisconception);
            	data.addPayload("freeText"             , misconceptionFreeText);
        	}
        	*/

			
			ByteArrayOutputStream out = new ByteArrayOutputStream();
			ObjectOutputStream oout = new ObjectOutputStream(out);
			oout.writeObject(data);
	        req.setParameter("file", "file.obj", new ByteArrayInputStream(out.toByteArray()));
			InputStream is = req.post();
			DocumentBuilder db = DocumentBuilderFactory.newInstance().newDocumentBuilder();
			Document response = db.parse(is);
			NodeList params = response.getElementsByTagName("param");
			Map responseMap = new HashMap();
			for (int i=0;i<params.getLength();i++) {
				Element param = (Element) params.item(i);
				responseMap.put(param.getAttribute("name"), 
						param.getAttribute("value"));
			}
			NodeList message = response.getElementsByTagName("message");
			if (message.getLength() > 0) {
				responseMap.put("response_message", message.item(0).getTextContent());
			}
			NodeList feedback = response.getElementsByTagName("feedback");
			if (feedback.getLength() > 0) {
				responseMap.put("response_feedback", feedback.item(0).getTextContent());
			}
			submitted = true;
			System.out.println(responseMap);
			return responseMap;
		} catch (MalformedURLException e) {
			e.printStackTrace();
			return submitExceptionResponse(e);
		} catch (IOException e) {
			e.printStackTrace();
			return submitExceptionResponse(e);
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
			return submitExceptionResponse(e);
		} catch (SAXException e) {
			e.printStackTrace();
			return submitExceptionResponse(e);
		}
    }

    private Map submitExceptionResponse(Exception e) {
    	Map m = new HashMap();
    	m.put("status", "ERROR");
    	m.put("response_message", e.getMessage());
    	return m;
    }


    protected int resolveGrade(Judgement j) {
        float points = 0;
        float maxPoints = 0;
        do {
            points += (float)sum(j.getPoints());
            maxPoints += (float)j.getMax();
            j = j.next;
        } while (j != null);
        return (int) ((points/maxPoints) * getMaxPoints()) ;
    }



    protected FDT[] getStructures() {
        return getExercisePanel().getStructures();
    }



    private static int sum(int[] array) {
        int s = 0;
        for (int i = 0; i < array.length; i++) {
            s = s + array[i];
        }
        return s;
    }


    //---- begin research setting code (should be moved)
    
    private static final boolean containsBasecase(ArrayList collectedCases, int[] baseCases){
        for (int i=0; i<baseCases.length; i++){
            if (collectedCases.contains(new Integer(baseCases[i])))
                return true;
        }
        return false;
    }
    
    public static void judgeMisconceptions(SimulationExerciseModel exercise, Animator a, ArrayList collectedExplanations, ArrayList collectedNumbers, Judgement normalCase){
        int countUserStates = 0;
        String explanation = "Normal"; // The initial value is not used
        Judgement j;

        /*
         * The base limit is used to recognize situations where the misconceived
         * algorithm only gets a single step better than the normal one.
         * 
         * These should not be considered as misconceived algorithms
         */

        int limit = normalCase.getFinalUserState() + 1;

        KnownMisconceptions miscs = (KnownMisconceptions) exercise;

        AnimatedStructures[] as = miscs.createMisconceptionSequences();
        String[] nicknames      = miscs.getMisconceptionNicknames();
        String[] explanations   = miscs.getGuidanceStrings();

        Jury normalJury = new Jury();

        boolean limitHasErrors = false;
        a.rewind();

        while(a.hasNextOperation()){
            countUserStates++;
            a.redo();
        }                

        a.rewind();


        if (normalCase.getPoints()[0] > 0){

            if (normalCase.hasErrorStates() && 
                    countUserStates != normalCase.getFinalUserState()){
                explanation = "Joidenkin oikeiden askelten jlkeen ratkaisussasi oli tunnistamattomia askeleita.</font><hr><font face=\"Arial, Helvetica\"> The correct algorithm was followed to some point, but then there "+
                "was a set of unrecognizable steps";
            }else{
                explanation = "Seurasit oikeaa algoritmia johonkin pisteeseen saakka, mutta lopetit kesken. </font><hr><font face=\"Arial, Helvetica\"> The correct algorithm was followed to some point, but you did not finish the whole exercise.";
            }
        }else{
            explanation = "Automaattinen tunnistusalgoritmi ei pystynyt tunnistamaan virhettsi. (jos tiedt mist on kyse, niin kirjoita tekstikenttn) </font><hr><font face=\"Arial, Helvetica\"> The steps you took in the algorithm could not be automatically matched to any previously known misconception.";
        }

        collectedExplanations.add(explanation);
        collectedNumbers.add(new Integer(-1));

        if (normalCase.getPoints()[0] == normalCase.getMax()){
            //Filter fully correct ones
            explanation = "Palauttamasi ratkaisu oli tysin oikein / Your solution was fully correct.";

            collectedExplanations.clear();
            collectedExplanations.add(explanation);
            collectedNumbers.add(new Integer(-1));

        }else if ( normalCase.wasEmpty() ){

            // and completely empty ones
            explanation = "Palauttamasi ratkaisu oli tysin tyhj / Your submitted solution sequence was completely empty";

            collectedExplanations.clear();
            collectedExplanations.add(explanation);
            collectedNumbers.add(new Integer(-1));

        }else{

            for (int i=0; i<as.length; i++){
                j = null;

                a.rewind();

                if (exercise instanceof content.interfaces.JudgeBlocks){
                    j = normalJury.getJudgementJoinedFDTs(
                            exercise.getAnswer(),
                            a,
                            as[i].getStructures(),
                            as[i].getAnimator());
                }
                else{
                    j = normalJury.getJudgementSeparateFDTs(
                            exercise.getAnswer(),
                            a,
                            as[i].getStructures(),
                            as[i].getAnimator());
                }

                //Cutoff smaller scores

                if (j.getFinalUserState() > limit){
                    limit = j.getFinalUserState();

                    explanation = "";
                    collectedExplanations.clear();
                    collectedNumbers.clear();

                    limitHasErrors = true;

                }

                if (j.getFinalUserState() >= limit ){ // Better than existing ones

                    if (j.hasErrorStates()){          // If there are error states they are considered only if the others are also erroneous

                        if (limitHasErrors && (containsBasecase(collectedNumbers, ((KnownMisconceptions)exercise).getBaseCases()[i]) && j.getFinalUserState() == limit) == false){ //Only show erroneous if all erroneous 
                            if (j.isComplete()){
                                explanation = /*"(checkMe CE)["+*/explanations[i]+" <br>"+howProbable(j.getFinalUserState(),countUserStates)+" <small>" + j.getFinalUserState() + "/"+countUserStates+" user steps explained (not finished)</small>";

                                collectedExplanations.add(explanation);
                                collectedNumbers.add(new Integer(i));

                            }
                            else{
                                explanation = /*"(IE)["+*/explanations[i]+" <br>"+howProbable(j.getFinalUserState(),countUserStates)+" <small>" + j.getFinalUserState() + "/"+countUserStates+" user steps explained (not finished)</small>";

                                collectedExplanations.add(explanation);
                                collectedNumbers.add(new Integer(i));

                            }
                        }
                        else{

                     //       System.err.print("Discarding "+nicknames[i]+ " " + j.getFinalUserState() + "/"+countUserStates +" because of existing full explanations ");
                     //       System.err.println(containsBasecase(collectedNumbers, ((KnownMisconceptions)exercise).getBaseCases()[i]) == true ? "(basecase)":"(erroneous)");

                        }
                    }else { // no error states

                        if (limitHasErrors && explanation.length() > 0){
                            explanation = "";//Erroneous states diminished to a note
                            collectedExplanations.clear();

                      // COULD THROW EXCEPTION     for (int k=0; k<collectedNumbers.size(); k++){
                      //          System.err.println("Discarding "+nicknames[((Integer)collectedNumbers.get(k)).intValue()]+ "because of newly found better points");                            
                      //      }

                            collectedNumbers.clear();

                        }


                        //basecases matter only if the limit does not change
                        if ((containsBasecase(collectedNumbers, ((KnownMisconceptions)exercise).getBaseCases()[i]) && j.getFinalUserState() == limit) == false){

                            if (j.isComplete()){
                                explanation = /*"(CNE)["+*/explanations[i]+" <br>"+howProbable(j.getFinalUserState(),countUserStates)+" <small>" + j.getFinalUserState() + "/"+countUserStates+" user steps explained (finished)</small>";

                                collectedExplanations.add(explanation);
                                collectedNumbers.add(new Integer(i));


                            }else{
                                explanation = /*"(checkMe NCNE)["+*/explanations[i]+" <br>"+howProbable(j.getFinalUserState(),countUserStates)+" <small>" + j.getFinalUserState() + "/"+countUserStates+" user steps explained (not finished)</small>";

                                collectedExplanations.add(explanation);
                                collectedNumbers.add(new Integer(i));


                            }
                        }
                        limitHasErrors = false;
                    }
                }else{
  //                  System.err.println("Discarding "+nicknames[i]+ "because of existing better points");                            
                }
            }
        }

        
        
        /*
         * Debug code for showing the misconcepted sequences
         * 
            for (int k=0; k<collectedNumbers.size(); k++){
                JFrame jf = new JFrame();
                int l = ((Integer)collectedNumbers.get(k)).intValue();

                if (l != -1){
                    jf.getContentPane().add(new StructurePanel(as[l].getStructures(), as[l].getAnimator()));
                    jf.setTitle(nicknames[l]);
                    jf.setVisible(true);
                }
            }
         */

    }

    private static String howProbable(int score, int max){
        if (max < 5){
            if ( max-score == 0)
                return "<font color=\"green\"><b>Very good match</b></font>";
            else if (max-score < 2)
                return "<font color=\"orange\"><b>Possible match, but contains unexplained steps</b>.</font>";
            else
                return "<font color=\"red\"><b>Weak match, contains a lot of unexplained steps.</b></font>";
        
        }else{
            if ( max-score == 0)
                return "<font color=\"green\"><b>Very good match</b></font>";
            else if (max-score < 3)
                return "<font color=\"orange\"><b>Possible match, but contains unexplained steps.</b></font>";
            else
                return "<font color=\"red\"><b>Weak match, contains a lot of unexplained steps.</b></font>";
        }
    }



}

