// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
package applications.trakla2.ui;

import java.awt.*;
import java.awt.event.*;
import java.util.Map;
import javax.swing.*;

import applications.exerciseapplet.ModelAnswerPanel;


import matrix.uitools.ComparisonPanel;
import prototype.ui.GradeDialog;

import content.ExerciseProperties;
import content.assessment.Judgement;
import content.interfaces.ComparableExercise;
import content.interfaces.GradeDialogLayout;
import content.interfaces.SimulationExerciseModel;

public class Trakla2GradeDialog extends GradeDialog {

    private boolean submitOk = false;
    private int grade = 0;
    private Trakla2ExerciseApplet applet; 

    protected JButton submitButton;
    protected JButton contButton;
    protected JLabel buttonInfo;

    public Trakla2GradeDialog(Judgement j, int grade, Trakla2ExerciseApplet a) {
        super(j);
        
        this.applet = a;
        this.grade = grade;
        
        doDialogLayout();
        
    }
    
    private void doDialogLayout(){
        
        SimulationExerciseModel exercise = applet.getExercise();
    	JTabbedPane tabbedPane = new JTabbedPane();
    	
    	boolean showModelAnswer = true;
    	boolean showComparison = true;
    	
    	if (exercise instanceof GradeDialogLayout){
    		showModelAnswer = ((GradeDialogLayout)exercise).showModelAnswerInGradeDialog();
    		showComparison = ((GradeDialogLayout)exercise).showComparisonInGradeDialog();
    	}
    	
    	int selectedTabIndex = 0;
        
    	// Show answer comparison
    	if (showComparison && exercise instanceof ComparableExercise) {
        	ComparisonPanel comparisonPanel = new ComparisonPanel(applet.getExercisePanel().getAnimator(), (ComparableExercise)exercise);
        	tabbedPane.addTab("Answer comparison", comparisonPanel);
    	}
    	else {
    		// show disabled tab
    		tabbedPane.add("Answer comparison", new JPanel());
    		tabbedPane.setEnabledAt(tabbedPane.getTabCount()-1, false); 
    		selectedTabIndex++;
    	}
    	// Show model answer
    	if(showModelAnswer && exercise instanceof SimulationExerciseModel) {
    		ModelAnswerPanel modelPanel = new ModelAnswerPanel((SimulationExerciseModel)exercise);

    		// set font in model answer
    		Font font = applet.getExercisePanel().getFont();
    		int style = font.getStyle();
    		int size = font.getSize();
    		modelPanel.getStructurePanel().setFont(new Font(font.getName(), style, size));

    		// notify VisualKey that fontSize changed
            matrix.visual.VisualKey.fontSizeChanged(size);
            
        	tabbedPane.addTab("Model answer", modelPanel);
    	}
    	else {
    		// show disabled tab
    		tabbedPane.add("Model answer", new JPanel());
    		tabbedPane.setEnabledAt(tabbedPane.getTabCount()-1, false);
    	}

    	tabbedPane.setSelectedIndex(selectedTabIndex);
  	  	getContentPane().add(tabbedPane, BorderLayout.CENTER);

        validate();

        // Submit exercise
        submit();
        
        pack();
        Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
        setSize(Math.min(screen.width, getWidth()+15), Math.min(screen.height, getHeight()+25));
    }


    protected JTextArea makeTextArea() {

        JTextArea t = new JTextArea("", DEFAULT_ROWS, COLUMNS);
        t.enableInputMethods(false);
        t.setEditable(false);
        return t;
    }



    protected JPanel makeButtonPanel() {
        ExerciseProperties lang = ExerciseProperties.getInstance();

        JPanel buttonPanel = new JPanel();
        buttonPanel.setLayout(new FlowLayout());

        submitButton = new JButton(lang.get("SUBMISSION_RETRY_BUTTON"));
        submitButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                submit();
            }
        });
        submitButton.setEnabled(false);
        buttonPanel.add(submitButton);

        contButton = new JButton(lang.get("SUBMISSION_CLOSE_BUTTON"));
        contButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                close();
            }
        });
        contButton.setEnabled(true);
        buttonPanel.add(contButton);


        return buttonPanel;
    }


    protected void submit() {
        ExerciseProperties lang = ExerciseProperties.getInstance();
        
        if (applet == null) throw new IllegalStateException("Trakla2ExerciseApplet not set");
        try {
            text.append(lang.get("SUBMISSION_WAITING"));
            validate();
            Map receipt = applet.submit(grade);
            String message = (String) receipt.get("response_message"); 
            if (receipt != null && !receipt.isEmpty() && receipt.get("status").equals("OK")) {
                submitOk = true;
                text.setText(lang.get("SUBMISSION_SUCCESS") + "\n\n");
                if (message != null && !message.trim().equals("")) {
                	text.append(message+"\n\n");
                }
                text.append(buf.toString());
		        text.append(lang.get("SUBMISSION_GRADE_MESSAGE") + " ");
                if (receipt.containsKey("submission-grade")) {
                    text.append(receipt.get("submission-grade") + " / ");
                } else {
                    text.append(grade + " / ");
                }
                text.append(applet.getMaxPoints() + ".");
                submitButton.setEnabled(false);
            } else {
                text.append(lang.get("SUBMISSION_FAILED"));
                text.append("\n" + message);
                submitButton.setEnabled(true);
            }
            
            // increase the size of textarea if there are more rows than in default area
            int lineCount = getLineCount(text.getText());
            if (lineCount > DEFAULT_ROWS){
            	text.setRows(Math.min(MAX_ROWS, lineCount));
            }

            //buttonPanel.remove(submitButton);
            //submitButton.setEnabled(false);
            contButton.setText(lang.get("SUBMISSION_CLOSE_BUTTON"));
            //	    contButton.repaint();
            //buttonPanel.repaint();
            //repaint();
            //buttonPanel.validate();
            validate();
        } catch (Exception e) {
            text.append(lang.get("SUBMISSION_FAILED") + e.getMessage());
            e.printStackTrace();
        }

    }

    public boolean submitOk() {
        return submitOk;
    }

    protected void close() {
        super.close();
    }

    // disabled functionality from superclass
    protected void text() {}
    
    /**
     * @param str
     * @return the line count of str
     */
    private int getLineCount(String str){
    	int count = 0;
    	int ind = 0;
    	while (ind < str.length()){
    		ind = 1 + str.indexOf("\n", ind);
    		count++;
    		if (ind == 0){
    			break;
    		}
    	}
    	return count;
    }

}
