// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
package applications.trakla2.util;

/*
 * Created on Apr 29, 2005
 * a group of very simple routines for comparing string arrays and finding out their
 * differences. Not currently used by other parts of the code.
 */

public class DifferenceUtils {

    /**
     * This method creates a difference array from the given parameter arrays.
     * The difference array holds all the elements from the first array not found in the second one
     * at the same positions. The first array should not have any null values.
     * 
     * @param a The first array to be compared
     * @param b The second array to be compared
     * @return the difference array, holding all the elements of a that do not match those of b
     */
    
    public static String[] createDifference(String[] a, String[] b){
        
        int smallerSize = Math.min(a.length, b.length);
        int biggerSize  = Math.max(a.length, b.length);
        
        String[] c = new String[biggerSize];
        
        for (int i=0; i<smallerSize; i++)
            if (!a[i].equals(b[i]))
                c[i] = a[i];
                
        return c;
      }

    /**
     * This method counts the differences between the given parameter arrays.
     * 
     * @param a The first array to be compared
     * @param b The second array to be compared
     * @return the difference between the arays as an integer
     */
          
      public static int countDifference(String[] a, String[] b){
        
        int sum = 0;
        int smallerSize = Math.min(a.length, b.length);
        int biggerSize  = Math.max(a.length, b.length);
        
        for (int i=0; i<smallerSize; i++)
            if (a==null || !a[i].equals(b[i]))
                sum++;
            
        sum += biggerSize - smallerSize;    
        return sum;
      }

      /**
       * This method counts the differences between the given parameter arrays.
       * Any null values in the first array are ignored. (filtered out)
       * 
       * @param a The first array to be compared
       * @param b The second array to be compared
       * @return the difference between the arays as an integer
       */

      public static int countDifferenceWithHoles(String[] a, String[] b){
        
        int sum = 0;
        int smallerSize = Math.min(a.length, b.length);
        int biggerSize  = Math.max(a.length, b.length);
        
        for (int i=0; i<smallerSize; i++)
            if (a!=null && !a[i].equals(b[i]))
                sum++;
            
        if (a.length == biggerSize)
            for (int i=smallerSize; i<biggerSize; i++)
                if (a[i]!=null)
                    sum++;
        return sum;
      }
    
      /**
       * This method calculates the number of non-null elements in the given array
       * 
       * @param a the array to be inspected
       * @return the number of non-null elements.
       */
      
      public static int countNonNulls(String[] a){
        
        int sum = 0;
        
        for (int i=0; i<a.length; i++)
            if (a!=null)
                sum++;
            
        return sum;
      
      }
}
