// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
/**
 * Linearizer class for creating String representations of FDTs
 * Implemented using the visitor pattern.
 * 
 * Typically used by calling the <b>linearize</b> method with
 * the FDT to be linearized.
 * 
 * @author Otto Seppl
 * @version 1.0
 * Created on Apr 28, 2005
 *
 */
package applications.trakla2.util;

import matrix.structures.FDT.Array;
import matrix.structures.FDT.LinkedList;
import matrix.structures.FDT.Graph;
import matrix.structures.FDT.Struct;
import matrix.structures.FDT.Tree;
import matrix.structures.FDT.FDT;
import matrix.structures.FDT.indexed.StaticLinkedList;
import matrix.structures.FDT.substructures.Vertex;

import matrix.structures.util.FDTVisitAssistant;
import matrix.structures.util.FDTVisitor;

public class StringLinearizer implements FDTVisitor{
    
    
	/**
	 * This method is the one that should be called to linearize the gived FDT structure.
     * The rest of the methods in this class are called by the FDTVisitAssistant
     *  
	 * @param fdt The FDT to be linearized to a single String
	 * @return the result of the linearization
	 */
          
      public static String linearize(FDT fdt){
         StringLinearizer linearizer = new StringLinearizer();
         return FDTVisitAssistant.accept(fdt, linearizer).toString();
      }

      //********** VISITOR METHODS ***********//
            
      public Object visitNonFDT(Object o){
      	return "["+o.toString()+"]";
      }
      
      public Object visit(FDT fdt){
      	return "["+fdt.getElement().toString()+"]";
      }
      
      public Object visit(Array a) {

        StringBuffer resultBuffer = new StringBuffer();
        
        int aFirst = a.getFirst();
        int aLast  = a.getLast();
       
        Object aObj;

        for(int i = aFirst; i <= aLast; i++) {
        	aObj = a.getObject(i);
            resultBuffer.append(FDTVisitAssistant.accept(aObj, this));
        }
        
        return resultBuffer;
      }
      
      public Object visit(Struct s){
          StringBuffer resultBuffer = new StringBuffer();
                   
          Object[] fields = s.getFields();
          resultBuffer.append("[");
          for(int i = 0; i < fields.length; i++) {        	  
              resultBuffer.append(FDTVisitAssistant.accept(fields[i], this));
              resultBuffer.append(" ");
          }
          resultBuffer.append("]");
          return resultBuffer;    	  
      }

      
      public Object visit(LinkedList a) {
        StringBuffer returnValue = new StringBuffer();
        
         Object aObj = a.getElement();
         if(aObj == null)
            returnValue.append(" ");
         else
            returnValue.append(FDTVisitAssistant.accept(aObj, this));
         
         while(a.hasNext()){
            a    = a.getNext();
            aObj = a.getElement();
            
            if(aObj == null)
                returnValue.append(" ");
             else
                returnValue.append(FDTVisitAssistant.accept(aObj, this));
         }
         
         return returnValue;
      }


        public Object visit(StaticLinkedList a) {
        	a.setFirst();
        	Object aObj;
        	StringBuffer returnValue = new StringBuffer();
            
            while(a.isInList()) {
            	aObj = a.currValue();
                if(aObj == null)
                    returnValue.append(" ");
                 else
                    returnValue.append(FDTVisitAssistant.accept(aObj, this));
                a.next();
            }
            return returnValue;
        }

      public Object visit(Tree a) {
      	StringBuffer returnValue = new StringBuffer();
        
        if (a==null)
            return " ";

        Object aObj = a.getElement();
        if(aObj == null)
            returnValue.append(" ");
         else
            returnValue.append(FDTVisitAssistant.accept(aObj, this));
        
        
        Tree[] aTree = a.getSubTrees();
        for(int i = 0; i < aTree.length; i++)
            returnValue.append(FDTVisitAssistant.accept(aTree[i], this));

        return returnValue;
      }
        
      public Object visit(Graph a) {
      	StringBuffer returnValue = new StringBuffer();
        
        if(a == null)
            return " ";
        
        Vertex[] aVertices = a.getVertices();
        Object ao;

        for(int i = 0; i < aVertices.length; i++) {
          ao = aVertices[i].getElement();
          if(ao == null)
            returnValue.append(" ");
          else
          	returnValue.append(FDTVisitAssistant.accept(ao, this));

        }
        return returnValue;
      }
/*
 * The following code is not used, and is thus commented out
 * 
      public static int countDifference(String a, String b){
        StringTokenizer aTokens = new StringTokenizer(a, "[]"); 
        StringTokenizer bTokens = new StringTokenizer(b, "[]");
        
        int sum = 0;
        
        while(aTokens.hasMoreElements() && bTokens.hasMoreElements()){
            if (!aTokens.nextToken().equals(bTokens.nextToken()))
                sum++;
        }

        while(aTokens.hasMoreElements()){
            aTokens.nextToken();
            sum++;
        }

        while(bTokens.hasMoreElements()){
            bTokens.nextToken();
            sum++;
        }
        
        return sum;
      }  
*/
      
}
