// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
package applications.trakla2.viewer;

import java.applet.Applet;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;

import matrix.util.configuration.MatrixConfiguration;

//public class TestApplet extends Applet implements ActionListener {
public class TestApplet extends Applet {
    
    private InetAddress address;
    private Label display;
    private Socket socket;
    
    private int port = -1;

    private String md5          = "";
    private String host         = "";
    private String fileLocation = "";
    
    public void init() {
	MatrixConfiguration.setCodebaseURL(getCodeBase().toString());
	MatrixConfiguration.getInstance();

        md5          = this.getParameter("md5");
        fileLocation = this.getParameter("file");
        
        try {
            port = Integer.parseInt(this.getParameter("port"));
        } catch (Exception e) {
            System.out.println("ERROR: No port parameter specified.");
            return;
        }
                
        host = getCodeBase().getHost();
        
        try {
            address = InetAddress.getByName(host);
        } catch (UnknownHostException e) {
            System.out.println("Sanity check failed - no such internet address");
            return;
        }
                
        //Set up the UI.
        GridBagLayout gridBag = new GridBagLayout();
        GridBagConstraints c = new GridBagConstraints();
        setLayout(gridBag);
        
        Label l1 = new Label("Socket created, internet address is "+address+"\nMD5 is "+md5+ "\nfile is " + fileLocation, Label.CENTER);
        c.anchor = GridBagConstraints.SOUTH;
        c.gridwidth = GridBagConstraints.REMAINDER;
        gridBag.setConstraints(l1, c); 
        add(l1);
        
        display = new Label("(no data received yet)", Label.CENTER);
        c.anchor = GridBagConstraints.NORTH;
        c.weightx = 1.0;
        c.fill = GridBagConstraints.HORIZONTAL;
        gridBag.setConstraints(display, c); 
        add(display);
        
        c.anchor = GridBagConstraints.SOUTH;
        c.gridwidth = 1;
        c.weightx = 0.0;
        c.weighty = 1.0;
        c.fill = GridBagConstraints.NONE;
                
        Button button = new Button("Get data");
        gridBag.setConstraints(button, c); 
        add(button);
        
        //button.addActionListener(this);
        button.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent evt) {
                    doIt();
                }
            });
    }
    
    public Insets getInsets() {
        return new Insets(4,4,5,5);
    }
    
    public void paint(Graphics g) {
        Dimension d = getSize();
        Color bg = getBackground();
        
        g.setColor(bg);
        g.draw3DRect(0, 0, d.width - 1, d.height - 1, true);
        g.draw3DRect(3, 3, d.width - 7, d.height - 7, false);
    }
    
    void doIt() {

        Object result = null;
        
        BufferedWriter os=null;
        ObjectInputStream is=null;
     
        
        try {
            System.out.println("TestApplet: trying to connect to " + host + ":" + port);
            socket = new Socket(host, port);
        } catch (IOException e) {
            System.out.println("Couldn't connect to host");
            return;
        }
        
        System.out.println("Socket created.");
        
        try{
            os = new BufferedWriter( new OutputStreamWriter(socket.getOutputStream()));
            is = new ObjectInputStream(socket.getInputStream());
        }catch(IOException e){
            System.err.println("Couldn't open the streams");
            e.printStackTrace();
        }
        
        try { 
            //Send in the md5 sum            
            os.write(md5);
            os.newLine();
            
            //Send in the file location            
            os.write(fileLocation);
            os.newLine();            
            
            os.flush();
                        
            result = is.readObject();
	    applications.trakla2.util.SubmissionViewer.openTrakla2Answer(null, (applications.trakla2.datalogging.ExerciseData)result);

        } catch (IOException e) {
            System.out.println("Something went wrong");
            e.printStackTrace();
            return;
        } catch (ClassNotFoundException e) {
            System.out.println("Class not found");
            e.printStackTrace();
            return;
            
        }
        
        display.setText("Got an object of class : "+result.getClass().getName());
    }
    
    /*
    public void actionPerformed(ActionEvent event) {        
        doIt();
    }
    */
}

