// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
package applications.trakla2.viewer;

import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.ServerSocket;
import java.net.Socket;
import java.security.NoSuchAlgorithmException;

import applications.trakla2.datalogging.SubmissionData;

public class TestServer {

    public static void main(String[] args) {
        ServerSocket ss=null;
        Socket s=null;
        
        String dataDirectory = "";
        String propertyFile = "";
        if(args != null && args.length >= 1) 
            propertyFile = args[0];
        else {            
            System.out.println("Usage: java application.trakla2.viewer.TestServer <properties file>");
            return;
        }
        
        // load properties
        try {
        BufferedInputStream in = new BufferedInputStream(new FileInputStream(args[0]));
        System.getProperties().load(in);
        in.close();
        } catch(IOException e) {
            System.err.println("Error reading property file: "+e);
            return;            
        }
        
        try{
            ss = new ServerSocket(Integer.parseInt(System.getProperty("submissionviewer.server.port")));
            dataDirectory = System.getProperty("trakla2.server.DATADIR");
            System.out.println("Starting exercise test server at port " + ss.getLocalPort() + ". Data directory is " + dataDirectory);
        }catch(IOException e){
            System.err.println("Socket creation failed "+e);
            return;
        }
        
        while(true){
            try{
                s = ss.accept();
                
                ObjectOutputStream os=null;
                BufferedReader is=null;
                
                os = new ObjectOutputStream (s.getOutputStream());
                is = new BufferedReader(new InputStreamReader(s.getInputStream()));
                
                //Read in the md5 sum                
                String md5 = is.readLine();
                
                //Read in the file location                
                String fileLocation = is.readLine();
                fileLocation = dataDirectory + fileLocation;
                
                ObjectInputStream ois = new ObjectInputStream(new FileInputStream(fileLocation));
                Object o = ois.readObject();
                
                // Calculate and compare here the md5 sum to the one received from the applet
                String fingerprint = "";
		if(o instanceof SubmissionData) {
		    fingerprint = getFingerPrint((SubmissionData)o);
		}
                
                if (fingerprint.equals(md5)){
                                
                  os.writeObject(o);
                  os.flush();

                }
                else{
		    //log the bad things
		    System.out.println("Invalid fingerprint, required: " + fingerprint + " got: " + md5);
                }
                  
                //close the connection
                os.close();
                is.close();
                s.close();
                
            }catch(IOException e){
                System.err.println("User session failed "+e);                    
            }
            catch(ClassNotFoundException e){
                System.err.println("Unknown local class type "+e);                                    
            }
        }
        
    }

    public static String getFingerPrint(SubmissionData data) {
	String print = "";
	try {
		String[] parts = data.getExerciseId().split("\\.");
        String courseId = "";
        int roundNumber = -1;
        int exerciseNumber = -1;
        if (parts.length >= 3) {
            courseId = parts[0];
            roundNumber = Integer.parseInt(parts[1]);
            exerciseNumber = Integer.parseInt(parts[2]);
        }
        
	    print = FingerprintHelper.calculateMD5(
						   FingerprintHelper.getFingerprint(courseId,
										    roundNumber, 
										    exerciseNumber,
										    data.getStudentid(),
										    data.getMaxPoints(), data.getExerciseClassName()));
	    
	} catch(NoSuchAlgorithmException e) {
	    e.printStackTrace();
	}
	return print;
    }
}
