// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/* $Id: ExerciseProperties.java,v 1.2 2008/01/23 08:32:12 tred Exp $ */

package content;

import java.io.IOException;
import java.util.Properties;
import matrix.util.Note;

/**
 * ExerciseProperties class is designed mainly for storing exercise specific strings used in
 * localizing exercises, although it can be used to store other types of variables in the future if needed.
 * 
 * The functionality applies the singleton design pattern.
 * 
 * @author Otto Seppl
 */

public class ExerciseProperties {

    private static ExerciseProperties instance;
    private Properties strings;
    
    /**
     * This constructor creates a new ExerciseProperties object. This should only be
     * done via the getInstance-method to ensure that there exists only one instance
     * of the class.
     * 
     * Upon initialization the default values are loaded from the file "exercisestrings.properties"
     */
    
    private ExerciseProperties() {
        strings = new Properties();
        try{
            strings.load(this.getClass().getClassLoader().getResourceAsStream("exercisestrings.properties"));
            
            //strings.list(System.out);
        } catch (IOException e) {
            Note.err(this, "file exercisestrings.properties not found");
            System.exit(-1);
        }
    }
    
    /**
     * Overrides or adds values to the set of strings kept by the class.
     * 
     * @param key The id of the string 
     * @param value The actual string
     */
    
    public void setString(String key, String value) {
        strings.setProperty(key, value);
    }
    
    /**
     * Returns a string thet corresponds to the given id.
     * 
     * @param key The id of the desired string.
     * @return the string that mathes the given id or an error message
     */
    
    public String get(String key) {
        return strings.getProperty(key, "[" + key + " not defined]");
    }
    
    /**
     * This method returns a singleton instance of the class.
     * 
     * @return An instance of the class.
     */
    
    public static ExerciseProperties getInstance() {
        if (instance == null)
            instance = new ExerciseProperties();
        
        return instance;
    }    
}
