// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package content.exercises;

import java.util.*;

import content.ExerciseProperties;
import content.interfaces.ConfigureVisualType;
import content.interfaces.ModelAnswerNames;
import content.interfaces.SimulationExerciseModel;
import content.interfaces.StyledExercise;
import content.interfaces.SwapBehaviour;

import matrix.animation.Animator;
import matrix.simulation.VisualTypeConf;
import matrix.structures.CDT.probe.*;
import matrix.structures.FDT.*;
import matrix.structures.FDT.LinkedList;
import matrix.structures.FDT.probe.*;

public class Analysis_Ranking2 implements SimulationExerciseModel, StyledExercise, ModelAnswerNames, ConfigureVisualType, SwapBehaviour {
	
	//BinSearchTree bst = null;
	Table itemsToBeRanked;
	LinkedList l1;
	private String[] keys;
	private String[] correctOrder;
	public static final boolean DEBUG = false;
	
	public Analysis_Ranking2() {
	}
	
	/**
	 * In this exercise a drag operation performs a swap
	 * @return
	 */
	public boolean getSwapBehaviour() {
		return true;
	}
	
	long seed = 1;
	
	/**
	 * Get the value of seed.
	 * @return value of seed.
	 */
	public long getSeed() {
		return seed;
	}
	
	/**
	 * Set the value of seed.
	 * @param v  Value to assign to seed.
	 */
	public void setSeed(long  v) {
		this.seed = v;
	}
	
	String[][] gf = new String[][] { // 1 < k < c
			new String[] { " log(log(n)) ", " ln(ln(n)) ", " lg(lg(n)) ", " c*lg(lg(n)) ", " log(k*log(n)) " },
			new String[] { " log(n) ", " ln(n) ", " lg(n) ", " log(2^log(n^k)) ", " log((2*k)^log(n)) " },
			new String[] { " (log(n))^k ", " log(n^log(n^2)) ", " log(n^log(n^log(n))) ", " log(n^(log(n))^c) ", " c*(log(n))^k " },
			new String[] { " 10^lg(n) ", " 2^(log(n))-1 ", " e^ln(n) ", " 2^ln(n) ", " 2^log(n) " },
			new String[] { " log(n^n) ", " (log(n^n))/2 ", " 0.5*log(n^n) ", "  c*log(n^n) ", " k*n*log(n)+c " },
			new String[] { " (log(n))^log(n) ", " (lg(n))^lg(n) ", " (ln(n))^ln(n) ", " (log(n))^((lg(n))/lg(2)) ", " (lg(n)/lg(2))^(lg(n)/lg(2)) " },
			new String[] { " c^n ", " k^n ", " 10^n ", " (n^k)*(c^n) ", " c^n+n^k " },
			new String[] { " n! ", " (n+1)! ", " (2*n)! ", " 0.1*n! ", " (n/2)! " },
			new String[] { " n^n ", " 2^(n*log(n)) ", " (1+c)^(n*log(n)) ", " (n^n)/n ", " n^(n-1)) " }
	};
	
	private String getInstance(int i, int n) {
		return gf[i][n];
	}
	
	private String getRandomInstance(int i) {
		Random R = new Random(seed);
		int n = Math.abs(R.nextInt() % 5);
		return getInstance(i, n);
	}
	
	private String[] randomRankedItems(long seed) {
		correctOrder = new String[] { "loglogn", "logn", "(logn)^k", "n", "(logn)^lognn", "k1log(n^n)", "c^n", "n!", "n^n" };
		String[] randomOrder = new String[] { "loglogn", "logn", "(logn)^k", "n", "(logn)^lognn", "k1log(n^n)", "c^n", "n!", "n^n" };
		for (int i=0; i<correctOrder.length; i++) {
			correctOrder[i] = getRandomInstance(i);
			randomOrder[i] = correctOrder[i];
		}
		return shuffle(randomOrder, correctOrder.length);
	}
	
	private String[] shuffle(String[] s, int length) {
		Random R = new Random(seed);
		for (int i=0; i<s.length; i++) {
			int n = Math.abs(R.nextInt() % length);
			String tmp = s[i];
			s[i] = s[n];
			s[n] = tmp;
		}
		return s;
	}
	
	public FDT[] init() {
		keys = randomRankedItems(seed);
		itemsToBeRanked = new Table(keys) { 
			private static final long serialVersionUID = -4601651163164028058L;

			public void setObject(Object obj, int address) {
				if (obj instanceof QueueImpl) {
					super.setObject(null, address);
					return;
				}
				super.setObject(obj, address);
			} 
		};
		l1 = new QueueImpl();
		FDT[] tmp  = { itemsToBeRanked, l1 };
		return tmp;
	}
	
	// layouts
	public String[] getStructureVisualisations() {
		return new String[] { "array", "list" };
	}
	
	public String[] getModelAnswerVisualisations() {
		return new String[] { "list"  };
	}
	
	// structure names
	public String[] getStructureNames() {
        ExerciseProperties lang = ExerciseProperties.getInstance();

        String[] tmp = { lang.get("ANALYSIS_RANKING_2_INPUT_TITLE"), 
                         lang.get("ANALYSIS_RANKING_2_SOLUTION_TITLE") };
        
		return tmp;
	}
	
	public String getDescription() {
		return "1 < k < c";
	}
	
	public FDT[] solve() {
		LinkedList modelSolution = new LinkedListImpl();
		LinkedList last;
		last = modelSolution;
		
		Animator a = Animator.getActiveAnimator();
		for (int i=0; i<correctOrder.length; i++) {
			a.startOperation();
			last.setNext(last.getNewNode(correctOrder[i]));
			last = last.getNext();
			a.endOperation();
		}
		FDT[] ans = { modelSolution.getNext() };
		return ans;
	}
	
	public FDT[] makeModelAnswer() {
		return solve();
	}
	
	public FDT[] getInitialStructures() {
		Table tbl = new Table(keys);
		return new FDT[] { tbl, new LinkedListImpl() };
	}
	
	// ?????
	public FDT[] getSimulatedStructures() {
		return new FDT[] { itemsToBeRanked, l1 };
	}
	
	public FDT[] getAnswer() {
		FDT[] tmp = { l1 };
		return tmp;
	}
	
	public String[] getModelAnswerNames() {
        ExerciseProperties lang = ExerciseProperties.getInstance();

        String[] s = { lang.get("ANALYSIS_RANKING_2_MODEL_SOLUTION_TITLE") };
		return s;
	}
	
	// ?????
	public SimulationExerciseModel getModelAnswer() {
		return null;
	}
	
	public VisualTypeConf[] conf() {
		VisualTypeConf ritems = new VisualTypeConf();
		ritems.enable("matrix.visual.VisualKey", VisualTypeConf.HIGHLIGHT_OPERATION);
		ritems.enable("matrix.visual.VisualKey", VisualTypeConf.DRAG_OPERATION);
		VisualTypeConf lis = new VisualTypeConf();
		lis.enable("matrix.visual.VisualList", VisualTypeConf.DROP_OPERATION);
		lis.enable("matrix.visual.VisualList", VisualTypeConf.HIGHLIGHT_OPERATION);
		return new VisualTypeConf[] { ritems, lis};
	}
	
	static final long serialVersionUID = 4685858434892753682L;
}
