//Matrix - algorithm simulation system
//Copyright (C) 2000- 2007 Matrix group
//(See the AUTHORS file and the author tag below for more information
//on the authors.)

//This program is free software; you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation; either version 2 of the License, or
//(at your option) any later version.

//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.

//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package content.exercises;

import java.util.*;

import content.ExerciseProperties;
import content.interfaces.ConfigureVisualType;
import content.interfaces.ModelAnswerNames;
import content.interfaces.MultipleQuestions;
import content.interfaces.SimulationExerciseModel;
import content.interfaces.StyledExercise;

import matrix.animation.Animator;
import matrix.simulation.VisualTypeConf;
import matrix.structures.FDT.*;
import matrix.structures.FDT.probe.*;

public abstract class Analysis_Selection implements MultipleQuestions, SimulationExerciseModel, StyledExercise, ModelAnswerNames, ConfigureVisualType {

    protected Table options, answer, correctSelections;
    protected String[] keys;
    public static final boolean DEBUG = false;

    public Analysis_Selection() {
    }

    // ---- MultipleQuestions ---- //
    public int numberOfQuestions() { return getNumberOfItems(); } // This exercise has two separate questions
    private int currentAnswer = 0;
    // this method is invoked before the exercise is judged
    // nrOfModelAnswer=0 for the first question; nrOfModelAnswer=1 for the second question
    public void changeAnimator(int i) { 
        currentAnswer = i;
    }

    long seed = 1;

    /**
     * Get the value of seed.
     * @return value of seed.
     */
    public long getSeed() {
        return seed;
    }

    /**
     * Set the value of seed.
     * @param v  Value to assign to seed.
     */
    public void setSeed(long  v) {
        this.seed = v;
    }

    public FDT[] init() {
        keys = getSelectionOptions();
        options = new Table(keys);
        answer = new Table(getNumberOfItems());
        for (int i=0; i<getNumberOfItems(); i++)
            answer.setObject(" ", i);
        correctSelections = new Table(getNumberOfItems());
        FDT[] tmp  = { options, answer };
        return tmp;
    }

    // layouts
    public String[] getStructureVisualisations() {
        return new String[] { "array", "array" };
    }

    public String[] getModelAnswerVisualisations() {
        return new String[] { "array"  };
    }

    // structure names
    public String[] getStructureNames() {
        ExerciseProperties lang = ExerciseProperties.getInstance();

        String[] tmp = { 
                lang.get("ANALYSIS_SELECTION_INPUT_TITLE"),
                lang.get("ANALYSIS_SELECTION_SOLUTION_TITLE")};

        return tmp;
    }

    protected abstract String[] getSelectionOptions();

    protected abstract int getNumberOfItems(); 

    protected abstract String getItem(int i);

    protected abstract int getNumberOfVariations();

    public abstract String getDescription(); 

    public FDT[] solve() {
        Animator a = Animator.getActiveAnimator();
        a.startOperation();
        correctSelections.setObject(correctSelections.getObject(currentAnswer),currentAnswer);
        a.endOperation();
        return new FDT[] { new Table(correctSelections.getObject(currentAnswer).toString()) };
    }

    protected int[] randomPositions() {
        int[] positions = new int[getNumberOfItems()];
        Random rand = new Random(getSeed());
        String usedNumbers = "";
        for (int i=0; i < getNumberOfItems();) {
            int num = rand.nextInt(getNumberOfVariations());
            if (usedNumbers.indexOf("" + num) == -1) {
                usedNumbers += "," + num;
                positions[i] = num;
                i++;
            }
        }
        return positions;
    }

    public FDT[] makeModelAnswer() {
        return new FDT[] { correctSelections };
    }

    public FDT[] getInitialStructures() {
        Table tbl = new Table(keys);
        return new FDT[] { tbl, new Table(getNumberOfItems()) };
    }

    // ?????
    public FDT[] getSimulatedStructures() {
        return new FDT[] { options, answer };
    }

    public FDT[] getAnswer() {
        return new FDT[] { new Table(answer.getObject(currentAnswer).toString()) };
    }

    public String[] getModelAnswerNames() {
        ExerciseProperties lang = ExerciseProperties.getInstance();

        String[] s = { lang.get("ANALYSIS_SELECTION_MODEL_ANSWER_TITLE") };
        return s;
    }

    // ????? What?? Is not needed??
    /*public SimulationExerciseModel getModelAnswer() {
	return null;
    }*/

    public VisualTypeConf[] conf() {
        VisualTypeConf sitems = new VisualTypeConf();
        sitems.enable("matrix.visual.VisualKey", VisualTypeConf.HIGHLIGHT_OPERATION);
        sitems.enable("matrix.visual.VisualKey", VisualTypeConf.DRAG_OPERATION);
        VisualTypeConf ans = new VisualTypeConf();
        ans.enable("matrix.visual.VisualArrayComponent", VisualTypeConf.DROP_OPERATION);
        ans.enable("matrix.visual.VisualArrayComponent", VisualTypeConf.HIGHLIGHT_OPERATION);
        return new VisualTypeConf[] { sitems, ans };
    }

    static final long serialVersionUID = -4358793927004125354L;
}
