// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package content.exercises;

import content.ExerciseProperties;

public class Analysis_Selection_RecursiveCode extends Analysis_Selection {

    private String[] codeFragments = {
            "int f(int n)\n   if (n <= 1)\n      return 1;\n   else\n      return f(n-1) + f(n-2);",
            "int f(int n)\n   if (n == 0)\n      return 1;\n   else\n      return n*f(n-1);",
            "int f(int x, int n)\n   switch (n):\n      case 0: return 1;\n      case 1: return x;\n      else:\n         if (odd(n)) then\n            return x*f(x*x, n/2)\n         else\n            return f(x*x, n/2)",
            "void f(link t) // link t is a pointer to binary tree\n   if (t != null)\n      visit(t); // O(1) operation\n      f(t->left)\n      f(t->right)",
            "void f(int l, int r) // n = r - l\n   if (r > l) then\n      m = (r+l)/2;\n      f(l,m)\n      f(m+1,r)\n      for (i=m..l) do\n         b[i] = a[i]\n      for (j=m+1..r) do\n         b[r+m+1-j] = a[j]\n      i = l\n      j = r\n      for (k=l..r) do\n         if (b[i] < b[j]) then\n            a[k] = b[i]\n            i = i + 1\n         else\n            a[k] = b[j]\n            j = j - 1",
            "void f(int l, int r) // n = r - l\n   if (r > l) then\n      i = partition(l,r); // O(n) operation\n      f(l,i-1)\n      f(i+1,r)",
            "int f(int l, int k, int r) // n = r - l\n   i = -1\n   if (l < r) then\n      i = partition(l,r) // O(n) operation\n      if (k<i) then\n         return f(l,k,i-1)\n      else if (k>i) then\n         return f(i+1,k,r)\n   return i" };

    private String[] correctAnswers = null;

    private String CONSTANT_TIME;
    private String LOGARITHMIC;
    private String LINEAR;
    private String LOGLINEAR;
    private String QUADRATIC;
    private String EXPONENTIAL;
    
    public Analysis_Selection_RecursiveCode() {
        super();
        ExerciseProperties lang = ExerciseProperties.getInstance();
        
        CONSTANT_TIME = " " + lang.get("ANALYSIS_SELECTION_CONSTANT_TIME")+ " ";
        LOGARITHMIC   = " " + lang.get("ANALYSIS_SELECTION_LOGARITHMIC")+ " ";
        LINEAR        = " " + lang.get("ANALYSIS_SELECTION_LINEAR")+ " ";
        LOGLINEAR     = " " + lang.get("ANALYSIS_SELECTION_LOGLINEAR")+ " ";
        QUADRATIC     = " " + lang.get("ANALYSIS_SELECTION_QUADRATIC")+ " ";
        EXPONENTIAL   = " " + lang.get("ANALYSIS_SELECTION_EXPONENTIAL")+ " ";
               
        correctAnswers = new String[]{ EXPONENTIAL, LINEAR, LOGARITHMIC,
                LINEAR, LOGLINEAR, QUADRATIC, QUADRATIC };
    }

    protected int getNumberOfVariations() {
        return codeFragments.length;
    }
    
    protected String[] getSelectionOptions() {
        return new String[] { CONSTANT_TIME, LOGARITHMIC, LINEAR, LOGLINEAR, QUADRATIC, EXPONENTIAL };
    }

    protected int getNumberOfItems() {
        return 3    ;
    }

    protected String getItem(int i) {
        return codeFragments[i];
    }

   
    public String getDescription() {
        String descr = "";
        int[] positions = randomPositions();
        for (int i=0; i < getNumberOfItems(); i++) {
            descr += i + ")\n" + getItem(positions[i]);
            if (i != getNumberOfItems()-1)
                descr += "\n\n";
            correctSelections.setObject(correctAnswers[positions[i]], i);
        }
        return descr;
    }

    static final long serialVersionUID = -916829416201596913L;
}
