// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
package content.exercises;

import java.util.Random;

import matrix.structures.CDT.probe.BinSearchTree;
import matrix.structures.FDT.FDT;
import matrix.structures.FDT.probe.Key;
import matrix.structures.FDT.probe.Table;
import matrix.structures.util.MatrixComparable;
import content.exercises.structures.ExerBin;
import content.exercises.structures.ExerBinSearchTree_DuplicatesRight;

/**
 * Binary search tree delete exercise in which the possible duplicates are 
 * in the right subtree and, thus, substitutes are searched from the right subtree
 * (the smallest key in the right subtree) if a node to be removed has two children. 
 * @author kstalnac
 *
 */
public class BST_Delete_DuplicatesRight extends BST_Delete {

    public BST_Delete_DuplicatesRight() {
        super();
    }
    
    public FDT[] init() {

        ExerBin bt;
        keys = "";
        // Generate tree
        int largest;
        int smallest;
        Random R = new Random(seed);
        do {
            // Generate good BST tree

            // 15-20 keys
            int n = Math.abs(R.nextInt()%6);
            // Can contain duplicates
            S = //matrix.util.RandomKey.createNoDuplicateUppercaseRandomKey(15)+
                matrix.util.RandomKey.createNoDuplicateUppercaseRandomKey(R,15+n);
            // Maximum height 6

            t = new Table(S);
            //bt = new ExerBin();

            bt = new ExerBin((new Key(""+S.charAt(0))));

            // Insert keys
            for (int i=1; i <S.length(); i++)
                bt.realInsert((MatrixComparable)new Key(""+S.charAt(i)));

            // Calc heights
            largest=bt.getLargestHeight();
            smallest=bt.getMinimumHeight();
            if (DEBUG)
                System.out.println("iteration "+smallest+":"+largest);

        } while (smallest < 3 || largest > 6);

        String newS= new String();
        String sep="";
        for (int i=0; i <MAARA; i++, sep=" ,") {
            char c =S.charAt(((MAARA-i-1)*S.length())/MAARA);
            newS+=""+ c;
            keys+= sep + c;
        }

        //keys = newS;
        table = new Table(newS);
        /* This is the only difference to the init method of the BST_Delete exercise
         * BinSearchTree changed to ExerBinSearchTree_DuplicatesRight
         */
        bt2 = new ExerBinSearchTree_DuplicatesRight();
        bt2.insert(t);
        FDT[]  s = { bt2 };
        return s;
    }

    public FDT[] getInitialStructures() {
        Table tbl = new Table(S);
        BinSearchTree ex = new ExerBinSearchTree_DuplicatesRight();
        ex.insert(tbl);
        return new FDT[] { ex };
    }

    private static final long serialVersionUID = -6585174550965584166L;
}
