// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package content.exercises;

import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.util.*;

import content.ExerciseProperties;
import content.exercises.structures.ActivationRecord;
import content.interfaces.*;
import matrix.animation.Animator;
import matrix.simulation.VisualTypeConf;
import matrix.structures.CDT.probe.*;
import matrix.structures.CDT.probe.Stack;
import matrix.structures.FDT.*;
import matrix.structures.FDT.LinkedList;
import matrix.structures.FDT.probe.*;
import matrix.structures.code.probe.PseudoCode;
import matrix.util.*;

public class BST_TraverseIn implements ComparableExercise, StyledExercise,
		ModelAnswerNames, ConfigureVisualType, LayoutExercise {

	BinSearchTree bst = null;

	LinkedList l1;

	private String keys;

	private PseudoCode vCode = null;

	private Stack callStack = null;

	public static final boolean DEBUG = false;

	private static final String[] frameLabels = { "tnode" };
	private static final int      frameCols = 1;
	private static final String[] code = {
			"_1 1 PROCEDURE TraverseInorder(TreeNode tnode) _/1",
			"",
			"_2 2 IF tnode != NULL _/2",
			"_3 3     TraverseInorder(tnode.getLeft()) _/3",
			"_4 4     visit(tnode) _/4",
			"_5 5     TraverseInorder(tnode.getRight()) _/5",
			"",
			"_6 6 RETURN _/6"
	};

	public BST_TraverseIn() {}

	long seed = 1;

	/**
	 * Get the value of seed.
	 * 
	 * @return value of seed.
	 */
	public long getSeed() {
		return seed;
	}

	/**
	 * Set the value of seed.
	 * 
	 * @param v
	 *            Value to assign to seed.
	 */
	public void setSeed(long v) {
		this.seed = v;
	}

	public FDT[] init() {
		bst = new BinSearchTree();
		keys = RandomKey.createNoDuplicateUppercaseRandomKey(
				new java.util.Random(seed), 10);
		Table t = new Table(keys);
		bst.insert(t);
		l1 = new QueueImpl();
		FDT[] tmp = { bst, l1 };
		return tmp;
	}

	public String[] getStructureVisualisations() {
		return new String[] { "layered tree", "list" };
	}

	public String[] getModelAnswerVisualisations() {
		return new String[] { "layered tree", "list", "array", "pseudocode" };
	}

	public String[] getStructureNames() {
        ExerciseProperties lang = ExerciseProperties.getInstance();
        String[] s = { 
                lang.get("BST_TRAVERSEIN_BST_TITLE"),
                lang.get("BST_TRAVERSEIN_VISITED_TITLE"),
        };
		return s;
	}

	public String getDescription() {
		return "";
	}

	private void setCodeLine(int line, Animator a) {
		a.startOperation();
		vCode.setLocation(line);
		a.endOperation();
	}

	private void pushStackFrame(BinaryTree node, int rline) {
		String value = "null";
		if (node != null && node.getElement() != null)
			value = node.getElement().toString();
		String[] values = { value };
		callStack.insert(new ActivationRecord(frameLabels, values, rline,
				frameCols));
	}

	private void prettyTraversal(BinaryTree t, QueueImpl l, int rline) {
		Animator a = Animator.getActiveAnimator();

		a.startOperation();
		pushStackFrame(t, rline);
		vCode.setLocation(1);
		a.endOperation();

		setCodeLine(2, a);
		if (t != null) {
			setCodeLine(3, a);
			prettyTraversal(t.getLeft(), l, 4);
			a.startOperation();
			callStack.pop(null);
			vCode.setLocation(4);
			a.endOperation();

			a.startOperation();
			l.insert(t.getElement().toString());
			vCode.setLocation(5);
			a.endOperation();

			prettyTraversal(t.getRight(), l, 6);
			a.startOperation();
			callStack.pop(null);
			vCode.setLocation(6);
			a.endOperation();
		} else {
			setCodeLine(6, a);
		}
	}

	public FDT[] solve() {
		LinkedList sl1, last1;
		FDT[] fd = getInitialStructures();
		sl1 = (LinkedList) fd[1];// new LinkedListImpl();
		last1 = sl1;
		// Table tbl = new Table(keys);
		BinaryTree bt = (BinaryTree) ((BinSearchTree) fd[0]).getElement();// (BinaryTree)
																			// bst.getElement();

		StringTokenizer pre = new StringTokenizer(Traverse.traverseInOrder(bt));
		String tmp;
		Animator a = Animator.getActiveAnimator();
		while (pre.hasMoreTokens()) {
			a.startOperation();
			try {
				tmp = pre.nextToken();
				last1.setNext(last1.getNewNode(tmp));
				last1 = last1.getNext();
			} catch (NoSuchElementException e) {
				Note
						.err(this,
								"Error during solve, one StringTokenizer shorter than pre");
			}
			a.endOperation();
		}
		FDT[] ans = { sl1.getNext() };
		return ans;
	}

	public FDT[] makeModelAnswer() {
        ExerciseProperties lang = ExerciseProperties.getInstance();

		FDT[] ini = getInitialStructures();
		QueueImpl lst;
		BinaryTree bt;

		bt = (BinaryTree) ((BinSearchTree) ini[0]).getElement();
		lst = (QueueImpl) ini[1];
		callStack = new Stack(8);
		if (vCode == null) {
			vCode = new PseudoCode();
			vCode.setCodeLines(code);
			vCode.setCodeName( lang.get("BST_TRAVERSEIN_PSEUDOCODE_NAME") );
		}
		vCode.setLocation(0);
		prettyTraversal(bt, lst, -1);
		Animator.getActiveAnimator().startOperation();
		callStack.pop(null);
		vCode.setLocation(0);
		Animator.getActiveAnimator().endOperation();
		return new FDT[] { bt, lst, callStack, vCode };
	}

	public FDT[] getInitialStructures() {
		Table tbl = new Table(keys);
		BinSearchTree tr = new BinSearchTree();
		tr.insert(tbl);
		return new FDT[] { tr, new QueueImpl() };
	}

	public FDT[] getSimulatedStructures() {
		return new FDT[] { bst, l1 };
	}

	public FDT[] getAnswer() {
		FDT[] tmp = { l1 };
		return tmp;
	}

	public String[] getModelAnswerNames() {
        ExerciseProperties lang = ExerciseProperties.getInstance();
        String[] s = { 
                lang.get("BST_TRAVERSEIN_BST_MODEL_TITLE"),
                lang.get("BST_TRAVERSEIN_VISITED_MODEL_TITLE"),
                lang.get("BST_TRAVERSEIN_CALLSTACK_MODEL_TITLE"),
                lang.get("BST_TRAVERSEIN_PSEUDOCODE_MODEL_TITLE")
        };
		return s;
	}

	public SimulationExerciseModel getModelAnswer() {
		return null;
	}

	public VisualTypeConf[] conf() {
		VisualTypeConf tre = new VisualTypeConf();
		tre.enable("matrix.visual.VisualKey", VisualTypeConf.HIGHLIGHT_OPERATION);
		tre.enable("matrix.visual.VisualKey", VisualTypeConf.DRAG_OPERATION);
		VisualTypeConf lis = new VisualTypeConf();
		lis.enable("matrix.visual.VisualList", VisualTypeConf.DROP_OPERATION);
		lis.enable("matrix.visual.VisualList", VisualTypeConf.HIGHLIGHT_OPERATION);
		VisualTypeConf sta = new VisualTypeConf();
		sta.setVisualConfValue("matrix.visual.VisualArray", VisualTypeConf.ROTATED, "true");
		VisualTypeConf cod  = new VisualTypeConf();
		return new VisualTypeConf[] { tre, lis, sta, cod };
	}

	public GridBagConstraints[] getConstraints() {
		GridBagConstraints c = new GridBagConstraints();
		c.insets    = new Insets(10, 10, 10, 10);
		c.gridwidth = GridBagConstraints.REMAINDER;
		
		return new GridBagConstraints[] {c, c};
	}

	public GridBagConstraints[] getModelAnswerConstraints() {
		Insets i = new Insets(10, 10, 10, 10);
		GridBagConstraints r[] = new GridBagConstraints[4];
		GridBagConstraints c;

		// Tree
		c = new GridBagConstraints();
		c.gridx = 0;
		c.gridy = 0;
		c.insets = i;
		c.gridheight = GridBagConstraints.REMAINDER;
		c.anchor = GridBagConstraints.PAGE_START;
		r[0] = c;
		// List
		c = new GridBagConstraints();
		c.gridx = 1;
		c.gridy = 0;
		c.insets = i;
		r[1] = c;
		// Stack
		c = new GridBagConstraints();
		c.gridx = 2;
		c.gridy = 0;
		c.insets = i;
		c.gridheight = GridBagConstraints.REMAINDER;
		c.anchor = GridBagConstraints.PAGE_START;
		r[2] = c;
		// Code
		c = new GridBagConstraints();
		c.gridx = 1;
		c.gridy = 1;
		c.insets = i;
		r[3] = c;

		return r;
	}
	
	public GridBagConstraints[] getComparisonConstraints() {
		return null; // use default
	}

	static final long serialVersionUID = 7537147638713474323L;

	public int[] getCompareIndices() {
		return new int[] {1};
	}

	public int[] getGradeIndices(){
        return null; // Used only in recovery mode
	}

	public boolean canRecover(){
		return false;
	}
	
	public String[] getCompareStructureVisualisations(){
		return null; // use default
	}
	
	public String[] getCompareStructureNames(){
		return null; // use default
	}    

	public VisualTypeConf[] getCompareVisualTypeConf(){
		return null; // use default
	}
}
