//Matrix - algorithm simulation system
//Copyright (C) 2000- 2007 Matrix group
//(See the AUTHORS file and the author tag below for more information
//on the authors.)
//
//This program is free software; you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation; either version 2 of the License, or
//(at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/* $Id: BST_search.java,v 1.17 2009/06/18 10:12:51 lhakulin Exp $ */

package content.exercises;

import java.awt.Color;

import java.util.Random;
import java.util.StringTokenizer;

import content.ExerciseProperties;
import content.interfaces.ComparableExercise;
import content.interfaces.ConfigureVisualType;
import content.interfaces.ModelAnswerNames;
import content.interfaces.MultipleQuestions;
import content.interfaces.SimulationExerciseModel;
import content.interfaces.SwapBehaviour;
import content.exercises.structures.ExerMinimizedRBTree;

import matrix.animation.Animator;
import matrix.simulation.VisualTypeConf;
import matrix.structures.CDT.probe.RedBlackNode;
import matrix.structures.FDT.FDT;
import matrix.structures.FDT.probe.Key;
import matrix.structures.FDT.probe.Table;
import matrix.structures.util.MatrixComparable;

public class BST_search implements MultipleQuestions, SimulationExerciseModel, 
								ModelAnswerNames, ConfigureVisualType, SwapBehaviour,
								ComparableExercise {
    ExerMinimizedRBTree user_BST_found, user_BST_failed;
    ExerMinimizedRBTree model_BST_found, model_BST_failed;
    String S_found = null;
    String S_failed = null;
    String keyToBeSearched = null;

    public static final boolean DEBUG = false;

    public BST_search() {
    }

    /**
     * In this exercise a drag operation performs a swap
     * @return
     */
    public boolean getSwapBehaviour() {
        return true;
    }

    long seed = 1;

    /**
     * Get the value of seed.
     * @return value of seed.
     */
    public long getSeed() {
        return seed;
    }

    /**
     * Set the value of seed.
     * @param v  Value to assign to seed.
     */
    public void setSeed(long  v) {
        this.seed = v;
    }


    public FDT[] init() {
    	
    	int randomTries = 200;
        Random R = new Random(seed);
        
        // if innerNodeFound == true, the search must end in an inner node in the found tree
        // otherwise the search must end in an inner node in the failed tree
        boolean innerNodeFound = R.nextBoolean();

        for (int triesInd=0; triesInd < randomTries; triesInd++) {
	        int n = Math.abs(R.nextInt()%6);
	        // Can contain duplicates
	        String head1, head2;
	        head1 = "ABCDE";
	        head2 = "ZYXWV";
	        String S = matrix.util.RandomKey.createNoDuplicateUppercaseRandomKey(R,14)+
	        matrix.util.RandomKey.createNoDuplicateUppercaseRandomKey(R,n);
	        if (n <= 3) {
	            S_failed = head1 + S;
	            S_found = head2 + S;
	        } else {
	            S_failed = head2 + S;
	            S_found = head1 + S;
	        }
	        // --- S can contain only 24 character in order to make sure there is at least one missing key --- //
	        user_BST_found = new ExerMinimizedRBTree();
	        user_BST_failed = new ExerMinimizedRBTree();
	
	        Table tbl[] = (Table[])getInitialStructures();
	
	        // found table
	        for (int i=0; i<tbl[0].size(); i++) {
	            user_BST_found = (ExerMinimizedRBTree)user_BST_found.superInsert(tbl[0].getObject(i));
	        }
	        // failed table
	        for (int i=0; i<tbl[1].size(); i++) {
	            user_BST_failed = (ExerMinimizedRBTree)user_BST_failed.superInsert(tbl[1].getObject(i));
	        }
	
	        // search a key not in the tree user_BST_failed
	        String inOrder = user_BST_failed.inOrder().trim(); // BST in sorted order
	        String allkeys = "K,L,M,N,O,P,Q,R,S,T,U,V,W,X,Y,Z,J,I,H,G,F,E,D,C,B,A";
	
	        StringTokenizer st = new StringTokenizer(allkeys, ",");
	
	        while (st.hasMoreTokens()) {
	            keyToBeSearched = st.nextToken();
	            if (inOrder.indexOf(keyToBeSearched) != -1)
	                continue;
	            break;
	        }
	
	        S_found += keyToBeSearched;
	        user_BST_found = (ExerMinimizedRBTree)user_BST_found.superInsert(new matrix.structures.FDT.probe.Key(keyToBeSearched));
	        
	        if (innerNodeFound && !isRootOrLeaf(user_BST_found, new Key(keyToBeSearched))){
	        	break;
	        }
	        if (!innerNodeFound && isFailedSearchInnerNode(user_BST_failed, new Key(keyToBeSearched))){
	        	break;
	        }

        }
        // Color found tree black
        user_BST_found.color(Color.black);

        // Color failed tree red
        user_BST_failed.color(Color.red);

        FDT[]  s = { user_BST_found, user_BST_failed };
        return s;
    }
    
    public String[] getStructureNames() {
        ExerciseProperties lang = ExerciseProperties.getInstance();
        String[] s = { lang.get("BST_SEARCH_TREE_1_LABEL"), lang.get("BST_SEARCH_TREE_2_LABEL") };
        return s;
    }


    public String getDescription() {
        ExerciseProperties lang = ExerciseProperties.getInstance();
        return lang.get("BST_SEARCH_EX_DESC_PART_1") +" "+ keyToBeSearched +" "+ lang.get("BST_SEARCH_EX_DESC_PART_2");
    }

    // ---- MultipleQuestions ---- //
    public int numberOfQuestions() { return 2; } // This exercise has two separate questions
    private int nrOfModelAnswer = 0;
    // this method is invoked before the exercise is judged
    // nrOfModelAnswer=0 for the first question; nrOfModelAnswer=1 for the second question
    public void changeAnimator(int i) { 
        nrOfModelAnswer = i;
    }

    public FDT[] solve() {
        //Note.out(this, "Solve");

        model_BST_found = new ExerMinimizedRBTree();
        model_BST_failed = new ExerMinimizedRBTree();

        Animator a = Animator.getActiveAnimator();

        Table tbl[] = (Table[])getInitialStructures();

        // found table
        for (int i=0; i<tbl[0].size(); i++) {
            model_BST_found = (ExerMinimizedRBTree)model_BST_found.superInsert(tbl[0].getObject(i));
        }
        // failed table
        for (int i=0; i<tbl[1].size(); i++) {
            model_BST_failed = (ExerMinimizedRBTree)model_BST_failed.superInsert(tbl[1].getObject(i));
        }

        // Color found tree black
        model_BST_found.color(Color.black);

        // Color failed tree red
        model_BST_failed.color(Color.red);

        a.startOperation();
        if (nrOfModelAnswer == 0)
            model_BST_found.search(new Key(keyToBeSearched));
        else
            model_BST_failed.search(new Key(keyToBeSearched));
        a.endOperation();

        return (nrOfModelAnswer == 0) ? new FDT[] { model_BST_found } : new FDT[] { model_BST_failed };
    }


    public String[] getModelAnswerNames() {
        return getStructureNames();
    }

    public FDT[] makeModelAnswer() {
        //Note.out(this, "make model answer");
        FDT[] models = new FDT[2];
        changeAnimator(0);
        models[0] = solve()[0];
        changeAnimator(1);
        models[1] = solve()[0];
        changeAnimator(0);
        return models;
    }

    public FDT[] getInitialStructures() {
        return new Table[] { new Table(S_found), new Table(S_failed) };
    }

    public FDT[] getSimulatedStructures() {
        return new FDT[] { user_BST_found, user_BST_failed };
    }

    public FDT[] getAnswer() {
        //Note.out(this, "get answer");
        return (nrOfModelAnswer == 0) ? new FDT[] { user_BST_found } : new FDT[] { user_BST_failed };
    }

    public VisualTypeConf[] conf() {
        VisualTypeConf tree1 = new VisualTypeConf();
        tree1.enable("matrix.visual.VisualLayeredTreeComponent", VisualTypeConf.HIGHLIGHT_OPERATION);

        VisualTypeConf tree2 = new VisualTypeConf();
        tree2.enable("matrix.visual.VisualLayeredTreeComponent", VisualTypeConf.HIGHLIGHT_OPERATION);

        return new VisualTypeConf[] { tree1, tree2 };
    }
    
    /* ComparableExercise interface */
    
	public int[] getCompareIndices(){
        return new int[] {0, 1};	
	}

	public int[] getGradeIndices(){
        return null; // Used only in recovery mode
	}

	public boolean canRecover(){
		return false;
	}
	
	public String[] getCompareStructureVisualisations(){
		return null; // use default
	}
	
	public String[] getCompareStructureNames(){
		return null; // use default
	}    
	
	public VisualTypeConf[] getCompareVisualTypeConf(){
		return null; // use default
	}

    
    /**
     * Searches a key from the RBTree and returns true if the node containing
     * the key is the root or a leaf node.
     * @param tree - the tree
     * @param key - the key to be searched
     * @return true if the node containing the key is root or a leaf
     */
    private boolean isRootOrLeaf(ExerMinimizedRBTree tree, Key key) {
    	int level = 0;
        RedBlackNode root = (RedBlackNode)tree.getElement();
        while (!root.getElement().equals(key)) {
        	level++;
            if (((MatrixComparable)root.getElement()).gt((MatrixComparable)key)) {
                if (root.getLeft() == null){
                    return false;
                }
                root = (RedBlackNode)root.getLeft();
            } else {
                if (root.getRight() == null){
                    return false;
                }
                root = (RedBlackNode)root.getRight();
            }
        }
        if ((root.getLeft() == null && root.getRight() == null) || level == 0){
        	return true;
        }
        return false;
    }
    
    /**
     * Searches a key from the RBTree and returns true if the search
     * is finished in inner node and the key is not found in the tree
     * @param tree - the tree
     * @param key - the key to be searched
     * @return true if the search stops in inner node and the key is not found
     */
    private boolean isFailedSearchInnerNode(ExerMinimizedRBTree tree, Key key) {
        RedBlackNode root = (RedBlackNode)tree.getElement();
        while (!root.getElement().equals(key)) {
            if (((MatrixComparable)root.getElement()).gt((MatrixComparable)key)) {
                if (root.getLeft() == null){
                	if (root.getRight() != null){
                		return true;
                	}
                	else {
                		return false;
                	}
                }
                root = (RedBlackNode)root.getLeft();
            } else {
                if (root.getRight() == null){
                	if (root.getLeft() != null){
                		return true;
                	}
                	else {
                		return false;
                	}
                }
                root = (RedBlackNode)root.getRight();
            }
        }
        return false;
    }

    static final long serialVersionUID = -7576022670921264541L;

}
