// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/* $Id: BTree_insert.java,v 1.15 2009/06/18 10:12:51 lhakulin Exp $ */

package content.exercises;

import java.util.Random;

import content.ExerciseProperties;
import content.interfaces.ButtonExercise;
import content.interfaces.ComparableExercise;
import content.interfaces.ConfigureVisualType;
import content.interfaces.ModelAnswerNames;
import content.interfaces.SimulationExerciseModel;
import content.interfaces.SwapBehaviour;
import content.exercises.structures.ExerBTree;

import matrix.animation.Animator;
import matrix.simulation.VisualTypeConf;
import matrix.structures.FDT.FDT;
import matrix.structures.FDT.probe.Table;
import matrix.util.Note;


public class BTree_insert implements SimulationExerciseModel, ModelAnswerNames, ButtonExercise,
                                   ConfigureVisualType, SwapBehaviour, ComparableExercise {
    Table t;
    ExerBTree btree;
    ExerBTree model_btree;
    public static final boolean DEBUG = false;
    // model solution animator marker
    String marker = "BTree_insert";
    private String S = null;

    public BTree_insert() {
    }

    /**
     * In this exercise a drag operation performs a swap
     * @return
     */
    public boolean getSwapBehaviour() {
        return true;
    }

    long seed = 1;

    /**
     * Get the value of seed.
     * @return value of seed.
     */
    public long getSeed() {
        return seed;
    }

    /**
     * Set the value of seed.
     * @param v  Value to assign to seed.
     */
    public void setSeed(long  v) {
        this.seed = v;
    }



    public FDT[] init() {
        Random R = new Random(seed);
	// test initial conditions
	/*
        do {
	    */
            // 15-20 keys
            int n = Math.abs(R.nextInt()%6);
            // Can contain duplicates
            S = matrix.util.RandomKey.createNoDuplicateUppercaseRandomKey(R,15)+
                matrix.util.RandomKey.createNoDuplicateUppercaseRandomKey(R,n);
	    /*
            // Maximum height 6
            testBST = new BinSearchTree();
            for(int i = 0; i < S.length(); i++)
                testBST.insert(new Key(""+S.charAt(i)));
            if (DEBUG)
                System.out.println("iteration - height "+testBST.getHeight());
        } while(testBST.getHeight() > 6);
	*/
        t = new Table(S);
        btree = new ExerBTree();
        FDT[]  s = { t, btree };
        return s;
    }

    public String[] getStructureNames() {
        ExerciseProperties lang = ExerciseProperties.getInstance();
        String[] s = { lang.get("BTREE_INSERT_INPUT_TITLE"), lang.get("BTREE_INSERT_TREE_TITLE") };
        return s;
    }

    public String getDescription() {
        return "";
    }

    public SimulationExerciseModel getModelAnswer() {
        return this;
    }

    public String[] getModelAnswerNames() {
        ExerciseProperties lang = ExerciseProperties.getInstance();
        String[] s = { lang.get("BTREE_INSERT_TREE_MODEL_TITLE") };
        return s;
   }


    public FDT[] solve() {
        Note.out(this, "Solve");
        model_btree = new ExerBTree();
        Table tbl = (Table)getInitialStructures()[0];
        Animator a = Animator.getActiveAnimator();
        for (int i=0; i<tbl.size(); i++) {
            a.startOperation();
            model_btree.insert(tbl.getObject(i));
            a.endOperation();
        }
        Note.out(this, "Solved");
        return new FDT[] { (FDT)model_btree };
    }

    public FDT[] makeModelAnswer() {
	return solve();
    }


    public FDT[] getAnswer() {
        FDT[] s = { btree };
        return s;
    }

    public FDT[] getInitialStructures() {
        return new FDT[] { new Table(S), new ExerBTree() };
    }

    public FDT[] getSimulatedStructures() {
        return new FDT[] { t, btree };
    }

    // ---------------------------------- ButtonExercise ---------------------------------- //

    public String[] buttonNames() {
        ExerciseProperties lang = ExerciseProperties.getInstance();
        String[] s = { lang.get("BTREE_INSERT_SPLIT_BUTTON_TEXT") };
        return s;
    }

    public String[] buttonCommands() {
        String[] s = { "reflectSelectedVisualType(exerciseReflectionCommand(split))" };
        return s;
    }

    // ---------------------------------- ConfigureVisualType  ---------------------------------- //

    public VisualTypeConf[] conf() {
        VisualTypeConf tabl = new VisualTypeConf();
        tabl.enable("matrix.visual.VisualKey", VisualTypeConf.HIGHLIGHT_OPERATION);
        tabl.enable("matrix.visual.VisualKey", VisualTypeConf.DRAG_OPERATION);
        VisualTypeConf tre = new VisualTypeConf();
        tre.enable("matrix.visual.VisualKey", VisualTypeConf.DROP_OPERATION);
        tre.enable("matrix.visual.VisualKey", VisualTypeConf.DRAG_OPERATION);
        tre.enable("matrix.visual.VisualKey", VisualTypeConf.HIGHLIGHT_OPERATION);
        tre.enable("matrix.visual.VisualArrayComponent", VisualTypeConf.DROP_OPERATION);
        tre.enable("matrix.visual.VisualArrayComponent", VisualTypeConf.HIGHLIGHT_OPERATION);
        //tre.enable("matrix.visual.VisualLayeredTreeComponent", VisualTypeConf.DROP_OPERATION);
        tre.enable("matrix.visual.VisualLayeredTreeComponent", VisualTypeConf.HIGHLIGHT_OPERATION);

        return new VisualTypeConf[] { tabl, tre};
    }

    /* ComparableExercise interface */
    
	public int[] getCompareIndices(){
        return new int[] {0};	
	}

	public int[] getGradeIndices(){
        return null; // Used only in recovery mode
	}

	public boolean canRecover(){
		return false;
	}
	
	public String[] getCompareStructureVisualisations(){
		return null; // use default
	}
	
	public String[] getCompareStructureNames(){
		return null; // use default
	}    
	
	public VisualTypeConf[] getCompareVisualTypeConf(){
		return null; // use default
	}

    static final long serialVersionUID = -3735922270591531741L;
}
