// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/* $Id: DFS.java,v 1.44 2009/06/18 12:43:51 lhakulin Exp $ */

package content.exercises;

import java.awt.Color;
import java.util.Random;

import matrix.animation.Animator;
import matrix.simulation.VisualTypeConf;
import matrix.structures.FDT.FDT;
import matrix.structures.FDT.probe.DirectedGraphImpl;
import matrix.structures.FDT.substructures.Vertex;
import matrix.util.Note;
import content.ExerciseProperties;
import content.exercises.structures.ExerVertex;
import content.exercises.structures.ExerVertexVisitableKey;
import content.interfaces.ConfigureVisualType;
import content.interfaces.GradeDialogLayout;
import content.interfaces.ModelAnswerNames;
import content.interfaces.ComparableExercise;
import content.interfaces.StyledExercise;
import content.interfaces.SwapBehaviour;

public class DFS implements ComparableExercise,StyledExercise, ModelAnswerNames, ConfigureVisualType,
							SwapBehaviour, GradeDialogLayout {
    private DirectedGraphImpl user;
    private DirectedGraphImpl model;
    private final String keys = "ABCDEFGHI";
    private int randomWanted = 3;
    private int randomTries = 100;
    private ExerVertex ModelVertex[];
    private String[] NodeList;
    public static final boolean DEBUG = false;
    
    public DFS() {
        
    }
    
    /**
     * In this exercise a drag operation performs a swap
     * @return
     */
    public boolean getSwapBehaviour() {
        return true;
    }
    
    public VisualTypeConf[] conf() {
        VisualTypeConf graph = new VisualTypeConf();
        graph.enable("matrix.visual.VisualKey", VisualTypeConf.HIGHLIGHT_OPERATION);  
        return new VisualTypeConf[] { graph };
    }
    
    
    // styled exercise
    
    public String[] getStructureVisualisations() {
        String[] s = { "kamada-kawai point graph" };
        return s;
    }
    
    
    public String[] getModelAnswerVisualisations() {
        String[] s = { "kamada-kawai point graph" };
        return s;
    }
    
    long seed = 1;
    
    /**
     * Get the value of seed.
     * @return value of seed.
     */
    public long getSeed() {
        return seed;
    }
    
    /**
     * Set the value of seed.
     * @param v  Value to assign to seed.
     */
    public void setSeed(long  v) {
        this.seed = v;
    }
    
    
    
    public FDT[] init() {
        
        int l = keys.length();
        Random R = new Random(seed);
        int i,n;
        
        
        ExerVertex V[] = new ExerVertex[l];
        ModelVertex = new ExerVertex[l];
        NodeList = new String[l];
        user = new DirectedGraphImpl();
        model = new DirectedGraphImpl();
        
        // Create vertices
        for(i=0; i<l; i++) {
            V[i]=new ExerVertex();
            V[i].setElement(new ExerVertexVisitableKey(""+keys.charAt(i), V[i]));
            ModelVertex[i]=new ExerVertex();
//            ModelVertex[i].setElement(new Key(""+keys.charAt(i)));
            ModelVertex[i].setElement(new ExerVertexVisitableKey(""+keys.charAt(i), ModelVertex[i]));

            NodeList[i] = "" + ((ExerVertexVisitableKey) V[i].getElement()).getLabel() + ": ";
        }
        
        // Path from root node to every node
        for(i=1; i<l; i++) {
            n = Math.abs(R.nextInt()%i);
            V[n].addSuccessor(V[i]);
            ModelVertex[n].addSuccessor(ModelVertex[i]);
            NodeList[n] += ((ExerVertexVisitableKey) V[i].getElement()).getLabel() + " ";
            V[i].addSuccessor(V[n]);
            ModelVertex[i].addSuccessor(ModelVertex[n]);
            NodeList[i] += ((ExerVertexVisitableKey) V[n].getElement()).getLabel() + " ";
        }
        
        // Add random nodes
        int randomCreated = 0;
        for(i=0; i<randomTries; i++) {
            n = Math.abs(R.nextInt()%l);
            if (DEBUG)
                System.out.println("i:"+i%l+" n:"+n);
            if ((!V[i%l].hasSuccessor(V[n])) && (i%l != n)) {
                V[i%l].addSuccessor(V[n]);
                NodeList[i%l] += ((ExerVertexVisitableKey)V[n].getElement()).getLabel() + " ";
                ModelVertex[i%l].addSuccessor(ModelVertex[n]);
                
                V[n].addSuccessor(V[i%l]);
                NodeList[n] += ((ExerVertexVisitableKey)V[i%l].getElement()).getLabel() + " ";
                ModelVertex[n].addSuccessor(ModelVertex[i%l]);
                randomCreated++;
                if (randomCreated >= randomWanted) break; 
            } 
        }
        
        user.setVertices(V);
        model.setVertices(ModelVertex);
        FDT[]  s = { user };
        return s;
    }
    
    public FDT[] init(Object input) {
        return init();
    }
    
    
    public String[] getStructureNames() {
        ExerciseProperties lang = ExerciseProperties.getInstance();

        String[] s = { lang.get("DFS_GRAPH_TITLE") };
        return s;
    }
    
    
    public String getDescription() {
        String S= "Start with node: " + keys.charAt(0) +"\n\n";
        for (int i=0; i<keys.length(); i++)
            S+=NodeList[i]+"\n";
        
        return S;
    }
    
    
    
    private void depthFirstSearch(ExerVertex v) {
        // If already visited, terminate
        if (v.visited() || v.finished()) return;
        
        Animator a = Animator.getActiveAnimator();
        
        // Mark visited
        a.startOperation();
        
        if (DEBUG) 
            System.out.println("visited:"+v.getElement());
        v.markVisited();
        
        a.endOperation();
        // Travel through successorlist
        Vertex[] vertices = v.getSuccessors();
        for (int i=0; i<vertices.length; i++)
            depthFirstSearch((ExerVertex)vertices[i]);
        
        a = Animator.getActiveAnimator();
        
        // Mark finished
        a.startOperation();
        if (DEBUG)
            System.out.println("finished:"+v.getElement());
        v.markFinished();
        a.endOperation();
    }
    
    
    public FDT[] solve() {
        Note.out(this, "Solve");
        
        Vertex[] vertices = model.getVertices();
        for (int i = 0; i < vertices.length; i++) {
            ((ExerVertex)vertices[i]).setColor(Color.white);
        }

        depthFirstSearch(ModelVertex[0]);
        
        Note.out(this, "Solved");
        return new FDT[] { model };
    }
    
    
    
    public FDT[] makeModelAnswer() {
        return solve();
    }
    
    
    public String[] getModelAnswerNames() {
        ExerciseProperties lang = ExerciseProperties.getInstance();

        String[] s = { lang.get("DFS_GRAPH_MODEL_TITLE") };
        return s;
    }
    
    public FDT[] getAnswer() {
        FDT[] s = { user };
        return s;
    }
    
    public FDT[] getInitialStructures() {
        throw new RuntimeException("not implemented");
    }
    
    /* ComparableExercise interface */

	public int[] getCompareIndices() {
		return new int[] {0};
	}
	
	public int[] getGradeIndices() {
		return new int[] {0};
	}
    
    public boolean canRecover() {
        return true;
    }
    
	public String[] getCompareStructureVisualisations(){
		return null; // use default
	}
	
	public String[] getCompareStructureNames(){
		return null; // use default
	}    

	public VisualTypeConf[] getCompareVisualTypeConf(){
		return null; // use default
	}
	
	/* GradeDialog interface */
	
	public boolean showModelAnswerInGradeDialog(){
		return false;
	}
	
	public boolean showComparisonInGradeDialog(){
		return true;
	}
	
    static final long serialVersionUID = 6124519500007486392L;
}
