//Matrix - algorithm simulation system
//Copyright (C) 2000- 2007 Matrix group
//(See the AUTHORS file and the author tag below for more information
//on the authors.)

//This program is free software; you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation; either version 2 of the License, or
//(at your option) any later version.

//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.

//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/* $Id: DST_Insert.java,v 1.25 2009/06/18 10:12:51 lhakulin Exp $ */

package content.exercises;

import content.ExerciseProperties;
import content.interfaces.ComparableExercise;
import content.interfaces.ConfigureVisualType;
import content.interfaces.ModelAnswerNames;
import content.interfaces.SimulationExerciseModel;
import content.interfaces.SwapBehaviour;
import matrix.animation.Animator;
import matrix.simulation.VisualTypeConf;
import matrix.structures.CDT.probe.DigitalSearchTree;
import matrix.structures.FDT.FDT;
import matrix.structures.FDT.probe.*;
import matrix.util.RandomKey;

public class DST_Insert implements SimulationExerciseModel, ModelAnswerNames, ConfigureVisualType,
									SwapBehaviour, ComparableExercise {

    Table t;
    BinTree bt;
    DigitalSearchTree dst;
    String S = null;
    public static final boolean DEBUG = false;

    public DST_Insert() {

    }

    /**
     * In this exercise a drag operation performs a swap
     * @return
     */
    public boolean getSwapBehaviour() {
        return true;
    }

    long seed = 1;

    /**
     * Get the value of seed.
     * @return value of seed.
     */
    public long getSeed() {
        return seed;
    }

    /**
     * Set the value of seed.
     * @param v  Value to assign to seed.
     */
    public void setSeed(long  v) {
        this.seed = v;
    }




    public FDT[] init() {
        S = RandomKey.createNoDuplicateUppercaseRandomKey(new java.util.Random(seed),10);
        t = new Table(S);
        bt = new BinTree();
        FDT[] tmp = { t, bt };
        return tmp;
    }


    public FDT[] init(Object input) {
        return init();
    }


    public String[] getStructureNames() {
        ExerciseProperties lang = ExerciseProperties.getInstance();

        String[] tmp = { lang.get("DST_INSERT_INPUT_TITLE"), lang.get("DST_INSERT_TREE_TITLE") };
        return tmp;
    }

    public String getDescription() {
        return "";
    }


    public static String printBinaryCodes() {
        StringBuffer buf = new StringBuffer();
        String sep = "";
        for (int c = 'A'; c < 'Z'; c++, sep = "\n") {
            if ((c - 'A') % 3 == 0) buf.append(sep);
            buf.append((char)c);
            buf.append(" = ");
            java.util.BitSet bs = matrix.util.KeyBuilder.getDigitalKey(
                    new String(new char[] { (char)c }));
            for (int i=0; i<7; i++)
                if (bs.get(i))
                    buf.append("1");
                else
                    buf.append("0");
            buf.append("  ");
        }
        return buf.toString();
    }


    public FDT[] solve() {
        dst = new DigitalSearchTree();
        Table tbl = (Table)getInitialStructures()[0];
        Animator a = Animator.getActiveAnimator();
        for (int i=0; i<tbl.size(); i++) {
            a.startOperation();
            dst.insert(tbl.getObject(i));
            a.endOperation();
        }
        return new FDT[] { dst.getRoot() };
    }



    public FDT[] makeModelAnswer() {
        return solve();
    }


    public FDT[] getInitialStructures() {
        return new FDT[] { new Table(S), new BinTree() };
    }

    public FDT[] getSimulatedStructures() {
        return new FDT[] { t, bt };
    }

    public FDT[] getAnswer() {
        FDT[] tmp = { bt };
        return tmp;
    }

    public String[] getModelAnswerNames() {
        ExerciseProperties lang = ExerciseProperties.getInstance();

        String[] s = { lang.get("DST_INSERT_TREE_MODEL_TITLE") };
        return s;
    }


    public SimulationExerciseModel getModelAnswer() {
        return this;
    }

    public VisualTypeConf[] conf() {
        VisualTypeConf tabl = new VisualTypeConf();
        tabl.enable("matrix.visual.VisualKey", VisualTypeConf.HIGHLIGHT_OPERATION);
        tabl.enable("matrix.visual.VisualKey", VisualTypeConf.DRAG_OPERATION);
        VisualTypeConf tre = new VisualTypeConf();
        tre.enable("matrix.visual.VisualKey", VisualTypeConf.DROP_OPERATION);
        tre.enable("matrix.visual.VisualKey", VisualTypeConf.HIGHLIGHT_OPERATION);
        tre.enable("matrix.visual.VisualLayeredTreeComponent", VisualTypeConf.DROP_OPERATION);
        tre.enable("matrix.visual.VisualLayeredTreeComponent", VisualTypeConf.HIGHLIGHT_OPERATION);

        return new VisualTypeConf[] { tabl, tre};
    }
    
    /* ComparableExercise interface */
    
	public int[] getCompareIndices(){
        return new int[] {0};	
	}

	public int[] getGradeIndices(){
        return null; // Used only in recovery mode
	}

	public boolean canRecover(){
		return false;
	}
	
	public String[] getCompareStructureVisualisations(){
		return null; // use default
	}
	
	public String[] getCompareStructureNames(){
		return null; // use default
	}    
	
	public VisualTypeConf[] getCompareVisualTypeConf(){
		return null; // use default
	}


    static final long serialVersionUID = 8738073562346980663L;
}
