/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2006 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package content.exercises;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.Rectangle;
import java.util.Arrays;
import java.util.Random;
import java.util.Vector;

import matrix.animation.Animator;
import matrix.decoration.LabelDecorator;
import matrix.simulation.VisualTypeConf;
import matrix.structures.FDT.FDT;
import matrix.structures.FDT.Struct;
import matrix.structures.FDT.probe.Key;
import matrix.structures.FDT.probe.Table;
import matrix.structures.memory.VirtualBoolean;
import matrix.structures.memory.VirtualInteger;
import matrix.structures.simulationextensions.Selectable;
import matrix.structures.spatial.Area;
import matrix.structures.spatial.PaintingStyleDecorator;
import matrix.structures.spatial.SpatialComparable;
import matrix.structures.spatial.FDT.probe.LineSegment;
import matrix.structures.spatial.FDT.probe.Point;
import matrix.structures.spatial.FDT.probe.SpatialElement;
import matrix.util.Note;
import content.ExerciseProperties;
import content.interfaces.*;


/**
 * A dynamic programming exercise where user has to select the correct
 * tasks to get maximum profit.
 */

public class DynamicProgrammingSelectTasks implements
        SimulationExerciseModel, StyledExercise, ModelAnswerNames,
        ConfigureVisualType, JudgeBlocks, LayoutExercise {

    private static final long serialVersionUID = -9071063606494335757L;

    private long seed = 1;
    private static int size = 5;

    private SumTable resultTable;
                
    // Exercise color and style scheme
    private static final Color LINE_COLOR = Color.BLACK;
    private static final Color LABEL_COLOR = Color.black;
    private static final PaintingStyleDecorator areaDecorator;
    private static final PaintingStyleDecorator lineDecorator;
    private static final PaintingStyleDecorator selectedLineDecorator;
    private static final PaintingStyleDecorator selectedSumLineDecorator;
    private static final PaintingStyleDecorator unselectedSumLineDecorator;
    
	// properties of the visualization area.
	private static final int AREA_WIDTH = 400;
	private static final int AREA_HEIGHT = 200;
	private static final int LINE_END_HEIGHT = 5;
	private static final int LINE_GAP = 35;
	private static final int SUM_LINE_GAP = 15;
	private static final int ELEMENT_GAP = 10;
	
    static {
    	
		areaDecorator = new PaintingStyleDecorator();
		areaDecorator.setLineColor(Color.white);
		areaDecorator.setFill(false);
        areaDecorator.setStroke(2, BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL);

        lineDecorator = new PaintingStyleDecorator();
        lineDecorator.setLineColor(LINE_COLOR);
        lineDecorator.setStroke(2, BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL);

        selectedLineDecorator = new PaintingStyleDecorator();
        selectedLineDecorator.setLineColor(new Color(0, 220, 0));
        selectedLineDecorator.setStroke(2, BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL);

        float[] pattern = { 3 , 5 };
        selectedSumLineDecorator = new PaintingStyleDecorator();
        selectedSumLineDecorator.setLineColor(new Color(0, 60, 160));
        selectedSumLineDecorator.setStroke(2, BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL, 3, pattern, 0);

        unselectedSumLineDecorator = new PaintingStyleDecorator();
        unselectedSumLineDecorator.setLineColor(Color.red);
        unselectedSumLineDecorator.setStroke(2, BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL);
    }


    public DynamicProgrammingSelectTasks() {
        super();
    }

    public FDT[] solveThis() {

        FDT[] structures = getInitialStructures();
        Table input      = (Table) structures[0];
        Table result      = (Table) structures[1];
        DynamicProgrammingArea area = (DynamicProgrammingArea)structures[2];
        
        Animator an = Animator.getActiveAnimator();
        an.startOperation();
        result.setObject(input.getObject(size - 1), size - 1);
        ((Result)result.getObject(size - 1)).assignSelect(true); // should be set to true
        an.endOperation();

        for (int i = size - 2; i >= 0; i--) {
            // j := aikaisin tapahtuma, jolle Alku[j] >= Loppu[i] tai j := n+1
            
            int currentEnd  = ((Integer)((Task)input.getObject(i)).getField(Task.END)).intValue();
            int bestDiff    = Integer.MAX_VALUE;
            int bestIndex   = size;
            
            for (int k=0; k<size; k++){
                
                int currentDiff = ((Integer)(((Task)input.getObject(k)).getField(Task.BEGIN))).intValue() - currentEnd ;
                if (currentDiff >= 0 && currentDiff < bestDiff){
                    bestDiff  = currentDiff;
                    bestIndex = k;
                }
            }
            
            int currentProfit   =  ((Integer)((Task)input.getObject(i)).getField(Task.PROFIT)).intValue();
            int cumulatedProfit =  ((Integer)((Result)result.getObject(bestIndex)).getField(Result.PROFIT)).intValue();            
            int previousProfit  =  ((Integer)((Result)result.getObject(i+1)).getField(Result.PROFIT)).intValue();
            
            if (currentProfit + cumulatedProfit >= previousProfit) {
                an.startOperation();
                ((Result)result.getObject(i)).assignSelect(true);
                result.setObject(input.getObject(i), i);
                result.setObject(result.getObject(bestIndex), i);
                an.endOperation();
            } else {
                an.startOperation();
                ((Result)result.getObject(i)).assignSelect(false);
                result.setObject(result.getObject(i + 1), i);
                an.endOperation();
            }
        }

        for (int i=0; i<size; i++){
            if (((Result)result.getObject(i)).getSelectionTime() != 0){
                int currentEnd  = ((Integer)((Task)input.getObject(i)).getField(Task.END)).intValue();

                for (int j=i+1; j<size; j++){
                    int currentBegin = ((Integer)(((Task)input.getObject(j)).getField(Task.BEGIN))).intValue();                    
                
                    if (((Result)result.getObject(j)).getSelectionTime() != 0 && currentBegin < currentEnd ){
                        an.startOperation();
                        ((Result)result.getObject(j)).assignSelect(false);
                        an.endOperation();
                    }
                    
                }
            }
                
        }
        
        return new FDT[] { input, result, area };
    }

    public FDT[] solve() {
        FDT[] structures = solveThis();
        return new FDT[] { structures[1] };
    }

    public FDT[] makeModelAnswer() {

        return solveThis();

    }

    public String[] getStructureVisualisations() {

        return new String[] { "array", "array" , "2D"};

    }

    public String[] getModelAnswerVisualisations() {

        return new String[] { "array", "array", "2D" };

    }

    public String[] getModelAnswerNames() {
        ExerciseProperties lang = ExerciseProperties.getInstance();
        return new String[] { 
                lang.get("DYNAMICPROGRAMMING_SELECT_TASKS_TASK_ARRAY_MODEL_TITLE"),
                lang.get("DYNAMICPROGRAMMING_SELECT_TASKS_RESULT_ARRAY_MODEL_TITLE"),
                lang.get("DYNAMICPROGRAMMING_SELECT_TASKS_VISUALIZATION_MODEL_TITLE")
        };

    }

    public VisualTypeConf[] conf() {    	

        VisualTypeConf taskConf = new VisualTypeConf();
        taskConf.enable("matrix.visual.VisualStruct", VisualTypeConf.DRAG_OPERATION);
        taskConf.enable("matrix.visual.VisualStruct",VisualTypeConf.HIGHLIGHT_OPERATION);

        VisualTypeConf resultConf = new VisualTypeConf();
        resultConf.enable("matrix.visual.VisualArrayComponent", VisualTypeConf.DROP_OPERATION);
        resultConf.enable("matrix.visual.VisualArrayComponent", VisualTypeConf.HIGHLIGHT_OPERATION);
        resultConf.enable("matrix.visual.VisualStruct", VisualTypeConf.DRAG_OPERATION);
        resultConf.enable("matrix.visual.VisualStruct", VisualTypeConf.HIGHLIGHT_OPERATION);

        VisualTypeConf areaConf = new VisualTypeConf();
		areaConf.enable("matrix.visual.VisualArea$AreaVisualization", VisualTypeConf.HIGHLIGHT_OPERATION);
        areaConf.setVisualConfValue("matrix.visual.VisualArea", VisualTypeConf.Y_AXIS_VISIBLE, "false");

        return new VisualTypeConf[] { taskConf, resultConf, areaConf };

    }
    
    public GridBagConstraints[] getConstraints() {
        
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 1;
        gbc.insets = new Insets(ELEMENT_GAP, 2, ELEMENT_GAP, 2);
        
        GridBagConstraints gbc2 = new GridBagConstraints();
        gbc2.gridx = 0;
        gbc2.gridy = 2;
        gbc2.insets = new Insets(ELEMENT_GAP, 2, ELEMENT_GAP, 2);

        GridBagConstraints gbc3 = new GridBagConstraints();
        gbc3.gridx = 0;
        gbc3.gridy = 0;
        gbc3.insets = new Insets(ELEMENT_GAP, 2, ELEMENT_GAP, 2);
        
        return new GridBagConstraints[]{ gbc, gbc2, gbc3 };
    }
    
    public GridBagConstraints[] getModelAnswerConstraints() {
    	
    	return getConstraints();
    }
    
	public GridBagConstraints[] getComparisonConstraints() {
		return null; // use default
	}
    
    public FDT[] init() {

        FDT[] fdts  = getInitialStructures();
        resultTable = (SumTable)fdts[1];
        
        return fdts;
    }

    public void setSeed(long v) {

        this.seed = v;
    }

    public long getSeed() {

        return seed;
    }

    public String[] getStructureNames() {
        ExerciseProperties lang = ExerciseProperties.getInstance();
        return new String[] { 
                lang.get("DYNAMICPROGRAMMING_SELECT_TASKS_TASK_ARRAY_TITLE"),
                lang.get("DYNAMICPROGRAMMING_SELECT_TASKS_RESULT_ARRAY_TITLE"),
                lang.get("DYNAMICPROGRAMMING_SELECT_TASKS_VISUALIZATION_TITLE")
        };

    }

    public FDT[] getAnswer() {

        return new FDT[] { resultTable };
    }

    public FDT[] getInitialStructures() {

        int[] begin;
        int[] end;
        int[] profit;
                
        
        Random rand = new Random(getSeed());
        Table taskTableLocal;
        SumTable resultTableLocal;
        
        begin = matrix.util.RandomKey.createSortedIntKeys(rand, 1, 28, size);
        end   = matrix.util.RandomKey.createIntKeys(rand, 2, 12, size);
        profit = matrix.util.RandomKey.createIntKeys(rand, 2, 99, size);
        
        // to get some overlap
        begin[size-1] = matrix.util.RandomKey.createIntKey(rand, begin[size-2]+1, begin[size-2]+1+end[size-2]-1);
        profit[size-1] = profit[size-2] - matrix.util.RandomKey.createIntKey(rand, 1, profit[size-2]-1);
        
        for (int i = 0; i < begin.length; i++) {
        	begin[i] *= 10;
            end[i] = begin[i] + (10*end[i]);
            if (end[i] > AREA_WIDTH){
            	end[i] = AREA_WIDTH;
            }
        }
                
        taskTableLocal = new Table(begin.length);
        for (int i=0; i<begin.length; i++){
            taskTableLocal.setObject(new Task(begin[i], end[i], profit[i]), i);
        }
        
        resultTableLocal = new SumTable(begin.length + 1);
        for (int i=0; i<begin.length+1; i++){
            resultTableLocal.setObject(new Result(), i);
        }
        DynamicProgrammingArea area = new DynamicProgrammingArea(taskTableLocal, resultTableLocal);
        area.setPaintingStyleDecorator(areaDecorator);
        
        return new FDT[] { taskTableLocal, resultTableLocal, area};

    }

    public String getDescription() {

        return "";
    }
    
    
    
    public static class LabelledLineSegment extends LineSegment implements LabelDecorator {
    	
    	private String label = "";
    	private boolean enabled;
    	private Color color;
        private VirtualBoolean selected;
    	
		private static final long serialVersionUID = -50793488810350435L;

		public LabelledLineSegment(int x1, int y1, int x2, int y2, PaintingStyleDecorator psd) {
        	super(x1, y1, x2, y2, psd);
        	selected = new VirtualBoolean(false, this, "Has this line been selected");
        	enabled = true;
        }
    	
        public boolean isLabelEnabled(){
        	return enabled;
        }        

        public void setLabelEnabled(boolean b){
        	enabled = b;
        }

        public void setLabel(String s){
        	label = s;
        }
 
        public String getLabel(){
        	return label;
        }

        public boolean isReferenceLabelEnabled(){
        	return false;
        }

        public void setReferenceLabelEnabled(boolean b){
            Note.warning(this, "Operation not implemented.");
        }

        public void setReferenceLabel(int i, String s){
            Note.warning(this, "Operation not implemented.");
        }

        public String getReferenceLabel(int i){
            Note.warning(this, "Operation not implemented.");
        	return "";
        }
        
        public void setLabelColor(Color c){
        	color = c;
        }
        
        public Color getLabelColor(){
        	return color;
        }
        
        public boolean isSelected(){
        	return selected.eval();
        }
        
        public void setSelected(boolean value){
        	selected.assign(value);
        }
   	
    }
    

    static class SumTable extends Table{
        
		private static final long serialVersionUID = 3958580033766278669L;

		public SumTable(int size){
            super(size);
        }
        
        public void setObject(Object obj, int address) {
            
            if (super.getObject(address) instanceof Key)
                super.setObject(obj, address);
            
            Struct incoming = (Struct)obj;
            Struct original = (Struct)super.getObject(address);
                         
            int originalProfit = ((Integer)original.getField(1)).intValue();
            int incomingProfit = 0;
            
            if (obj instanceof Task)
                incomingProfit = ((Integer)incoming.getField(Task.PROFIT)).intValue();
            else
                incomingProfit = ((Integer)incoming.getField(Result.PROFIT)).intValue();
            
            original.setField(new Integer(incomingProfit + originalProfit), Result.PROFIT);
            
            // task is a beginning of a selected event chain if a value from
            // tasks is summed into it
            if (original instanceof Result && obj instanceof Task){
            	((Result)original).beginOfEventChain.assign(true);
            }
            
            super.setObject(original, address);
        }
    }

    static class Task implements Struct{

		private static final long serialVersionUID = 745195614311205830L;

		private String[] taskNames;
        
        public final static int BEGIN  = 1;
        public final static int END    = 2;
        public final static int PROFIT = 3;
        
        private Object[] values;
        
        public Task(int begin, int end, int profit){
            values = new Object[4];
            values[0] = "";
            values[1] = new Integer(begin);
            values[2] = new Integer(end);
            values[3] = new Integer(profit);
            setTaskNames();
        }
        
        private void setTaskNames(){
            ExerciseProperties lang = ExerciseProperties.getInstance();
            taskNames = new String[4];
            taskNames[0] = "";
            taskNames[1] = lang.get("DYNAMICPROGRAMMING_SELECT_TASKS_TASK_BEGIN_TEXT");
            taskNames[2] = lang.get("DYNAMICPROGRAMMING_SELECT_TASKS_TASK_END_TEXT");
            taskNames[3] = lang.get("DYNAMICPROGRAMMING_SELECT_TASKS_TASK_PROFIT_TEXT");
        }
        
        public Object getField(int location) {
            return values[location];
        }

        public String getFieldName(int location) {
            return taskNames[location];
        }

        public String[] getFieldNames() {
            return taskNames;
        }

        public Object[] getFields() {
            return values;
        }

        public void setField(Object value, int location) {
            values[location] = (Integer)value;
        }

        public Object getElement() {
            return null;
        }

        public void setElement(Object o) {
        }

        public boolean equals(Object o){
            
            if (o instanceof Task){
                Task r = (Task)o;

                return (values[1].equals(r.values[1]) &&
                        values[2].equals(r.values[2]) &&
                        values[3].equals(r.values[3])); 
            }
            return false;
        }

        
    }

    static class Result implements Struct, Selectable{
        
		private static final long serialVersionUID = -9146106549939385700L;
		
		public static final int PROFIT   = 1; 
        public static final int SELECTED = 2; 

        private String[] resultNames;

        private VirtualInteger profit;
        private VirtualBoolean selected;
        private VirtualBoolean hasBeenSelected;
        private VirtualBoolean beginOfEventChain;
        private VirtualInteger dropCount;
        
        public Result(){
            profit   = new VirtualInteger(0, this, "profit field");
            selected = new VirtualBoolean(false, this, "selected field");
            beginOfEventChain = new VirtualBoolean(false, this, "begin of a chain of events");
            dropCount = new VirtualInteger(0, this, "drop count");
            hasBeenSelected = new VirtualBoolean(false, this, "has the struct been selected before");
            setResultNames();
        }
        
        private void setResultNames(){
            ExerciseProperties lang = ExerciseProperties.getInstance();
            resultNames = new String[3];
            resultNames[0] = "";
            resultNames[1] = lang.get("DYNAMICPROGRAMMING_SELECT_TASKS_RESULT_PROFITSUM_TEXT");
            resultNames[2] = lang.get("DYNAMICPROGRAMMING_SELECT_TASKS_RESULT_SELECTED_TEXT");
        }
        
        public Object getField(int location) {
            if (location == 0)
                return "";
            else if (location == 1)
                return new Integer(profit.eval());
            else
                return new Boolean(selected.eval());
        }

        public String getFieldName(int location) {
            return resultNames[location];
        }

        public String[] getFieldNames() {
            return resultNames;
        }

        public Object[] getFields() {
            return new Object[]{"", new Integer(profit.eval()), new Boolean(selected.eval())};
        }

        public void setField(Object value, int location) {
            if (location == 1)
                profit.assign(((Integer)value).intValue());
            else if (location == 2)
                selected.assign(((Boolean)value).booleanValue());
        }

        
        public Object getElement() {
            return null;
        }

        public void setElement(Object o) {
        }

        public long getSelectionTime() {            
            return selected.eval()?1:0;
        }

        public void setSelect(long globalClockTimestamp) {
            Animator.getActiveAnimator().startOperation();
            selected.assign(!selected.eval());
            hasBeenSelected.assign(true);
            Animator.getActiveAnimator().endOperation();
        }
        
        public void assignSelect(boolean value){
        	Animator.getActiveAnimator().startOperation();
        	selected.assign(value);
            hasBeenSelected.assign(true);
        	Animator.getActiveAnimator().endOperation();            
        }
        
        public boolean equals(Object o){
            if (o instanceof Result){
                Result r = (Result)o;
                if (profit.equals(r.profit) && selected.equals(r.selected)){
                    return true;
                }
            }
            return false;
        }
        
        public String toString(){
            return "["+profit.eval() + " "+selected.eval()+"]";
        }
        
        public boolean isChainBeginning(){
        	if (dropCount.eval() > 1){
        		return true;
        	}
        	return false;
        }
        
    }

    /**
     * An area which represents the information in tasktable and resulttable
     * in dynamic programming exercise
     * @author lhakulin
     *
     */
    static class DynamicProgrammingArea implements Area {
    	
		private static final long serialVersionUID = 2173005778844978975L;
		
		private final Rectangle boundingBox;
        private PaintingStyleDecorator paintingStyleDecorator;
        private Table taskTable;
        private SumTable sumTable;
        
        private static final int ARROW_LENGTH = 30;
        
    	public DynamicProgrammingArea(Table taskTable, SumTable sumTable){
    		this.taskTable = taskTable;
    		this.sumTable = sumTable;
            this.boundingBox = new Rectangle(0, 0, AREA_WIDTH, AREA_HEIGHT);
            this.paintingStyleDecorator = new PaintingStyleDecorator();
    	}
    	
        public Object getElement() {
            return null;
        }
        public void setElement(Object o) {
        }
        // ** Implementation of interface Area **

        public void addSubArea(Area a) {
        	// Not implemented
        }

        public void removeSubArea(Area a) {
        	// Not implemented
        }

        public Area[] getSubAreas() {
        	// Not implemented
            return new Area[0];
        }

        public void delete(SpatialComparable e) {
        	// Not implemented
        }

        public void insert(SpatialComparable e) {
        	// Not implemented
        }        

        public Rectangle getBoundingPolygon() {
            return this.boundingBox;
        }

    
        public SpatialElement[] getSpatialElements() {
        	

            Vector vec = new Vector();
            for (int i=0; i < taskTable.size(); i++){
            	Task task = (Task)taskTable.getObject(i);
            	Result result = (Result)sumTable.getObject(i);
        		int x1 = ((Integer)task.getField(Task.BEGIN)).intValue();
        		int x2 = ((Integer)task.getField(Task.END)).intValue();
        		int y1 = (i+1)*LINE_GAP;
        		int y2 = (i+1)*LINE_GAP;

        		PaintingStyleDecorator decorator = lineDecorator;
        		if (result.selected.eval()){
        			decorator = selectedLineDecorator;
        		}
        		
        		LabelledLineSegment line = new LabelledLineSegment(x1, y1, x2, y2, decorator);
        		LabelledLineSegment beginLine = new LabelledLineSegment(x1, y1-LINE_END_HEIGHT, x1, y1+LINE_END_HEIGHT, decorator);
        		LabelledLineSegment endLine = new LabelledLineSegment(x2, y2-LINE_END_HEIGHT, x2, y2+LINE_END_HEIGHT, decorator);
        		line.setLabel(task.getField(Task.PROFIT) + "");
        		line.setLabelColor(LABEL_COLOR);
        		vec.add(line);
        		vec.add(beginLine);
        		vec.add(endLine);

        		LabelledLineSegment sumLine;
        		if (result.selected.eval() || (result.beginOfEventChain.eval() && result.hasBeenSelected.eval())){
        			sumLine = new LabelledLineSegment(x1, y1-SUM_LINE_GAP, AREA_WIDTH, y2-SUM_LINE_GAP, selectedSumLineDecorator);
            		vec.add(new Point(x1, y2-SUM_LINE_GAP, selectedSumLineDecorator));
        		}
        		else {
        			int endX = x1 + ARROW_LENGTH;
        			int a = 5;
        			sumLine = new LabelledLineSegment(x1, y1-SUM_LINE_GAP, endX, y2-SUM_LINE_GAP, unselectedSumLineDecorator);        			
            		vec.add(new Point(x1, y2-SUM_LINE_GAP, unselectedSumLineDecorator));
            		vec.add(new LineSegment(endX-a, y2-SUM_LINE_GAP+a, endX, y1-SUM_LINE_GAP, unselectedSumLineDecorator));
            		vec.add(new LineSegment(endX-a, y2-SUM_LINE_GAP-a, endX, y1-SUM_LINE_GAP, unselectedSumLineDecorator));
        		}
        		sumLine.setLabel(result.getField(Result.PROFIT) + "");
        		vec.add(sumLine);
        		
            }
            
            SpatialElement[] spatialElements = new SpatialElement[vec.size()];
            vec.toArray(spatialElements);
            return spatialElements;
        }
          
        public boolean hasPaintingStyleDecorator(){
            return (paintingStyleDecorator != null);
        }

        public void setPaintingStyleDecorator(PaintingStyleDecorator psd){
            paintingStyleDecorator = psd;
        }

        public PaintingStyleDecorator getPaintingStyleDecorator(){
            return paintingStyleDecorator;
        }        
    }
}
