//Matrix - algorithm simulation system
//Copyright (C) 2000- 2007 Matrix group
//(See the AUTHORS file and the author tag below for more information
//on the authors.)

//This program is free software; you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation; either version 2 of the License, or
//(at your option) any later version.

//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.

//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package content.exercises;

import java.awt.Color;
import java.util.ArrayList;
import java.util.Random;
import content.ExerciseProperties;
import content.exercises.SelectionSort.BarTableKey;
import content.interfaces.ConfigureVisualType;
import content.interfaces.JudgeBlocks;
import content.interfaces.ModelAnswerNames;
import content.interfaces.ComparableExercise;
import matrix.animation.Animator;
import matrix.decoration.StyleSheet;
import matrix.decoration.StyleSheetAdapter;
import matrix.decoration.Styled;
import matrix.simulation.VisualTypeConf;
import matrix.structures.FDT.FDT;
import matrix.structures.FDT.Struct;
import matrix.structures.FDT.probe.Table;
import matrix.structures.memory.VirtualBoolean;
import matrix.structures.memory.VirtualInteger;
import matrix.structures.memory.VirtualObject;
import matrix.structures.simulationextensions.Selectable;
import matrix.util.Note;

/**
 * A hashing excercise which contains operations add, search and remove.
 * Probing method can be linear, quadratic or double hashing.
 * Probing method and the amount of add, search and remove operations can be set
 * via exercise properties. 
 * 
 * @author lhakulin
 *
 */

public class Hash_Operations implements ComparableExercise, ModelAnswerNames, ConfigureVisualType, JudgeBlocks {

    private static final int PROBE_TYPE_LINEAR = 0;
    private static final int PROBE_TYPE_QUADRATIC = 1;
    private static final int PROBE_TYPE_DOUBLEHASH = 2;
    private static final String EMPTY = "";
    
    private Table inputDataTable;
    private Table currentInput;
    private Table userSolutionTable;

    private static final int size = 19;
    private static final int secondHashParameter = 7;
    private static final int duplicates = 5;
    private static final int moreDuplicates = 3;
    private static final int forcedBeginAdditions = 4; // minimum amount of additions in the beginning
    
    // default values of parameters
	private static final int defaultAddDataSize = 15;
	private static final int defaultSearchDataSize = 3;
	private static final int defaultRemoveDataSize = 3;
	private static final int defaultFailedSearchDataSize = 1;
	private static final int defaultInitialDataSize = 0;
    
    static final int min = 100;
    static final int max = 999;

    private String taskAddText; 
    private String taskRemoveText; 
    private String taskSearchText;     
    
    private int probeType = PROBE_TYPE_LINEAR; // default is linear
    private int initialDataSize;

    public static final boolean DEBUG = false;

    private long seed = 1;

    public Hash_Operations() {
    }

    /**
     * Get the value of seed.
     * @return value of seed.
     */
    public long getSeed() {
        return seed;
    }

    /**
     * Set the value of seed.
     * @param v  Value to assign to seed.
     */
    public void setSeed(long  v) {
        this.seed = v;
    }

    /**
     * Gets the texts used in exercise from exercise properties
     *
     */
    private void initTexts(){
    	ExerciseProperties lang = ExerciseProperties.getInstance();
        taskAddText = lang.get("HASH_OPERATIONS_TASK_ADD_TEXT");
        taskSearchText = lang.get("HASH_OPERATIONS_TASK_SEARCH_TEXT");
        taskRemoveText = lang.get("HASH_OPERATIONS_TASK_REMOVE_TEXT");
    }
    
    /**
     * Gets the amount of add operations from exercise properties.
     * Uses default value if the value from exercise properties is not valid
     * @return amount of add operations
     */
    private int getAddDataSize(){
    	int addDataSize;
        ExerciseProperties lang = ExerciseProperties.getInstance();
        try {
            int value = Integer.parseInt(lang.get("HASH_OPERATIONS_OPERATION_ADD_COUNT").trim());
            addDataSize = Math.max(value, duplicates+1);
            addDataSize = Math.min(addDataSize, size);
		} catch (NumberFormatException e) {
			addDataSize = defaultAddDataSize;
		}
		return addDataSize;
    }

    /**
     * Gets the amount of search operations from exercise properties.
     * Uses default value if the value from exercise properties is not valid
     * @return amount of search operations
     */
    private int getSearchDataSize(int addDataSize){
    	int searchDataSize;
        ExerciseProperties lang = ExerciseProperties.getInstance();
        try {
            int value = Integer.parseInt(lang.get("HASH_OPERATIONS_OPERATION_SEARCH_COUNT").trim());			
            searchDataSize = Math.min(value, addDataSize);
		} catch (NumberFormatException e) {
			searchDataSize = defaultSearchDataSize;
		}
		return searchDataSize;
    }
    
    /**
     * Gets the amount of failed search operations from exercise properties.
     * Uses default value if the value from exercise properties is not valid
     * @return amount of failed search operations
     */
    private int getFailedSearchDataSize(int searchDataSize){
    	int failedSearchDataSize;
        ExerciseProperties lang = ExerciseProperties.getInstance();
        try {
            int value = Integer.parseInt(lang.get("HASH_OPERATIONS_OPERATION_FAILED_SEARCH_COUNT").trim());			
            failedSearchDataSize = Math.min(value, searchDataSize);
		} catch (NumberFormatException e) {
			failedSearchDataSize = defaultFailedSearchDataSize;
		}
		return failedSearchDataSize;
    }
    
    /**
     * Gets the amount of remove operations from exercise properties.
     * Uses default value if the value from exercise properties is not valid
     * @return amount of remove operations
     */
    private int getRemoveDataSize(int addDataSize){
    	int removeDataSize;
        ExerciseProperties lang = ExerciseProperties.getInstance();
        try {
            int value = Integer.parseInt(lang.get("HASH_OPERATIONS_OPERATION_REMOVE_COUNT").trim());
            removeDataSize = Math.min(value, addDataSize);
		} catch (NumberFormatException e) {
			removeDataSize = defaultRemoveDataSize;
		}
    	return removeDataSize;
    }
    
    /**
     * Gets the amount keys initially in the table from exercise properties.
     * Uses default value if the value from exercise properties is not valid
     * @return amount of initial keys in the table
     */
    private int getInitialDataSize(int addDataSize){
    	int initialDataSize;
        ExerciseProperties lang = ExerciseProperties.getInstance();
        try {
            int value = Integer.parseInt(lang.get("HASH_OPERATIONS_INITIAL_COUNT").trim());
            initialDataSize = Math.min(value, size-addDataSize);
		} catch (NumberFormatException e) {
			initialDataSize = defaultInitialDataSize;
		}
    	return initialDataSize;
    }
    
    /**
     * Sets the probing method got from exercise properties.
     * Uses default value if the value from exercise properties is not valid
     */
    private void setProbeType(){
        ExerciseProperties lang = ExerciseProperties.getInstance();
        try {
            int value = Integer.parseInt(lang.get("HASH_OPERATIONS_PROBETYPE").trim());
            if (value == PROBE_TYPE_LINEAR || value == PROBE_TYPE_QUADRATIC || value == PROBE_TYPE_DOUBLEHASH){
            	probeType = value;
            }
		} catch (NumberFormatException e) {
			// using default probeType
		}

    }

    public FDT[] init() {
    	
    	initTexts();
        setProbeType();
    	    	
    	int addDataSize = getAddDataSize();
        int searchDataSize = getSearchDataSize(addDataSize);
        int removeDataSize = getRemoveDataSize(addDataSize);
        int failedSearchDataSize = getFailedSearchDataSize(searchDataSize);
        initialDataSize = getInitialDataSize(addDataSize);
        		        
        Random R = new Random(seed);
        boolean newInit = true;
        
        while (newInit && addDataSize > 0){
        
        	int inputData[] = new int[addDataSize];	
	        // create random data
	        for(int i = 0; i < addDataSize-duplicates; i++) {
	            inputData[i] = Math.abs(R.nextInt()%(max-min))+min;
	        }
	
	        // Create duplicates
	        for(int i = addDataSize-duplicates; i < addDataSize; i++) {
	            do {
	                inputData[i] = inputData[(0+i+duplicates-addDataSize)%(duplicates-moreDuplicates)];
	                inputData[i] -= (R.nextInt(addDataSize))*size;
	            } while(inputData[i]<min || inputData[i] > max);
	        }
	                
	        inputDataTable = new Table(initialDataSize + inputData.length + searchDataSize + removeDataSize);
	        
	        // the initial data 
	        for (int i=0; i < initialDataSize; i++){
	        	int value = Math.abs(R.nextInt()%(max-min))+min;
        		inputDataTable.setObject(new HashTask(value, taskAddText, probeType, size), i);
	        }
	        
	        int inputInd = 0;
	        int searchCount = 0;
	        int removeCount = 0;
	        int failedSearchCount = 0;
	        ArrayList hashTableContent = new ArrayList();
	        	        
	        for (int i = initialDataSize; i < inputDataTable.size(); i++){
	        	int f = R.nextInt(inputDataTable.size());
	        	
	        	// add
	        	if (i < forcedBeginAdditions+initialDataSize ||
	        			(searchCount >= searchDataSize && removeCount >= removeDataSize) ||
	        			(hashTableContent.size() == 0) || (f < addDataSize && inputInd < inputData.length)){
	        			        		
	            	hashTableContent.add(new Integer(inputData[inputInd]));
	        		inputDataTable.setObject(new HashTask(inputData[inputInd++], taskAddText, probeType, size), i);
	        	}
	        	// search
	        	else if ((removeCount >= removeDataSize) ||
	        				(f < addDataSize+searchDataSize && searchCount < searchDataSize) ||
	        				(searchCount < searchDataSize && hashTableContent.size() <= 1)){
	        		
	        		int tmp = R.nextInt(searchDataSize+failedSearchDataSize);
	        		// failed search
	        		if ((tmp < failedSearchDataSize || searchCount+(failedSearchDataSize-failedSearchCount) >= searchDataSize) &&
	        				failedSearchCount < failedSearchDataSize){
	        			
	        			int value;
	        			do {
	        				value = Math.abs(R.nextInt()%(max-min))+min;
	        			} while(hashTableContent.contains(new Integer(value)));
	        			
		        		inputDataTable.setObject(new HashTask(value, taskSearchText, probeType, size), i);
	        			failedSearchCount++;
	        		}
	        		// successfull search
	        		else {
		        		int ind = R.nextInt(hashTableContent.size());
		        		inputDataTable.setObject(new HashTask(((Integer)hashTableContent.get(ind)).intValue(), taskSearchText, probeType, size), i);
	        		}
	        		searchCount++;
	        	}
	        	// remove
	        	else {
	        		int ind = R.nextInt(hashTableContent.size());
	        		inputDataTable.setObject(new HashTask(((Integer)hashTableContent.remove(ind)).intValue(), taskRemoveText, probeType, size), i);
	        		removeCount++;
	        	}
	        }
	        currentInput = new Table(1);
	        currentInput.setObject(inputDataTable.getObject(0), 0);
	        userSolutionTable = new SelectableTable(size, inputDataTable, currentInput);
	        
	        // add the inital keys
	        for (int i=0; i < initialDataSize; i++){
   	        	int keyValue = ((Integer)(((HashTask)inputDataTable.getObject(i)).getField(HashTask.VALUE))).intValue();
	        	int hashValue = keyValue % size;
	            int doubleHashValue = secondHashParameter - (keyValue % secondHashParameter);
	            ((SelectableTable)userSolutionTable).clearSelections();          
	            probe((SelectableTable)userSolutionTable, hashValue, doubleHashValue, false);
	        }
	        
	        // if the exercise is not solvable, decrease addDataSize by one
	        // and init the exercise again
	        SelectableTable testTable = (SelectableTable)getInitialStructures()[1];
	        if (!solveTable(testTable, false)){
	        	addDataSize--;
	        	newInit = true;
	            Note.warning(this, "Exercise not solvable. Decreasing addDataSize.");
	        }
	        else {
	        	newInit = false;
	        }
        }
        
        
        return new FDT[] { currentInput, userSolutionTable };
    }


    public FDT[] init(Object o) {
        return init();
    }

    public String[] getStructureNames() {
        ExerciseProperties lang = ExerciseProperties.getInstance();
        return new String[] { 
                lang.get( "HASH_OPERATIONS_INPUT_TITLE" ),
                lang.get( "HASH_OPERATIONS_HASHTABLE_TITLE" )
        };
    }

    public String getDescription() {
        ExerciseProperties lang = ExerciseProperties.getInstance();
        String prompt1 =  lang.get( "HASH_OPERATIONS_PROMPT_PART_1" ); // "The hash function is h(k) = k "
        String prompt2 =  lang.get( "HASH_OPERATIONS_PROMPT_PART_2" ); // " mod "

        String s = prompt1 + " " + prompt2 + " " + size + "\n";
        
        if (probeType == PROBE_TYPE_DOUBLEHASH){
            String prompt3 =  lang.get( "HASH_OPERATIONS_PROMPT_PART_3" ); // "The second hash function is h(k) = k "
            String prompt4 =  lang.get( "HASH_DOUBLEHASH_PROMPT_PART_4" ); // " - (k mod "
            s += prompt3 + " " + prompt4 + " " + secondHashParameter + " )\n";
        }

        s += lang.get( "HASH_OPERATIONS_PROBING_TYPE_TEXT" ); // "probing type:"
        
        String typeString = "";
        if (probeType == PROBE_TYPE_LINEAR){
        	typeString = lang.get( "HASH_OPERATIONS_LINEAR_PROBING_TEXT" );
        }
        else if (probeType == PROBE_TYPE_QUADRATIC){
        	typeString = lang.get( "HASH_OPERATIONS_QUADRATIC_PROBING_TEXT" );
        }
        else if (probeType == PROBE_TYPE_DOUBLEHASH){
        	typeString = lang.get( "HASH_OPERATIONS_DOUBLE_PROBING_TEXT" );
        }
        s += " " + typeString;
        
        return s;
    }
    

    /**
     * Does the hashing to the hashTable and chech's whether the exercise is solvable
     * @param hashTable
     * @param animatorOn true if operations are saved in animator
     * @return true if hashing is possible, false if it's not possible
     */
    private boolean solveTable(SelectableTable hashTable, boolean animatorOn){

    	boolean solvable = true;
    	
        for(int i = initialDataSize; i < inputDataTable.size(); i++) {
        	HashTask task = ((HashTask)inputDataTable.getObject(i));
        	int hashValue = (((Integer)task.getField(HashTask.VALUE)).intValue()) % size;
            int doubleHashValue = secondHashParameter - (((Integer)task.getField(HashTask.VALUE)).intValue() % secondHashParameter);

            hashTable.clearSelections();
          	solvable = probe(hashTable, hashValue, doubleHashValue, animatorOn);

            if (solvable == false){
            	return false;
            }
        }
        return true;
    }
    
    public FDT[] solve() {
    	
        FDT[] initialStructures = getInitialStructures();
        Table input =  (Table)initialStructures[0];
        SelectableTable modelSolution = (SelectableTable)initialStructures[1];
        solveTable(modelSolution, true);
        
        return new FDT[] { input, modelSolution };
    }
    

    protected boolean probe(SelectableTable modelSolution, int hashValue, int doubleHashValue, boolean animatorOn){
    	for (int i=0; i < size; i++){
            if (probeType == PROBE_TYPE_LINEAR && 
            		modelSolution.selected((HashCell)modelSolution.getObject((hashValue+i)%size), animatorOn)){
            	return true;
            }   		
            else if (probeType == PROBE_TYPE_QUADRATIC && 
            		modelSolution.selected((HashCell)modelSolution.getObject((hashValue+i*i)%size), animatorOn)){
            	return true;
            }  		
            else if (probeType == PROBE_TYPE_DOUBLEHASH && 
            		modelSolution.selected((HashCell)modelSolution.getObject((hashValue+i*doubleHashValue)%size), animatorOn)){
            	return true;
            }   		
    	}    	
    	return false;
    }

    public FDT[] makeModelAnswer() {
        return solve();
    }

    public FDT[] getInitialStructures() {
        Table current = new Table();
        current.setObject(inputDataTable.getObject(0), 0);
        Table resultTable = new SelectableTable(size, inputDataTable, current);   
        
        // add the inital keys
        for (int i=0; i < initialDataSize; i++){
	       	int keyValue = ((Integer)(((HashTask)inputDataTable.getObject(i)).getField(HashTask.VALUE))).intValue();
        	int hashValue = keyValue % size;
            int doubleHashValue = secondHashParameter - (keyValue % secondHashParameter);
            ((SelectableTable)resultTable).clearSelections();          
            probe((SelectableTable)resultTable, hashValue, doubleHashValue, false);
        }

        return new FDT[] { current, resultTable };
    }

    public FDT[] getSimulatedStructures() {
        return new FDT[] { inputDataTable, userSolutionTable };
    }
    
    public String[] getModelAnswerNames() {
        ExerciseProperties lang = ExerciseProperties.getInstance();
        return new String[] { lang.get( "HASH_OPERATIONS_INPUT_MODEL_TITLE"), lang.get( "HASH_OPERATIONS_HASHTABLE_MODEL_TITLE" ) };
    }


    public FDT[] getAnswer() {
        return new FDT[] { currentInput, userSolutionTable };
    }

    public ComparableExercise getModelAnswer() {
        return this;
    }

    public VisualTypeConf[] conf() {
        VisualTypeConf tabl = new VisualTypeConf();
        tabl.setVisualConfValue("matrix.visual.VisualArray",VisualTypeConf.INDEXED, "false");
        VisualTypeConf hashtabl = new VisualTypeConf();
        hashtabl.enable("matrix.visual.VisualStruct",VisualTypeConf.HIGHLIGHT_OPERATION);
        return new VisualTypeConf[] { tabl, hashtabl };
    }

    public int[] getCompareIndices() {
        return new int[] {0, 1};
    }

	public int[] getGradeIndices(){
        return null; // Used only in recovery mode
	}

	public boolean canRecover(){
		return false;
	}
	
	public String[] getCompareStructureVisualisations(){
		return null; // use default
	}
	
	public String[] getCompareStructureNames(){
		return null; // use default
	}      
	
	public VisualTypeConf[] getCompareVisualTypeConf(){
		return null; // use default
	}

    static final long serialVersionUID = -37657283323350211L;
    
    
    /**
     * The hash table of the exercise.
     * @author lhakulin
     *
     */
    public static class SelectableTable extends Table {
    	    	
    	protected Table input; // all the input data of the exercise
    	protected Table currentInput; // current input task
    	protected VirtualInteger current;
    	
        private String taskAddText; 
        private String taskSearchText; 
        private String taskRemoveText; 
        private String taskFinishedText; 
        private String graveStoneText; 
    	
    	public SelectableTable(int size, Table inputData, Table ci){
    		super();
    		input = inputData;
    		currentInput = ci;
    		current = new VirtualInteger(0, this, "current index of input table");
    		for (int i=0; i < size; i++){
    			this.setObject(new HashCell(EMPTY, this, i), i);
    		}
    		setTaskTexts();
    	}
    	
        private void setTaskTexts(){
            ExerciseProperties lang = ExerciseProperties.getInstance();
            taskAddText = lang.get("HASH_OPERATIONS_TASK_ADD_TEXT");
            taskSearchText = lang.get("HASH_OPERATIONS_TASK_SEARCH_TEXT");
            taskRemoveText = lang.get("HASH_OPERATIONS_TASK_REMOVE_TEXT");
            taskFinishedText = lang.get("HASH_OPERATIONS_TASK_FINISHED_TEXT");
            graveStoneText = lang.get("HASH_OPERATIONS_GRAVESTONE_TEXT");
        }
    	
    	/**
    	 * Called when one of the cells is selected
    	 * Performs an operation (add, search or remove) according to the currentInput
    	 * @param cell - the cell which was selected
    	 * @return true if currentInput is updated
    	 */
    	protected boolean selected(HashCell cell, boolean animatorOn){
    		
    		if (currentFinished()){
    			return false;
    		}
    		
    		boolean result = false;
    		if (animatorOn){
    			Animator.getActiveAnimator().startOperation();
    		}
    		cell.assignSelect(!cell.isSelected());
    		HashTask task; 
    		if (currentInput.getObject(0) instanceof HashTask){
    			task = (HashTask)currentInput.getObject(0);
    		}
    		else {
    			task = (HashTask)input.getObject(current.eval());
    		}
    		    		
    		if (task.getField(HashTask.OPERATION).equals(taskAddText)){
    			if (cell.getField(HashCell.VALUE).toString().equals(EMPTY) ||
    					cell.getField(HashCell.VALUE).toString().equals(graveStoneText)){
    	    		cell.setField(task.getField(HashTask.VALUE), HashCell.VALUE);
    	    		this.setObject(cell, cell.getIndex()); // needed to get the latest action highlighted when animating
    	    		clearSelections();
    	    		current.inc();
    	    		currentInput.setObject(input.getObject(current.eval()), 0);
    	    		result = true;
    			}
    		}
    		else if (task.getField(HashTask.OPERATION).equals(taskRemoveText)){
    			if (cell.getField(HashCell.VALUE).equals(task.getField(HashTask.VALUE).toString()) ||
    					cell.getField(HashCell.VALUE).equals(EMPTY)){
    				if (!cell.getField(HashCell.VALUE).equals(EMPTY)){
    					cell.setField(graveStoneText, HashCell.VALUE);
    				}
    	    		this.setObject(cell, cell.getIndex()); // needed to get the latest action highlighted when animating
    	    		clearSelections();
    	    		current.inc();
    	    		currentInput.setObject(input.getObject(current.eval()), 0);
    	    		result = true;
    			}
    		}
    		else if (task.getField(HashTask.OPERATION).equals(taskSearchText)){
    			if (cell.getField(HashCell.VALUE).toString().equals(task.getField(HashTask.VALUE).toString()) ||
    					cell.getField(HashCell.VALUE).equals(EMPTY)){
    	    		clearSelections();
    	    		current.inc();
    	    		currentInput.setObject(input.getObject(current.eval()), 0);
    	    		result = true;
    			}
    		}    		
    		
    		if (currentInput.getObject(0) == null){
    			currentInput.setObject(taskFinishedText, 0);
    		}
    		if (animatorOn){
    			Animator.getActiveAnimator().endOperation();
    		}
    		
    		return result;
    	}
    	
    	protected void clearSelections(){
    		for (int i=0; i < this.size(); i++){
    			((HashCell)getObject(i)).assignSelect(false);
    		}
    	}
    	
    	protected boolean currentFinished(){
    		return current.eval() > input.getLast();
    	}

		protected void setTaskAddText(String taskAddText) {
			this.taskAddText = taskAddText;
		}

		protected void setTaskFinishedText(String taskFinishedText) {
			this.taskFinishedText = taskFinishedText;
		}

		protected void setTaskRemoveText(String taskRemoveText) {
			this.taskRemoveText = taskRemoveText;
		}

		protected void setTaskSearchText(String taskSearchText) {
			this.taskSearchText = taskSearchText;
		}   

		protected void setGraveStoneText(String graveStoneText) {
			this.graveStoneText = graveStoneText;
		}
		
		/**
		 * @return true if all keys are either disabled or empty
		 */
		public boolean allDisabledOrEmpty(){
			for (int i=getFirst(); i <= getLast(); i++){
				HashCell hc = (HashCell)getObject(i);
				if (!hc.isDisabled() && !hc.getField(HashCell.VALUE).equals(EMPTY)){
					return false;
				}
			}
			return true;
		}

		static final long serialVersionUID = 8668291347493297645L;
    }
    
    /**
     * A hash task contains the value of the key and operation
     * @author lhakulin
     *
     */
    public static class HashTask implements Struct {

		private static final long serialVersionUID = 6332395742256549987L;

		private VirtualBoolean selected;
        private String[] taskNames;
        private VirtualInteger value;
        private String operation;
        private int probeType;
        private String modString;
        private int tableSize;
        
        public static final int VALUE = 1; 
        public static final int OPERATION = 2; 
        public static final int HASH_CALCULATION = 3; 

        public HashTask(int val, String op, int type, int size){
            value = new VirtualInteger(val, this, "value field");
            operation = op;
            selected = new VirtualBoolean(false, this, "selected field");
            probeType = type;
            tableSize = size;
            setTaskNames();
        }
        
        private void setTaskNames(){
            ExerciseProperties lang = ExerciseProperties.getInstance();
            if (probeType == PROBE_TYPE_DOUBLEHASH){
                taskNames = new String[5];            	
                taskNames[4] = "";
            }
            else {
                taskNames = new String[4];
            }
            taskNames[0] = "";
            taskNames[1] = lang.get("HASH_OPERATIONS_TASK_KEY_TEXT");
            taskNames[2] = lang.get("HASH_OPERATIONS_TASK_OPERATION_TEXT");
            taskNames[3] = lang.get("HASH_OPERATIONS_TASK_CALCULATION_TEXT");
            
            modString = lang.get( "HASH_OPERATIONS_PROMPT_PART_2" ).trim();
        }
        
        
        public Object getField(int location) {
            if (location == 0){
                return "";
            }
            else if (location == VALUE){
                return new Integer(value.eval());
            }
            else if (location == OPERATION){
                return operation;
            }
            else if (location == HASH_CALCULATION){
                int hashValue = (value.eval()) % tableSize;
            	return value.eval() + " " + modString + " " + tableSize + " = " + hashValue;
            }
            else {
                int hashValue = secondHashParameter - ((value.eval()) % secondHashParameter);
            	return secondHashParameter + " - (" + value.eval() + " " + modString + " " + secondHashParameter + ") = " + hashValue;            	
            }
        }

        public String getFieldName(int location) {
            return taskNames[location];
        }

        public String[] getFieldNames() {
            return taskNames;
        }

        public Object[] getFields() {
            return new Object[]{"", new Integer(value.eval()), operation.toString()};
        }

        public void setField(Object val, int location) {
            if (location == VALUE){
                value.assign(((Integer)val).intValue());
            }
            else if (location == OPERATION){
            	operation = val.toString();
            }
        }

        public Object getElement() {
            return null;
        }

        public void setElement(Object o) {
        }

        public boolean equals(Object o){
            
            if (o instanceof HashTask){
                HashTask r = (HashTask)o;
                return (value.equals(r.value) &&
                        operation.toString().equals(r.operation.toString()) &&
                        selected.equals(r.selected)); 
            }
            return false;
        }
        


        
    }

    /**
     * A HashCell contains the value of the hashed key.
     * Calls the parent table when selected
     * 
     * @author lhakulin
     *
     */
    public static class HashCell implements Struct, Selectable, Styled{

		private static final long serialVersionUID = 6332890425565493641L;

		private VirtualBoolean selected;
		private SelectableTable parent;
		private VirtualBoolean disabled; // used in rehash exercise
    	
        final String[] taskNames = {"",""};
        VirtualObject value;
        private int index;
        
        public static final int VALUE = 1; 
        
        public HashCell(Object val, SelectableTable p, int i){
            value = new VirtualObject(val, this, "value field");
            selected = new VirtualBoolean(false, this, "selected field");
            disabled = new VirtualBoolean(false, this, "disabled field");
            parent = p;
            index = i;
        }
        
        public int getIndex(){
        	return index;
        }
        
        public Object getField(int location) {
            if (location == 0){
                return "";
            }
            else {
                return value.toString();
            }
        }

        public String getFieldName(int location) {
            return taskNames[location];
        }

        public String[] getFieldNames() {
            return taskNames;
        }

        public Object[] getFields() {
            return new Object[]{"", value};
        }

        public void setField(Object val, int location) {
            if (location == VALUE){
                value.setObject(val);
            }
        }

        public Object getElement() {
            return null;
        }

        public void setElement(Object o) {
        }

        public boolean equals(Object o){
            
            if (o instanceof HashCell){
            	HashCell r = (HashCell)o;
                return (value.toString().equals(r.value.toString()) &&
                        selected.equals(r.selected) &&
                        index == r.index); 
            }
            return false;
        }
        public long getSelectionTime() {            
            return selected.eval()?1:0;
        }

        public void setSelect(long globalClockTimestamp) {
       		parent.selected(this, true);            
        }
        
        public void assignSelect(boolean v){
        	selected.assign(v);
        }
        
        protected boolean isSelected(){
        	return selected.eval();
        }
        
        public StyleSheet getStyleSheet() {
        	
        	StyleSheet sh = new StyleSheetAdapter();
                
            sh.setDefaultPenColor(Color.black);
            sh.setFocusedPenColor(Color.black);
            sh.setSelectedPenColor(Color.BLUE);
            
            sh.setFocusedBackgroundColor(Color.white);
            sh.setSelectedBackgroundColor(Color.white);
            
            if (disabled.eval()){
                sh.setDefaultBorderColor(Color.gray);            	
            }
            else {
            	sh.setDefaultBorderColor(Color.white);
            }
            sh.setFocusedBorderColor(Color.BLUE);
            sh.setSelectedBorderColor(Color.GREEN);

            return sh;
        }
        
        public void setDisabled(boolean d){
        	disabled.assign(d);
        }
        
        public boolean isDisabled(){
        	return disabled.eval();
        }
        
        public SelectableTable getParent(){
        	return parent;
        }
        
    }

}
