//Matrix - algorithm simulation system
//Copyright (C) 2000- 2007 Matrix group
//(See the AUTHORS file and the author tag below for more information
//on the authors.)

//This program is free software; you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation; either version 2 of the License, or
//(at your option) any later version.

//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.

//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/*$Id: Heap_DeleteMin.java,v 1.11 2009/06/18 11:04:47 lhakulin Exp $*/

package content.exercises;

import java.util.Random;

import content.ExerciseProperties;
import content.interfaces.ButtonExercise;
import content.interfaces.ComparableExercise;
import content.interfaces.StyledExercise;
import content.interfaces.ConfigureVisualType;
import content.interfaces.ModelAnswerNames;
import content.interfaces.SimulationExerciseModel;
import content.interfaces.SwapBehaviour;
import content.exercises.structures.ExerHeap;
import matrix.animation.Animator;
import matrix.simulation.VisualTypeConf;
import matrix.structures.FDT.FDT;
import matrix.structures.FDT.probe.Key;
import matrix.structures.FDT.probe.Table;
import matrix.structures.code.probe.PseudoCode;
import matrix.structures.util.MatrixComparable;
import matrix.util.RandomKey;

public class Heap_DeleteMin extends AbstractSimulationExercise implements SimulationExerciseModel,
									ModelAnswerNames, ConfigureVisualType, ButtonExercise,
									SwapBehaviour, StyledExercise, ComparableExercise {

    Table t;
    ExerHeap bt, bh;

    private int[] S = null;
    public static final boolean DEBUG = false;

    public static final String[] code = new String[]{
        "_1 PROCEDURE DeleteMin(Node q)",
        "",
        "_2 Swap the root node with the last node on the lowest level",
        "   and decrease the heap-size by one (here, the last node is deleted)._/2",
        "_3 Restore the heap order property by calling MIN-HEAPIFY on the root node_/3,/1",
        "",
        "_4,5 PROCEDURE MIN-HEAPIFY(Node q)_/5",
        "",
        "_6 2   heap order property between q and its children is checked_/6",
        "_7 3   IF the heap order property is violated",
        "3.1    swap the nodes p and q in which p is the smallest children_/7",
        "3.2    continue from 2 until",
        "    _8 q reaches a position where the heap order property",
        "    is satisfied or reaches a leaf node_/4,/8"
    };

    public void c(Animator a, PseudoCode code, int location){
        a.startOperation();
        code.setLocation(location);
        a.endOperation();            
    }


    /**
     * In this exercise a drag operation performs a swap
     * @return
     */
    public boolean getSwapBehaviour() {
        return true;
    }

//  styled exercise

    public String[] getStructureVisualisations() {
        String[] s = { "array" , "layered tree" };
        return s;
    }


    public String[] getModelAnswerVisualisations() {
        String[] s = { "array", "layered tree", "pseudo" };
        return s;
    }

    public FDT[] init() {
        final int keys = 13; // the number of keys in the initial heap
        Random r = new java.util.Random(seed);
        do {
            S = RandomKey.createIntKeys(r, 10, 99, keys); // between 10 and 99, thus can be compared lexicographically
        } while (!isAcceptable());
        t = new Table(S);
        bt = new ExerHeap(15);
        for(int i = 0; i < t.size(); i++)
            bt.insert(bt.new DeletableKey(t.getObject(i)));
        FDT[] s = { bt, bt };
        return s;
    }

    /*
     * @return true if the current instance of input (see init()) is acceptable
     */
    private boolean isAcceptable() {
        ExerHeap test = new ExerHeap(15);
        Table t = new Table(S);
        int recursions = 0;
        for (int i=0; i<t.size(); i++)
            test.insert(t.getObject(i));
        for (int i=0; i<3; i++)
            test.deleteMin();
        recursions=test.getNumberOfRecursions();
        //matrix.util.Note.out(this, "recursions="+recursions);
        if ((recursions < 9) || (recursions > 10)) // almost the same amount of work in all instances
            return false;
        if (!test.hasInterrupted())
            return false;
        if (test.getMaxNumberOfRecursionsDuringSingleCall() < 4)
            return false;
        if (!test.hasEqualChildren())
            return false;
        return true;
    }

    public FDT[] init(Object o) {
        return init();
    }

    public String[] getStructureNames() {
        ExerciseProperties lang = ExerciseProperties.getInstance();
        return new String[] { 
                lang.get( "HEAP_DELETEMIN_HEAP_ARRAY_TITLE" ),
                lang.get( "HEAP_DELETEMIN_HEAP_TREE_TITLE" )
        };
    }

    public String getDescription() {
        return "";
    }

    public FDT[] makeModelAnswer() {
        return solveModel(true);
    }

    public FDT[] solve(){
        return new FDT[]{solveModel(false)[0]};
    }

    public FDT[] solveModel(final boolean model) {
        ExerciseProperties lang = ExerciseProperties.getInstance();
        
        final PseudoCode pseudoCode = new PseudoCode();
        pseudoCode.setCodeLines(code);
        pseudoCode.setCodeName( lang.get("HEAP_DELETEMIN_PSEUDOCODE_NAME") );
        pseudoCode.setLocation(1);

        final Animator a = Animator.getActiveAnimator();

        bh = new ExerHeap(15) {

            private static final long serialVersionUID = 4093740609312959944L;

            public void deleteMin() {
                if (model)a.startOperation();
                MatrixComparable tmp = (MatrixComparable)getObject(keys.eval() - 1);

                setObject(tmp, 0);
                setObject(new Key(EMPTY), keys.eval() - 1);
                if (model)a.endOperation();
                keys.dec();
                c(a, pseudoCode, 3);
                minHeapify(0);
            }

            public void minHeapify(int i) {

                if (!model)
                    super.minHeapify(i);

                // if(i >= keys)
                    if(i >= getSize()) {
                        c(a, pseudoCode, 8);
                        return;
                    }
                    MatrixComparable p = (MatrixComparable)getObject(i);
                    MatrixComparable lc = (MatrixComparable)getObject(leftChild(i));
                    MatrixComparable rc = (MatrixComparable)getObject(rightChild(i));

                    c(a, pseudoCode, 6);

                    if(lc == null || lc.equals(EMPTY)) {
                        c(a, pseudoCode, 8);
                        return;
                    }
                    if(rc == null || rc.equals(EMPTY)) { 
                        if(p.gt(lc) && (lc!=null && !lc.equals(EMPTY))) {
                            c(a, pseudoCode, 7);
                            a.startOperation();
                            swap(i, leftChild(i));
                            a.endOperation();
                            minHeapify(leftChild(i));
                        } else { c(a, pseudoCode, 8); }
                        return;
                    }
                    if(lc.gt(rc)) {
                        if(p.gt(rc) && (rc!=null && !rc.equals(EMPTY))) {
                            c(a, pseudoCode, 7);
                            a.startOperation();
                            swap(i, rightChild(i));
                            a.endOperation();
                            minHeapify(rightChild(i));
                        } else { c(a, pseudoCode, 8); }
                    } else {
                        if(p.gt(lc) && (lc!=null && !lc.equals(EMPTY))) {
                            c(a, pseudoCode, 7);
                            a.startOperation();
                            swap(i, leftChild(i));
                            a.endOperation();
                            minHeapify(leftChild(i));
                        } else { c(a, pseudoCode, 8); }
                    }
            }
        };

        for(int i = 0; i < t.size(); i++) {
            bh.insert(t.getObject(i));
        }
        for(int i = 0; i < 3; i++) {
            if (!model)a.startOperation();
            c(a, pseudoCode, 2);
            bh.deleteMin();
            if (!model)a.endOperation();
        }
        FDT[] s = { bh, bh, pseudoCode };
        return s;
    }

    public FDT[] getInitialStructures() {
        ExerHeap eh = new ExerHeap(15);
        return new FDT[] { eh, eh };
    }

    public FDT[] getSimulatedStructures() {
        return new FDT[] { bt, bt };
    }

    public String[] getModelAnswerNames() {
        ExerciseProperties lang = ExerciseProperties.getInstance();
        return new String[] { 
                lang.get( "HEAP_DELETEMIN_HEAP_ARRAY_MODEL_TITLE" ),
                lang.get( "HEAP_DELETEMIN_HEAP_TREE_MODEL_TITLE" ),
                lang.get( "HEAP_DELETEMIN_PSEUDOCODE_MODEL_TITLE" )
        };
    }


    public FDT[] getAnswer() {
        return new FDT[] { bt };
    }

    public SimulationExerciseModel getModelAnswer() {
        return this;
    }

    public String[] buttonNames() {
        ExerciseProperties lang = ExerciseProperties.getInstance();
        String[] s = { lang.get("HEAP_DELETEMIN_DECREMENT_BUTTON_TITLE") };
        return s;
    }

    public void decrementHeapsize() {
        // matrix.util.Note.out(this, "decrementheapsize");
        Animator.getActiveAnimator().startOperation();
        bt.decrementHeapsize();
        Animator.getActiveAnimator().endOperation();
    }

    public String[] buttonCommands() {
        String[] s = { "/decrementHeapsize" };
        return s;
    }

    public VisualTypeConf[] conf() {
        VisualTypeConf tabl = new VisualTypeConf();
        tabl.enable("matrix.visual.VisualKey", VisualTypeConf.HIGHLIGHT_OPERATION);
        tabl.enable("matrix.visual.VisualKey", VisualTypeConf.DROP_OPERATION);
        tabl.enable("matrix.visual.VisualKey", VisualTypeConf.DRAG_OPERATION);
        tabl.enable("matrix.visual.VisualArrayComponent", VisualTypeConf.HIGHLIGHT_OPERATION);
        tabl.enable("matrix.visual.VisualArrayComponent", VisualTypeConf.DROP_OPERATION);
        VisualTypeConf tre = new VisualTypeConf();
        tre.enable("matrix.visual.VisualKey", VisualTypeConf.HIGHLIGHT_OPERATION);
        tre.enable("matrix.visual.VisualKey", VisualTypeConf.DROP_OPERATION);
        tre.enable("matrix.visual.VisualKey", VisualTypeConf.DRAG_OPERATION);
        tre.enable("matrix.visual.VisualLayeredTreeComponent", VisualTypeConf.HIGHLIGHT_OPERATION);
        tre.enable("matrix.visual.VisualLayeredTreeComponent", VisualTypeConf.DROP_OPERATION);
        VisualTypeConf code  = new VisualTypeConf();
        return new VisualTypeConf[] { tabl, tre, code };
    }
    
    /* ComparableExercise interface */
    
	public int[] getCompareIndices(){
        return new int[] {1};	
	}

	public int[] getGradeIndices(){
        return null; // Used only in recovery mode
	}

	public boolean canRecover(){
		return false;
	}
	
	public String[] getCompareStructureVisualisations(){
		return null; // use default
	}
	
	public String[] getCompareStructureNames(){
		return null; // use default
	}    
	
	public VisualTypeConf[] getCompareVisualTypeConf(){
		return null; // use default
	}

    static final long serialVersionUID = -7671756502477641250L;
}
