//Matrix - algorithm simulation system
//Copyright (C) 2000- 2007 Matrix group
//(See the AUTHORS file and the author tag below for more information
//on the authors.)

//This program is free software; you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation; either version 2 of the License, or
//(at your option) any later version.

//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.

//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/*$Id: Heap_Max_Insert.java,v 1.6 2009/06/18 10:12:51 lhakulin Exp $*/

package content.exercises;

import java.util.Random;

import content.ExerciseProperties;
import content.interfaces.ComparableExercise;
import content.interfaces.ConfigureVisualType;
import content.interfaces.ModelAnswerNames;
import content.interfaces.SimulationExerciseModel;
import content.interfaces.SwapBehaviour;
import content.exercises.structures.ExerMaxHeap;
import matrix.animation.Animator;
import matrix.simulation.VisualTypeConf;
import matrix.structures.FDT.FDT;
import matrix.structures.FDT.probe.Table;
import matrix.structures.code.probe.PseudoCode;
import matrix.structures.util.MatrixComparable;
import matrix.util.RandomKey;

public class Heap_Max_Insert extends AbstractSimulationExercise implements SimulationExerciseModel,
										ModelAnswerNames, ConfigureVisualType, SwapBehaviour,
										ComparableExercise{

    private static final long serialVersionUID = 7109083489109342550L;
    Table t;
    ExerMaxHeap bt, bh;

    private int[] S = null;
    public static final boolean DEBUG = false;

    public static final String[] code = new String[]{
        "_0,1 PROCEDURE INSERT(Node q)",
        "",
        "_2 Insert new node in the leftmost free position on the lowest level._/2",
        "_3 Restore heap property by calling SIFT-UP on the new node_/3,/1",
        "",
        "_4,5 PROCEDURE SIFT-UP(Node q)_/5",
        "",
        "_6 2   heap order property between q and its parent node p is checked_/6",
        "_7 3   IF the heap order property is violated",
        "3.1    swap the nodes p and q_/7",
        "3.2    continue from 2 until",
        "    _8 q reaches a position where the heap order property",
        "    is satisfied or reaches the root node_/4,/8,/0"
    };

    public void c(Animator a, PseudoCode code, int location) {
        a.startOperation();
        code.setLocation(location);
        a.endOperation();            
    }

    /**
     * In this exercise a drag operation performs a swap
     * @return
     */
    public boolean getSwapBehaviour() {
        return true;
    }

    public FDT[] init() {
        final int asize = 13; // number of items to be inserted
        Random r = new java.util.Random(seed);
        do {
            S = RandomKey.createIntKeys(r, 10, 99, asize); // between 10 and 99, thus can be compared lexicographically
        } while (!isAcceptable());
        t = new Table(S);
        bt = new ExerMaxHeap(15);
        FDT[] s = { t, bt };
        return s;
    }

    /*
     * @return true if the current instance of input (see init()) is acceptable
     */
    boolean isAcceptable() {
        boolean duplicate = false;
        for (int i=1; i<S.length; i++)
            if (S[i] == S[i-1])
                duplicate = true;
        if (!duplicate)
            return false; // make sure we have at least one duplicate
        ExerMaxHeap test = new ExerMaxHeap(15);
        Table t = new Table(S);
        int swaps = 0;
        for (int i=0; i<t.size(); i++)
            swaps += test.insert(t.getObject(i));
        if ((swaps < t.size()+1) || (swaps > t.size()+3))
            return false; // too few or too many swaps
        if (!test.hasCollision())
            return false; // no collision encountered
        return true;
    }

    public FDT[] init(Object o) {
        return init();
    }

    public String[] getStructureNames() {
        ExerciseProperties lang = ExerciseProperties.getInstance();
        return new String[] { 
                lang.get( "HEAP_MAX_INSERT_INPUT_TITLE" ),
                lang.get( "HEAP_MAX_INSERT_HEAP_TREE_TITLE" )
        };
    }

    public String getDescription() {
        return "";
    }

    public FDT[] makeModelAnswer() {
        return solveModel(true);
    }

    public FDT[] solve(){
        return new FDT[]{solveModel(false)[0]};
    }

    public FDT[] solveModel(final boolean model) {
        ExerciseProperties lang = ExerciseProperties.getInstance();

        final PseudoCode pseudoCode = new PseudoCode();
        pseudoCode.setCodeLines(code);
        pseudoCode.setCodeName( lang.get( "HEAP_MAX_INSERT_PSEUDOCODE_NAME" ) );
        pseudoCode.setLocation(1);

        final Animator a = Animator.getActiveAnimator();

        bh = new ExerMaxHeap(15) {

            private static final long serialVersionUID = 7044184753695498886L;

            public int insert(Object o) {
                int swaps = 0;
                if (DEBUG)
                    System.out.println("Size:" + keys);
                a.startOperation();
                pseudoCode.setLocation(2);
                setObject(o, keys.eval());
                keys.inc();
                a.endOperation();

                c(a, pseudoCode, 3);
                swaps = bubble(keys.eval()-1);
                c(a, pseudoCode, 8);

                return swaps;
            }

            protected int bubble(int n) {
                MatrixComparable p = (MatrixComparable)getObject(parent(n));
                MatrixComparable t = (MatrixComparable)getObject(n);
                c(a, pseudoCode, 6);

                if (p == null || p.lt(t)) {
                    c(a, pseudoCode, 7);
                    if (model)a.startOperation();
                    swap(n,parent(n));
                    if (model)a.endOperation();
                    c(a, pseudoCode, 6);
                    return bubble(parent(n)) + 1;
                } return 0;
            }

            public void maxHeapify(int i) {

                if (!model)
                    super.maxHeapify(i);

                // if(i >= keys)
                if(i >= getSize())
                    return;
                MatrixComparable p = (MatrixComparable)getObject(i);
                MatrixComparable lc = (MatrixComparable)getObject(leftChild(i));
                MatrixComparable rc = (MatrixComparable)getObject(rightChild(i));


                if(lc == null || lc.equals(EMPTY))
                    return;
                if(rc == null || rc.equals(EMPTY)) { 
                    if(p.lt(lc) && (lc!=null && !lc.equals(EMPTY))) {
                        a.startOperation();
                        swap(i, leftChild(i));
                        a.endOperation();
                        maxHeapify(leftChild(i));
                    }
                    return;
                }
                if(lc.lt(rc)) {
                    if(p.lt(rc) && (rc!=null && !rc.equals(EMPTY))) {
                        a.startOperation();
                        swap(i, rightChild(i));
                        a.endOperation();
                        maxHeapify(rightChild(i));
                    }
                } else {
                    if(p.lt(lc) && (lc!=null && !lc.equals(EMPTY))) {
                        a.startOperation();
                        swap(i, leftChild(i));
                        a.endOperation();
                        maxHeapify(leftChild(i));
                    }
                }
            }			
        };
        Table tbl = (Table)getInitialStructures()[0];
        for(int i = 0; i < tbl.size(); i++) {
            if (!model)a.startOperation();
            bh.insert(tbl.getObject(i));
            if (!model)a.endOperation();
        }
        c(a, pseudoCode, 0);
        FDT[] s = { bh, pseudoCode };
        return s;
    }

    public FDT[] getInitialStructures() {
        return new FDT[] { new Table(S), new ExerMaxHeap(15) };
    }

    public FDT[] getSimulatedStructures() {
        return new FDT[] { t, bt };
    }

    public String[] getModelAnswerNames() {
        ExerciseProperties lang = ExerciseProperties.getInstance();
        return new String[] { 
                lang.get( "HEAP_MAX_INSERT_HEAP_TREE_MODEL_TITLE" ),
                lang.get( "HEAP_MAX_INSERT_PSEUDOCODE_MODEL_TITLE" )
        };
    }

    public FDT[] getAnswer() {
        return new FDT[] { bt };
    }

    public SimulationExerciseModel getModelAnswer() {
        return this;
    }

    public VisualTypeConf[] conf() {
        VisualTypeConf tabl = new VisualTypeConf();
        tabl.enable("matrix.visual.VisualKey", VisualTypeConf.HIGHLIGHT_OPERATION);
        tabl.enable("matrix.visual.VisualKey", VisualTypeConf.DRAG_OPERATION);
        VisualTypeConf tre = new VisualTypeConf();
        tre.enable("matrix.visual.VisualKey", VisualTypeConf.HIGHLIGHT_OPERATION);
        tre.enable("matrix.visual.VisualKey", VisualTypeConf.DROP_OPERATION);
        tre.enable("matrix.visual.VisualKey", VisualTypeConf.DRAG_OPERATION);
        tre.enable("matrix.visual.VisualLayeredTreeComponent", VisualTypeConf.HIGHLIGHT_OPERATION);
        tre.enable("matrix.visual.VisualLayeredTreeComponent", VisualTypeConf.DROP_OPERATION);
        return new VisualTypeConf[] { tabl, tre };
    }
    
    /* ComparableExercise interface */
    
	public int[] getCompareIndices(){
        return new int[] {0};	
	}

	public int[] getGradeIndices(){
        return null; // Used only in recovery mode
	}

	public boolean canRecover(){
		return false;
	}
	
	public String[] getCompareStructureVisualisations(){
		return null; // use default
	}
	
	public String[] getCompareStructureNames(){
		return null; // use default
	}    

	public VisualTypeConf[] getCompareVisualTypeConf(){
		return null; // use default
	}

}
