//Matrix - algorithm simulation system
//Copyright (C) 2000- 2007 Matrix group
//(See the AUTHORS file and the author tag below for more information
//on the authors.)

//This program is free software; you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation; either version 2 of the License, or
//(at your option) any later version.

//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.

//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package content.exercises;

import content.ExerciseProperties;
import content.interfaces.*;
import matrix.animation.Animator;
import matrix.simulation.VisualTypeConf;
import content.exercises.structures.*;
import matrix.structures.FDT.FDT;
import matrix.structures.FDT.probe.*;
import matrix.structures.CDT.probe.*;
import java.util.*;

public class InfixToPostfixExpressions implements SimulationExerciseModel,
StyledExercise, ModelAnswerNames, ConfigureVisualType, JudgeBlocks, ComparableExercise {

    //ATRIBUTES


    private Table table1;

    private StackImplInfixToPostFix stack1;

    private QueueImpl queue1;

    private String result;

    private long seed = 1;

    //METHODS

    public FDT[] solve() {

        Table tab = (Table) getInitialStructures()[0];
        StackImpl st = (StackImplInfixToPostFix) getInitialStructures()[1];
        QueueImpl qu = (QueueImpl) getInitialStructures()[2];
        int index = 0;
        Key k, k2;
        Animator an = Animator.getActiveAnimator();

        while (index < tab.size()) {

            an.startOperation();

            k = (Key) tab.next();

            if (k.toString().indexOf('*') != -1) {

                boolean fin = false;

                while (!fin) {

                    k2 = (Key) st.getElement();

                    if (k2 != null) {

                        if (k2.toString().indexOf('*') != -1) {

                            qu.insert(k2);
                            st.delete(k2);
                            an.endOperation();
                            an.startOperation();
                        } else {

                            fin = true;
                            st.insert(k);

                        }

                    } else {

                        fin = true;
                        st.insert(k);

                    }

                }

            } else if (k.toString().indexOf('+') != -1) {

                boolean fin2 = false;

                while (!fin2) {

                    k2 = (Key) st.getElement();

                    if (k2 != null) {

                        if ((k2.toString().indexOf('*') != -1)
                                || (k2.toString().indexOf('+') != -1)) {

                            qu.insert(k2);
                            st.delete(k2);
                            an.endOperation();
                            an.startOperation();
                        } else {

                            fin2 = true;
                            st.insert(k);

                        }

                    } else {

                        fin2 = true;
                        st.insert(k);

                    }

                }

            } else if (k.toString().indexOf(')') != -1) {

                k2 = (Key) st.getElement();

                if (k2 != null) {

                    while (k2.toString().indexOf('(') == -1) {

                        qu.insert(k2);
                        st.delete(k2);
                        an.endOperation();
                        an.startOperation();
                        k2 = (Key) st.getElement();

                    }
                    st.delete(k2);
                }

            } else if (k.toString().indexOf('(') != -1) {

                st.insert(k);

            } else {

                qu.insert(k);

            }

            an.endOperation();
            index++;
        }

        while (!st.isEmpty()) {

            an.startOperation();
            k2 = (Key) st.getElement();
            qu.insert(k2);
            st.delete(k2);
            an.endOperation();

        }

        return new FDT[] { st, qu };
    }

    public FDT[] makeModelAnswer() {

        return solve();

    }

    public String[] getStructureVisualisations() {

        return new String[] { "list", "list", "list" };

    }

    public String[] getModelAnswerVisualisations() {

        return new String[] { "list", "list" };

    }

    public String[] getModelAnswerNames() {
        ExerciseProperties lang = ExerciseProperties.getInstance();
        return new String[] { 
                lang.get( "INFIXTOPOSTFIX_STACK_MODEL_TITLE" ),
                lang.get( "INFIXTOPOSTFIX_POSTFIXEXPRESSION_MODEL_TITLE" )
        };
    }

    public VisualTypeConf[] conf() {

        VisualTypeConf tabl1 = new VisualTypeConf();
        tabl1.enable("matrix.visual.VisualKey", VisualTypeConf.HIGHLIGHT_OPERATION);
        tabl1.enable("matrix.visual.VisualKey", VisualTypeConf.DRAG_OPERATION);

        VisualTypeConf stac = new VisualTypeConf();
        stac.enable("matrix.visual.VisualList", VisualTypeConf.DROP_OPERATION);
        stac.enable("matrix.visual.VisualList", VisualTypeConf.HIGHLIGHT_OPERATION);
        stac.enable("matrix.visual.VisualKey", VisualTypeConf.DRAG_OPERATION);
        stac.enable("matrix.visual.VisualKey", VisualTypeConf.HIGHLIGHT_OPERATION);

        VisualTypeConf tabl2 = new VisualTypeConf();
        tabl2.enable("matrix.visual.VisualList", VisualTypeConf.HIGHLIGHT_OPERATION);
        tabl2.enable("matrix.visual.VisualList", VisualTypeConf.DROP_OPERATION);

        return new VisualTypeConf[] { tabl1, stac, tabl2 };

    }

    public FDT[] init() {

        String[] setA = { "a", "b", "c", "d", "e", "f", "g", "h", "i", "1", "2", "3", "4", "5", "6", "7", "8", "9" };
        String[] setB = { "+", "*", "+", "*", "+(", "*(", ")+", ")*" };

        int depth = 0;
        result = "";
        Random r = new Random(seed);

        int numOfParenthesis = 0;

        //For checking precedence
        boolean wasPlus = false;
        boolean wasMult = false;
        boolean plusMult = false;
        boolean multPlus = false;

        int varCount = 0;
        int index = 0;

        while (numOfParenthesis < 2 || !plusMult || !multPlus || depth > 1) {
            result = "";
            numOfParenthesis = 0;
            depth = 0;
            multPlus = false;
            plusMult = false;
            wasMult = false;
            wasPlus = false;

            for (int i = 0; i < 5; i++) {
                result += setA[r.nextInt(setA.length)];
                varCount++;

                if (depth > 0 && varCount > 1)
                    index = r.nextInt(setB.length);
                else
                    index = r.nextInt(setB.length - 2);


                if (index % 2 == 0 && wasMult == true)
                    multPlus = true;
                if (index % 2 == 1 && wasPlus == true)
                    plusMult = true;

                wasPlus = ((index % 2) == 0);
                wasMult = ((index % 2) == 1);

                if (index == 4 || index == 5) {
                    depth++;
                    numOfParenthesis++;
                    varCount = 0;
                    wasPlus = false;
                    wasMult = false;
                }

                if (index == 6 || index == 7) {
                    depth--;
                    wasPlus = false;
                    wasMult = false;
                }

                result += setB[index];
            }
            result += setA[r.nextInt(setA.length)];

            if (varCount == 0)
                result += setB[r.nextInt(2)] + setA[r.nextInt(setA.length)];

            for (int i = 0; i < depth; i++) {
                result += ")";
            }
        }

        queue1 = new QueueImpl();
        stack1 = new StackImplInfixToPostFix();
        table1 = new Table(result);
        return new FDT[] { table1, stack1, queue1 };

    }

    public void setSeed(long v) {

        this.seed = v;
    }

    public long getSeed() {

        return seed;
    }

    public String[] getStructureNames() {
        ExerciseProperties lang = ExerciseProperties.getInstance();
        return new String[] { 
                lang.get( "INFIXTOPOSTFIX_INFIXEXPRESSION_TITLE" ),
                lang.get( "INFIXTOPOSTFIX_STACK_TITLE" ),
                lang.get( "INFIXTOPOSTFIX_POSTFIXEXPRESSION_TITLE" )
        };
    }

    public FDT[] getAnswer() {

        return new FDT[] { stack1, queue1 };
    }

    public FDT[] getInitialStructures() {

        return new FDT[] { new Table(result), new StackImplInfixToPostFix(),
                new QueueImpl() };

    }

    public String getDescription() {

        return "";
    }
    
    /* ComparableExercise interface */
    
	public int[] getCompareIndices(){
        return new int[] {0, 1};	
	}

	public int[] getGradeIndices(){
        return null; // Used only in recovery mode
	}

	public boolean canRecover(){
		return false;
	}
	
	public String[] getCompareStructureVisualisations(){
		return null; // use default
	}
	
	public String[] getCompareStructureNames(){
		return null; // use default
	}    
	
	public VisualTypeConf[] getCompareVisualTypeConf(){
		return null; // use default
	}

    static final long serialVersionUID = -8169820934240394313L;
}
