// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package content.exercises;
import content.ExerciseProperties;
import content.interfaces.*;
import matrix.animation.Animator;
import matrix.simulation.VisualTypeConf;
import matrix.structures.FDT.FDT;
import matrix.structures.FDT.probe.*;
import matrix.structures.memory.VirtualInteger;
import matrix.structures.spatial.Area;
import matrix.structures.spatial.PaintingStyleDecorator;
import matrix.structures.spatial.SpatialComparable;
import matrix.structures.spatial.FDT.probe.SpatialElement;
import matrix.util.Note;
import matrix.util.RandomKey;
import java.awt.*;
import java.util.Arrays;
import java.util.Random;
import java.util.Vector;

/**
 * Insertionsort algorithm exercise where the end part
 * of the input table is already in order.
 * 
 * @author lhakulin
 *
 */

public class InsertionSortPartlyOrdered implements SimulationExerciseModel, StyledExercise, ModelAnswerNames,
											ConfigureVisualType, JudgeBlocks, SwapBehaviour, LayoutExercise,
											ComparableExercise {

    private Table table;
    private Table tempTable; //Table which size is one. It is used as a temp variable when doing the exercise.
    private long seed = 1;
    private String[] s;
    private String tempString;
    private String initialValueOfTmp;
    
	private static final int ELEMENT_GAP = 5;

    private static final PaintingStyleDecorator areaDecorator;
    private static final PaintingStyleDecorator barDecorator;
    private static final PaintingStyleDecorator barDecoratorOrdered;
    
    static {
		areaDecorator = new PaintingStyleDecorator();
		areaDecorator.setOutline(false);
		areaDecorator.setFill(false);
        areaDecorator.setStroke(2, BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL);
		
		barDecorator = new PaintingStyleDecorator();
		barDecorator.setLineColor(Color.black);
        barDecorator.setStroke(2, BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL);
        barDecorator.setFocusFillColor(barDecorator.getFillColor());

        barDecoratorOrdered = new PaintingStyleDecorator();
        barDecoratorOrdered.setLineColor(Color.black);
        barDecoratorOrdered.setFillColor(new Color(0,80,0));
        barDecoratorOrdered.setTransparency(0.6);
        barDecoratorOrdered.setStroke(2, BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL);
        barDecoratorOrdered.setFocusFillColor(barDecoratorOrdered.getFillColor());
    }


    public boolean getSwapBehaviour() {
        return false;
    }

    public FDT[] solve(){

        FDT[] structures = getInitialStructures();
        InsertionBarTable tab = (InsertionBarTable) structures[0];
        Table temp = (Table) structures[1];
        Animator an = Animator.getActiveAnimator();

        for (int i = 1; i < tab.size(); i++) { 

            an.startOperation();
            temp.setObject(tab.getObject(i), 0);
            tab.selectIndex(i);
            an.endOperation();
            int tempKey= Integer.parseInt(((Key)tab.getObject(i)).getComparisonString());

            int j = i;

            while ((j > 0) && (Integer.parseInt(((Key)tab.getObject(j-1)).getComparisonString()) > tempKey)) {
                an.startOperation();
                tab.setObject((Key)tab.getObject(j-1),j);
                an.endOperation();
                j= j-1;
            } 

            an.startOperation();
            tab.setObject(temp.getObject(0),j);
            an.endOperation();
        }
        return new FDT[]{tab, temp, tab};
    }


    public FDT[] makeModelAnswer(){

        return solve();

    }
    public String[] getStructureVisualisations(){

        return new String[]  {"array", "array", "2D"};

    }

    public String[] getModelAnswerVisualisations(){

        return new String[] {"array", "array", "2D"};

    }
    
    public String[] getModelAnswerNames() {
        ExerciseProperties lang = ExerciseProperties.getInstance();
        return new String[] { 
                lang.get( "INSERTIONSORTPARTLYORDERED_INPUT_MODEL_TITLE" ),
                lang.get( "INSERTIONSORTPARTLYORDERED_TEMPORARY_MODEL_TITLE" ),
                lang.get( "INSERTIONSORTPARTLYORDERED_INPUT_BARGRAPH_MODEL_TITLE" )
        };
    }

    public VisualTypeConf[] conf(){

        VisualTypeConf tabl1 = new VisualTypeConf();
        tabl1.enable("matrix.visual.VisualKey", VisualTypeConf.HIGHLIGHT_OPERATION);

        tabl1.enable("matrix.visual.VisualKey", VisualTypeConf.DRAG_OPERATION);
        tabl1.enable("matrix.visual.VisualKey", VisualTypeConf.DROP_OPERATION);
        tabl1.enable("matrix.visual.VisualArrayComponent", VisualTypeConf.DROP_OPERATION);

        VisualTypeConf tabl2 = new VisualTypeConf();
        tabl2.enable("matrix.visual.VisualKey", VisualTypeConf.HIGHLIGHT_OPERATION);
        tabl2.enable("matrix.visual.VisualKey", VisualTypeConf.DRAG_OPERATION);
        tabl2.enable("matrix.visual.VisualKey", VisualTypeConf.DROP_OPERATION);
        tabl2.enable("matrix.visual.VisualArrayComponent", VisualTypeConf.HIGHLIGHT_OPERATION);
        tabl2.enable("matrix.visual.VisualArrayComponent", VisualTypeConf.DROP_OPERATION);
        tabl2.setVisualConfValue("matrix.visual.VisualArray", VisualTypeConf.INDEXED, "false");
        
		VisualTypeConf barGraph = new VisualTypeConf();
        barGraph.enable("matrix.visual.VisualArea$AreaVisualization", VisualTypeConf.HIGHLIGHT_OPERATION);
        barGraph.setVisualConfValue("matrix.visual.VisualArea", VisualTypeConf.AXIS_VISIBLE, "false");

        return new VisualTypeConf[] { tabl1, tabl2, barGraph};

    }

    public FDT[] init(){

        s=new String[16];
        tempString = " ";
        Random random = new Random(seed);

        // end part of the table is in order and consists of larger numbers
        int dBegin[];
        int dEnd[];
        dBegin = matrix.util.RandomKey.createIntKeys(random,0,20,8);
        dEnd = matrix.util.RandomKey.createIntKeys(random,15,30,8);
        Arrays.sort(dEnd);
    
        for (int i=0;i<dBegin.length;i++){
            s[i] = dBegin[i] + "";
        }
        for (int i=0;i<dEnd.length;i++){
            s[i+dBegin.length] = dEnd[i] + "";
        }
        initialValueOfTmp = RandomKey.createIntKey(random, -9, 0) + "";

        FDT[] fdts = getInitialStructures();
        table = (InsertionBarTable)fdts[0];
        tempTable = (Table)fdts[1];
        return fdts;
    }

    public void setSeed(long v){

        this.seed = v;
    }


    public long getSeed(){

        return seed;
    }

    public String[] getStructureNames() {
        ExerciseProperties lang = ExerciseProperties.getInstance();
        return new String[] { 
                lang.get( "INSERTIONSORTPARTLYORDERED_INPUT_TITLE" ),
                lang.get( "INSERTIONSORTPARTLYORDERED_TEMPORARY_TITLE" ),
                lang.get( "INSERTIONSORTPARTLYORDERED_INPUT_BARGRAPH_TITLE" )
        };
    }

    public FDT[] getAnswer(){

        return new FDT[]{table, tempTable, table};
    }
    public FDT[] getInitialStructures(){
        InsertionBarTable ct = new InsertionBarTable(s);
        ct.selectIndex(0);
        ct.setPaintingStyleDecorator(areaDecorator);
        ct.setBarPaintingStyleDecorator(barDecorator);
        ct.setOrderedBarPaintingStyleDecorator(barDecoratorOrdered);

        Table t = new Table(tempString);

        t.setObject(initialValueOfTmp, 0); 
        return new FDT[] { ct, t, ct};

    }
    public String getDescription(){

        return "";
    }
    
    public GridBagConstraints[] getConstraints() {
        
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 1;
        gbc.insets = new Insets(ELEMENT_GAP, 2, ELEMENT_GAP, 2);
        
        GridBagConstraints gbc2 = new GridBagConstraints();
        gbc2.gridx = 0;
        gbc2.gridy = 2;
        gbc2.insets = new Insets(ELEMENT_GAP, 2, ELEMENT_GAP, 2);

        GridBagConstraints gbc3 = new GridBagConstraints();
        gbc3.gridx = 0;
        gbc3.gridy = 0;
        gbc3.insets = new Insets(ELEMENT_GAP, 2, ELEMENT_GAP, 2);
        
        return new GridBagConstraints[]{ gbc, gbc2, gbc3 };
    }
    
    public GridBagConstraints[] getModelAnswerConstraints() {
    	return getConstraints();
    }
    
	public GridBagConstraints[] getComparisonConstraints() {
		return null; // use default
	}
    
    /* ComparableExercise interface */
    
	public int[] getCompareIndices(){
        return new int[] {0, 1, 2};	
	}

	public int[] getGradeIndices(){
        return null; // Used only in recovery mode
	}

	public boolean canRecover(){
		return false;
	}
	
	public String[] getCompareStructureVisualisations(){
		return null; // use default
	}
	
	public String[] getCompareStructureNames(){
		return null; // use default
	}    
	
	public VisualTypeConf[] getCompareVisualTypeConf(){
		return null; // use default
	}
    

    /**
     * A table which is sorted.
     * Can also be visualized as a bar graph.
     * @author lhakulin
     *
     */
    static final class InsertionBarTable extends Table implements Area {
    	
    	private VirtualInteger highEndInd;

        private final Rectangle boundingBox;
        private PaintingStyleDecorator paintingStyleDecorator;
        private PaintingStyleDecorator barDecorator;
        private PaintingStyleDecorator barDecoratorOrdered;
        
        private static final int BAR_WIDTH = 29;
        private static final int HEIGHT_SCALE = 5;
    	private static final int BOTTOM_BORDER_HEIGHT = 20;

        public InsertionBarTable(String[] ss) {
            super(ss);
            for(int i=0; i < ss.length; i++){
                setObject(new InsertionBarTableKey(ss[i], i, this), i);
            }
        	int width = ss.length * BAR_WIDTH + 25;
            this.boundingBox = new Rectangle(0, 0, width, 200);
            this.barDecorator = new PaintingStyleDecorator();
            this.paintingStyleDecorator = new PaintingStyleDecorator();
            highEndInd = new VirtualInteger(0, this, "Index of the highest ordered element");
        }
        
        public void selectIndex(int index) {
        	
        	highEndInd.assign(index);
        	super.selectIndex(0);
        	
        	if (index == 0){
        		// needed when highlighting only the first index
        		super.selectIndex(-1);
        	}
        	else {
        		super.selectIndex(index);
        	}
        	
        }
        
        /**
         * Higlight all the indexes of the table from beginning to the key k
         * @param k - Last key to be highlighted
         */
        public void updateIndexHighlighting(InsertionBarTableKey k){
        	for (int i=highEndInd.eval(); i <= getLast(); i++){
        		InsertionBarTableKey a = (InsertionBarTableKey)getObject(i);
        		if (a.equals(k)){
        			selectIndex(i);
        			break;
        		}
        	}
        }        
        
        // ** Implementation of interface Area **

        public void addSubArea(Area a) {
        	// Not implemented
        }

        public void removeSubArea(Area a) {
        	// Not implemented
        }

        public Area[] getSubAreas() {
        	// Not implemented
            return new Area[0];
        }

        public void delete(SpatialComparable e) {
        	// Not implemented
        }

        public void insert(SpatialComparable e) {
        	// Not implemented
        }        

        public Rectangle getBoundingPolygon() {
            return this.boundingBox;
        }

        public SpatialElement[] getSpatialElements() {
        	
            Vector vec = new Vector();
            for (int i = this.getFirst(); i <= this.getLast(); i++){
            	if (getObject(i) instanceof Key){
            		try {
                		int value = Integer.parseInt(((Key)getObject(i)).getVisualizationString()) * HEIGHT_SCALE;
                		matrix.structures.spatial.FDT.probe.Rectangle rec = new matrix.structures.spatial.FDT.probe.Rectangle(i*BAR_WIDTH, BOTTOM_BORDER_HEIGHT, BAR_WIDTH, value);
                		if (i <= highEndInd.eval()){
                			rec.setPaintingStyleDecorator(this.barDecoratorOrdered);
                		}
                		else {
                			rec.setPaintingStyleDecorator(this.barDecorator);
                		}
                		vec.add(rec);
                		vec.add(new matrix.structures.spatial.FDT.probe.Label(i*BAR_WIDTH + BAR_WIDTH/2, 10, ""+i));
					} catch (Exception e) {
		                Note.err(this, "Unable to parse integer value of key " + getObject(i).toString());
					}
            	}
            }
            
            SpatialElement[] spatialElements = new SpatialElement[vec.size()];
            vec.toArray(spatialElements);
            return spatialElements;
        }
                
        public boolean hasPaintingStyleDecorator(){
            return (paintingStyleDecorator != null);
        }

        public void setPaintingStyleDecorator(PaintingStyleDecorator psd){
            paintingStyleDecorator = psd;
        }

        public PaintingStyleDecorator getPaintingStyleDecorator(){
            return paintingStyleDecorator;
        }
        
        public void setBarPaintingStyleDecorator(PaintingStyleDecorator psd){
            barDecorator = psd;
        }
        public void setOrderedBarPaintingStyleDecorator(PaintingStyleDecorator psd){
            barDecoratorOrdered = psd;
        }
        
        static final long serialVersionUID = 8668291687539197711L;   
    }

    /**
     * A key which is used in the InsertionBarTable in InsertionSort exercise
     * @author lhakulin
     *
     */
    public static final class InsertionBarTableKey extends Key {

        private InsertionBarTable parent;

        public InsertionBarTableKey(String keyText, int i, InsertionBarTable p){
            super(keyText);
            
            // indexed are always length of 2
            if (i <= 9){
            	setComparisonString(keyText+"0"+i);
            }
            else {
            	setComparisonString(keyText+i);
            }
            setVisualizationString(keyText);
            parent = p;
        }
        
        public void pressed(){
        	Animator.getActiveAnimator().startOperation();
        	parent.updateIndexHighlighting(this);        
        	Animator.getActiveAnimator().endOperation();
        }

        static final long serialVersionUID = 5356225316236393671L;    
    }
    
    static final long serialVersionUID = 5324781321629168531L;
}
