// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/*$Id: InterpolationSearch.java,v 1.39 2009/06/12 10:42:55 lhakulin Exp $*/

package content.exercises;

import content.ExerciseProperties;
import content.interfaces.ComparableExercise;
import content.interfaces.ConfigureVisualType;
import content.interfaces.ModelAnswerNames;
import matrix.animation.AnimatedStructures;
import matrix.animation.Animator;
import matrix.simulation.VisualTypeConf;
import matrix.structures.FDT.*;
import matrix.structures.FDT.probe.Table;

public class InterpolationSearch extends BinarySearch implements ModelAnswerNames, ConfigureVisualType,
										ComparableExercise {
    
    public static final boolean DEBUG = false;
    
    public InterpolationSearch() {
        
    }
    
    public InterpolationSearch(int[] sortedKeys, int key) {
        super(sortedKeys,key);
    }
    

    public String[] getStructureNames() {
        ExerciseProperties lang = ExerciseProperties.getInstance();
        String[] s = { lang.get("INTERPOLATION_SEARCH_TABLE_TITLE"), lang.get("INTERPOLATION_SEARCH_MIDPOINT_LIST_TITLE") };
        return s;
    }

    public String getDescription() {
        ExerciseProperties lang = ExerciseProperties.getInstance();
        return lang.get("INTERPOLATION_SEARCH_PROMPT") + " " +keyToSearch;        
    }
    
    public String[] getModelAnswerNames() {
        ExerciseProperties lang = ExerciseProperties.getInstance();

        String[] s = { lang.get("INTERPOLATION_SEARCH_TABLE_MODEL_TITLE") };
        return s;
    }
    
    
    protected int getMiddleOne(int left, int right) {
        
        if (left >= right)
            return left;
        
        int mid = left +  Math.abs( (keyToSearch - keyArray[left]) *
            (right - left) ) /
            ( keyArray[right] - keyArray[left] );
        //Note.out(this,"calc middle one=" + mid);
        return mid;
    }
    
    public FDT[] solve() {
        //Note.out(this, "Solve");
        FDT[] initialStructures = getInitialStructures();  
        
        LinkedList modelList = (LinkedList)initialStructures[1];
        LinkedList last = modelList;
        Table model = (Table)initialStructures[0];
        Animator a = Animator.getActiveAnimator();
        int low = 0;
        int high = model.size() - 1;
        int mid = 0;
        int i = 0;
        while (keyArray[low] < keyToSearch && keyArray[high] >= keyToSearch) {
            a.startOperation();
            mid = getMiddleOne(low,high);
            if (DEBUG) {
                System.out.println("Iteration: "+i++);
                System.out.println("Low: "+low);
                System.out.println("High: "+high);
                System.out.println("Mid: "+mid);
                System.out.println("Key: "+keyArray[mid]);
            }
            if (model.getObject(mid) != null ) {
                last.setNext(last.getNewNode(model.getObject(mid)));
                last = last.getNext();
            }
            if( keyArray[ mid ] < keyToSearch)
                low = mid + 1;
            else if( keyArray[ mid ] > keyToSearch )
                high = mid - 1;
            else {
                a.endOperation();
                return new FDT[] { modelList.getNext() };  /* Found mid*/
            }
            a.endOperation();
        }
        /* not added to model answer
         //compare with the key at low
          a.startOperation();
          last.setNext(last.getNewNode(model.getObject(low)));
          last = last.getNext();
          a.endOperation();
          */
        
        //Note.out(this, "Solved");
        return new FDT[] { modelList.getNext() };
    }
    
    public VisualTypeConf[] conf() {
        VisualTypeConf tabl = new VisualTypeConf();
        tabl.enable("matrix.visual.VisualKey", VisualTypeConf.HIGHLIGHT_OPERATION);
        tabl.enable("matrix.visual.VisualKey", VisualTypeConf.DRAG_OPERATION);
        VisualTypeConf lis = new VisualTypeConf();
        lis.enable("matrix.visual.VisualKey", VisualTypeConf.DROP_OPERATION);
        lis.enable("matrix.visual.VisualListComponent", VisualTypeConf.DROP_OPERATION);
        lis.enable("matrix.visual.VisualList", VisualTypeConf.DROP_OPERATION);
        lis.enable("matrix.visual.VisualList", VisualTypeConf.HIGHLIGHT_OPERATION);
        return new VisualTypeConf[] { tabl, lis};
    }
    
    /* ComparableExercise interface */
    
	public int[] getCompareIndices(){
        return new int[] {0};	
	}

	public int[] getGradeIndices(){
        return new int[] {0};
	}

	public boolean canRecover(){
		return false;
	}
    
    /* KnownMisconceptions interface */
    
    public matrix.animation.AnimatedStructures[] createMisconceptionSequences(){
        AnimatedStructures[] sequences = new AnimatedStructures[3];
        
        sequences[0] = makeMisconception(false, true);
        sequences[1] = makeMisconception(true,  false);
        sequences[2] = makeMisconception(true,  true);
        
        return sequences;
    }
    
    public String[] getMisconceptionNicknames(){
        return new String[]{
            "Wrong limits",
            "Rounds up",
            "Limits+Round"
        };      
    }
    
    public String[] getGuidanceStrings(){
        return new String[]{
            "Advice missing",
            "Advice missing",
            "Advice missing"
        };     
    }
    
    private AnimatedStructures makeMisconception(boolean roundUp, boolean limits) {
        FDT[] initialStructures = getInitialStructures();  
                
        Table model   = (Table)initialStructures[0];
        LinkedList modelList = (LinkedList)initialStructures[1];
        
        //This is the answer sequence
        LinkedList last = modelList;
        
        //Saving the old animator
        Animator oldAnimator = Animator.getActiveAnimator();
        
        //Creating a brand new Animator for this sequence and structures only
        Animator a = new Animator();
        Animator.setActiveAnimator(a);   
        
        int low = 0;
        int high = model.size()-1;
        int mid = 0;
        
        //Wrong algorithms can do endless recursions and endless iterations.
        //GiveUp gives and upper limit for this. Never iterate more than giveUp times.
        int giveUp = model.size();
        
        while (low <= high) {
            a.startOperation();
            mid = getMiddleOne(low + (roundUp? 1 : 0),high);
            
            last.setNext(last.getNewNode(model.getObject(mid)));
            last = last.getNext();
            try {
                if( keyArray[mid] < keyToSearch)
                    low  = mid + (limits? 0 : 1);
                else if( keyArray[ mid ] > keyToSearch )
                    high = mid - (limits? 0 : 1);
                else {
                    a.endOperation();
                    break; //found the key at midpoint
                }
            }  catch(ArrayIndexOutOfBoundsException e) {
                //Note.out(this,e+"index = "+mid);
                a.endOperation();
                break;   //the key was not found
            }
            a.endOperation();
            if (giveUp-- == 0)
                break;
        }
        
        //Returning the old Animator
        Animator.setActiveAnimator(oldAnimator);
        
        return new AnimatedStructures(new FDT[]{modelList.getNext()}, a);
    }
    
    
    static final long serialVersionUID = 2428841166095544234L;
}
