//Matrix - algorithm simulation system
//Copyright (C) 2000- 2007 Matrix group
//(See the AUTHORS file and the author tag below for more information
//on the authors.)

//This program is free software; you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation; either version 2 of the License, or
//(at your option) any later version.

//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.

//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package content.exercises;

import content.ExerciseProperties;
import content.interfaces.*;
import matrix.animation.Animator;
import matrix.simulation.VisualTypeConf;
import content.exercises.structures.*;
import matrix.structures.FDT.FDT;
import matrix.structures.FDT.probe.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.JButton;
import java.util.Random;
import matrix.util.*;

public class IteratedMergeSort implements SimulationExerciseModel,
								AWTComponentUtilizer, StyledExercise, ModelAnswerNames,
								ConfigureVisualType, JudgeBlocks, ComparableExercise {

    //ATTRIBUTES


    private Table mergeStudentTable;

    private Table aidStudentTable;

    private long seed = 1;

    private String s, h;

    private Application app;


    //METHODS

    public void setApplication(Application a) {
        app = a;
    }

    public Application getApplication() {
        return app;
    }

    public FDT[] solve() {

        int elements = 2;
        Animator an = Animator.getActiveAnimator();
        Table mergeTable = (Table) getInitialStructures()[0];
        Table aidTable = (Table) getInitialStructures()[1];

        int index = 0;

        while (elements - 1 <= mergeTable.size()) {

            index = 0;
            while (index + elements - 1 <= mergeTable.size()) {

                int left = index, right = index + elements / 2;
                int pos = 0;

                while (left < index + elements / 2 && right < index + elements) {
                    an.startOperation();

                    if (((Key) mergeTable.getObject(left)).leq((Key) mergeTable.getObject(right))) {
                        aidTable.setObject(mergeTable.getObject(left), index + pos);
                        left++;
                    } else {
                        aidTable.setObject(mergeTable.getObject(right), index + pos);
                        right++;
                    }
                    pos++;

                    an.endOperation();

                }

                if (left < index + elements / 2)
                    for (; left < index + elements / 2; left++) {
                        an.startOperation();
                        aidTable.setObject(mergeTable.getObject(left), index + pos);
                        pos++;
                        an.endOperation();
                    }

                else
                    for (; right < index + elements; right++) {
                        an.startOperation();
                        aidTable.setObject(mergeTable.getObject(right), index + pos);
                        pos++;
                        an.endOperation();
                    }

                index = index + elements;

            }
            an.startOperation();
            for (int i = 0; i < mergeTable.size(); i++) {

                mergeTable.setObject(aidTable.getObject(i), i);
                aidTable.setObject(new Key(""), i);
            }

            an.endOperation();

            elements = elements * 2;

        }

        return new FDT[] { mergeTable, aidTable };
    }

    public FDT[] makeModelAnswer() {

        return solve();

    }

    public String[] getStructureVisualisations() {

        return new String[] { "array", "array" };

    }

    public String[] getModelAnswerVisualisations() {

        return new String[] { "array", "array" };

    }

    public String[] getModelAnswerNames() {
        ExerciseProperties lang = ExerciseProperties.getInstance();
        return new String[] { 
                lang.get("ITERATEDMERGESORT_INPUT_MODEL_TITLE" ), 
                lang.get("ITERATEDMERGESORT_AUX_TABLE_MODEL_TITLE" ) 
        };

    }

    public VisualTypeConf[] conf() {

        VisualTypeConf tabl1 = new VisualTypeConf();
        tabl1.enable("matrix.visual.VisualKey", VisualTypeConf.HIGHLIGHT_OPERATION);
        tabl1.enable("matrix.visual.VisualKey", VisualTypeConf.DRAG_OPERATION);
        tabl1.enable("matrix.visual.VisualArray", VisualTypeConf.DROP_OPERATION);
        tabl1.enable("matrix.visual.VisualArrayComponent", VisualTypeConf.HIGHLIGHT_OPERATION);

        VisualTypeConf tabl2 = new VisualTypeConf();
        tabl2.enable("matrix.visual.VisualKey", VisualTypeConf.HIGHLIGHT_OPERATION);
        tabl2.enable("matrix.visual.VisualArray", VisualTypeConf.DROP_OPERATION);
        tabl2.enable("matrix.visual.VisualKey", VisualTypeConf.DROP_OPERATION);
        tabl2.enable("matrix.visual.VisualArrayComponent", VisualTypeConf.DROP_OPERATION);
        tabl2.enable("matrix.visual.VisualArrayComponent", VisualTypeConf.HIGHLIGHT_OPERATION);
        return new VisualTypeConf[] { tabl1, tabl2 };

    }

    public FDT[] init() {

        s = "";

        int d[];
        d = matrix.util.RandomKey.createIntKeys(new Random(seed), 0, 10, 16);

        for (int i = 0; i < d.length; i++) {
            s = s + d[i];
        }

        h = "                ";
        mergeStudentTable = new Table(s);
        aidStudentTable = new TableMergeSort(h);

        return new FDT[] { mergeStudentTable, aidStudentTable };

    }

    public void setSeed(long v) {

        this.seed = v;
    }

    public long getSeed() {

        return seed;
    }

    public String[] getStructureNames() {
        ExerciseProperties lang = ExerciseProperties.getInstance();
        return new String[] { 
                lang.get("ITERATEDMERGESORT_INPUT_TITLE" ), 
                lang.get("ITERATEDMERGESORT_AUX_TABLE_TITLE" ) 
        };

    }

    public FDT[] getAnswer() {

        return new FDT[] { mergeStudentTable, aidStudentTable };
    }
    
    /* ComparableExercise interface */
    
	public int[] getCompareIndices(){
        return new int[] {0, 1};	
	}

	public int[] getGradeIndices(){
        return null; // Used only in recovery mode
	}

	public boolean canRecover(){
		return false;
	}
	
	public String[] getCompareStructureVisualisations(){
		return null; // use default
	}
	
	public String[] getCompareStructureNames(){
		return null; // use default
	}    
	
	public VisualTypeConf[] getCompareVisualTypeConf(){
		return null; // use default
	}

    // Buttons

    public Component[] getAWTComponents() {
        ExerciseProperties lang = ExerciseProperties.getInstance();
        JButton push = new JButton( lang.get("ITERATEDMERGESORT_MOVE_BUTTON_TEXT") );
        push.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                pushButton();
            }
        });

        return new Component[] { push };
    }

    public GridBagConstraints[] getAWTComponentConstraints() {
        return null;
    }

    public FDT[] getInitialStructures() {

        return new FDT[] { new Table(s), new Table(h) };

    }

    public String getDescription() {

        return "";
    }

    public void pushButton() {

        int index = 0;
        Key k;

        Animator.getActiveAnimator().startOperation();
        while (index < aidStudentTable.size()) {

            k = (Key) aidStudentTable.next();

            if (k != null && k.toString().trim().equals("") == false) {

                mergeStudentTable.setObject(aidStudentTable.getObject(index), index);
                aidStudentTable.setObject(new Key(""), index);

            }

            index++;
        }
        Animator.getActiveAnimator().endOperation();
        aidStudentTable.reset();
        mergeStudentTable.reset();
        app.validate();
        app.validateAnimator();

    }
    static final long serialVersionUID = -7037190818058554913L;
}
