/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package content.exercises;

import content.ExerciseProperties;
import content.interfaces.*;
import matrix.animation.Animator;
import matrix.simulation.VisualTypeConf;
import content.exercises.structures.*;
import matrix.structures.FDT.FDT;
import matrix.structures.FDT.Struct;
import matrix.structures.FDT.probe.*;
import matrix.structures.code.probe.PseudoCode;
import matrix.structures.memory.VirtualBoolean;
import matrix.structures.memory.VirtualInteger;
import matrix.structures.CDT.probe.Stack;
import java.awt.*;
import java.awt.event.*;
import javax.swing.JButton;
import java.util.Random;
import matrix.util.*;
import matrix.decoration.*;

public class KnapSackProblem01 extends TracingExercise implements SimulationExerciseModel, AWTComponentUtilizer, 
/*StyledExercise,*/ ModelAnswerNames, ConfigureVisualType, JudgeBlocks,
HasRotatedContainers, LayoutExercise {

    //ATTRIBUTES

    private String w;
    private Key[] keys; 

    private Table weightsStudentTable;    
    private Table valueStudentTable;

    private MatrixImpl studentKnap;

    private PseudoCode vCode = null;

    private Table weightsTable, valueTable;

    private MatrixImpl dpMatrix;

    private int maxWeight = 6;

    private int numberOfItems = 5;

    private VirtualInteger pointer_i;
    private VirtualInteger pointer_j;
    private SelectableKey inFocus = null;
    private SelectableKey valueKey = null;
    private SelectableKey knapKey = null;

    public static final String[] code = new String[] {
        "_1 1 PROCEDURE dynProgKnap01(knap, a, c, n, b) _/1",
        "",
        "_2 2 Init first row of dynamic programming matrix to 0 _/2",
        "_3 3 for each item i in 1..n _/3",
        "_4 4    for each weight y in 0..b _/3 _/4",
        "_5 5        if y > a[i-1] _/5",
        "_6 6            knap[i][y] = max{knap[i-1][y], knap[i-1][y-a[i-1]] + c[i]} _/6",
        "_7 7        else _/7",
        "_8 8            if y == a[i-1] _/8",
        "_9 9                knap[i][y] = max{knap[i-1][y], c[i-1]} _/9",
        "_10 10          else _/10",
        "_11 11              knap[i][y] = knap[i-1][y] _/11",
        "_12 12 END _/12"
    };

    //METHODS


    private void dynProgKnap01 (MatrixImpl knap, Table weights, Table value, int n, int b, boolean model, int imax, int jmax) {

        Animator a  = Animator.getActiveAnimator();
        a.disposeUndo();
        for (int i=1; i<=n; i++) {

            //Note.out(this, "knap="+knap);

            int a_i_1 = Integer.parseInt(weights.getObject(i-1).toString()); // a_i_1 = a[i-1]
            for (int y=0; y<=b; y++) {
                if ((i == imax) && (y == jmax)) return;


                //Note.out(this, "y="+y);
                //Note.out(this, "object = "+knap.getObject(i-1, y).toString());
                int knap_i_1_y = Integer.parseInt(knap.getObject(i-1, y).toString()); // knap_i_1_y = knap[i-1][y]
                int knapiy = 0; // knapiy = knap[i][y]
                //gotoLine(a, 5, model);
                a.startOperation();
                if (model) 
                    vCode.setLocation(5);
                ((SelectableKey)knap.getObject(i, y)).setFocus(true); 
                a.endOperation();
                if (y > a_i_1) {
                    a.startOperation();

                    if (vCode != null)
                        vCode.setLocation(6);
                    int knap_i_1_y_a = Integer.parseInt(knap.getObject(i-1, y-a_i_1).toString()); // knap_i_1_y_a = knap[i-1][y-a[i-1]]
                    //Note.out(this, "i="+i);
                    //Note.out(this, "value="+value);

                    int c_i = Integer.parseInt(value.getObject(i-1).toString()); // c_i = c[i]
                    knapiy = Math.max(knap_i_1_y, knap_i_1_y_a + c_i);

                    a.endOperation();
                } else {
                    gotoLine(a, 8, model);
                    if (y == a_i_1) {
                        a.startOperation();

                        if (vCode != null)
                            vCode.setLocation(9);
                        int c_i_1 = Integer.parseInt(value.getObject(i-1).toString()); // c_i_1 = c[i-1]
                        knapiy = Math.max(knap_i_1_y, c_i_1);
                        a.endOperation();
                    } else {
                        a.startOperation();

                        if (vCode != null)
                            vCode.setLocation(11); 
                        knapiy = knap_i_1_y;
                        a.endOperation();
                    }
                }
                a.startOperation();
                knap.setObject(new SelectableKey(knapiy+"", true), i, y);
                if (vCode != null)
                    vCode.setLocation(3);  
                ((SelectableKey)knap.getObject(i, y)).setFocus(false);
                a.endOperation();
            }
        }
        gotoLine(a, 12, model);
    }

    private void c(Animator a, int line, boolean model) {
        if (model) {
            a.startOperation();

            vCode.setLocation(line);
            a.endOperation();
        }
    }

    private void gotoLine(Animator a, int line, boolean model) {
        c(a, line, model);
    }

    private void solvePartially(MatrixImpl knap, Table w, Table v, int i, int j) {
        dynProgKnap01(knap, w, v, w.size(), maxWeight, false, i, j);
    }

    public FDT[] solve() {
        ExerciseProperties lang = ExerciseProperties.getInstance();

        FDT[] fdt = getInitialStructures();
        weightsTable = (Table) fdt[0];
        valueTable = (Table) fdt[1];
        dpMatrix = (MatrixImpl) fdt[2];

        if (vCode == null) {
            vCode = new PseudoCode();
            vCode.setCodeLines(code);
            vCode.setCodeName( lang.get("KNAPSACK_PSEUDOCODE_NAME") );
        }

        dynProgKnap01(dpMatrix, weightsTable, valueTable, weightsTable.size(), maxWeight, false, numberOfItems, maxWeight);

        FDT[] ans = new FDT[dpMatrix.getRows()];
        for (int i=0; i<dpMatrix.getRows(); i++)
            ans[i] = (FDT) dpMatrix.getObject(i);
        return ans;   
    }

    public FDT[] makeModelAnswer() {
        ExerciseProperties lang = ExerciseProperties.getInstance();

        FDT[] fdt = getInitialStructures();
        weightsTable = (Table) fdt[0];
        valueTable = (Table) fdt[1];
        dpMatrix = (MatrixImpl) fdt[2];

        if (vCode == null) {
            vCode = new PseudoCode();
            vCode.setCodeLines(code);
            vCode.setCodeName( lang.get("KNAPSACK_PSEUDOCODE_NAME") );
        }
        vCode.setLocation(3);

        dynProgKnap01(dpMatrix, weightsTable, valueTable, weightsTable.size(), maxWeight, true, numberOfItems+1, maxWeight);

        return new FDT[] { weightsTable, valueTable, dpMatrix, vCode };
    }


    public String[] getModelAnswerNames() {
        ExerciseProperties lang = ExerciseProperties.getInstance();
        return new String[] { 
                lang.get("KNAPSACK_A_WEIGHTS_MODEL_TITLE"),
                lang.get("KNAPSACK_C_VALUES_MODEL_TITLE"),
                lang.get("KNAPSACK_KNAP_MODEL_TITLE"),
                lang.get("KNAPSACK_PSEUDOCODE_MODEL_TITLE")
        };    }

    public VisualTypeConf[] conf() {

        VisualTypeConf tabl1 = new VisualTypeConf();
        //tabl1.enable("matrix.visual.VisualKey", VisualTypeConf.HIGHLIGHT_OPERATION);
        //tabl1.enable("matrix.visual.VisualKey", VisualTypeConf.DRAG_OPERATION);
        //tabl1.enable("matrix.visual.VisualArray", VisualTypeConf.DROP_OPERATION);
        //tabl1.enable("matrix.visual.VisualArrayComponent", VisualTypeConf.HIGHLIGHT_OPERATION);

        VisualTypeConf tabl2 = new VisualTypeConf();
        tabl2.enable("matrix.visual.VisualKey", VisualTypeConf.HIGHLIGHT_OPERATION);
        tabl2.enable("matrix.visual.VisualArray", VisualTypeConf.DROP_OPERATION);
        tabl2.enable("matrix.visual.VisualKey", VisualTypeConf.DROP_OPERATION);
        tabl2.enable("matrix.visual.VisualArrayComponent", VisualTypeConf.DROP_OPERATION);
        tabl2.enable("matrix.visual.VisualArrayComponent", VisualTypeConf.HIGHLIGHT_OPERATION);

        VisualTypeConf matrix = new VisualTypeConf();
        matrix.enable("matrix.visual.VisualKey", VisualTypeConf.HIGHLIGHT_OPERATION);
        matrix.enable("matrix.visual.VisualArray", VisualTypeConf.DROP_OPERATION);
        matrix.enable("matrix.visual.VisualKey", VisualTypeConf.DROP_OPERATION);
        matrix.enable("matrix.visual.VisualArrayComponent", VisualTypeConf.DROP_OPERATION);
        matrix.enable("matrix.visual.VisualArrayComponent", VisualTypeConf.HIGHLIGHT_OPERATION);

        VisualTypeConf code  = new VisualTypeConf();

        return new VisualTypeConf[] { tabl1, tabl2, matrix, code };

    }

    public FDT[] init() {

        w = "";

        int d[];
        d = matrix.util.RandomKey.createIntKeys(new Random(seed), 1, 5, numberOfItems);
        for (int i = 0; i < d.length; i++) {
            w = w + d[i];
        }

        int dv[];
        keys = new Key[numberOfItems];
        dv = matrix.util.RandomKey.createIntKeys(new Random(), -2, 3, numberOfItems);
        for (int i = 0; i < dv.length; i++) {
            keys[i] = new SelectableKey(""+(Math.abs(d[i] + dv[i]) + 1), false);
        }

        weightsTable = new Table(w);
        valueStudentTable = new Table(keys);
        studentKnap = new MatrixImplKS(numberOfItems+1, maxWeight+1);

        for (int i=0; i <= numberOfItems; i++) {
            for (int j=0; j <= maxWeight; j++) {
                if (i == 0)
                    studentKnap.setObject(new SelectableKey("0", true), i, j);
                else
                    studentKnap.setObject(new SelectableKey("?", true), i, j);
            }
        }

        pointer_i = new VirtualInteger(2, studentKnap, "[i]");
        pointer_j = new VirtualInteger(3, studentKnap, "[y]");

        solvePartially(studentKnap, weightsTable, valueStudentTable, pointer_i.eval(), pointer_j.eval());
        ((SelectableKey)studentKnap.getObject(pointer_i.eval(), pointer_j.eval())).setFocus(true);

        return new FDT[] { weightsTable, valueStudentTable, studentKnap }; 
    }

    public String[] getStructureNames() {
        ExerciseProperties lang = ExerciseProperties.getInstance();

        return new String[] { 
                lang.get("KNAPSACK_A_WEIGHTS_TITLE"),
                lang.get("KNAPSACK_C_VALUES_TITLE"),
                lang.get("KNAPSACK_KNAP_TITLE")
        };
    }

    public FDT[] getAnswer() {
        FDT[] ans = new FDT[studentKnap.getRows()];
        for (int i=0; i<studentKnap.getRows(); i++)
            ans[i] = (FDT) studentKnap.getObject(i);
        return ans;   
    }

    public FDT[] getInitialStructures() {        
        MatrixImpl knap = new MatrixImplKS(numberOfItems+1, maxWeight+1);
        for (int i=0; i <= numberOfItems; i++) {
            for (int j=0; j <= maxWeight; j++) {
                if (i == 0)
                    knap.setObject(new SelectableKey("0", true), i, j);
                else
                    knap.setObject(new SelectableKey("?", true), i, j);
            }
        }
        return new FDT[] { new Table(w), new Table(keys), knap };        
    }

    public String getDescription() { 
        return "";
    }

    // Buttons

    public Component[] getAWTComponents() {
        ExerciseProperties lang = ExerciseProperties.getInstance();

        JButton push = new JButton( lang.get("KNAPSACK_ASSIGN_BUTTON_TEXT") +" " );

        push.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                pushButton();
            }
        });

        return new Component[] { push };
    }

    public void pushButton() {
        Animator.getActiveAnimator().startOperation();
        ((SelectableKey)studentKnap.getObject(pointer_i.eval(), pointer_j.eval())).setFocus(false);
        if (valueKey != knapKey) { // if either of them is != null
            try {
                int knapVal = 0, valVal = 0;
                if (knapKey != null)
                    knapVal = Integer.parseInt(knapKey.toString());
                if (valueKey != null)
                    valVal = Integer.parseInt(valueKey.toString());
                if (valueKey != null) { // 1st or 2nd cases in lines 5 and 7
                    if (knapKey != null) { // case 1.2, max is knap + c

                        studentKnap.setObject(new SelectableKey(""+(knapVal+valVal), true), pointer_i.eval(), pointer_j.eval());

                    } else { // case 2.2, max is c
                        studentKnap.setObject(new SelectableKey(valueKey.toString(), true), pointer_i.eval(), pointer_j.eval());
                    }           
                } else { // 1st case 1.1 or 2nd case 2.1 or 3rd case in line 10
                    studentKnap.setObject(new SelectableKey(knapKey.toString(), true), pointer_i.eval(), pointer_j.eval());
                }

                pointer_j.assign((pointer_j.eval() + 1) % (maxWeight+1));        
                if (pointer_j.eval() == 0)
                    pointer_i.assign((pointer_i.eval() + 1) % (numberOfItems+1));
                ((SelectableKey)studentKnap.getObject(pointer_i.eval(), pointer_j.eval())).setFocus(true);
            } catch(Exception e) {
                Note.show(app, ExerciseProperties.getInstance().get("KNAPSACK_ERROR_MESSAGE") );
                return;
            }
        }
        Animator.getActiveAnimator().endOperation();

        app.validate();
        app.validateAnimator();
        // clear selections
        if (knapKey != null)
            knapKey.select();
        knapKey = null;
        if (valueKey != null)
            valueKey.select();
        valueKey = null;
    }

    private void highlightCurrent(MatrixImpl knap, int i, int j) {
        ((SelectableKey) knap.getObject(i, j)).stealFocus();
    }

    public GridBagConstraints[] getAWTComponentConstraints() {
        return null;
    }

    public GridBagConstraints[] getConstraints () {
        Insets i = new Insets(10, 10, 10, 10);
        GridBagConstraints c;

        c = new GridBagConstraints();
        c.insets    = i;
        c.gridwidth =  GridBagConstraints.REMAINDER;
        return new GridBagConstraints[] { c, c, c };
    }

    public GridBagConstraints[] getModelAnswerConstraints() {
        Insets i = new Insets(10, 10, 10, 10);
        GridBagConstraints[] r = new GridBagConstraints[4];
        GridBagConstraints   c;

        // Array
        c = new GridBagConstraints();
        c.insets = i;
        c.gridx  = 0;
        c.gridy  = 0;
        r[0]     = c;

        // Array
        c = new GridBagConstraints();
        c.insets     = i;
        c.gridx      = 1;
        c.gridy      = 0;
        c.gridheight = GridBagConstraints.REMAINDER;
        c.anchor     = GridBagConstraints.PAGE_START;
        r[1]         = c;

        // Matrix
        c = new GridBagConstraints();
        c.insets     = i;
        c.gridx      = 0;
        c.gridy      = 1;
        r[2]         = c;

        // Pseudocode
        c = new GridBagConstraints();
        c.insets    = i;
        c.gridx     = 1;
        c.gridy     = 1;
//      c.gridwidth = GridBagConstraints.REMAINDER;
        c.anchor    = GridBagConstraints.PAGE_START;
        r[3]        = c;

        return r;
    }
    
	public GridBagConstraints[] getComparisonConstraints() {
		return null; // use default
	}

    public boolean[] getContainerRotations() {
        return new boolean[] { false, false, false };
    }

    public boolean[] getModelAnswerRotations() {
        return new boolean[] { false, false, false, false };
    }
    
    public boolean [] getComparisonRotations() {
    	return null; // use default
    }


    class SelectableKey extends Key implements Visitable, Styled {
        VirtualBoolean selected = new VirtualBoolean(false, this, "[selected]");
        boolean keyInKnapTable = false;

        public SelectableKey(String k, boolean keyInKnapTable) {
            super(k);
            this.keyInKnapTable=keyInKnapTable;
        }

        public void setFocus(boolean val) {
            selected.assign(val);
        }

        public void stealFocus() {
            if (inFocus != null)
                inFocus.setFocus(false);
            inFocus = this;
            setFocus(true);   
        }

        /* Select a key. Only one key at a time can be selected both from the 
         * Matrix and value Table.
         * @overrides matrix.structures.FDT.probe.Key#select()
         */
        public void select() {
            /*
            if (selected) { // We do either this: clear reference...
                if (keyInKnapTable) {
                    if (knapKey == this)
                        knapKey = null;
                } else {
                    if (valueKey == this)
                        valueKey = null;
                }
            }
             */
            selected.toggle(); // toggle selection

            if (selected.eval()) { // ...or this: set reference and clear previous selection
                if (keyInKnapTable) {
                    if (knapKey != null)
                        knapKey.select(); // unselect the previous one in Matrix
                    knapKey = this; // do this only after deselecting the previous one
                } else {
                    if (valueKey != null)
                        valueKey.select(); // unselect the previous one in valueTable
                    valueKey = this; // do this only after deselecting the previous one
                }
            }

            Note.out(this, "selected = "+selected +" for "+this);
            Note.out(this, "knapKey = "+knapKey);
            Note.out(this, "valueKey = "+valueKey);

        }

        public boolean isVisited() {
            return selected.eval();
        }

        public void setVisited(boolean val) {
            selected.assign(val);
        }

        public StyleSheet getStyleSheet() {
            return new StyleSheetAdapter() {
                public Color getVisitedBackgroundColor() {
                    return Color.gray;
                }
                public Color getVisitedFocusedBackgroundColor() {
                    return Color.gray;
                }
                public Color getVisitedPenColor() {
                    return Color.white;
                }
                public Color getFocusedBackgroundColor() {
                    return Color.white;
                }
                public Color getFocusedPenColor() {
                    return Color.blue;
                }
                private static final long serialVersionUID = 1L;
            };
        }

        /**
         * 
         */
        private static final long serialVersionUID = 1L;
    }

    class MatrixImplKS extends MatrixImpl {
        public MatrixImplKS(int r, int c) {
            super(r,c);
            setIndexName("  ", 0);
        }
    }

    static final long serialVersionUID = 2953048071727138376L;    

}
