/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package content.exercises;

import matrix.animation.Animator;
import matrix.simulation.VisualTypeConf;
import matrix.structures.FDT.FDT;
import matrix.structures.FDT.probe.*;
import matrix.structures.code.probe.PseudoCode;
import matrix.structures.memory.VirtualBoolean;
import matrix.structures.memory.VirtualInteger;
import java.awt.*;
import java.util.Random;

import content.ExerciseProperties;

import matrix.util.*;
import matrix.decoration.*;

/**
 * @author archie
 *
 */
public class LongestCommonSubsequenceMatrix extends TracingExercise {

    //ATRIBUTES

    private String firstString, secondString; // parameters

    private MatrixImpl studentMat; // student matrix 

    private PseudoCode vCode = null; 

    private MatrixImpl lcsmMatrix; // model answer matrix

    private int m = 0; // matrix size (rows)
    private int n = 0; // matrix size (cols)

    private SelectableKey inFocus = null; // cell in focus

    private final static String firstStringOptions = "ACGT"; // alphabet for the 1st string
    private final static String secondStringOptions = firstStringOptions; // alphabet for the snd string
    private final static int len1stString = 4; // lenght of the string 1
    private final static int len2ndString = 4; // length of the string 2
    private final static int init_i = len1stString/2; // partially solved exercise up to i rows
    private final static int init_j = len2ndString/2; // partially solved exercise up to j cols

    public static final String[] code = new String[] {
        "_1 1 PROCEDURE LCSM(mat, m, n) _/1",
        "_1 // m is the length of the first String _/1",
        "_1 // n is the length of the second String _/1",
        "_2 2 for (i=0; i less than or equal to m; i++) _/2",
        "_3 3    for (j=0; j less that or equal to n; j++) _/3",
        "_4 4        if (i == 0) // Entries on first row _/4",
        "_5 5            mat[i][j] = 0; _/5",
        "_6 6        else if (j == 0) // Entries on first column _/6",
        "_7 7            mat[i][j] = 0; _/7",
        "_8 8        else if (first[i] == second[j]) // Same char on both Strings _/8",
        "_9 9          mat[i][j] = mat[i-1][j-1]+1 _/9",
        "_10 10      else if (first[i] != second[j]) // Different chars on both Strings _/10",
        "_11 11        mat[i][j] = max{ mat[i-1][j], mat[i][j-1] } _/11",
        "_12 12 END _/12"
    };

    //METHODS

    // helper to access matrix cells
    private SelectableKey getKey(MatrixImpl mat, int i, int j) {
        return (SelectableKey) mat.getObject(i, j);
    }

    /**
     * This is the actual algorithm for this exercice.
     * @param mat - matrix to be filled
     * @param m - number of rows
     * @param n - number of cols
     * @param model - flag to indivate that model solution (code animation) is constructed in this run
     * @param imax - solve only partially to row imax
     * @param jmax - solve only partially to col jmax
     */
    private int lcsm(MatrixImpl mat, int m, int n, boolean model, int imax, int jmax) {
        Animator a  = Animator.getActiveAnimator();
        a.disposeUndo();
        for (int i=0; i<=m; i++) {
            for (int j=0; j<=n; j++) { 
                gotoLine(a, 4, model);
                if (i == 0) {
                    gotoLine(a, 5, model);
                    set(a, mat, i, j, 0);
                }
                else {
                    gotoLine(a, 6, model);
                    if (j == 0) {
                        gotoLine(a, 7, model);
                        set(a, mat, i, j, 0);
                    } else {
                        gotoLine(a, 8, model);
                        if (first(i) == second(j)) {
                            gotoLine(a, 9, model); 
                            int sum = 1+Integer.parseInt(mat.getObject(i-1,j-1).toString());
                            set(a, mat, i, j, sum);
                        } else {
                            gotoLine(a, 10, model);
                            if (first(i) != second(j)) {
                                gotoLine(a, 11, model);
                                int x = Integer.parseInt(mat.getObject(i-1,j).toString());
                                int y = Integer.parseInt(mat.getObject(i,j-1).toString());
                                int max = Math.max(x, y);
                                set(a, mat, i, j, max);
                            }
                        }
                    }
                }
                if ((i == imax) & (j > jmax)) return -1;
            }
        }
        gotoLine(a, 12, model);
        return Integer.parseInt(mat.getObject(m,n).toString());
    }

    // helper to set & highlight cell
    private void set(Animator a, MatrixImpl m, int i, int j, int x) {
        a.startOperation();
        getKey(m, i, j).stealFocus(); 
        getKey(m, i, j).changeKeyValue(x+"");
        a.endOperation();
    }

    // helper to compare strings
    private int first(int i) {
        return firstString.charAt(i-1);
    }

    //  helper to compare strings
    private int second(int i) {
        return secondString.charAt(i-1);
    }

    // helper to produce code animation
    private void c(Animator a, int line, boolean model) {
        if (model) {
            a.startOperation();
            vCode.setLocation(line);
            a.endOperation();
        }
    }

    // helper to produce code animation
    private void gotoLine(Animator a, int line, boolean model) {
        c(a, line, model);
    }

    // helper/facade to call actual algorithm
    private void solvePartially(MatrixImpl mat, int i, int j) {
        lcsm(mat, m, n, false, i, j);
    }

    public FDT[] solve() {
        ExerciseProperties lang = ExerciseProperties.getInstance();

        FDT[] fdt = getInitialStructures();
        lcsmMatrix = (MatrixImpl) fdt[0];

        if (vCode == null) {
            vCode = new PseudoCode();
            vCode.setCodeLines(code);
            vCode.setCodeName(lang.get("LCS_PSEUDOCODE_NAME"));
        }

        lcsm(lcsmMatrix, m, n, false, m, n);

        FDT[] ans = new FDT[m+1];
        for (int i=0; i<=m; i++)
            ans[i] = (FDT) lcsmMatrix.getObject(i);
        return ans;   
    }

    public FDT[] makeModelAnswer() {
        ExerciseProperties lang = ExerciseProperties.getInstance();

        FDT[] fdt = getInitialStructures();
        lcsmMatrix = (MatrixImpl) fdt[0];

        if (vCode == null) {
            vCode = new PseudoCode();
            vCode.setCodeLines(code);
            vCode.setCodeName(lang.get("LCS_PSEUDOCODE_NAME"));
        }
        vCode.setLocation(3);

        lcsm(lcsmMatrix, m, n, true, m, n);

        return new FDT[] { lcsmMatrix, vCode };
    }


    public String[] getModelAnswerNames() {
        ExerciseProperties lang = ExerciseProperties.getInstance();
        return new String[] { lang.get("LCS_MATRIX_MODEL_TITLE"), lang.get("LCS_PSEUDOCODE_MODEL_TITLE") };
    }

    public VisualTypeConf[] conf() {

        VisualTypeConf matrix = new VisualTypeConf();
        matrix.enable("matrix.visual.VisualKey", VisualTypeConf.HIGHLIGHT_OPERATION);
        //matrix.enable("matrix.visual.VisualKey", VisualTypeConf.DROP_OPERATION);
        //matrix.enable("matrix.visual.VisualKey", VisualTypeConf.DRAG_OPERATION);
        //matrix.enable("matrix.visual.VisualArray", VisualTypeConf.DROP_OPERATION);
        //matrix.enable("matrix.visual.VisualArrayComponent", VisualTypeConf.DROP_OPERATION);
        //matrix.enable("matrix.visual.VisualArrayComponent", VisualTypeConf.HIGHLIGHT_OPERATION);

        VisualTypeConf code  = new VisualTypeConf();

        return new VisualTypeConf[] { matrix, code };
    }

    // accept only cases in which certain number appears at most twice
    private boolean accept(int t[]) {
        int counter[] = new int[t.length]; 
        for (int i=0; i<counter.length; i++)
            counter[i] = 0;
        for (int i=0; i<t.length; i++)
            counter[t[i]]++; // calculate #number of appearances into counter table
        for (int i=0; i<counter.length; i++)
            if (counter[i] > 2) // reject if a number appears three or more times
                return false;
        for (int i=0; i<counter.length; i++)
            if (counter[i] == 1) // accept only if some number appears exactly once 
                return true;
        return false;
    }

    public FDT[] init() {
        int match = 0;
        new Random(seed);
        do {
            firstString = secondString = "";
            int f[], s[];
            do {
                f = matrix.util.RandomKey.createIntKeys(new Random(), 0, 3, len1stString);
            } while(!accept(f));
            do {
                s = matrix.util.RandomKey.createIntKeys(new Random(), 0, 3, len2ndString);
            } while (!accept(s));
            for (int i=0; i<len1stString; i++)
                firstString += firstStringOptions.charAt(f[i]);
            for (int i=0; i<len2ndString; i++)
                secondString += secondStringOptions.charAt(s[i]);

            m = firstString.length();
            n = secondString.length();
            studentMat = new MatrixImplLCSM(m+1, n+1);

            for (int i=0; i <= m; i++) {
                for (int j=0; j <= n; j++) {
                    studentMat.setObject(new SelectableKey("0", false), i, j);
                }
            }
            match = lcsm(studentMat, m, n, false, m, n);
        } while (!(match == Math.min(len1stString, len2ndString)/2));

        studentMat = new MatrixImplLCSM(m+1, n+1);

        for (int i=0; i <= m; i++) {
            for (int j=0; j <= n; j++) {
                studentMat.setObject(new SelectableKey("0", false), i, j);
            }
        }

        solvePartially(studentMat, init_i, init_j);

        return new FDT[] { studentMat }; 
    }

    public String[] getStructureNames() {
        ExerciseProperties lang = ExerciseProperties.getInstance();
        return new String[] { lang.get("LCS_MATRIX_TITLE") };
    }

    public FDT[] getAnswer() {
        FDT[] ans = new FDT[m+1];
        for (int i=0; i<=m; i++)
            ans[i] = (FDT) studentMat.getObject(i);
        return ans;   
    }

    public FDT[] getInitialStructures() {        
        MatrixImpl mat = new MatrixImplLCSM(m+1, n+1);
        for (int i=0; i <= m; i++) {
            for (int j=0; j <= n; j++) {
                mat.setObject(new SelectableKey("0", false), i, j);
            }
        }

        return new FDT[] { mat };        
    }

    public String getDescription() { 
        return "";
    }

    class SelectableKey extends Key implements Visitable, Styled {
        VirtualBoolean selected = new VirtualBoolean(false, this, "[selected]");
        VirtualInteger virtualString = new VirtualInteger(this, "0");

        public SelectableKey(String k, boolean highlight) {
            super(k);
        }

        public void setFocus(boolean val) {
            selected.assign(val);
            inFocus = this;
        }

        public void stealFocus() {
            if (inFocus != null)
                inFocus.setFocus(false);
            inFocus = this;
            setFocus(true);
        }    

        public void changeKeyValue(String s) {
            super.changeKeyValue(s);
            setVisualizationString(s);
            //setComparisonString(s);
            //this.setOtherStrings();
        }

        /* Select a key. Only one key at a time can be selected both from the 
         * Matrix and value Table.
         * @overrides matrix.structures.FDT.probe.Key#select()
         */
        public void select() {      
            Animator.getActiveAnimator().startOperation();
            this.changeKeyValue(""+(Integer.parseInt(this.getVisualizationString())+1));
            stealFocus();
            Animator.getActiveAnimator().endOperation();
        }

        public boolean isVisited() {
            return selected.eval();
        }

        public void setVisited(boolean val) {
            selected.assign(val);
        }

        public StyleSheet getStyleSheet() {
            return new StyleSheetAdapter() {
                public Color getVisitedBackgroundColor() {
                    return Color.gray;
                }
                public Color getVisitedFocusedBackgroundColor() {
                    return Color.gray;
                }
                public Color getVisitedPenColor() {
                    return Color.white;
                }
                public Color getFocusedBackgroundColor() {
                    return Color.white;
                }
                public Color getFocusedPenColor() {
                    return Color.blue;
                }
                /**
                 * 
                 */
                private static final long serialVersionUID = -965792110564393586L;
            };
        }

        public String getComparisonString() {
            return virtualString.toString();
        }

        //Visualization String
        public String getVisualizationString() {
            return virtualString.toString();
        }

        public void setVisualizationString(String s) {
            virtualString.assign(Integer.parseInt(s));
        }

        /**
         * 
         */
        private static final long serialVersionUID = -6365310182324298462L;
    }

    class MatrixImplLCSM extends MatrixImpl {
        public MatrixImplLCSM(int r, int c) {
            super(r,c);
            for (int i=0;i<r;i++)
                if (i==0)
                    setIndexName(" ", 0);
                else
                    setIndexName(firstString.charAt(i-1)+"", i);
            for (int i=0;i<c;i++)
                if (i==0)
                    setColumnIndexName(" ", 0);
                else
                    setColumnIndexName(secondString.charAt(i-1)+"", i);
        }

        /**
         * 
         */
        private static final long serialVersionUID = 3121522880738461625L;
    }

    static final long serialVersionUID = 2953048071727138376L;
}
