// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
package content.exercises;

import content.ExerciseProperties;
import content.interfaces.*;
import java.util.Random;
import matrix.animation.Animator;
import matrix.simulation.VisualTypeConf;
import content.exercises.structures.StackImplOper;
import matrix.structures.FDT.FDT;
import matrix.structures.FDT.probe.Key;
import matrix.structures.FDT.probe.Table;
import matrix.util.RandomKey;

public class PostfixExpressions implements SimulationExerciseModel,
        StyledExercise, ModelAnswerNames, ConfigureVisualType, ComparableExercise {
    // 
    // public postfixExpressions()
    // {
    // seed = 1L;
    // }

    public FDT[] solve() {
        Table table = (Table) getInitialStructures()[0];
        StackImplOper stackimploper = (StackImplOper) getInitialStructures()[1];
        int i = 0;
        Animator animator = Animator.getActiveAnimator();
        for (; i < table.size(); i++) {
            animator.startOperation();
            Key key = (Key) table.next();
            if (key.toString().indexOf('+') != -1
                    || key.toString().indexOf('*') != -1) {
                if (stackimploper.hasNext()) {
                    Object obj = stackimploper.ADTPop();
                    Object obj1 = stackimploper.ADTPop();
                    int j = Integer.parseInt(obj.toString());
                    int k = Integer.parseInt(obj1.toString());
                    int l;
                    if (key.toString().indexOf('+') != -1)
                        l = j + k;
                    else
                        l = j * k;
                    stackimploper.insert(new Key(l + ""));
                }
            } else {
                stackimploper.insert(key);
            }
            animator.endOperation();
        }

        return (new FDT[] { stackimploper });
    }

    public FDT[] makeModelAnswer() {
        return solve();
    }

    public String[] getStructureVisualisations() {
        return (new String[] { "list", "list" });
    }

    public String[] getModelAnswerVisualisations() {
        return (new String[] { "list" });
    }

    public String[] getModelAnswerNames() {
        ExerciseProperties lang = ExerciseProperties.getInstance();
        return new String[] { lang.get("POSTFIXEXPRESSIONS_STACK_MODEL_TITLE") };
    }
    
    public VisualTypeConf[] conf() {
    	ExerciseProperties lang = ExerciseProperties.getInstance();
    	boolean stackVertical = lang.get("POSTFIXEXPRESSIONS_STACK_VERTICAL").trim().toLowerCase().equals("true");
    	
        VisualTypeConf visualtypeconf = new VisualTypeConf();
        visualtypeconf.enable("matrix.visual.VisualKey", 4);
        visualtypeconf.enable("matrix.visual.VisualKey", 2);
        VisualTypeConf visualtypeconf1 = new VisualTypeConf();
        visualtypeconf1.enable("matrix.visual.VisualList", 1);
        visualtypeconf1.enable("matrix.visual.VisualList", 4);
        
        if (stackVertical){
        	visualtypeconf1.setVisualConfValue("matrix.visual.VisualList", VisualTypeConf.ROTATED, "true");
        	visualtypeconf1.setVisualConfValue("matrix.visual.VisualList", VisualTypeConf.FLIPPEDY, "true");
        }
        return (new VisualTypeConf[] { visualtypeconf, visualtypeconf1 });
    }

    public FDT[] init() {
    	Random random = new Random(seed);
        s = "";
        int numbers = RandomKey.createIntKey(random, 7, 9);
        int counter = 0; //counts the difference between operands and operators

        for (int i = 0; i < numbers;) {
            if (counter < 2) { //we can put only number to the expression
                s = s + RandomKey.createIntKey(random, 1, 10);
                counter++;
                i++;
            } else {
                int numberOrOperator = RandomKey.createIntKey(random, 0, 2);
                if (numberOrOperator == 0) { //adding operator
                    int operatorType = RandomKey.createIntKey(random, 0, 2);
                    if (operatorType == 0) {//adding plus operator
                        s = s + "+";
                        counter--;
                    } else {//adding multiply operator
                        s = s + "*";
                        counter--;
                    }
                } else { //adding number
                    s = s + RandomKey.createIntKey(random, 1, 10);
                    counter++;
                    i++;
                }
            }
        }

        //Lastly, add enough operators in the end of expression
        for (; counter > 1; counter--) {
            int operatorType = RandomKey.createIntKey(random, 0, 2);
            if (operatorType == 0) { //adding plus operator
                s = s + "+";
            } else { //adding multiply operator
                s = s + "*";
            }
        }

        table1 = new Table(s);
        stack1 = new StackImplOper();
        return (new FDT[] { table1, stack1 });
    }

    public void setSeed(long l) {
        seed = l;
    }

    public long getSeed() {
        return seed;
    }

    public String[] getStructureNames() {
        ExerciseProperties lang = ExerciseProperties.getInstance();
        return new String[] { 
                lang.get("POSTFIXEXPRESSIONS_POSTIX_EXPRESSION_TITLE"),
                lang.get("POSTFIXEXPRESSIONS_OPERAND_STACK_TITLE")};
    }

    public FDT[] getAnswer() {
        return (new FDT[] { stack1 });
    }

    public FDT[] getInitialStructures() {
        return (new FDT[] { new Table(s), new StackImplOper() });
    }

    public String getDescription() {
        ExerciseProperties lang = ExerciseProperties.getInstance();
        return lang.get("POSTFIXEXPRESSIONS_DESCRIPTION_TEXT");
    }
    
    /* ComparableExercise interface */
    
	public int[] getCompareIndices(){
        return new int[] {0};	
	}

	public int[] getGradeIndices(){
        return null; // Used only in recovery mode
	}

	public boolean canRecover(){
		return false;
	}
	
	public String[] getCompareStructureVisualisations(){
		return null; // use default
	}
	
	public String[] getCompareStructureNames(){
		return null; // use default
	}    
	
	public VisualTypeConf[] getCompareVisualTypeConf(){
		return null; // use default
	}

    private Table table1;

    private StackImplOper stack1;

    private String s;

    private long seed;

    private static final long serialVersionUID = 6083073112550496292L;
}
