//Matrix - algorithm simulation system
//Copyright (C) 2000- 2007 Matrix group
//(See the AUTHORS file and the author tag below for more information
//on the authors.)

//This program is free software; you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation; either version 2 of the License, or
//(at your option) any later version.

//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.

//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package content.exercises;

import java.util.Random;

import content.ExerciseProperties;
import content.interfaces.ConfigureVisualType;
import content.interfaces.GradeDialogLayout;
import content.interfaces.JudgeBlocks;
import content.interfaces.ModelAnswerNames;
import content.interfaces.ComparableExercise;
import content.interfaces.StyledExercise;
import content.interfaces.SwapBehaviour;
import content.exercises.structures.ExerVertexVisitableKey;
import content.exercises.structures.PriorityExerVertex;

import matrix.simulation.VisualTypeConf;
import matrix.structures.FDT.*;
import matrix.structures.FDT.probe.Key;
import matrix.structures.FDT.probe.UndirectedGraphImpl;
import matrix.structures.FDT.substructures.Vertex;

public class Prim implements ComparableExercise, ModelAnswerNames, StyledExercise , ConfigureVisualType, 
							JudgeBlocks, SwapBehaviour, GradeDialogLayout {
    private int randomWanted = 6;
    private int randomTries = 100;
    protected PriorityExerVertex ModelVertex[];
    protected UndirectedGraphImpl usergraph;
    protected UndirectedGraphImpl modelgraph;
    protected String[] NodeList;
    protected final String keys = "ABCDEFGHJ";

    public static final boolean DEBUG = false;


    // constructor

    public Prim() {}

    /**
     * In this exercise a drag operation performs a swap
     * @return
     */
    public boolean getSwapBehaviour() {
        return true;
    }

    // styled exercise

    public String[] getStructureVisualisations() {
        String[] s = { "Kamada-Kawai point graph" };
        return s;
    }


    public String[] getModelAnswerVisualisations() {
        String[] s = { "Kamada-Kawai point graph" };
        return s;
    }

    long seed = 1;

    /**
     * Get the value of seed.
     * @return value of seed.
     */
    public long getSeed() {
        return seed;
    }

    /**
     * Set the value of seed.
     * @param v  Value to assign to seed.
     */
    public void setSeed(long  v) {
        this.seed = v;
    }

    private void createSymmetricEdge(PriorityExerVertex u, PriorityExerVertex v, int weight) {
        //Note.out(this, "create edge " + u.toString() + "-" + v.toString());
        u.insertSuccessor(v, weight);
        v.insertSuccessor(u, weight);
    }

    private void createAdjacencyList(String[] list, PriorityExerVertex[] t) {
        for (int i=0; i<t.length; i++)
            NodeList[i] = "" + ((ExerVertexVisitableKey)t[i].getElement()).getLabel() + ": ";
        for (int i=0; i<t.length; i++) {
            PriorityExerVertex u = t[i];
            Vertex v[] = u.getSuccessors();
            for (int j=0; j<v.length; j++) {
                list[i] += ((ExerVertexVisitableKey)v[j].getElement()).getLabel() + "" + t[i].getWeight((PriorityExerVertex)v[j]) + " ";
            }
        }
    }

    public FDT[] init() {
        int l = keys.length(); // keys is the string of alphabet nodes in the graph
        Random R = new Random(seed);
        int i, n;

        // V and ModelVertex are arrays of nodes that forms the graph for user and for the model answer, respectively
        PriorityExerVertex V[] = new PriorityExerVertex[l]; 
        ModelVertex = new PriorityExerVertex[l];
        NodeList = new String[l]; // adjacensy list representation of the graph
        usergraph = new UndirectedGraphImpl(); //  G=(V,E), i.e., usergraph=(V,E);
        modelgraph = new UndirectedGraphImpl();

        // Create vertices V
        for(i=0; i<l; i++) {
            V[i]=new PriorityExerVertex();
            V[i].setElement(new ExerVertexVisitableKey(""+keys.charAt(i), V[i]));
            ModelVertex[i]=new PriorityExerVertex();
            ModelVertex[i].setElement(new ExerVertexVisitableKey(""+keys.charAt(i), ModelVertex[i]));
        }

        // create weighted edges E, weight w = 1,2,3,...
        int w = 3; // w=1 and w=2 are reserved for later use 
        // Path from root node to every node except the last node (therefore, l = l - 1)
        l = l - 1; 
        for(i=1; i<l; i++) {
            n = Math.abs(R.nextInt()%i);
            createSymmetricEdge(V[n], V[i], w); 
            createSymmetricEdge(ModelVertex[n], ModelVertex[i], w); 
            w++;
        }

        // Add random edges
        int randomCreated = 0;
        for(i = 0; i<randomTries; i++) {
            n = Math.abs(R.nextInt()%l);
            if (DEBUG)
                System.out.println("i:"+i%l+" n:"+n);
            if ((!V[i%l].hasSuccessor(V[n])) && (i%l != n)) {
                createSymmetricEdge(V[i%l], V[n], w); 
                createSymmetricEdge(ModelVertex[i%l], ModelVertex[n], w); 
                randomCreated++;
                if (randomCreated >= randomWanted) break; 
                w++;
            } 
        }
        // creating a small edge w = 2
        for(; i<randomTries; i++) {
            n = Math.abs(R.nextInt()%l);
            if (DEBUG)
                System.out.println("i:"+i%l+" n:"+n);
            if ((!V[i%l].hasSuccessor(V[n])) && (i%l != n)) {
                createSymmetricEdge(V[i%l], V[n], 2); 
                createSymmetricEdge(ModelVertex[i%l], ModelVertex[n], 2); 
                randomCreated++;
                if (randomCreated >= randomWanted) break; 
            } 
        }
        // dealing with the last node (at V[l] because l was decreased before)
        // we make sure there occurs at least one update while running Prim's algorithm
        // by providing edges: W(A,last) = max_weight; W(last, second_last) = min_weight
        createSymmetricEdge(V[l], V[0], w); 
        createSymmetricEdge(ModelVertex[l], ModelVertex[0], w); 
        w++;
        createSymmetricEdge(V[l-1], V[l], 1); 
        createSymmetricEdge(ModelVertex[l-1], ModelVertex[l], 1); 

        V[0].markVisited();
        ModelVertex[0].markVisited();

        createAdjacencyList(NodeList, V);
        usergraph.setVertices(V);
        modelgraph.setVertices(ModelVertex);

        FDT[]  s = { usergraph };
        return s;
    }

    public FDT[] makeModelAnswer() {
        return solve();
    }


    public FDT[] init(Object input) {
        return init();
    }


    public String[] getStructureNames() {
        ExerciseProperties lang = ExerciseProperties.getInstance();
        String[] s = { lang.get("PRIM_GRAPH_TITLE") };
        return s;
    }


    public String getDescription() {
        ExerciseProperties lang = ExerciseProperties.getInstance();

        String S= lang.get("PRIM_INSTRUCTIONS")+" " + keys.charAt(0) +".\n\n";
        for (int i=0; i<keys.length(); i++)
            S+=NodeList[i]+"\n";
        return S;
    }

    public FDT[] solve() {
        //Note.out(this, "Solve");

        MST_Prim prim = new MST_Prim(ModelVertex, ModelVertex[0]);
        prim.run();
        return new FDT[] { modelgraph };
    }

    public String[] getModelAnswerNames() {
        ExerciseProperties lang = ExerciseProperties.getInstance();
        String[] s = { lang.get("PRIM_GRAPH_MODEL_TITLE") };
        return s;
    }

    public FDT[] getAnswer() {
        FDT[] s = { usergraph };
        return s;
    }

    public FDT[] getInitialStructures() {
        throw new RuntimeException("not implemented");
    }

    public VisualTypeConf[] conf() {
        VisualTypeConf graph_vtc = new VisualTypeConf();
        graph_vtc.enable("matrix.visual.VisualReference", VisualTypeConf.HIGHLIGHT_OPERATION);
        VisualTypeConf stree_vtc = new VisualTypeConf();
        stree_vtc.enable("matrix.visual.VisualLayeredTreeComponent", VisualTypeConf.HIGHLIGHT_OPERATION);
        stree_vtc.enable("matrix.visual.VisualLayeredTreeComponent", VisualTypeConf.DROP_OPERATION);
        return new VisualTypeConf[] { graph_vtc, stree_vtc };
    }

    public int[] getCompareIndices() {
        return new int[] {0};
    }

    public int[] getGradeIndices() {
        return new int[] {0};
    }

    public boolean canRecover() {
        return true;
    }
    
	public String[] getCompareStructureVisualisations(){
		return null; // use default
	}
	
	public String[] getCompareStructureNames(){
		return null; // use default
	}    
	
	public VisualTypeConf[] getCompareVisualTypeConf(){
		return null; // use default
	}
	
	/* GradeDialog interface */
	
	public boolean showModelAnswerInGradeDialog(){
		return false;
	}
	
	public boolean showComparisonInGradeDialog(){
		return true;
	}

    static final long serialVersionUID = -5017014843164584479L;
}
