//Matrix - algorithm simulation system
//Copyright (C) 2000- 2007 Matrix group
//(See the AUTHORS file and the author tag below for more information
//on the authors.)

//This program is free software; you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation; either version 2 of the License, or
//(at your option) any later version.

//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.

//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/* $Id: RBT_Insert.java,v 1.39 2009/06/18 10:12:50 lhakulin Exp $ */

package content.exercises;

import java.util.Random;

import matrix.animation.Animator;
import matrix.simulation.VisualTypeConf;
import matrix.structures.CDT.probe.RBTree;
import matrix.structures.FDT.FDT;
import matrix.structures.FDT.probe.Table;
import matrix.util.Note;
import content.ExerciseProperties;
import content.exercises.structures.ExerRBTree;
import content.interfaces.ButtonExercise;
import content.interfaces.ButtonGroups;
import content.interfaces.ComparableExercise;
import content.interfaces.ConfigureVisualType;
import content.interfaces.ModelAnswerNames;
import content.interfaces.SimulationExerciseModel;
import content.interfaces.SwapBehaviour;
//import ptn.*;

public class RBT_Insert implements SimulationExerciseModel, ButtonExercise, ButtonGroups, ModelAnswerNames,
									ConfigureVisualType, SwapBehaviour, ComparableExercise {

    Table t;
    ExerRBTree user;
    RBTree model;
    String S = null;
    private boolean doubleRotationDisabled;
    private boolean manualColoring;

    public static final boolean DEBUG = false;

    public RBT_Insert() {
    }

    public RBT_Insert(String initTable) { t = new Table(initTable); }

    /**
     * In this exercise a drag operation performs a swap
     * @return the used swap behaviour, in this exercise, true
     */
    public boolean getSwapBehaviour() {
        return true;
    }

    long seed = 1;

    /**
     * Get the value of seed.
     * @return value of seed.
     */
    public long getSeed() {
        return seed;
    }

    /**
     * Set the value of seed.
     * @param v  Value to assign to seed.
     */
    public void setSeed(long  v) {
        this.seed = v;
    }


    public FDT[] init() {
    	
    	this.initParameters();

        // 9-13 keys
        Random R = new Random(seed);
        int n = Math.abs(R.nextInt()%4);
        // Can contain duplicates
        S = matrix.util.RandomKey.createNoDuplicateUppercaseRandomKey(R,9)+
        matrix.util.RandomKey.createNoDuplicateUppercaseRandomKey(R,n);

        t = new Table(S);
        user = new ExerRBTree(this.manualColoring);
        FDT[]  s = { t, user };
        return s;
    }
    
    private void initParameters(){
        ExerciseProperties lang = ExerciseProperties.getInstance();

        if (lang.get("RBT_INSERT_MANUAL_COLORING").trim().equals("true")){
        	manualColoring = true;
        }
        else {
        	manualColoring = false;
        }
        
        if (lang.get("RBT_INSERT_DOUBLE_ROTATIONS_DISABLED").trim().equals("true")){
        	doubleRotationDisabled = true;
        }
        else {
        	doubleRotationDisabled = false;
        }
        
    }

    public String[] getStructureNames() {
        ExerciseProperties lang = ExerciseProperties.getInstance();
        return new String[] { 
                lang.get("RBT_INSERT_INPUT_TITLE"), 
                lang.get("RBT_INSERT_TREE_TITLE") 
        };
    }

    public String getDescription() {
        return "";
    }

    public String[] buttonNames() {
        ExerciseProperties lang = ExerciseProperties.getInstance();
        
        if (this.doubleRotationDisabled) {
            return new String[] {
                    lang.get("RBT_INSERT_TOGGLE_COLOR_BUTTON_TEXT"),
                    lang.get("RBT_INSERT_ROTATE_LEFT_BUTTON_TEXT"),
                    lang.get("RBT_INSERT_ROTATE_RIGHT_BUTTON_TEXT")
            };
        	
        }
        return new String[] {
                lang.get("RBT_INSERT_TOGGLE_COLOR_BUTTON_TEXT"),
                lang.get("RBT_INSERT_ROTATE_LEFT_BUTTON_TEXT"),
                lang.get("RBT_INSERT_ROTATE_RIGHT_BUTTON_TEXT"),
                lang.get("RBT_INSERT_DOUBLE_ROTATE_LEFT_BUTTON_TEXT"),
                lang.get("RBT_INSERT_DOUBLE_ROTATE_RIGHT_BUTTON_TEXT")
        };
    }

    public String[] buttonCommands() {
    	
        if (this.doubleRotationDisabled) {
            return new String[] {"reflectSelectedVisualType(exerciseReflectionCommand(toggleColor))",
                    "reflectSelectedVisualType(exerciseReflectionCommand(rotateLeft))",
                    "reflectSelectedVisualType(exerciseReflectionCommand(rotateRight))"
            };
        	
        }
        return new String[] {"reflectSelectedVisualType(exerciseReflectionCommand(toggleColor))",
                "reflectSelectedVisualType(exerciseReflectionCommand(rotateLeft))",
                "reflectSelectedVisualType(exerciseReflectionCommand(rotateRight))",
                "reflectSelectedVisualType(exerciseReflectionCommand(doubleRotateLeft))",
                "reflectSelectedVisualType(exerciseReflectionCommand(doubleRotateRight))"
        };
    }


    public int[] buttonGroups() {
        int[] n = new int[2];
        n[0] = 1;
        n[1] = 3;
        return n;
    }


    public FDT[] solve() {
        Note.out(this, "Solve");
        model = new RBTree();
        Table tbl = (Table)getInitialStructures()[0];
        Animator a = Animator.getActiveAnimator();
        for (int i=0; i<tbl.size(); i++) {
            a.startOperation();
            model = (RBTree)model.insert(tbl.getObject(i));
            a.endOperation();
        }
        Note.out(this, "Solved");
        return new FDT[] { model };
    }



    public FDT[] makeModelAnswer() {
        return solve();
    }


    public FDT[] getInitialStructures() {
        return new FDT[] { new Table(S), new ExerRBTree() };
    }

    public FDT[] getSimulatedStructures() {
        return new FDT[] { t, user };
    }

    public FDT[] getAnswer() {
        FDT[] s = { user };
        return s;
    }

    public String[] getModelAnswerNames() {
        ExerciseProperties lang = ExerciseProperties.getInstance();
        return new String[] { lang.get("RBT_INSERT_TREE_MODEL_TITLE") };
    }

    //public SimulationExerciseModel getModelAnswer() {
    //return null;
    //}


    public VisualTypeConf[] conf() {
        VisualTypeConf tabl = new VisualTypeConf();
        tabl.enable("matrix.visual.VisualKey", VisualTypeConf.HIGHLIGHT_OPERATION);
        tabl.enable("matrix.visual.VisualKey", VisualTypeConf.DRAG_OPERATION);
        VisualTypeConf tre = new VisualTypeConf();
        tre.enable("matrix.visual.VisualKey", VisualTypeConf.DROP_OPERATION);
        //tre.enable("matrix.visual.VisualKey", VisualTypeConf.HIGHLIGHT_OPERATION);
        tre.enable("matrix.visual.VisualLayeredTreeComponent", VisualTypeConf.DROP_OPERATION);
        tre.enable("matrix.visual.VisualLayeredTreeComponent", VisualTypeConf.HIGHLIGHT_OPERATION);

        return new VisualTypeConf[] { tabl, tre};
    }
    
    /* ComparableExercise interface */
    
	public int[] getCompareIndices(){
        return new int[] {0};	
	}

	public int[] getGradeIndices(){
        return null; // Used only in recovery mode
	}

	public boolean canRecover(){
		return false;
	}
	
	public String[] getCompareStructureVisualisations(){
		return null; // use default
	}
	
	public String[] getCompareStructureNames(){
		return null; // use default
	}    
	
	public VisualTypeConf[] getCompareVisualTypeConf(){
		return null; // use default
	}


    static final long serialVersionUID = -7576022670921264541L;
}
