//Matrix - algorithm simulation system
//Copyright (C) 2000- 2007 Matrix group
//(See the AUTHORS file and the author tag below for more information
//on the authors.)

//This program is free software; you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation; either version 2 of the License, or
//(at your option) any later version.

//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.

//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package content.exercises;

import java.awt.*;
import java.awt.event.*;
import java.util.Random;
import javax.swing.JButton;

import content.ExerciseProperties;
import content.exercises.RecursiveQuicksort2.QuickSortStackFrame;
import content.exercises.structures.ColorableKey;
import content.interfaces.AWTComponentUtilizer;
import content.interfaces.ConfigureVisualType;
import content.interfaces.HasRotatedContainers;
import content.interfaces.JudgeBlocks;
import content.interfaces.ModelAnswerNames;
import content.interfaces.SimulationExerciseModel;
import content.interfaces.StyledExercise;
import content.interfaces.SwapBehaviour;

import matrix.animation.Animator;
import matrix.simulation.VisualTypeConf;
import matrix.structures.CDT.probe.Stack;
import matrix.structures.FDT.*;
import matrix.structures.FDT.advanced.StyledArray;
import matrix.structures.FDT.probe.Key;
import matrix.structures.FDT.probe.Table;
import matrix.structures.memory.*;
import matrix.structures.simulationextensions.Selectable;
import matrix.util.*;
import matrix.visual.VisualType;


public class RadixExchangeSort2 implements SimulationExerciseModel,
    StyledExercise, ModelAnswerNames,
    ConfigureVisualType, AWTComponentUtilizer,
    JudgeBlocks, SwapBehaviour/*, LayoutExercise*/ {

    private Application app;

    public static final int BIT_INDEX = 0;
    public static final int LEFT_INDEX = 1;
    public static final int RIGHT_INDEX = 2;


    public void setApplication(Application a) {
        app=a;
    }

    public Application getApplication() {
        return app;
    }

    /* how many odd numbers are required in the array to be sorted */
    public static final int ODD_NUMBER_COUNT = 5;

    public static final int ARRAY_SIZE = 10;

    /* indices of the left and right border in the parameter table */
    public static final int LEFT_BORDER = 0;
    public static final int RIGHT_BORDER = 1;


    /* length of keys */
    public static final int KEY_BITS = 3;


    private Table userTable;
    private SelectionTable userBitSelect;
    private Stack userStack;

    protected String[] keyArray = null;
    protected String[] labels   = null;
    
    public static final boolean DEBUG = false;

    /**
     * In this exercise a drag operation performs a swap
     * @return
     */
    public boolean getSwapBehaviour() {
        return true;
    }

    long seed = 1;

    /**
     * Get the value of seed.
     * @return value of seed.
     */
    public long getSeed() {
        return seed;
    }

    /**
     * Set the value of seed.
     * @param v  Value to assign to seed.
     */
    public void setSeed(long  v) {
        this.seed = v;
    }

    public FDT[] init() {
        java.util.Random r =  new java.util.Random(seed);
        BitTable bitTable = null;
        do {
            bitTable = createBitTable(r, 0, 25, KEY_BITS, ARRAY_SIZE);
        } while (bitTable.oddCount < ODD_NUMBER_COUNT);
        
        keyArray = bitTable.table;
        labels  = bitTable.labels;
        
        FDT[] rvs =  getInitialStructures();

        userTable     = (Table)rvs[0];
        userBitSelect = (SelectionTable)rvs[1];
        userStack     = (Stack)rvs[2];

        return rvs;
    }

    public FDT[] getInitialStructures(){

        Table newTable = makeKeyTable(keyArray, labels);

        SelectableStruct[] bitKeys = new SelectableStruct[KEY_BITS];
        for(int i=0; i<bitKeys.length; i++)
            bitKeys[i]=new SelectableStruct(bitKeys.length -1 - i);

        SelectionTable newBitSelect = new SelectionTable(bitKeys);
        newTable.selectRange(0, ARRAY_SIZE - 1);

        Stack newStack = new Stack();
        pushParameters(newStack, new int[]{KEY_BITS-1,0,ARRAY_SIZE-1});

        FDT[]  s = { newTable, newBitSelect, newStack };
        return s;
    }

    public String[] getStructureNames() {
        ExerciseProperties lang = ExerciseProperties.getInstance();
        return new String[] { 
                lang.get("RADIXSORT_TABLE_TITLE"),
                lang.get("RADIXSORT_BIT_SELECTION_TITLE"),
                lang.get("RADIXSORT_CALL_STACK_TITLE") 
        };
    }

    public String[] getStructureVisualisations() {
        String[] s = { "array", "array", "array" };
        return s;
    }

    public String[] getModelAnswerVisualisations() {
        return getStructureVisualisations();
    }

    public String[] getModelAnswerNames() {
        ExerciseProperties lang = ExerciseProperties.getInstance();
        return new String[] { 
                lang.get("RADIXSORT_TABLE_MODEL_TITLE"), 
                lang.get("RADIXSORT_BIT_SELECTION_MODEL_TITLE"), 
                lang.get("RADIXSORT_CALL_STACK_MODEL_TITLE") 
        };
    }

    public SimulationExerciseModel getModelAnswer() {
        return this;
    }

    public VisualTypeConf[] conf() {
        VisualTypeConf tabl = new VisualTypeConf();


        tabl.enable("matrix.visual.VisualStruct", VisualTypeConf.HIGHLIGHT_OPERATION);
        tabl.enable("matrix.visual.VisualStruct", VisualTypeConf.DRAG_OPERATION);
        tabl.enable("matrix.visual.VisualStruct", VisualTypeConf.DROP_OPERATION);
        tabl.enable("matrix.visual.VisualArrayComponent", VisualTypeConf.HIGHLIGHT_OPERATION);
        tabl.setVisualConfValue("matrix.visual.VisualArray", VisualTypeConf.RANGE_SELECTION, "true");

        VisualTypeConf bitSelect = new VisualTypeConf();
        bitSelect.enable("matrix.visual.VisualStruct", VisualTypeConf.HIGHLIGHT_OPERATION);

        VisualTypeConf stack = new VisualTypeConf();
        return new VisualTypeConf[] { tabl, bitSelect, stack };
    }


    public FDT[] getAnswer() {
        FDT[] s = { userTable/*,  userBitSelect*/, userStack };
        return s;
    }


    public FDT[] init(Object input) {
        return init();
    }


    public FDT[] solve() {
        return solveHighlight(false);
    }

    public FDT[] makeModelAnswer() {
        return solveHighlight(true);
    }

    public FDT[] solveHighlight(boolean hlight){
        FDT[] structures = getInitialStructures();
        Table mTable   = (Table)structures[0];
        Table mBorders = (Table)structures[1];
        Stack mStack   = (Stack)structures[2];

        Animator a = Animator.getActiveAnimator();
        radixExchangeSort(a, mTable, (SelectionTable)mBorders, mStack,  hlight);

        a.startOperation();
        popParameters(mStack);
        a.endOperation();

        if (hlight)
            return new FDT[] {mTable, mBorders, mStack};
        else
            return new FDT[] {mTable, mStack};
            
    }

    //========================================================

    // --------UsesSplit begin
    public int[] getPanelInfo(){
        return new int[]{0,0,1};
    }

    public int getNumberOfPanels(){
        return 2;
    }
    // --------UsesSplit end


    // --------RotatedContainers begin
    public boolean[] getContainerRotations(){
        return new boolean[]{false, false, true};
    }
    public boolean[] getModelAnswerRotations() {
        return getContainerRotations();
    }
    // --------RotatedContainers end



    public String getDescription() {
        return "";

        /*
	    "RadixExchangeSort( Array, Stack){\n\n"+
	    "  peek values from the top of the recursion stack (bit, leftIndex, rightIndex)\n\n"+
	    "  if (rightIndex > leftIndex AND bit >= 0){\n"+
	    "    split = ExchangeSort(Array, bit, leftIndex, rightIndex)\n\n"+
            "    if Array[split] == 0\n"+
            "       split = split + 1\n\n"+
	    "    PUSH to stack -> (bit, leftIndex, split-1)\n"+
	    "    PUSH to stack -> (bit, split, rightIndex)\n\n"+
            "    RadixExchangeSort(Array, Stack)\n"+
	    "    POP the top item off the stack\n\n"+
            "    RadixExchangeSort(Array, Stack)\n"+
	    "    POP the top item off the stack\n"+
	    "  }\n"+
	    "}\n\n"+
	    "ExchangeSort(Array, bit, leftIndex, rightIndex){\n\n"+
	    "  i = leftIndex\n"+
	    "  j = rightIndex\n"+
	    "  while (j != i){\n"+
	    "    while ( Array[i].bit == 0 AND i < j)\n"+
	    "      i = i+1\n\n"+
	    "    while ( Array[j].bit == 1 AND i < j)\n"+
	    "      j = j-1\n\n"+
	    "    SWAP array elements i and j\n"+
	    "  }\n"+
	    "}";
         */

    }

    //========================================================


    /**
       Sorts the given table using recursive radix exchange sort.

       @param a animator for animating the algorithm
       @param table the table to be sorted
       @param size the size of the table
       @param stack the recursion stack
       @param highlightOn a flag indicating whether to highlight keys in the table when
       the algorithm proceeds. Highlighting is in generating the model solution.
     */
    static void radixExchangeSort(Animator a, Table table, SelectionTable bt, Stack stack, boolean highlightOn) {

        //PEEK the top of the stack (get parameters)

        Object top = stack.getTopOfStack();
        if (top instanceof Key)
            top = ((Key)top).getObject();

        RadixStackFrame paramsTable = (RadixStackFrame)top;	
        int[] params = getParameters(paramsTable);

        int left  = params[LEFT_INDEX];
        int right = params[RIGHT_INDEX];
        int bit   = params[BIT_INDEX];

        try{

            int leftCursor, rightCursor;

            if (right > left && bit >= 0){
                leftCursor  = left;
                rightCursor = right;

                while (leftCursor != rightCursor){
                    while ( bit(getParameter(table,leftCursor), bit) == 0 &&
                            leftCursor < rightCursor )
                        leftCursor++;

                    while ( bit(getParameter(table,rightCursor), bit) == 1 &&
                            leftCursor < rightCursor )
                        rightCursor--;

                    swap(a, table, leftCursor, rightCursor, highlightOn);

                }

                if ( bit(getParameter(table,rightCursor), bit) == 0 )
                    rightCursor++;

                if (bit > 0){		 


                    int[] leftPartition  = new int[3];
                    int[] rightPartition = new int[3];

                    leftPartition[LEFT_INDEX]  = left;
                    leftPartition[RIGHT_INDEX] = rightCursor-1;
                    leftPartition[BIT_INDEX]   = bit-1;

                    rightPartition[LEFT_INDEX]  = rightCursor;
                    rightPartition[RIGHT_INDEX] = right;
                    rightPartition[BIT_INDEX]   = bit-1;

                    if (rightCursor-1 > left){
                        if(highlightOn){
                            a.startOperation();	
                            table.selectRange(left, rightCursor -1);
                            bt.setSelectedBit(bit - 1);
                            a.endOperation();	
                        }

                        pushParametersAnimated(stack, leftPartition, a);
                        radixExchangeSort(a, table, bt, stack, highlightOn);
                        popParametersAnimated(stack, table, bt, a);
                    }

                    if (rightCursor < right){
                        if(highlightOn){
                            a.startOperation();	
                            table.selectRange(rightCursor, right);
                            bt.setSelectedBit(bit - 1);
                            a.endOperation();	
                        }

                        pushParametersAnimated(stack, rightPartition, a);
                        radixExchangeSort(a, table, bt, stack, highlightOn);
                        popParametersAnimated(stack, table, bt, a);		    		 
                    }

                }
            }
            else{

            }
        }catch(NumberFormatException e){
            Note.show(null, "Illegal value in table.");	    
        }

    }	


    static void swap(Animator a, Table table, int hi, int lo, boolean highlightOn){
        if (hi == lo)
            return;

        a.startOperation();
        table.swap(hi,lo);

        a.endOperation();	
    }





    static void highlightTable(Table t, int lo, int hi, Color color) {
        for (int i = lo; i <= hi; i++) {
            Object o = t.getObject(i);
            if (o instanceof ColorableKey) {
                highlightKey(((ColorableKey)o), color);
            }
        }
    }


    static void highlightKey(Object k, Color c) {
        if (k instanceof ColorableKey)
            ((ColorableKey)k).getStyleSheet().setDefaultBackgroundColor(c);
    }



    static void clearHighlight(Table t) {
        for (int i = 0; i < t.size(); i++) {
            Object o = t.getObject(i);
            if (o instanceof ColorableKey) {
                ((ColorableKey)o).setDefaultBackgroundColor(Color.white);
            }
        }
    }

    static void pushParametersAnimated(Stack stack, int[] params, Animator a) {
        a.startOperation();
        pushParameters(stack, params);
        a.endOperation();
    }

    static void pushParameters(Stack stack, int[] params) {
        RadixStackFrame stackFrame = new RadixStackFrame(params[1], params[2], params[0]);
        stack.insert(stackFrame);
    }


    static int[] popParametersAnimated(Stack stack, Table table, SelectionTable bt, Animator a) {
        a.startOperation();
        int [] arr=popParameters(stack);

        int [] newTop = peekParameters(stack);
        table.selectRange(newTop[1], newTop[2]);	
        bt.setSelectedBit(newTop[0]);   

        a.endOperation();
        return arr;
    }

    static int[] popParameters(Stack stack) {
        int[] params = peekParameters(stack);
        stack.pop(null);
        return params;
    }

    static int[] peekParameters(Stack stack) {
        RadixStackFrame params;

        Object top = stack.getTopOfStack();
        if (top instanceof Key)
            params = (RadixStackFrame)((Key)top).getObject();
        else
            params = (RadixStackFrame)top;

        if (params==null)
            return null;	

        return getParameters(params);
    }



    //---conversion methods for int tables----------------
    static void setParameters(Table t, int[] params) {
        for (int i = 0; i < params.length; i++) {
            t.setObject(params[i]+"", i);
        }
    }

    static int[] getParameters(RadixStackFrame params) {

        int[] p = new int[3];
        p[0] = params.getBit();
        p[1] = params.getLeft();
        p[2] = params.getRight();

        return p;
    }

    static int getParameter(Table params, int index){

        BitStruct b = (BitStruct)(params.getObject(index));
        
        try{
            String s = ((String)b.getField(0));
            return Integer.valueOf(s).intValue();
        } catch (NumberFormatException e) {		
            return -1;
        } catch (NullPointerException e) {
            return -1;
        }

    }
    //---------------------------------------------------------


    static int bit(int x, int which) {
        return ((x >> which) & 1);
    }



    /**
       Creates a bit array.

       @param r random number generator
       @param bits number of bits used
       @param size number of characters of the key to create
     */
    public BitTable createBitTable(Random r, int min, int max, int bits,
            int size) {
        
        String[] labels = new String[size];
        String[] data = new String[size];
        
        int i = 0;
        int oddCount = 0;
        final int BITMASK = (1<<bits) - 1;
        
        while (i < size) {
            // Ascii codes of letters A-Z are from 1 to 26.
            int rand = ((int)(r.nextDouble() * 32))+1;
            data[i] = binaryRepresentation(rand, bits);
            labels[i] = (rand & BITMASK) + "";
            if (rand % 2 == 1)
                oddCount++;
            i++;
        }
        return new BitTable(labels, data, oddCount);
    }



    public static String binaryRepresentation(int n, int bits) {
        StringBuffer buf = new StringBuffer();
        for (int i = bits-1; i >= 0; i--) {
            buf.append(((n >> i) % 2) == 0 ? "0":"1");
        }
        return buf.toString();
    }



    // Buttons

    public Component[] getAWTComponents() {
        ExerciseProperties lang = ExerciseProperties.getInstance();

        JButton push = new JButton( lang.get("RADIXSORT_CALL_BUTTON_TEXT") );
        push.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                pushButton();
            }
        });

        JButton pop = new JButton( lang.get("RADIXSORT_RETURN_BUTTON_TEXT") );
        pop.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                popButton();
            }
        });

        return new Component[]{push, pop};
    }

    public GridBagConstraints[] getAWTComponentConstraints() {
        return null;
    }


    boolean containsStructure(VisualType vt, FDT s) {
        if (vt == null)
            return false;
        else if (vt.getStructure() == null)
            return false;
        else if (vt.getStructure() == s) 
            return true;
        else if (!(vt.getParent() instanceof VisualType))
            return false;
        else
            return containsStructure((VisualType)vt.getParent(), s);
    }




    void pushButton() {
        int[] params =new int[3]; 

        params[0] = userBitSelect.getSelectedBit();
        params[1] = userTable.selectionStart();
        params[2] = userTable.selectionEnd();

        /*getParameters(userParameters);
	for (int i = 0; i < params.length; i++) {
	    if (params[i] < 0) {
		Note.show(null, "Select parameters.");
		return;
	    }
	    }*/
        getApplication().getAnimator().startOperation();
        pushParameters(userStack, params);
        getApplication().getAnimator().endOperation();
        //clearTable(userParameters);

        validateApplication();
    }

    void popButton() {

        //in this exercise one shouldn't pop the final element off the stack
        if (userStack.getStackSize()==0)
            return;

        getApplication().getAnimator().startOperation();
        int[] parameters = popParameters(userStack);

        //>ADDED
        if (userStack.getStackSize() == 0){
        }
        else{
            Object top = userStack.getTopOfStack();
            if (top instanceof Key)
                top = ((Key)top).getObject();

            RadixStackFrame paramsTable = (RadixStackFrame)top;	
            parameters = getParameters(paramsTable);
            //<ADDED
        }

        Animator.getActiveAnimator().endOperation();
        //setParameters(userParameters, parameters);

        if (userStack.getStackSize() != 0){
            userTable.selectRange(parameters[1], parameters[2]);
            userBitSelect.setSelectedBit(parameters[0]);
        }

        validateApplication();
    }


    void clearTable(Table t) {
        for (int i = 0; i < t.size(); i++) {
            t.setObject(" ", i);
        }
    }


    void validateApplication() {
        if (app != null) {
            app.validate();
            app.validateAnimator();
        }
    }



    private Table makeKeyTable(String[] keys, String[] labels) {
        Table keyTable = new Table(keys.length);
        BitStruct[] vKeys = new BitStruct[keys.length];
        
        for (int i = 0; i < keys.length; i++) {
            keyTable.setObject(new BitStruct(labels[i], keys[i]), i);
        } 
        return keyTable;
    }

    static class BitStruct implements Struct {

        String[] labels;
        String bits;
        
        public BitStruct(String label, String bits) {
            this.labels = new String[]{label, ""};
            this.bits = bits;
        }
        
        
        
        public Object getField(int location) {
            if (location == 0)
                return labels[0];
            return bits;
        }

        public String getFieldName(int location) {
            return labels[location];
        }

        public String[] getFieldNames() {
            return labels;
        }

        public Object[] getFields() {
            return new String[]{null, bits};
        }

        public void setField(Object value, int location) {
        }

        public Object getElement() {
            return null;
        }

        public void setElement(Object o) {
        }

        public boolean equals(Object o) {
            if (o instanceof BitStruct) {
                BitStruct b = (BitStruct)o;
                return b.bits.equals(bits);
            }
            return false;
        }
        
        public String toString(){
            return "[key: "+labels[0]+" bits: "+bits+"]";
        }

    }


    static class BitTable {
        String[] table;
        String[] labels;
        int oddCount;
        BitTable(String[] labels_, String[] table_, int oddCount_) {
            labels = labels_;
            table = table_;
            oddCount = oddCount_;
        }
    }


    static class SelectionTable extends Table implements StyledArray {
        SelectableStruct current;

        SelectionTable(SelectableStruct[] args) {
            super(KEY_BITS);

            for (int i=0; i<args.length; i++){
                setObject(args[i], i);
                args[i].setParent(this);
            }

            current = args[0];
            current.set();
        }

        public String getIndexName(int address) {
            return "";      
        }

        public void setSelected(SelectableStruct caller){
            current.clear();
            caller.set();
            current = caller;
        }

        public int getSelectedBit(){
            return current.getBitNumber();
        }

        public void setSelectedBit(int index){
            Note.out(this, "set selected " + index);
            Note.out(this, "that is " + getObject(index));
            setSelected((SelectableStruct)getObject(KEY_BITS-1-index));
        }
        static final long serialVersionUID = 5178438448608582653L;
    }

    
    static class SelectableStruct implements Selectable, Struct {

        String[] labels = {"", "Bit Number", "Selected"};

        VirtualBoolean isSelected = new VirtualBoolean(false, this, "Has this key been selected");
        SelectionTable parent;
        
        int bitNumber;
        
        public SelectableStruct(int bitNumber){
            this.bitNumber = bitNumber;
        }
        
        public void setParent(SelectionTable parent){
            this.parent = parent;
        }
        
        public long getSelectionTime() {
            return isSelected.eval() ? 1 : Selectable.NOT_SELECTED;
        }

        public void setSelect(long globalClockTimestamp) {
            Animator.getActiveAnimator().startOperation();
            parent.setSelected(this);
            Animator.getActiveAnimator().endOperation();
        }

        public String getFieldName(int location) {
            return labels[location];
        }
        
        public String[] getFieldNames() {
            return labels;
        }

        public Object getField(int location) {
            switch (location) {
            case 0 : return "Click to select";
            case 1 : return ""+bitNumber;
            default : return ""+isSelected.eval();
            }
        }

        public int getBitNumber(){
            return bitNumber;
        }
        
        public Object[] getFields() {
            return new String[]{"Click to select", ""+bitNumber, ""+isSelected.eval()};
        }

        public void setField(Object value, int location) {
            if (location == 2){
                if(((Boolean)value).booleanValue())
                    set();
                else
                    clear();                
            }
        }

        public void set(){
            isSelected.assign(true);
        }

        public void clear(){
            isSelected.assign(false);
        }
        
        public Object getElement() {
            return null;
        }

        public void setElement(Object o) {
        }

        public boolean equals(Object other) {           
            return bitNumber == ((SelectableStruct)other).bitNumber 
                && isSelected.equals(((SelectableStruct)other).isSelected);
        }
        
        public String toString(){
            return "[bit no.: "+bitNumber+" selected: "+isSelected.eval()+"]";
        }

    }
    
    static class RadixStackFrame implements Struct {

        private static final long serialVersionUID = 1250154066878117737L;

        private Integer left, right, bit;
        private String[] fieldNames = {"Stack frame", "Left", "Right", "Bit"};
        
        public RadixStackFrame(int left, int right, int bit) {
            this.left  = new Integer(left);
            this.right = new Integer(right);
            this.bit = new Integer(bit);
        }
        
        public Object getField(int location) {
            switch(location) {
              case 0 : return "";
              case 1 : return left;
              case 2 : return right;
              default: return bit;
            }
        }

        public int getLeft(){
            return left.intValue();
        }

        public int getRight(){
            return right.intValue();
        }

        public int getBit(){
            return bit.intValue();
        }

        
        public String getFieldName(int location) {
            return fieldNames[location];
        }

        public String[] getFieldNames() {
            return fieldNames;
        }

        public Object[] getFields() {
            return new Object[]{null, left, right, bit};
        }

        public void setField(Object value, int location) {
            switch (location) {
            case 0 : return;
            case 1 : 
                left = (Integer)value;
                return;
            case 2 : 
                right = (Integer)value;
                return;
            default :
                bit = (Integer)value;
                return;
            }
        }

        // Not used but required by FDT
        public Object getElement() {
            return null;
        }

        public void setElement(Object o) {
        }
        
        public boolean equals(Object o){
            if (o instanceof RadixStackFrame){
                RadixStackFrame otherFrame = (RadixStackFrame) o;
                return  left.equals(otherFrame.left) 
                    && right.equals(otherFrame.right)
                    && bit.equals(otherFrame.bit);

            }
            return false;
        }
        
        public String toString(){
            return "[left: "+left+" right: "+right+" bit: "+bit+"]";
        }
    }

    
    static final long serialVersionUID = 4482070511108742137L;
}
