// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package content.exercises;
import content.ExerciseProperties;
import content.interfaces.*;
import matrix.animation.Animator;
import matrix.decoration.StyleSheet;
import matrix.decoration.StyleSheetAdapter;
import matrix.decoration.Styled;
import matrix.simulation.VisualTypeConf;
import matrix.structures.FDT.FDT;
import matrix.structures.FDT.probe.*;
import matrix.structures.memory.VirtualBoolean;
import matrix.structures.memory.VirtualInteger;
import matrix.structures.simulationextensions.Selectable;
import matrix.structures.spatial.Area;
import matrix.structures.spatial.PaintingStyleDecorator;
import matrix.structures.spatial.SpatialComparable;
import matrix.structures.spatial.FDT.probe.CubicBezier;
import matrix.structures.spatial.FDT.probe.LineSegment;
import matrix.structures.spatial.FDT.probe.Point;
import matrix.structures.spatial.FDT.probe.SpatialElement;
import matrix.util.Application;
import matrix.util.Note;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Random;
import java.util.Vector;
import javax.swing.JButton;

/**
 * Selectionsort algorithm exercise.
 * The algorithm is simulated by selecting the smallest key of the
 * unsorted part of the table. Key can be selected either by clicking
 * the key or the corresponding bar.
 * 
 * @author lhakulin
 *
 */

public class SelectionSort implements SimulationExerciseModel, StyledExercise, ModelAnswerNames, 
									ConfigureVisualType, JudgeBlocks, LayoutExercise,
									AWTComponentUtilizer, ComparableExercise {

    private BarTable table;
    private Table minimumTable;
    private long seed = 1;
    private String[] s;
    private transient Application app;

    private static final PaintingStyleDecorator areaDecorator;
    private static final PaintingStyleDecorator barDecorator;
    private static final PaintingStyleDecorator barDecoratorSelected;
    private static final PaintingStyleDecorator barDecoratorOrdered;
    private static final PaintingStyleDecorator minimumLineDecorator;
    private static final PaintingStyleDecorator swapLineDecorator;
    
	private static final int ELEMENT_GAP = 5;

    static {
		areaDecorator = new PaintingStyleDecorator();
		areaDecorator.setOutline(false);
		areaDecorator.setFill(false);
        areaDecorator.setStroke(2, BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL);
		
		barDecorator = new PaintingStyleDecorator();
		barDecorator.setLineColor(Color.black);
        barDecorator.setStroke(2, BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL);
        barDecorator.setFocusFillColor(barDecorator.getFillColor());

        barDecoratorSelected = new PaintingStyleDecorator();
		barDecoratorSelected.setLineColor(Color.black);
		barDecoratorSelected.setFillColor(Color.red);
		barDecoratorSelected.setTransparency(0.5);
        barDecoratorSelected.setStroke(2, BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL);
        barDecoratorSelected.setFocusFillColor(barDecoratorSelected.getFillColor());

        barDecoratorOrdered = new PaintingStyleDecorator();
        barDecoratorOrdered.setLineColor(Color.black);
        barDecoratorOrdered.setFillColor(new Color(0,80,0));
        barDecoratorOrdered.setTransparency(0.6);
        barDecoratorOrdered.setStroke(2, BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL);
        barDecoratorOrdered.setFocusFillColor(barDecoratorOrdered.getFillColor());
        
        float[] pattern = { 3 , 5 };
        minimumLineDecorator = new PaintingStyleDecorator();
        minimumLineDecorator.setLineColor(Color.red);
        minimumLineDecorator.setStroke(2, BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL, 3, pattern, 0);

        swapLineDecorator = new PaintingStyleDecorator();
        swapLineDecorator.setLineColor(new Color(180, 0, 0));
        swapLineDecorator.setFill(false);
        swapLineDecorator.setStroke(2, BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL);
    }


    public FDT[] solve(){

        FDT[] structures = getInitialStructures();
        BarTable table = (BarTable) structures[0];
        Table minimum = (Table) structures[1];

        for (int i = 0; i < table.size()-1; i++) { 
        	int min = Integer.MAX_VALUE;
        	for (int j=i; j < table.size(); j++){
        		int current = Integer.parseInt(((Key)table.getObject(j)).getVisualizationString()); 
        		if (current < min){
        			min = current;
        			BarTableKey key = (BarTableKey)table.getObject(j);
        			key.select();
        		}
        	}
        	table.doSwap();
        }
        return new FDT[]{table, minimum, table};
    }


    public FDT[] makeModelAnswer(){

        return solve();

    }
    public String[] getStructureVisualisations(){

        return new String[]  {"array", "array", "2D"};

    }

    public String[] getModelAnswerVisualisations(){

        return new String[] {"array", "array", "2D"};

    }
    
    public String[] getModelAnswerNames() {
        ExerciseProperties lang = ExerciseProperties.getInstance();
        return new String[] { 
                lang.get( "SELECTIONSORT_INPUT_MODEL_TITLE" ),
                lang.get( "SELECTIONSORT_MINIMUM_MODEL_TITLE" ),
                lang.get( "SELECTIONSORT_BARGRAPH_MODEL_TITLE" )
        };
    }

    public VisualTypeConf[] conf(){

        VisualTypeConf tabl1 = new VisualTypeConf();
        tabl1.enable("matrix.visual.VisualKey", VisualTypeConf.HIGHLIGHT_OPERATION);

        VisualTypeConf tabl2 = new VisualTypeConf();
        tabl2.setVisualConfValue("matrix.visual.VisualArray", VisualTypeConf.INDEXED, "false");
        
		VisualTypeConf barGraph = new VisualTypeConf();
        barGraph.enable("matrix.visual.VisualArea$AreaVisualization", VisualTypeConf.HIGHLIGHT_OPERATION);
        barGraph.setVisualConfValue("matrix.visual.VisualArea", VisualTypeConf.AXIS_VISIBLE, "false");
        
        return new VisualTypeConf[] { tabl1, tabl2, barGraph};

    }

    public FDT[] init(){

        s=new String[16];
        Random random = new Random(seed);

        int d[];
        d=matrix.util.RandomKey.createIntKeys(random,3,30,16);
        for (int i=0;i<s.length;i++){
            s[i] = d[i] + "";
        }
        
        FDT[] fdts = getInitialStructures();
        table = (BarTable)fdts[0];
        minimumTable = (Table)fdts[1];
        return fdts;
    }
    
    public void setSeed(long v){
        this.seed = v;
    }


    public long getSeed(){
        return seed;
    }

    public String[] getStructureNames() {
        ExerciseProperties lang = ExerciseProperties.getInstance();
        return new String[] { 
                lang.get( "SELECTIONSORT_INPUT_TITLE" ),
                lang.get( "SELECTIONSORT_MINIMUM_TITLE" ),
                lang.get( "SELECTIONSORT_BARGRAPH_TITLE" )
        };
    }

    public FDT[] getAnswer(){
    	
        return new FDT[]{table, minimumTable, table};
    }
    
    public FDT[] getInitialStructures(){
    	
        Table minTable = new Table();
        BarTable barTable = new BarTable(s, minTable);
        barTable.setPaintingStyleDecorator(areaDecorator);
        barTable.setBarPaintingStyleDecorator(barDecorator);
        barTable.setSelectedBarPaintingStyleDecorator(barDecoratorSelected);
        barTable.setOrderedBarPaintingStyleDecorator(barDecoratorOrdered);
        return new FDT[] { barTable, minTable, barTable };

    }
    
    /* ComparableExercise interface */
    
	public int[] getCompareIndices(){
        return new int[] {0, 1, 2};	
	}

	public int[] getGradeIndices(){
        return null; // Used only in recovery mode
	}

	public boolean canRecover(){
		return false;
	}
	
	public String[] getCompareStructureVisualisations(){
		return null; // use default
	}
	
	public String[] getCompareStructureNames(){
		return null; // use default
	}    
    
	public VisualTypeConf[] getCompareVisualTypeConf(){
		return null; // use default
	}
	
    public String getDescription(){

        return "";
    }
    
    public GridBagConstraints[] getConstraints() {
        
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 1;
        gbc.insets = new Insets(ELEMENT_GAP, 2, ELEMENT_GAP, 2);
        
        GridBagConstraints gbc2 = new GridBagConstraints();
        gbc2.gridx = 0;
        gbc2.gridy = 2;
        gbc2.insets = new Insets(ELEMENT_GAP, 2, ELEMENT_GAP, 2);

        GridBagConstraints gbc3 = new GridBagConstraints();
        gbc3.gridx = 0;
        gbc3.gridy = 0;
        gbc3.insets = new Insets(ELEMENT_GAP, 2, ELEMENT_GAP, 2);
        
        return new GridBagConstraints[]{ gbc, gbc2, gbc3 };
    }
    
    public GridBagConstraints[] getModelAnswerConstraints() {
    	return getConstraints();
    }
    
    public GridBagConstraints[] getComparisonConstraints() {
    	return null; // use default
    }


    public Component[] getAWTComponents() {
        ExerciseProperties lang = ExerciseProperties.getInstance();
    	JButton swap = new JButton(lang.get("SELECTIONSORT_SWAP_BUTTON_TEXT"));
        swap.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                swapButton();
            }
        });
        
        return new Component[]{ swap };
    }

    public GridBagConstraints[] getAWTComponentConstraints() {
        
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 3;
        gbc.gridwidth = 1;
        gbc.insets = new Insets(2, 2, 2, 2);
                
        return new GridBagConstraints[]{ gbc};
    }

    public void setApplication(Application a) {
        app=a;
    }

    private void swapButton(){
    	table.doSwap();
    	app.validate();
    }
    

    /**
     * A table which can also be visualized as a bar graph.
     * @author lhakulin
     *
     */
    static class BarTable extends Table implements Area {

        private final Rectangle boundingBox;
        private PaintingStyleDecorator paintingStyleDecorator;
        private PaintingStyleDecorator barDecorator;
        private PaintingStyleDecorator barDecoratorSelected;
        private PaintingStyleDecorator barDecoratorOrdered;
        private Table currentMin;
        private VirtualInteger selectedIndex;
        private VirtualInteger orderedIndex; // index of the highest ordered key
        
        private static final int BAR_WIDTH = 29;
        private static final int HEIGHT_SCALE = 5;
        private static final int AREA_HEIGHT = 200;        
    	private static final int BOTTOM_BORDER_HEIGHT = 20;
       
        public BarTable(String[] s, Table currentMin) {
            super(s);
            for(int i=0; i < s.length; i++){
                setObject(new BarTableKey(s[i], i, this), i);
            }
        	int width = s.length * BAR_WIDTH + 25;
            this.boundingBox = new Rectangle(0, 0, width, AREA_HEIGHT);
            this.barDecorator = new PaintingStyleDecorator();
            this.paintingStyleDecorator = new PaintingStyleDecorator();
            this.currentMin = currentMin;
            selectedIndex = new VirtualInteger(-1, this, "Index of the selected element");
            orderedIndex = new VirtualInteger(-1, this, "Index of the highest ordered element");
        }
            	
        // ** Implementation of interface Area **

        public void addSubArea(Area a) {
        	// Not implemented
        }

        public void removeSubArea(Area a) {
        	// Not implemented
        }

        public Area[] getSubAreas() {
        	// Not implemented
            return new Area[0];
        }

        public void delete(SpatialComparable e) {
        	// Not implemented
        }

        public void insert(SpatialComparable e) {
        	// Not implemented
        }        

        public Rectangle getBoundingPolygon() {
            return this.boundingBox;
        }

        /**
         * Creates rectangles according to the BarTableKeys in the table
         */
        public SpatialElement[] getSpatialElements() {
        	
        	int maxValue = AREA_HEIGHT/2;
        	
            Vector vec = new Vector();
            for (int i = this.getFirst(); i <= this.getLast(); i++){
            	if (getObject(i) instanceof BarTableKey){
            		try {
                		int value = Integer.parseInt(((Key)getObject(i)).getVisualizationString()) * HEIGHT_SCALE;
                		if (value > maxValue && i > orderedIndex.eval() && i <= selectedIndex.eval()){
                			maxValue = value;                			
                		}
                		SelectableRectangle rec = new SelectableRectangle(i*BAR_WIDTH, BOTTOM_BORDER_HEIGHT, BAR_WIDTH, value, (BarTableKey)getObject(i));
                		if (i == selectedIndex.eval()){
                			rec.setPaintingStyleDecorator(this.barDecoratorSelected);
                			// minimum line
                			vec.add(new LineSegment((i+1)*BAR_WIDTH, value + BOTTOM_BORDER_HEIGHT, (this.getLast()+1)*BAR_WIDTH, value + BOTTOM_BORDER_HEIGHT, minimumLineDecorator));
                		}
                		else if (i <= orderedIndex.eval() || orderedIndex.eval() >= getLast()-1){
                			rec.setPaintingStyleDecorator(this.barDecoratorOrdered);
                		}
                		else {
                			rec.setPaintingStyleDecorator(this.barDecorator);
                		}
                		vec.add(rec);
                		vec.add(new matrix.structures.spatial.FDT.probe.Label(i*BAR_WIDTH + BAR_WIDTH/2, 10, ""+i));
					} catch (Exception e) {
		                Note.err(this, "Unable to parse integer value of key " + getObject(i).toString());
					}
            	}
            }
            vec.addAll(getSwapSpline(maxValue + 20));
            SpatialElement[] spatialElements = new SpatialElement[vec.size()];
            vec.toArray(spatialElements);
            return spatialElements;
        }
        
        /**
         * Creates a spline connecting the two bars to be swapped
         * @param maxHeight - highest point of the line (which above the highest bar between the two bars)
         * @return - a vector containing the elements of the spline
         */
        private Vector getSwapSpline(int maxHeight){
        	Vector elements = new Vector();
            if (selectedIndex.eval() >= 0){
	            double startX = (orderedIndex.eval()+1)*BAR_WIDTH + BAR_WIDTH/2;
	            double endX = selectedIndex.eval()*BAR_WIDTH + BAR_WIDTH/2;
	            double startY = 0;
	            double endY = 0;
	            if (orderedIndex.eval() >= getLast()){
	            	return elements;
	            }
            	startY = BOTTOM_BORDER_HEIGHT + Double.parseDouble(((Key)getObject(orderedIndex.eval()+1)).getVisualizationString()) * HEIGHT_SCALE;
    			endY = BOTTOM_BORDER_HEIGHT + Double.parseDouble(((Key)getObject(selectedIndex.eval())).getVisualizationString()) * HEIGHT_SCALE;

    			if (selectedIndex.eval() != orderedIndex.eval()+1){
    				
//    				Point2D[] linePoints = new Point2D[3];
//    				linePoints[0] = new Point2D.Double(startX, startY);
//    				linePoints[1] = new Point2D.Double(startX + (endX-startX)/2, maxHeight);
//    				linePoints[2] = new Point2D.Double(endX, endY);
//    				Point2D[][] points = GeometryFunctions.splineInterpolationCurve(linePoints);
//
//    				for (int i=0; i < 2; i++){
//    					CubicBezier bezier = new CubicBezier(points[i][0].getX(), points[i][0].getY(), points[i][1].getX(), points[i][1].getY(),
//    										points[i][2].getX(), points[i][2].getY(), points[i][3].getX(), points[i][3].getY());
//    					bezier.setPaintingStyleDecorator(swapLineDecorator);
//    			        elements.add(bezier);	                					
//    				}
    				
    				// Using simpler bezier, because splineInterpolationCurve doesn't always return
    				// a smooth line
    				CubicBezier bezier = new CubicBezier(startX, startY, startX, startY+50, endX, endY+50, endX, endY);
					bezier.setPaintingStyleDecorator(swapLineDecorator);
			        elements.add(bezier);	                					
    				
    			}
	            elements.add(new Point(startX, startY, swapLineDecorator));
	            elements.add(new Point(endX, endY, swapLineDecorator));
            }
            
            return elements;
        }
        
        public boolean hasPaintingStyleDecorator(){
            return (paintingStyleDecorator != null);
        }

        public void setPaintingStyleDecorator(PaintingStyleDecorator psd){
            paintingStyleDecorator = psd;
        }

        public PaintingStyleDecorator getPaintingStyleDecorator(){
            return paintingStyleDecorator;
        }
        
        public void setBarPaintingStyleDecorator(PaintingStyleDecorator psd){
            barDecorator = psd;
        }
        public void setSelectedBarPaintingStyleDecorator(PaintingStyleDecorator psd){
            barDecoratorSelected = psd;
        }
        public void setOrderedBarPaintingStyleDecorator(PaintingStyleDecorator psd){
            barDecoratorOrdered = psd;
        }
        
        /**
         * Called when a key (or a bar) is selected.
         * Updates the selected key and bar of the table
         * @param key - the selected key
         */
        public void keySelected(Key key){
        	this.currentMin.setObject(new StyledKey(getKeyIndex(key) + ""), 0);
        	for (int i=this.getFirst(); i <= this.getLast(); i++){
        		if (key.equals(this.getObject(i))){
        			selectedIndex.assign(i);
        		}
        		else {
        			((BarTableKey)getObject(i)).setSelected(false);
        		}
        	}
        }
        
        /**
         * Used when highlighting the ordered part of the table
         */
        public void selectIndex(int index) {
        	super.selectIndex(0);
        	
        	if (index == 0){
        		// needed when highlighting only the first index
        		super.selectIndex(-1);
        	}
        	else {
        		super.selectIndex(index);
        	}        	
        }
        
        /**
         * Swaps the selected key with the first unordered key
         *
         */
        public void doSwap(){
        	if (this.selectedIndex.eval() >= 0 && orderedIndex.eval() < this.getLast()){
	        	Animator.getActiveAnimator().startOperation();
	        	for (int i=0; i <= this.getLast(); i++){
	        		BarTableKey key = (BarTableKey)this.getObject(i);
	        		key.setSelected(false);
	        	}
	        	super.swap(orderedIndex.inc(), selectedIndex.eval());
	        	
	        	// if the unsorted part is only the last key, the array is sorted
	        	if (orderedIndex.eval() >= this.getLast()-1){
	        		orderedIndex.inc();
	        	}
	        	this.selectIndex(orderedIndex.eval());
	        	this.currentMin.setObject(null, 0);
	        	this.selectedIndex.assign(-1);
	        	Animator.getActiveAnimator().endOperation();
        	}
        }
        
        /**
         * Gets the index of the given key
         * @param key
         * @return the index of the key or -1 if the key was not found
         */
        private int getKeyIndex(Key key){
        	for (int i=0; i <= getLast(); i++){
        		if (key.equals(getObject(i))){
        			return i;
        		}
        	}
        	return -1;
        }

        static final long serialVersionUID = 8668291684539197254L;   
    }

    /**
     * A key which is used in a BarTable
     * @author lhakulin
     *
     */
    static class BarTableKey extends Key implements Styled{

        private VirtualBoolean selected = new VirtualBoolean(false, this, "Has this key been selected");
        private BarTable parentTable;
        
        public BarTableKey(String keyText, int i, BarTable p){
            super(keyText);
            setComparisonString(keyText+i); // to identify duplicates
            setVisualizationString(keyText);
            parentTable = p;
        }

        public void setSelected(boolean b){
            selected.assign(b);
        }

        public void select(){
        	Animator.getActiveAnimator().startOperation();
            selected.assign(!selected.eval());
            parentTable.keySelected(this);
            Animator.getActiveAnimator().endOperation();
        }
        
        public StyleSheet getStyleSheet(){
        	
        	StyleSheet sh = new StyleSheetAdapter();
        	
        	if (BarTableKey.this.selected.eval()){
        		sh.setDefaultPenColor(Color.white);
        		sh.setDefaultBackgroundColor(Color.red);
            	sh.setFocusedBackgroundColor(Color.red);
        	}

        	return sh;        	
        }

        static final long serialVersionUID = 5356245316236393677L;    
    }
    
    /**
     * A selectable rectangle which is used in a BarTable
     * Each SelectableRectangle represents one BarTableKey
     * @author lhakulin
     *
     */
    static class SelectableRectangle extends matrix.structures.spatial.FDT.probe.Rectangle implements Selectable {

		private VirtualBoolean selected = new VirtualBoolean(false, this, "Has this key been selected");
		private BarTableKey key;
    	
    	public SelectableRectangle(double x, double y, double w, double h, BarTableKey key) {
    		super(x, y, w, h);
    		this.key = key;
    	}
        public long getSelectionTime() {            
            return selected.eval()?1:0;
        }

        public void setSelect(long globalClockTimestamp) {
        	Animator.getActiveAnimator().startOperation();
            selected.assign(!selected.eval());
            key.select();
            Animator.getActiveAnimator().endOperation();
        }

        private static final long serialVersionUID = 2942374711421559203L;
    }

    /**
     * A styled key.
     * Used to hold the current minimum value.
     * @author lhakulin
     *
     */
    static class StyledKey extends Key implements Styled {
    	
		private static final long serialVersionUID = -6347253492393485127L;

		private StyleSheet sh;
    	
    	public StyledKey(String s){
    		super(s);
    		initStyleSheet();
    	}
    	
    	private void initStyleSheet(){
        	sh = new StyleSheetAdapter();
        	sh.setDefaultBackgroundColor(Color.red);
        	sh.setVisitedBackgroundColor(Color.red);
        	sh.setDefaultPenColor(Color.white);
        	sh.setVisitedPenColor(Color.white);
    	}
    	
        public StyleSheet getStyleSheet(){
        	return sh;
        }
    }
    
    
    static final long serialVersionUID = 5324781323499168437L;
}
