//Matrix - algorithm simulation system
//Copyright (C) 2000- 2007 Matrix group
//(See the AUTHORS file and the author tag below for more information
//on the authors.)

//This program is free software; you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation; either version 2 of the License, or
//(at your option) any later version.

//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.

//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package content.exercises;

import java.util.Comparator;
import java.util.Random;

import content.ExerciseProperties;
import content.interfaces.ButtonExercise;
import content.interfaces.ComparableExercise;
import content.interfaces.ConfigureVisualType;
import content.interfaces.ModelAnswerNames;
import content.interfaces.SimulationExerciseModel;
import content.interfaces.SimulationExerciseSelfAssessment;
import content.interfaces.SwapBehaviour;
import content.exercises.structures.ExerAVL;

import matrix.animation.Animator;
import matrix.simulation.VisualTypeConf;
import matrix.structures.CDT.probe.AVLTree;
import matrix.structures.FDT.FDT;
import matrix.structures.FDT.probe.Table;
import matrix.util.Note;

public class Tree_Rotations2 implements SimulationExerciseSelfAssessment, ButtonExercise, ModelAnswerNames,
										ConfigureVisualType, SwapBehaviour, ComparableExercise {


    private Table t;
    private ExerAVL ex,exOld;
    private int doubleRot=0;
    private static int MIN=6;
    private AVLTree avl;
    private String S;
    public static final boolean DEBUG = true;


    private String marker = "Tree Rotations";

    public Tree_Rotations2() {
    }

    /**
     * In this exercise a drag operation performs a swap
     * @return
     */
    public boolean getSwapBehaviour() {
        return true;
    }

    long seed = 1;

    /**
     * Get the value of seed.
     * @return value of seed.
     */
    public long getSeed() {
        return seed;
    }

    /**
     * Set the value of seed.
     * @param v  Value to assign to seed.
     */
    public void setSeed(long  v) {
        this.seed = v;
    }


    public int[] judge() {
        ExerAVL res = (ExerAVL)(getInitialStructures()[0]);
        int i=doubleRot;
        res.realInsert(t.getObject(i));
        int points = 0;
        if (res.equals(ex))
            points = 1;
        int[] results = new int[1];
        results[0]=points;
        return results;
    }

    public int getMaxPoints() {
        return 1;
    }

    public String getFeedbackString() {
        // TODO Auto-generated method stub
        return null;
    }

    public FDT[] init() {

        int i;
        boolean doubleNeeded; 
        boolean rootChanged;
        Random R = new Random(seed);
        do {
            doubleRot = 0;
            rootChanged = false;
            doubleNeeded = false;
            ex=new ExerAVL();
            // Can contain duplicates.
            S = matrix.util.RandomKey.createNoDuplicateUppercaseRandomKey(R,24);

            t = new Table(S);


            for(i = 0; i < S.length(); i++) {
                ex.realInsert(t.getObject(i));
                if (ex.doubleWasNeeded() && !doubleNeeded && i > MIN)
                    doubleNeeded=true;
                if (doubleNeeded && doubleRot <  MIN) {
                    doubleRot = i;
                    rootChanged = ex.rootWasChanged();
                }
            }


            if (DEBUG)
                System.out.println("iteration ["+S+"] double:"+doubleRot+" root:"+rootChanged);
        } while(doubleNeeded == false || doubleRot < MIN || rootChanged);


        ex = new ExerAVL();
        // Double
        for (i = 0; i < doubleRot; i++)
            ex.realInsert(t.getObject(i));
        ex.dumbInsert(t.getObject(i));
        FDT[] s = {ex};

        return s;
    }


    public String[] getStructureNames() {
        ExerciseProperties lang = ExerciseProperties.getInstance();
        String[] s =  { lang.get("TREE_ROTATIONS2_AVL_TREE_TITLE") };
        return s;
    }

    public String getDescription() {
        ExerciseProperties lang = ExerciseProperties.getInstance();
        return lang.get("TREE_ROTATIONS2_DESCRIPTION_TEXT");
    }

    public String[] buttonNames() {
        ExerciseProperties lang = ExerciseProperties.getInstance();
        String[] s = {lang.get("TREE_ROTATIONS2_UPDATE_BUTTON_TEXT")};
        return s;
    }


    public String[] buttonCommands() {
        String[] s = {"updateReferences"};
        return s;
    }

    public SimulationExerciseModel getModelAnswer() {
        return this;
    }

    public FDT[] solve() {
        Note.out(this, "Solve");
        ExerAVL res = (ExerAVL)(getInitialStructures()[0]);
        int i=doubleRot;

        Animator a = Animator.getActiveAnimator();

        a.startOperation();
        res.realInsert(t.getObject(i));
        a.endOperation();

        Note.out(this, "Solved");
        return new FDT[] {res };
    }



    public FDT[] makeModelAnswer() {
        return solve();
    }

    public String[] getModelAnswerNames() {
        ExerciseProperties lang = ExerciseProperties.getInstance();
        String[] s =  { lang.get("TREE_ROTATIONS2_MODEL_AVL_TREE_TITLE") };
        return s;
    }

    public FDT[] getInitialStructures() {
        ExerAVL res = new ExerAVL();

        for (int i = 0; i < doubleRot; i++)
            res.realInsert(t.getObject(i));
        return new FDT[] { res };
    }

    public FDT[] getAnswer() {

        return new FDT[] {ex };
    }


    public VisualTypeConf[] conf() {

        VisualTypeConf tre = new VisualTypeConf();

        //tre.enable("matrix.visual.VisualKey", VisualTypeConf.HIGHLIGHT_OPERATION);
        //tre.enable("matrix.visual.VisualKey", VisualTypeConf.DROP_OPERATION);
        //tre.enable("matrix.visual.VisualKey", VisualTypeConf.DRAG_OPERATION);

        tre.enable("matrix.visual.VisualLayeredTreeComponent",
                VisualTypeConf.HIGHLIGHT_OPERATION);
        tre.enable("matrix.visual.VisualLayeredTreeComponent", VisualTypeConf.DROP_OPERATION);
        tre.enable("matrix.visual.VisualReference", VisualTypeConf.HIGHLIGHT_OPERATION);
        tre.enable("matrix.visual.VisualReference", VisualTypeConf.DRAG_OPERATION);

        return new VisualTypeConf[] { tre};
    }
    
    /* ComparableExercise interface */
    
	public int[] getCompareIndices(){
        return new int[] {0};	
	}

	public int[] getGradeIndices(){
        return null; // Used only in recovery mode
	}

	public boolean canRecover(){
		return false;
	}
	
	public String[] getCompareStructureVisualisations(){
		return null; // use default
	}
	
	public String[] getCompareStructureNames(){
		return null; // use default
	}    
	
	public VisualTypeConf[] getCompareVisualTypeConf(){
		return null; // use default
	}

    static final long serialVersionUID = -3394028505321162930L;
}
