//Matrix - algorithm simulation system
//Copyright (C) 2000- 2007 Matrix group
//(See the AUTHORS file and the author tag below for more information
//on the authors.)

//This program is free software; you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation; either version 2 of the License, or
//(at your option) any later version.

//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.

//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package content.exercises;

import java.awt.*;
import java.awt.event.*;
import java.util.Random;
import javax.swing.JButton;

import content.ExerciseProperties;
import content.exercises.structures.ColorableKey;
import content.interfaces.AWTComponentUtilizer;
import content.interfaces.ComparableExercise;
import content.interfaces.ConfigureVisualType;
import content.interfaces.HasRotatedContainers;
import content.interfaces.JudgeBlocks;
import content.interfaces.ModelAnswerNames;
import content.interfaces.SimulationExerciseModel;
import content.interfaces.StyledExercise;
import content.interfaces.SwapBehaviour;

import matrix.animation.Animator;
import matrix.simulation.VisualTypeConf;
import matrix.structures.CDT.probe.Stack;
import matrix.structures.FDT.*;
import matrix.structures.FDT.advanced.StyledArray;
import matrix.structures.FDT.probe.Key;
import matrix.structures.FDT.probe.Table;
import matrix.structures.memory.*;
import matrix.util.*;
import matrix.visual.VisualType;


public class TrueRecursiveRadixExchangeSort implements SimulationExerciseModel,
    										StyledExercise, ModelAnswerNames,
    										ConfigureVisualType, AWTComponentUtilizer,
    										HasRotatedContainers,/* UsesSplit,*/ JudgeBlocks,
    										SwapBehaviour, ComparableExercise {

    private Application app;

    public static final int BIT_INDEX = 0;
    public static final int LEFT_INDEX = 1;
    public static final int RIGHT_INDEX = 2;


    public void setApplication(Application a) {
        app=a;
    }

    public Application getApplication() {
        return app;
    }

    /* how many odd numbers are required in the array to be sorted */
    public static final int ODD_NUMBER_COUNT = 5;

    public static final int ARRAY_SIZE = 10;

    /* indices of the left and right border in the parameter table */
    public static final int LEFT_BORDER = 0;
    public static final int RIGHT_BORDER = 1;


    /* length of keys */
    public static final int KEY_BITS = 3;


    private Table userTable;
    private BitSelectionTable userBitSelect;
    private Stack userStack;

    protected String[] keyArray = null;
    public static final boolean DEBUG = false;

    /**
     * In this exercise a drag operation performs a swap
     * @return
     */
    public boolean getSwapBehaviour() {
        return true;
    }

    long seed = 1;

    /**
     * Get the value of seed.
     * @return value of seed.
     */
    public long getSeed() {
        return seed;
    }

    /**
     * Set the value of seed.
     * @param v  Value to assign to seed.
     */
    public void setSeed(long  v) {
        this.seed = v;
    }

    public FDT[] init() {
        java.util.Random r =  new java.util.Random(seed);
        BitTable bitTable = null;
        do {
            bitTable = createBitTable(r, 0, 25, KEY_BITS, ARRAY_SIZE);
        } while (bitTable.oddCount < ODD_NUMBER_COUNT);
        keyArray = bitTable.table;

        FDT[] rvs =  getInitialStructures();

        userTable     = (Table)rvs[0];
        userBitSelect = (BitSelectionTable)rvs[1];
        userStack     = (Stack)rvs[2];

        return rvs;
    }

    public FDT[] getInitialStructures(){

        Table newTable = new Table(makeKeys(keyArray));

        SelectableKey[] bitKeys = new SelectableKey[KEY_BITS];
        for(int i=0; i<bitKeys.length; i++)
            bitKeys[i]=new SelectableKey(bitKeys.length -1 - i);

        BitSelectionTable newBitSelect = new BitSelectionTable(bitKeys);
        newTable.selectRange(0, ARRAY_SIZE - 1);

        Stack newStack = new Stack();
        pushParameters(newStack, new int[]{KEY_BITS-1,0,ARRAY_SIZE-1});

        FDT[]  s = { newTable, newBitSelect, newStack };
        return s;
    }

    public String[] getStructureNames() {
        ExerciseProperties lang = ExerciseProperties.getInstance();
        return new String[] { 
                lang.get("RADIXSORT_TABLE_TITLE"),
                lang.get("RADIXSORT_BIT_SELECTION_TITLE"),
                lang.get("RADIXSORT_CALL_STACK_TITLE") 
        };
    }

    public String[] getStructureVisualisations() {
        String[] s = { "array", "array", "array" };
        return s;
    }

    public String[] getModelAnswerVisualisations() {
        return getStructureVisualisations();
    }

    public String[] getModelAnswerNames() {
        ExerciseProperties lang = ExerciseProperties.getInstance();
        return new String[] { 
                lang.get("RADIXSORT_TABLE_MODEL_TITLE"), 
                lang.get("RADIXSORT_BIT_SELECTION_MODEL_TITLE"), 
                lang.get("RADIXSORT_CALL_STACK_MODEL_TITLE") 
        };
    }

    public SimulationExerciseModel getModelAnswer() {
        return this;
    }

    public VisualTypeConf[] conf() {
        VisualTypeConf tabl = new VisualTypeConf();


        tabl.enable("matrix.visual.VisualKey", VisualTypeConf.HIGHLIGHT_OPERATION);
        tabl.enable("matrix.visual.VisualKey", VisualTypeConf.DRAG_OPERATION);
        tabl.enable("matrix.visual.VisualKey", VisualTypeConf.DROP_OPERATION);
        tabl.enable("matrix.visual.VisualArrayComponent", VisualTypeConf.HIGHLIGHT_OPERATION);
        tabl.setVisualConfValue("matrix.visual.VisualArray", VisualTypeConf.RANGE_SELECTION, "true");

        VisualTypeConf bitSelect = new VisualTypeConf();
        bitSelect.enable("matrix.visual.VisualKey", VisualTypeConf.HIGHLIGHT_OPERATION);

        VisualTypeConf stack = new VisualTypeConf();
        return new VisualTypeConf[] { tabl, bitSelect, stack };
    }


    public FDT[] getAnswer() {
        FDT[] s = { userTable,  userBitSelect, userStack };
        return s;
    }


    public FDT[] init(Object input) {
        return init();
    }


    public FDT[] solve() {
        return solveHighlight(false);
    }

    public FDT[] makeModelAnswer() {
        return solveHighlight(true);
    }

    public FDT[] solveHighlight(boolean hlight){
        FDT[] structures = getInitialStructures();
        Table mTable   = (Table)structures[0];
        Table mBorders = (Table)structures[1];
        Stack mStack   = (Stack)structures[2];

        Animator a = Animator.getActiveAnimator();
        radixExchangeSort(a, mTable, (BitSelectionTable)mBorders, mStack,  hlight);

        a.startOperation();
        popParameters(mStack);
        a.endOperation();

        return new FDT[] {mTable, mBorders, mStack};
    }
    
    /* ComparableExercise interface */
    
	public int[] getCompareIndices(){
        return new int[] {0, 1, 2};	
	}

	public int[] getGradeIndices(){
        return null; // Used only in recovery mode
	}

	public boolean canRecover(){
		return false;
	}
	
	public String[] getCompareStructureVisualisations(){
		return null; // use default
	}
	
	public String[] getCompareStructureNames(){
		return null; // use default
	}    
	
	public VisualTypeConf[] getCompareVisualTypeConf(){
		return null; // use default
	}

    //========================================================

    // --------UsesSplit begin
    public int[] getPanelInfo(){
        return new int[]{0,0,1};
    }

    public int getNumberOfPanels(){
        return 2;
    }
    // --------UsesSplit end


    // --------RotatedContainers begin
    public boolean[] getContainerRotations(){
        return new boolean[]{false, false, true};
    }
    public boolean[] getModelAnswerRotations() {
        return getContainerRotations();
    }
    public boolean [] getComparisonRotations() {
    	return null; // use default
    }
    // --------RotatedContainers end



    public String getDescription() {
        return "";

        /*
	    "RadixExchangeSort( Array, Stack){\n\n"+
	    "  peek values from the top of the recursion stack (bit, leftIndex, rightIndex)\n\n"+
	    "  if (rightIndex > leftIndex AND bit >= 0){\n"+
	    "    split = ExchangeSort(Array, bit, leftIndex, rightIndex)\n\n"+
            "    if Array[split] == 0\n"+
            "       split = split + 1\n\n"+
	    "    PUSH to stack -> (bit, leftIndex, split-1)\n"+
	    "    PUSH to stack -> (bit, split, rightIndex)\n\n"+
            "    RadixExchangeSort(Array, Stack)\n"+
	    "    POP the top item off the stack\n\n"+
            "    RadixExchangeSort(Array, Stack)\n"+
	    "    POP the top item off the stack\n"+
	    "  }\n"+
	    "}\n\n"+
	    "ExchangeSort(Array, bit, leftIndex, rightIndex){\n\n"+
	    "  i = leftIndex\n"+
	    "  j = rightIndex\n"+
	    "  while (j != i){\n"+
	    "    while ( Array[i].bit == 0 AND i < j)\n"+
	    "      i = i+1\n\n"+
	    "    while ( Array[j].bit == 1 AND i < j)\n"+
	    "      j = j-1\n\n"+
	    "    SWAP array elements i and j\n"+
	    "  }\n"+
	    "}";
         */

    }

    //========================================================


    /**
       Sorts the given table using recursive radix exchange sort.

       @param a animator for animating the algorithm
       @param table the table to be sorted
       @param size the size of the table
       @param stack the recursion stack
       @param highlightOn a flag indicating whether to highlight keys in the table when
       the algorithm proceeds. Highlighting is in generating the model solution.
     */
    static void radixExchangeSort(Animator a, Table table, BitSelectionTable bt, Stack stack, boolean highlightOn) {


        //PEEK the top of the stack (get parameters)

        Object top = stack.getTopOfStack();
        if (top instanceof Key)
            top = ((Key)top).getObject();

        Table paramsTable = (Table)top;	
        int[] params = getParameters(paramsTable);

        int left  = params[LEFT_INDEX];
        int right = params[RIGHT_INDEX];
        int bit   = params[BIT_INDEX];

        try{

            int leftCursor, rightCursor;

            if (right > left && bit >= 0){
                leftCursor  = left;
                rightCursor = right;

                while (leftCursor != rightCursor){
                    while ( bit(getParameter(table,leftCursor), bit) == 0 &&
                            leftCursor < rightCursor )
                        leftCursor++;

                    while ( bit(getParameter(table,rightCursor), bit) == 1 &&
                            leftCursor < rightCursor )
                        rightCursor--;

                    swap(a, table, leftCursor, rightCursor, highlightOn);

                }

                if ( bit(getParameter(table,rightCursor), bit) == 0 )
                    rightCursor++;

                if (bit > 0){		 


                    int[] leftPartition  = new int[3];
                    int[] rightPartition = new int[3];

                    leftPartition[LEFT_INDEX]  = left;
                    leftPartition[RIGHT_INDEX] = rightCursor-1;
                    leftPartition[BIT_INDEX]   = bit-1;

                    rightPartition[LEFT_INDEX]  = rightCursor;
                    rightPartition[RIGHT_INDEX] = right;
                    rightPartition[BIT_INDEX]   = bit-1;

                    if (rightCursor-1 > left){
                        if(highlightOn){
                            a.startOperation();	
                            table.selectRange(left, rightCursor -1);
                            bt.setSelectedKeyValue(bit - 1);
                            a.endOperation();	
                        }

                        pushParametersAnimated(stack, leftPartition, a);
                        radixExchangeSort(a, table, bt, stack, highlightOn);
                        popParametersAnimated(stack, table, bt, a);
                    }

                    if (rightCursor < right){
                        if(highlightOn){
                            a.startOperation();	
                            table.selectRange(rightCursor, right);
                            bt.setSelectedKeyValue(bit - 1);
                            a.endOperation();	
                        }

                        pushParametersAnimated(stack, rightPartition, a);
                        radixExchangeSort(a, table, bt, stack, highlightOn);
                        popParametersAnimated(stack, table, bt, a);		    		 
                    }

                }
            }
            else{

            }
        }catch(NumberFormatException e){
            Note.show(null, "Illegal value in table.");	    
        }

    }	


    static void swap(Animator a, Table table, int hi, int lo, boolean highlightOn){
        if (hi == lo)
            return;

        a.startOperation();
        table.swap(hi,lo);

        a.endOperation();	
    }





    static void highlightTable(Table t, int lo, int hi, Color color) {
        for (int i = lo; i <= hi; i++) {
            Object o = t.getObject(i);
            if (o instanceof ColorableKey) {
                highlightKey(((ColorableKey)o), color);
            }
        }
    }


    static void highlightKey(Object k, Color c) {
        if (k instanceof ColorableKey)
            ((ColorableKey)k).getStyleSheet().setDefaultBackgroundColor(c);
    }



    static void clearHighlight(Table t) {
        for (int i = 0; i < t.size(); i++) {
            Object o = t.getObject(i);
            if (o instanceof ColorableKey) {
                ((ColorableKey)o).setDefaultBackgroundColor(Color.white);
            }
        }
    }

    static void pushParametersAnimated(Stack stack, int[] params, Animator a) {
        a.startOperation();
        pushParameters(stack, params);
        a.endOperation();
    }

    static void pushParameters(Stack stack, int[] params) {
        Table t = new Table(params.length);
        setParameters(t, params);

        stack.insert((FDT)t);
    }


    static int[] popParametersAnimated(Stack stack, Table table, BitSelectionTable bt, Animator a) {
        a.startOperation();
        int [] arr=popParameters(stack);

        int [] newTop = peekParameters(stack);
        bt.setSelectedKeyValue(newTop[0]);	
        table.selectRange(newTop[1], newTop[2]);	

        a.endOperation();
        return arr;
    }

    static int[] popParameters(Stack stack) {
        int[] params = peekParameters(stack);
        stack.pop(null);
        return params;
    }

    static int[] peekParameters(Stack stack) {
        Table params;

        Object top = stack.getTopOfStack();
        if (top instanceof Key)
            params = (Table)((Key)top).getObject();
        else
            params = (Table)top;

        if (params==null)
            return null;	

        return getParameters(params);
    }



    //---conversion methods for int tables----------------
    static void setParameters(Table t, int[] params) {
        for (int i = 0; i < params.length; i++) {
            t.setObject(params[i]+"", i);
        }
    }

    static int[] getParameters(Table params) {


        int[] p = new int[params.size()];
        for (int i = 0; i < p.length; i++) {
            try {
                String s=params.getObject(i).toString();
                p[i] = Integer.valueOf(s).intValue();
            } catch (NumberFormatException e) {		
                p[i] = -1;
            } catch (NullPointerException e) {
                p[i] = -1;
            }
        }
        return p;
    }

    static int getParameter(Table params, int index){

        try {
            String s=params.getObject(index).toString();
            return Integer.valueOf(s).intValue();
        } catch (NumberFormatException e) {		
            return -1;
        } catch (NullPointerException e) {
            return -1;
        }

    }
    //---------------------------------------------------------


    static int bit(int x, int which) {
        return ((x >> which) & 1);
    }



    /**
       Creates a bit array.

       @param r random number generator
       @param bits number of bits used
       @param size number of characters of the key to create
     */
    public BitTable createBitTable(Random r, int min, int max, int bits,
            int size) {
        String[] str = new String[size];
        int i = 0;
        int oddCount = 0;
        while (i < size) {
            // Ascii codes of letters A-Z are from 1 to 26.
            int rand = ((int)(r.nextDouble() * 32))+1;
            str[i] = binaryRepresentation(rand, bits);
            if (rand % 2 == 1)
                oddCount++;
            i++;
        }
        return new BitTable(str, oddCount);
    }



    public static String binaryRepresentation(int n, int bits) {
        StringBuffer buf = new StringBuffer();
        for (int i = bits-1; i >= 0; i--) {
            buf.append(((n >> i) % 2) == 0 ? "0":"1");
        }
        return buf.toString();
    }



    // Buttons

    public Component[] getAWTComponents() {
        ExerciseProperties lang = ExerciseProperties.getInstance();

        JButton push = new JButton( lang.get("RADIXSORT_CALL_BUTTON_TEXT") );
        push.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                pushButton();
            }
        });

        JButton pop = new JButton( lang.get("RADIXSORT_RETURN_BUTTON_TEXT") );
        pop.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                popButton();
            }
        });

        return new Component[]{push, pop};
    }

    public GridBagConstraints[] getAWTComponentConstraints() {
        return null;
    }


    boolean containsStructure(VisualType vt, FDT s) {
        if (vt == null)
            return false;
        else if (vt.getStructure() == null)
            return false;
        else if (vt.getStructure() == s) 
            return true;
        else if (!(vt.getParent() instanceof VisualType))
            return false;
        else
            return containsStructure((VisualType)vt.getParent(), s);
    }




    void pushButton() {
        int[] params =new int[3]; 

        params[0] = userBitSelect.getSelectedKeyValue();
        params[1] = userTable.selectionStart();
        params[2] = userTable.selectionEnd();

        /*getParameters(userParameters);
	for (int i = 0; i < params.length; i++) {
	    if (params[i] < 0) {
		Note.show(null, "Select parameters.");
		return;
	    }
	    }*/
        getApplication().getAnimator().startOperation();
        pushParameters(userStack, params);
        getApplication().getAnimator().endOperation();
        //clearTable(userParameters);

        validateApplication();
    }

    void popButton() {

        //in this exercise one shouldn't pop the final element off the stack
        if (userStack.getStackSize()==0)
            return;

        getApplication().getAnimator().startOperation();
        int[] parameters = popParameters(userStack);

        //>ADDED
        if (userStack.getStackSize() == 0){
        }
        else{
            Object top = userStack.getTopOfStack();
            if (top instanceof Key)
                top = ((Key)top).getObject();

            Table paramsTable = (Table)top;	
            parameters = getParameters(paramsTable);
            //<ADDED
        }

        Animator.getActiveAnimator().endOperation();
        //setParameters(userParameters, parameters);

        if (userStack.getStackSize() != 0){
            userTable.selectRange(parameters[1], parameters[2]);
            userBitSelect.setSelectedKeyValue(parameters[0]);
        }

        validateApplication();
    }


    void clearTable(Table t) {
        for (int i = 0; i < t.size(); i++) {
            t.setObject(" ", i);
        }
    }


    void validateApplication() {
        if (app != null) {
            app.validate();
            app.validateAnimator();
        }
    }



    private ColorableKey[] makeKeys(String[] keys) {
        ColorableKey[] vKeys = new ColorableKey[keys.length];
        for (int i = 0; i < keys.length; i++) {
            vKeys[i] = new ColorableKey(keys[i]);
        } 
        return vKeys;
    }




    static class BitTable {
        String[] table;
        int oddCount;
        BitTable(String[] table_, int oddCount_) {
            table = table_;
            oddCount = oddCount_;
        }
    }





    static class BitSelectionTable extends Table implements StyledArray {
        SelectableKey current;

        BitSelectionTable(SelectableKey[] args) {
            super(args);

            for (int i=0; i<args.length; i++){
                args[i].setParent(this);
            }

            current = args[0];
            current.set();


        }

        public String getIndexName(int address) {
            return "";	    
        }

        public void setSelectedKey(SelectableKey caller){
            current.clear();
            caller.set();
            current = caller;
        }

        public int getSelectedKeyValue(){
            return current.getValue();
        }

        public void setSelectedKeyValue(int index){
            Note.out(this, "set selected " + index);
            Note.out(this, "that is " + getObject(index));
            setSelectedKey((SelectableKey)getObject(KEY_BITS-1-index));
        }
        static final long serialVersionUID = 5178438448608582653L;
    }

    static class SelectableKey extends content.exercises.structures.ColorableKey{
        BitSelectionTable parent;
        int keyValue;
        VirtualBoolean isSelected = new VirtualBoolean(false, this, "Has this key been selected");

        public boolean isSelected(){
            //Note.out(this, "focusrequest "+isSelected.eval());
            return isSelected.eval();
        }

        public void set(){
            //Note.out(this, "set "+keyValue);
            isSelected.assign(true);
        }

        public void clear(){
            isSelected.assign(false);
        }

        public SelectableKey(int keyValue){
            super("Bit "+keyValue);
            this.keyValue = keyValue;
        }

        public void setParent(BitSelectionTable parent){
            this.parent = parent;
        }

        public void select(){
            //Note.out(this, "select Entity "+this);
            parent.setSelectedKey(this);
        }	

        public int getValue(){
            return keyValue;
        }

        public Color getVisitedFocusedPenColor(){
            return getVisitedPenColor();
        }

        public Color getFocusedPenColor(){
            return getDefaultPenColor();
        }

        public Color getVisitedFocusedBackgroundColor(){
            return getVisitedBackgroundColor();
        }

        public Color getFocusedBackgroundColor(){
            return getDefaultBackgroundColor();	
        }



        static final long serialVersionUID = -3244060625096939248L;
    }

    static final long serialVersionUID = 4482070511108742137L;
}
