// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
package content.exercises.sda;

import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.Serializable;
import java.util.Random;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;

import matrix.animation.Animator;
import matrix.simulation.VisualTypeConf;
import matrix.structures.CDT.probe.StackImpl;
import matrix.structures.FDT.FDT;
import matrix.structures.FDT.advanced.StyledArray;
//import matrix.structures.FDT.probe.Table;
import matrix.structures.memory.VirtualObject;
import matrix.structures.simulationextensions.Selectable;
//import matrix.structures.spatial.Area;
import matrix.structures.spatial.FDT.probe.Point;
import matrix.structures.spatial.FDT.probe.PolyLine;
import matrix.structures.spatial.FDT.probe.Rectangle;
import matrix.structures.spatial.FDT.probe.SpatialElementTable;
import matrix.util.Application;
import matrix.util.Note;

import content.ExerciseProperties;
import content.exercises.sda.structures.PointGenerator;
import content.exercises.sda.structures.PointGenerator.IllegalConstraintsException;
import content.interfaces.AWTComponentUtilizer;
import content.interfaces.ComparableExercise;
import content.interfaces.ConfigureVisualType;
import content.interfaces.JudgeBlocks;
import content.interfaces.LayoutExercise;
import content.interfaces.ModelAnswerNames;
import content.interfaces.SimulationExerciseModel;
import content.interfaces.StyledExercise;

public class ClosestPairOfPoints implements SimulationExerciseModel, StyledExercise,  ConfigureVisualType, AWTComponentUtilizer,
ActionListener, JudgeBlocks, LayoutExercise, ModelAnswerNames, ComparableExercise {

	/**
	 * 
	 */
	private static final long serialVersionUID = -6748337553282209882L;
	// table contains the elements for which the closest pair of points
	// needs to be found
	// auxtable is auxiliary table used for the merge operation.
    // areatable is used to show the area view
	private SpatialElementTable table, auxTable, areaTable, modelAux, modelArea;
	private StackImpl stack, modelStack;
	// holds the closest pair of points
	private SpatialElementTable closest, modelClosest;
	private long seed;
	private java.awt.Point[] points;
	private double mindist = Double.MAX_VALUE;

	private VirtualObject select1, select2;
	private VirtualObject minimumDistance = null;
	private VirtualObject midPoint = null;
	
    private transient Application app;
    
	// number of points
	private static final int POINT_COUNT = 8;
	// borders for the area
	private static final int MINX = 10;
	private static final int MINY = 10;
	private static final int MAXX = 200;
	private static final int MAXY = 130;
	private static final int MINDIST = 20;
	
	public FDT[] solve() {
		mindist = Double.MAX_VALUE;
	    SpatialElementTable modelTable = new SpatialElementTable(POINT_COUNT);
        modelStack = new StackImpl();
	    modelAux = new SpatialElementTable(POINT_COUNT);
        modelArea = new SpatialElementTable(POINT_COUNT, MAXX+10,MAXY+10);
        modelClosest = new ModelTable(2);
        modelClosest.setObject(null, 0);
        modelClosest.setObject(null,1);
        for(int i = 0; i < modelTable.size(); i++) {
        	char key = (char)('a'+i);
            Point p = new Point((int)points[i].getX(), (int)points[i].getY(), ""+key);
            modelTable.setObject(p,i);
            modelArea.setObject(p,i);
            modelAux.setObject(null,i);
        }
        Animator a = Animator.getActiveAnimator();
        modelTable = sortByX(modelTable);
        closestPair(modelTable, 0, modelTable.size()-1, a);
        return new FDT[] {modelTable,modelAux,modelClosest,modelArea,modelStack};
    }

    private void closestPair(SpatialElementTable a, int p, int r, Animator anim) {
        if(p < r) {
        	anim.startOperation();
        	a.selectIndex(p);
        	a.selectIndex(r);
            int q = (p+r)/2;
            int middle = (int)((Point)a.getObject(q)).getX();
        	Point mid = (Point)a.getObject(middle);
        	modelStack.ADTPush(new StackElement("["+p+","+r+"]",mid));
        	anim.endOperation();
            closestPair(a,p,q,anim);
            closestPair(a,q+1,r,anim);
            Note.out(this, "Middle is " + middle);
            if(!(a.selectionStart() == p && a.selectionEnd() == r)) {
            	anim.startOperation();	
            	a.selectIndex(p);
            	a.selectIndex(r);
            	anim.endOperation();
            }
            mergeByY(a,p,q,r,anim);
            Point p1, p2, p3, p4;
            p1 = p2 = p3 = p4 = null;
            int i = p;
            int oldSize = modelArea.size();
            // operation started in mergeByY
            //anim.startOperation();
            double md = mindist;
            double dist = mindist;
            if(md > MAXX) { 
            	md = middle;
            	dist = MAXX/2;
            }
            Rectangle area = new Rectangle(middle-md, 0, dist*2, MAXY+10);
            modelArea.setObject(area, POINT_COUNT);
            anim.endOperation();
            while(i <= r) {
                Point current = (Point)a.getObject(i);
                if(Math.abs(current.getX() - middle) < mindist) {
                    check(current,p1,anim,middle);
                    check(current,p2,anim,middle);
                    check(current,p3,anim,middle);
                    check(current,p4,anim,middle);
                    p1 = p2;
                    p2 = p3;
                    p3 = p4;
                    p4 = current;
                }
                i++;
            }
            int size = modelArea.size();
            anim.startOperation();
            for(i = oldSize; i < size; i++) {
            	modelArea.setObject(null,i);
            }
            modelArea.setObject(null, POINT_COUNT);
            modelArea.selectRange(0,oldSize);
            modelStack.ADTPop();
            anim.endOperation();
        }
    }
    
    private void mergeByY(SpatialElementTable a, int p, int q, int r, Animator anim) {
        int pc = p;
        int qc = q+1;
        anim.startOperation();
        for(int i = p; i <= r; i++) {
            Point pp, qp;
        	// read points, unless we're out of scope area
            if(pc > q) {
        		pp = null;
        	} else {
        		pp = (Point)a.getObject(pc);
        	}
        	if(qc > r) {
        		qp = null;
        	} else {
        		qp = (Point)a.getObject(qc);
        	}
        	
        	// see which point to add next
        	if(pc > q) {
                modelAux.setObject(qp,i);
                qc++;
            } else if(qc > r) {
                modelAux.setObject(pp,i);
                pc++;
            } else if(pp.getY() <= qp.getY()) {
                modelAux.setObject(pp,i);
                pc++;
            } else {
                modelAux.setObject(qp,i);
                qc++;
            }
        }
        anim.endOperation();
        anim.startOperation();
        for(int i = p; i <=r; i++) {
            a.setObject(modelAux.getObject(i),i);
        	modelAux.setObject(null, i);
        }
        //anim.endOperation();
        // operation ended in parent method, bad hack!
    }
    
    private void check(Point p, Point o, Animator a, int middle) {
    	if(p == null || o == null) return;
        double dist = Math.sqrt((p.getX()-o.getX())*(p.getX()-o.getX()) + 
                (p.getY()-o.getY())*(p.getY()-o.getY()));
        a.startOperation();
        PolyLine pol = new PolyLine(new Point[] {p,o}, "" + p.getElement() + "-" + o.getElement());
        modelArea.setObject(pol,modelArea.size());
        a.endOperation();
        if(dist < mindist) {
            mindist = dist;
            a.startOperation();
            Rectangle newr = new Rectangle(middle-mindist, 0, mindist*2, MAXY+10);
            modelArea.setObject(newr, POINT_COUNT);
            modelClosest.setObject(o,0);
            modelClosest.setObject(p,1);
            a.endOperation();
        }
    }
    
    private SpatialElementTable sortByX(SpatialElementTable t) {
        for(int i = 0; i < t.size(); i++) {
            Point tmp = (Point)t.getObject(i);
            int location = i;
            for(int j = i+1; j < t.size(); j++) {
                Point p = (Point)t.getObject(j);
                if(p.getX() < tmp.getX()) {
                    tmp = p;
                    location = j;
                }
            }
            Point pair = (Point)t.getObject(i);
            t.setObject(tmp, i);
            t.setObject(pair,location);
        }
        return t;
    }
    
	public FDT[] makeModelAnswer() {
		return solve();
	}

	public FDT[] init() {
		stack = new StackImpl();
		table = new SpatialElementTable(POINT_COUNT);
		auxTable = new SpatialElementTable(POINT_COUNT);
		areaTable = new SpatialElementTable(POINT_COUNT, MAXX+10, MAXY+10);
        minimumDistance = new VirtualObject(null, null, "Minimum known distance between two points");
        minimumDistance.setMem(0, new Double(Double.MAX_VALUE));
        midPoint = new VirtualObject(null, null, "Current middle point in the algorithm");
        midPoint.setMem(0, new Integer(MAXX/2));
        closest = new ExerciseTable(2);
        select1 = new VirtualObject(null, null, "User-selected point no. 1");
        select2 = new VirtualObject(null, null, "User-selected point no. 2");
        try {
			points = PointGenerator.generatePoints(MINX,MINY,MAXX,MAXY,POINT_COUNT,MINDIST, new Random());
		} catch(IllegalConstraintsException e) {
			Note.err(this, "Cannot initialize, no point set generated");
			return null;
		}
		for(int i = 0; i< points.length; i++) {
			char key = (char)('a'+i);
            Point p = new SelectablePoint((int)points[i].getX(), (int)points[i].getY(), "" + key);
            table.setObject(p,i);
            areaTable.setObject(p,i);
            auxTable.setObject(null,i);
        }
        closest.setObject(null, 0);
        closest.setObject(null, 1);
        table = sortByX(table);
        return new FDT[] {table, auxTable, closest, areaTable, stack};
	}

	public void setSeed(long seed) {
	    this.seed = seed;
    }

	public long getSeed() {
	    return seed;
    }

	public String[] getStructureNames() {
		ExerciseProperties lang = ExerciseProperties.getInstance();
		String[] s = new String[] { lang.get("CLOSESTPAIROFPOINTS_TABLE_TITLE"), lang.get("CLOSESTPAIROFPOINTS_AUX_TABLE_TITLE"), 
				lang.get("CLOSESTPAIROFPOINTS_CLOSEST_PAIR_TITLE"), lang.get("CLOSESTPAIROFPOINTS_AREA"), lang.get("CLOSESTPAIROFPOINTS_CALL_STACK")
		};
		return s;
		//return new String[] {"Points", "Auxiliary table", "Closest pair", "Area", "Call stack" };
	}

	public String[] getModelAnswerNames() {
		return getStructureNames();
	}

	
	public FDT[] getAnswer() {
	    return new FDT[] {table, auxTable, closest, areaTable, stack};
    }

	public FDT[] getInitialStructures() {
	    SpatialElementTable t = new SpatialElementTable(POINT_COUNT);
        return new FDT[] {t, new SpatialElementTable(POINT_COUNT), t, new SpatialElementTable(2)};
    }

	public String getDescription() {
	    return "";
		//return "Find the closest pair of points";
    }

	public String[] getStructureVisualisations() {
		return new String[] {"array", "array", "array", "2D", "list"};
	}

	public String[] getModelAnswerVisualisations() {
        //return new String[] {"array", "array", "list", "2D", "array"};
		return getStructureVisualisations();
	}

	public VisualTypeConf[] conf() {
		VisualTypeConf input = new VisualTypeConf();
        input.setVisualConfValue("matrix.visual.VisualArray", VisualTypeConf.RANGE_SELECTION, "true");
		input.enable("matrix.visual.VisualArrayComponent", VisualTypeConf.DROP_OPERATION);
		input.enable("matrix.visual.VisualArrayComponent", VisualTypeConf.DRAG_OPERATION);
		input.enable("matrix.visual.VisualArrayComponent", VisualTypeConf.HIGHLIGHT_OPERATION);
		VisualTypeConf aux = new VisualTypeConf();
		aux.enable("matrix.visual.VisualArrayComponent", VisualTypeConf.DROP_OPERATION);
		aux.enable("matrix.visual.VisualArrayComponent", VisualTypeConf.DRAG_OPERATION);
		aux.enable("matrix.visual.VisualArrayComponent", VisualTypeConf.HIGHLIGHT_OPERATION);
		aux.enable("matrix.visual.VisualStruct-matrix.structures.spatial.FDT.probe.Point", VisualTypeConf.MINIMIZED);
		VisualTypeConf area = new VisualTypeConf();
		area.enable("matrix.visual.VisualArea$AreaVisualization", VisualTypeConf.HIGHLIGHT_OPERATION);
		//area.enable("matrix.visual.VisualArea$AreaVisualization", VisualTypeConf.DRAG_OPERATION);
		VisualTypeConf minpair = new VisualTypeConf();
		minpair.enable("matrix.visual.VisualArrayComponent", VisualTypeConf.DROP_OPERATION);
		minpair.enable("matrix.visual.VisualArrayComponent", VisualTypeConf.DRAG_OPERATION);
		minpair.enable("matrix.visual.VisualArrayComponent", VisualTypeConf.HIGHLIGHT_OPERATION);
		VisualTypeConf stack = new VisualTypeConf();
		stack.setVisualConfValue("matrix.visual.VisualList", VisualTypeConf.ROTATED, "true");
		stack.setVisualConfValue("matrix.visual.VisualList", VisualTypeConf.FLIPPEDY, "true");
		return new VisualTypeConf[] {input, aux, minpair, area, stack};
	}

	private String dist = "Distance: ";
	private String mdist = "Min. distance: ";
	private String move = "Move";
	private String call = "Call";
	private String ret = "Return";
	private transient JLabel distLabel, mdistLabel;

	private void setButtonStrings() {
		ExerciseProperties lang = ExerciseProperties.getInstance();
		dist = lang.get("CLOSESTPAIROFPOINTS_DIST_STRING");
		mdist = lang.get("CLOSESTPAIROFPOINTS_MINDIST_STRING");
		move = lang.get("CLOSESTPAIROFPOINTS_MOVE_BUTTON");
		call = lang.get("CLOSESTPAIROFPOINTS_CALL_BUTTON");
		ret = lang.get("CLOSESTPAIROFPOINTS_RET_BUTTON");
	}
	
    public Component[] getAWTComponents() {
    	setButtonStrings();
    	GridBagLayout gbl = new GridBagLayout();
        JPanel p = new JPanel(gbl);
        distLabel = new JLabel(dist + " N/A");
        mdistLabel = new JLabel(mdist + " " + MAXX);
        JButton moveButton = new JButton(move);
        JButton callButton = new JButton(call);
        JButton retButton = new JButton(ret);
        moveButton.setActionCommand(move);
        callButton.setActionCommand(call);
        retButton.setActionCommand(ret);
        moveButton.addActionListener(this);
        callButton.addActionListener(this);
        retButton.addActionListener(this);
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.gridwidth = GridBagConstraints.REMAINDER;
        p.add(distLabel, gbc);
        p.add(mdistLabel, gbc);
        p.add(moveButton, gbc);
        p.add(callButton, gbc);
        p.add(retButton, gbc);
        return new Component[] {p};
    }

    public GridBagConstraints[] getAWTComponentConstraints() {
        return null;
    }
    
    /* ComparableExercise interface */
    
	public int[] getCompareIndices(){
        return new int[] {0, 1, 2, 3, 4};	
	}

	public int[] getGradeIndices(){
        return null; // Used only in recovery mode
	}

	public boolean canRecover(){
		return false;
	}
	
	public String[] getCompareStructureVisualisations(){
		return null;
	}
	
	public String[] getCompareStructureNames(){
		return null;
	}
	
	public VisualTypeConf[] getCompareVisualTypeConf(){
		return null;
	}


    public void setApplication(Application a) {
	    app = a;
	}

	public Application getApplication()  {
		return app;
	}
	
	
	public void actionPerformed(ActionEvent e) {
		Note.out(this, "action with event " + e.getActionCommand());
		Animator a = Animator.getActiveAnimator();
		if(move.equals(e.getActionCommand())) {
			a.startOperation();			
			double middle = MAXX/2;
			if(!stack.ADTIsEmpty()) {
				StackElement elem = (StackElement)stack.ADTPeek();
				/*
				String s = elem.getString();
				int tmp = s.indexOf(',');
				int tmp2 = s.indexOf(']');
				int start = Integer.parseInt(s.substring(1,tmp));
				int end = Integer.parseInt(s.substring(tmp+1,tmp2));
				*/
				middle = (int)elem.getMiddle().getX();
				Note.out(this, "Midpoint set to " + middle);
				midPoint.setMem(0,new Integer((int)middle));
			}
			for(int i = 0; i < table.size(); i++) {
				if(auxTable.getObject(i) != null) {
					table.setObject(auxTable.getObject(i),i);
					auxTable.setObject(null, i);
				}
			}
            double md = ((Double)minimumDistance.getObject()).doubleValue();
            double dist = md;
            if(md > MAXX) { 
            	md = middle;
            	dist = MAXX/2;
            }
            Rectangle area = new Rectangle(middle-md, 0, dist*2, MAXY+10);
            areaTable.setObject(area, POINT_COUNT);
			a.endOperation();
		} else if(call.equals(e.getActionCommand())) {
			a.startOperation();
			int start = table.selectionStart();
			int end = table.selectionEnd();
			int middle = (start+end)/2;
			Point mid = (Point)table.getObject(middle);
			stack.ADTPush(new StackElement("["+start+","+end+"]", mid));
			a.endOperation();
		} else if(ret.equals(e.getActionCommand())) {
			if(!stack.ADTIsEmpty()) {
				a.startOperation();
				stack.ADTPop();
				if(!stack.ADTIsEmpty()) {
					String s = ((StackElement)stack.ADTPeek()).toString();
					int tmp = s.indexOf(',');
					int tmp2 = s.indexOf(']');
					int start = Integer.parseInt(s.substring(1,tmp));
					int end = Integer.parseInt(s.substring(tmp+1,tmp2));
					table.selectIndex(start);
					if(!table.isSelection()) {
						table.selectIndex(start);					
					}
					table.selectIndex(end);
					if(!table.isSelection()) {
						table.selectIndex(end);					
					}
				} else {
					if(table.isSelection()) {
						int start = table.selectionStart();
						table.selectIndex(start);
					}
				}
				int size = areaTable.size();
				for(int i = POINT_COUNT; i < size; i++) {
					areaTable.setObject(null,i);
				}
				areaTable.selectRange(0,POINT_COUNT);
				a.endOperation();
			}
		}
		getApplication().validate();
	}

	public GridBagConstraints[] getConstraints() {
	    Insets i = new Insets(10,10,10,10);
        GridBagConstraints[] gbc = new GridBagConstraints[5];
        GridBagConstraints c = new GridBagConstraints();
        c.insets = i;
        c.gridwidth = GridBagConstraints.REMAINDER;
        gbc[0] = c;
        gbc[2] = c;
        gbc[4] = c;
        
        c = new GridBagConstraints();
        c.insets = i;
        c.gridwidth = GridBagConstraints.RELATIVE;
        gbc[1] = c;
        gbc[3] = c;
        return gbc;	
	}  

	public GridBagConstraints[] getModelAnswerConstraints() {
	    return getConstraints();
    }
	
	public GridBagConstraints[] getComparisonConstraints() {
		return null; // use default
	}

	class SelectablePoint extends Point implements Selectable {

		/**
		 * 
		 */
		private static final long serialVersionUID = -6884447772638686014L;
		private long stamp;
		
		public SelectablePoint(double x, double y, String key) {
			super(x,y,key);
			stamp = Selectable.NOT_SELECTED;
		}
		
		public void setSelect(long globalClockTimestamp) {
			// there has been selection of two elements already, clear them
			SelectablePoint old1, old2;
			old1 = (SelectablePoint)select1.getMem(0);
			old2 = (SelectablePoint)select2.getMem(0);
			// first case, this point has been selected
			if(stamp != Selectable.NOT_SELECTED) {
				if(old1 == this) {
					select1.setMem(0,null);	
				} else if(old2 == this) {
					select2.setMem(0,null);
				} else {
					// Error, shouldn't happen
					Note.err(this, "Marked as selected, but not in select1 or select2");
				}
				stamp = Selectable.NOT_SELECTED;				
				return;
			} 
			// second case, both points have been selected but this one hasn't; this shouldn't happen in current scheme
			if(old1 != null &&  old2 != null) {
				Note.err(this, "oops");
				/*
				old1.setStamp(Selectable.NOT_SELECTED);
				old2.setStamp(Selectable.NOT_SELECTED);
				select2.setMem(0,null);
				select1.setMem(0,this);
				*/
			} else if(old1 == null) { // first point not selected
				select1.setMem(0, this);
				old1 = this;
			} else if(old2 == null) { // second point not selected
				select2.setMem(0, this);
				old2 = this;
			}
			stamp = globalClockTimestamp;
			// old1 and old2 are reused here for calculating distance between them
			if(old1 != null && old2 != null) {
				Animator a = Animator.getActiveAnimator();
				a.startOperation();
				// this switch needs to be done in order for the grading to work
				// since model answer always draws the lines beginning from the point
				// with greater Y value
				if(old1.getY() < old2.getY()) {
					SelectablePoint tmp = old1;
					old1 = old2;
					old2 = tmp;
				}
				PolyLine pol = new PolyLine(new Point[] {old1,old2}, "" + old1.getElement() + "-" + old2.getElement());
				areaTable.setObject(pol,areaTable.size());
				double distance = Math.sqrt((old1.getX()-old2.getX())*(old1.getX()-old2.getX())+(old1.getY()-old2.getY())*(old1.getY()-old2.getY()));
				String dString = "" + distance;
				int dot = dString.indexOf('.');
				distLabel.setText(dist+" "+dString.substring(0,dot+2));
				old1.setStamp(Selectable.NOT_SELECTED);
				old2.setStamp(Selectable.NOT_SELECTED);
				select2.setMem(0,null);
				select1.setMem(0,null);
				a.endOperation();
			}
		}

		public void setStamp(long tstamp) {
			stamp = tstamp;
		}
		
		public long getSelectionTime() {
			return stamp;
		}
		
	}

	class ModelTable extends SpatialElementTable implements StyledArray {
		
		public ModelTable(int size) {
			super(size);
		}

		public String getIndexName(int address) {
			if(address == 0) {
				return "cp1";
			}
			if(address == 1) {
				return "cp2";
			}
			return "" + address;
		}
		
	}
	
	class ExerciseTable extends SpatialElementTable implements StyledArray {
		
		/**
		 * 
		 */
		private static final long serialVersionUID = -5022840774775260082L;

		public ExerciseTable(int size) {
			super(size);
		}

		// needed in order to avoid NullPointerException during intialization
		public void randomize() {
			return;
		}
		
		public void setObject(Object o, int adress) {
			super.setObject(o,adress);
			Object o1, o2;
			o1 = getObject(0);
			o2 = getObject(1);
			if(o instanceof Point && o1 instanceof Point && o2 instanceof Point) {
				Point p1, p2;
				p1 = (Point)o1;
				p2 = (Point)o2;
				double distance = Math.sqrt((p1.getX()-p2.getX())*(p1.getX()-p2.getX())+(p1.getY()-p2.getY())*(p1.getY()-p2.getY()));
				if(minimumDistance != null) {
					double oldDist = ((Double)minimumDistance.getObject()).doubleValue();
					minimumDistance.setMem(0,new Double(distance));
					String dString = "" + distance;
					int dot = dString.indexOf('.');
					mdistLabel.setText(mdist + " " + dString.substring(0,dot+2));
					// if the area to be searched is shown
					if(areaTable.size() > POINT_COUNT) {
						// case of no shortest pair candidate existing
						if(oldDist > MAXX) {
							if(stack.ADTIsEmpty()) {
								Note.out(this, "stack empty");
								return;
							}
							String s = stack.ADTPeek().toString();
							int tmp = s.indexOf(',');
							int tmp2 = s.indexOf(']');
							int start = Integer.parseInt(s.substring(1,tmp));
							int end = Integer.parseInt(s.substring(tmp+1,tmp2));
							if(end-start > 1) {
								Note.out(this, "not on deepest level of recursion");
								return;
							}
							Point pi1 = (Point)table.getObject(start);
							Point pi2 = (Point)table.getObject(end);
							if(pi1.getX() < pi2.getX()) {
								oldDist = pi1.getX();
							} else {
								oldDist = pi2.getX();
							}
						}
						//Rectangle old = (Rectangle)areaTable.getObject(POINT_COUNT);
						//double x = old.getX();
						double middle = ((Integer)midPoint.getMem(0)).doubleValue();
						areaTable.setObject(new Rectangle(middle-distance, 0, distance*2, MAXY+10), POINT_COUNT);
					}
				}
			}
		}

		public String getIndexName(int address) {
			if(address == 0) {
				return "cp1";
			}
			if(address == 1) {
				return "cp2";
			}
			return "" + address;
		}
		
	}

	private class StackElement implements Serializable {

		/**
		 * 
		 */
		private static final long serialVersionUID = -6941524525150850117L;
		private String value;
		private Point mid;
		
		public StackElement(String v, Point m) {
			value = v;
			mid = m;
		}
		
		public Point getMiddle() {
			return mid;
		}
	
		public String getString() {
			return value;
		}

		public String toString() {
			return value;
		}
		
		public boolean equals(Object o) {
			if(o instanceof StackElement) {
				StackElement other = (StackElement)o;
				return value.equals(other.getString()) && mid.equals(other.getMiddle());
			}
			return false;
		}
		
	}
	
}
