package content.exercises.sda;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import java.awt.geom.QuadCurve2D;
import java.util.Comparator;
import java.util.Iterator;
import java.util.Random;
import java.util.Vector;

import javax.swing.JButton;

import matrix.animation.Animator;
import matrix.simulation.VisualTypeConf;
import matrix.structures.CDT.CDT;
import matrix.structures.FDT.Array;
import matrix.structures.FDT.FDT;
import matrix.structures.FDT.Struct;
import matrix.structures.memory.VirtualBoolean;
import matrix.structures.memory.VirtualDouble;
import matrix.structures.memory.VirtualObject;
import matrix.structures.simulationextensions.DeleteEnabled;
import matrix.structures.simulationextensions.Selectable;
import matrix.structures.spatial.Area;
import matrix.structures.spatial.PaintingStyleDecorator;
import matrix.structures.spatial.SpatialComparable;
import matrix.structures.spatial.FDT.probe.Circle;
import matrix.structures.spatial.FDT.probe.LineSegment;
import matrix.structures.spatial.FDT.probe.Parabola;
import matrix.structures.spatial.FDT.probe.Point;
import matrix.structures.spatial.FDT.probe.Polygon;
import matrix.structures.spatial.FDT.probe.SpatialElement;
import matrix.structures.spatial.FDT.probe.Triangle;
import matrix.util.Application;
import matrix.util.Note;
import content.ExerciseProperties;
import content.exercises.sda.structures.GeometryFunctions;
import content.exercises.sda.structures.PointGenerator;
import content.exercises.sda.structures.SimpleStack;
import content.exercises.sda.structures.SinglyLinkedList;
import content.exercises.sda.structures.SortedList;
import content.exercises.sda.structures.Triangulation;
import content.exercises.sda.structures.PointGenerator.IllegalConstraintsException;
import content.exercises.sda.structures.Triangulation.VisualElements;
import content.interfaces.AWTComponentUtilizer;
import content.interfaces.ComparableExercise;
import content.interfaces.ConfigureVisualType;
import content.interfaces.Exercise;
import content.interfaces.LayoutExercise;
import content.interfaces.ModelAnswerNames;
import content.interfaces.SimulationExercise;
import content.interfaces.SimulationExerciseModel;
import content.interfaces.StyledExercise;

/**
 * @author Juha Helminen
 *
 */
public class Fortune implements Exercise, SimulationExercise,
                                SimulationExerciseModel, StyledExercise,
                                ConfigureVisualType, ModelAnswerNames,
                                AWTComponentUtilizer, LayoutExercise,
                                ComparableExercise/*,
                                SimulationExerciseSelfAssessment*/ {
    
    // For debugging
    private static final boolean STATIC_SEED = false;
    private static final long STATIC_SEED_VALUE = 1190740486499L;
    // 1190714298377
    // 1187170616842
    // 1190710611744
    // 1190711937499
    
    private static final long SELECTED = Selectable.clock.getNextTimeStamp();
    
    // The number of points.
    private static final int INPUT_SIZE = 12;
    // The minimum allowed distance between two points.
    private static final int MIN_DISTANCE = 80;
    // The minimum allowed distance from the points to the border.
    private static final int BORDER_X = 70;
    private static final int BORDER_Y = 75;
    // The size of the exercise Area.
    private static final int WIDTH = 450;
    private static final int HEIGHT = 350;
    
    private static final int MAX_X = WIDTH - BORDER_X / 2 - 15;

    private long seed = System.currentTimeMillis();
    
    // ** Color and style scheme **

    private static final Rectangle BOUNDING_BOX_NONE = new Rectangle(0,0,0,0);
    protected static final Color PARABOLA_COLOR = Color.BLUE;
    protected static final Color ARC_COLOR = Color.CYAN;
    protected static final Color SELECTED_ARC_COLOR = Color.GREEN;
    protected static final Color SWEEPLINE_COLOR = Color.YELLOW;
    protected static final Color POINT_COLOR = Color.BLACK;
    protected static final Color VORONOI_VERTEX_COLOR = Color.GRAY;
    protected static final Color DELAUNAY_POINT_COLOR = Color.WHITE;
    protected static final Color VORONOI_COLOR = Color.GRAY;
    protected static final Color VORONOI_FILL_COLOR = Color.BLACK;
    protected static final double VORONOI_TRANSPARENCY = 0;
    protected static final Color DELAUNAY_COLOR = Color.WHITE;
    protected static final Color DELAUNAY_FILL_COLOR = Color.BLACK;
    protected static final double DELAUNAY_TRANSPARENCY = 0;
    protected static final Color MISSING_DELAUNAY_COLOR = Color.WHITE;
    protected static final Color WRONG_DELAUNAY_COLOR = Color.RED;
    protected static final Color BISECTOR_COLOR = Color.RED;
    protected static final Color CURRENT_CIRCLE_COLOR = Color.MAGENTA;
    protected static final Color CURRENT_CIRCLE_FILL_COLOR = Color.MAGENTA;
    protected static final Color CIRCLE_COLOR = Color.RED;
    protected static final Color CIRCLE_FILL_COLOR = Color.RED;
    protected static final Color CIRCLE_CENTER_COLOR = new Color(100, 100, 150);
    protected static final Color CIRCLE_EVENT_COLOR = Color.MAGENTA;
    protected static final double CIRCLE_TRANSPARENCY = 0.035;
    protected static final Color TRIANGLE_FILL_COLOR = Color.WHITE;
    protected static final double TRIANGLE_TRANSPARENCY = 0.12;
    protected static final Color TEST_LINE_COLOR = Color.CYAN;
    protected static final PaintingStyleDecorator vertexDecorator;
    protected static final PaintingStyleDecorator voronoiVertexDecorator;
    protected static final PaintingStyleDecorator invisiblePointDecorator;
    protected static final PaintingStyleDecorator voronoiDecorator;
    protected static final PaintingStyleDecorator delaunayDecorator;
    protected static final PaintingStyleDecorator triangleDecorator;
    protected static final PaintingStyleDecorator beachLineDecorator;
    protected static final PaintingStyleDecorator sweeplineDecorator;
    protected static final PaintingStyleDecorator circleDecorator;
    protected static final PaintingStyleDecorator currentCircleDecorator;
    protected static final PaintingStyleDecorator circleCenterDecorator;
    protected static final PaintingStyleDecorator circleEventDecorator;
    protected static final PaintingStyleDecorator testLineDecorator;
    protected static final PaintingStyleDecorator arcDecorator;
    protected static final PaintingStyleDecorator selectedArcDecorator;
    static {
        invisiblePointDecorator = new PaintingStyleDecorator();
        invisiblePointDecorator.setOutline(false);
        
        vertexDecorator = new PaintingStyleDecorator();
        vertexDecorator.setLineColor(POINT_COLOR);
        vertexDecorator.setStroke(1, BasicStroke.CAP_ROUND, BasicStroke.JOIN_BEVEL);

        voronoiVertexDecorator = new PaintingStyleDecorator();
        voronoiVertexDecorator.setLineColor(VORONOI_VERTEX_COLOR);
        voronoiVertexDecorator.setStroke(2, BasicStroke.CAP_ROUND, BasicStroke.JOIN_BEVEL);

        voronoiDecorator = new PaintingStyleDecorator();
        voronoiDecorator.setLineColor(VORONOI_COLOR);
        voronoiDecorator.setFillColor(VORONOI_FILL_COLOR);
        voronoiDecorator.setTransparency(VORONOI_TRANSPARENCY);
        voronoiDecorator.setStroke(2, BasicStroke.CAP_ROUND, BasicStroke.JOIN_BEVEL);

        delaunayDecorator = new PaintingStyleDecorator();
        delaunayDecorator.setLineColor(DELAUNAY_COLOR);
        delaunayDecorator.setFillColor(DELAUNAY_FILL_COLOR);
        delaunayDecorator.setTransparency(DELAUNAY_TRANSPARENCY);
        delaunayDecorator.setStroke(2, BasicStroke.CAP_ROUND, BasicStroke.JOIN_BEVEL);

        float[] testLineDashPattern = { 5, 5 };
        testLineDecorator = new PaintingStyleDecorator();
        testLineDecorator.setLineColor(TEST_LINE_COLOR);
        testLineDecorator.setStroke(1, BasicStroke.CAP_ROUND, BasicStroke.JOIN_BEVEL, 10, testLineDashPattern, 0);

        
        float[] circumcircleDashPattern = { 5, 5 };
        circleDecorator = new PaintingStyleDecorator();
        circleDecorator.setLineColor(CIRCLE_COLOR);
        circleDecorator.setFillColor(CIRCLE_FILL_COLOR);
        circleDecorator.setTransparency(CIRCLE_TRANSPARENCY);
        circleDecorator.setStroke(1, BasicStroke.CAP_ROUND, BasicStroke.JOIN_BEVEL, 10, circumcircleDashPattern, 0);

        float[] currentCircleDashPattern = { 5, 5 };
        currentCircleDecorator = new PaintingStyleDecorator();
        currentCircleDecorator.setLineColor(CURRENT_CIRCLE_COLOR);
        currentCircleDecorator.setFillColor(CURRENT_CIRCLE_FILL_COLOR);
        currentCircleDecorator.setTransparency(CIRCLE_TRANSPARENCY);
        currentCircleDecorator.setStroke(1, BasicStroke.CAP_ROUND, BasicStroke.JOIN_BEVEL, 10, currentCircleDashPattern, 0);

        circleCenterDecorator = new PaintingStyleDecorator();
        circleCenterDecorator.setLineColor(CIRCLE_CENTER_COLOR);

        circleEventDecorator = new PaintingStyleDecorator();
        circleEventDecorator.setLineColor(CIRCLE_EVENT_COLOR);
        
        triangleDecorator = new PaintingStyleDecorator();
        triangleDecorator.setFillColor(TRIANGLE_FILL_COLOR);
        triangleDecorator.setTransparency(TRIANGLE_TRANSPARENCY);
        triangleDecorator.setOutline(false);

        beachLineDecorator = new PaintingStyleDecorator();
        beachLineDecorator.setLineColor(PARABOLA_COLOR);
        beachLineDecorator.setFill(false);
        beachLineDecorator.setStroke(2, BasicStroke.CAP_ROUND, BasicStroke.JOIN_BEVEL);

        arcDecorator = new PaintingStyleDecorator();
        arcDecorator.setLineColor(ARC_COLOR);
        arcDecorator.setFill(false);
        arcDecorator.setStroke(2, BasicStroke.CAP_ROUND, BasicStroke.JOIN_BEVEL);

        selectedArcDecorator = new PaintingStyleDecorator();
        selectedArcDecorator.setLineColor(SELECTED_ARC_COLOR);
        selectedArcDecorator.setFill(false);
        selectedArcDecorator.setStroke(2, BasicStroke.CAP_ROUND, BasicStroke.JOIN_BEVEL);

        float[] sweeplineDecoratorDashPattern = { 10, 5, 5, 5 };
        sweeplineDecorator = new PaintingStyleDecorator();
        sweeplineDecorator.setLineColor(SWEEPLINE_COLOR);
        sweeplineDecorator.setStroke(2, BasicStroke.CAP_ROUND, BasicStroke.JOIN_BEVEL, 10, sweeplineDecoratorDashPattern, 0);
    }
    
    protected class TriangulationVisualElements extends VisualElements {

        protected Point newVertex(java.awt.Point vertex) {
            return new Point(vertex, vertexDecorator);
        }

        protected LineSegment newDelaunayEdge(java.awt.Point orig, java.awt.Point dest) {
            return new LineSegment(orig, dest, delaunayDecorator);        
        }
        
        protected Triangle newTriangle(java.awt.Point[] vertices) {
            return new Triangle(vertices, triangleDecorator);
        }
        
        protected Point newVoronoiVertex(java.awt.Point vertex) {
            return new Point(vertex, voronoiVertexDecorator);
        }
        
        protected LineSegment newVoronoiEdge(java.awt.Point orig, java.awt.Point dest) {
            return new LineSegment(orig, dest, voronoiDecorator);        
        }
        
        protected Polygon newVoronoiRegion(java.awt.Point[] vertices) {
            return null;
        }

        protected void hideDelaunay() {}
        
        protected void showDelaunay() {}
        
        protected void hideVoronoi() {}

        private static final long serialVersionUID = 4482392082455412991L;
    }

    
    private java.awt.Point[] input;
    private Triangulation triangulation;
    private VirtualDouble sweepLineX;
    private EventQueue eventQueue;
    private BeachLine beachLine; 
    private Slot currentEvent;
    private Slot sweepLine;
    
    public FDT[] getInitialStructures() {
        SimpleStack problemVis = new SimpleStack(new Rectangle(WIDTH, HEIGHT, 0, 0));
        return new FDT[]{ problemVis };
    }
    
    public FDT[] init() {
        // For debugging
        Note.out(this, "The random number generator seed for this exercise is: " + this.getSeed());
        Random r = new Random(this.getSeed());
        
        // Generate the exercise input.
        final int BORDER_POINT_COUNT = 8;
        final int MAX_TRIES = 5000;
        int tryCounter = 0;
        java.awt.Point[] randomInput;
        while (true) {
            try {
                randomInput = PointGenerator.generatePoints(BORDER_X, BORDER_Y,
                                                            WIDTH - BORDER_X, HEIGHT - BORDER_Y,
                                                            INPUT_SIZE - BORDER_POINT_COUNT, MIN_DISTANCE, r);
            } catch (IllegalConstraintsException e) {
                Note.err(this, e.getMessage());
                Note.show(this, "Could not generate input for the exercise. Please restart the exercise.");
                return null;
            }
    
            this.input = new java.awt.Point[randomInput.length + BORDER_POINT_COUNT];
            for (int i = 0; i < randomInput.length; i++) {
                this.input[i] = randomInput[i];
            }
            this.input[randomInput.length] = new java.awt.Point(BORDER_X / 2 + 50, BORDER_Y / 2);
            this.input[randomInput.length + 1] = new java.awt.Point(BORDER_X / 2 + 15, HEIGHT - BORDER_Y / 2);
            this.input[randomInput.length + 2] = new java.awt.Point(WIDTH - BORDER_X / 2 - 50, HEIGHT - BORDER_Y / 2);
            this.input[randomInput.length + 3] = new java.awt.Point(MAX_X, BORDER_Y / 2);
            this.input[randomInput.length + 4] = new java.awt.Point(WIDTH / 2 + 50, BORDER_Y / 3);
            this.input[randomInput.length + 5] = new java.awt.Point(BORDER_X / 3, HEIGHT / 2);
            this.input[randomInput.length + 6] = new java.awt.Point(WIDTH - BORDER_X / 3, HEIGHT / 2);
            this.input[randomInput.length + 7] = new java.awt.Point(WIDTH / 2 - 50, HEIGHT - BORDER_Y / 3);

            if (GeometryFunctions.delaunay(this.input) == null) {
                if (tryCounter++ >= MAX_TRIES) {
                    Note.err(this, "Could not generate input for the exercise.");
                    Note.show(this, "Could not generate input for the exercise. Please restart the exercise.");
                    return null;
                }
            } else {
                break;
            }
        }
        
        FDT[] structures = this.getInitialStructures();
        SimpleStack problemVis = (SimpleStack)structures[0];
        
        this.triangulation = new Triangulation(new TriangulationVisualElements(), WIDTH, HEIGHT);
        this.eventQueue = new EventQueue(new EventComparator());
        for (int i = 0; i < this.input.length; i++) {
            this.triangulation.addVertex(this.input[i]);
            this.eventQueue.insert(new SiteEvent(this.input[i]));
        }
        Iterator iterator = this.eventQueue.iterator();
        SiteEvent siteEvent;
        int k = 1;
        while (iterator.hasNext()) {
            siteEvent = (SiteEvent)iterator.next();
            siteEvent.createVisualization(k++);
        }
        problemVis.addSubArea(this.triangulation);
        problemVis.addSubArea(this.eventQueue);

        this.triangulation.setDrawVoronoi(Triangulation.VORONOI_NONE);
        this.triangulation.setDrawTrianglesVoronoi(false);
        this.triangulation.setDrawVoronoiVertices(false);

        this.sweepLineX = new VirtualDouble(0, null, "sweepLineX");
        this.sweepLine = new Slot(new LineSegment((int)this.sweepLineX.eval(), 0, 
                                                  (int)this.sweepLineX.eval(), HEIGHT, 
                                                  sweeplineDecorator));
        problemVis.addSubArea(this.sweepLine);
        
        this.currentEvent = new Slot();
        problemVis.addSubArea(this.currentEvent);
        this.beachLine = new BeachLine(this.sweepLineX, 
                                       this.eventQueue, 
                                       this.triangulation);
        problemVis.addSubArea(this.beachLine);

        return new FDT[]{ problemVis, this.eventQueue, this.beachLine, this.currentEvent };
    }
    
    public String[] getStructureVisualisations() {
        return new String[]{"2D", "array", "array", "array"};
    }

    public String[] getStructureNames() {
        // TODO
    	ExerciseProperties lang = ExerciseProperties.getInstance();
    	String[] s = new String[] { lang.get("FORTUNE_AREA"), lang.get("FORTUNE_EVENT_QUEUE_TABLE"),
    			lang.get("FORTUNE_BEACHLINE_TABLE"), lang.get("FORTUNE_CURRENT_EVENT_TABLE")
    	};
    	return s;
        //return new String[]{ "", "Event queue", "Beach line", "Current event" };
    }

    public String getDescription() {
        return "";
    }

    public FDT[] getAnswer() {
        return new FDT[]{this.beachLine};
    }

    public FDT[] makeModelAnswer() {
        FDT[] modelStructures = this.getInitialStructures();

        Triangulation modelTriangulation = new Triangulation(new TriangulationVisualElements(), 
                                                             WIDTH, 
                                                             HEIGHT);
        modelTriangulation.setDrawVoronoi(Triangulation.VORONOI_NONE);
        modelTriangulation.setDrawVoronoiVertices(false);
        modelTriangulation.setDrawDelaunay(true);
        for (int i = 0; i < this.input.length; i++) {
            modelTriangulation.addVertex(this.input[i]);
        }
        SimpleStack problemVisualization = ((SimpleStack)modelStructures[0]);
        problemVisualization.addSubArea(modelTriangulation);
        
        VirtualDouble sweepLineX = new VirtualDouble(0, null, "sweepLineX");
        Slot sweepLine = new Slot(
                new LineSegment((int)sweepLineX.eval(), 0, (int)sweepLineX.eval(), HEIGHT, sweeplineDecorator));
        problemVisualization.addSubArea(sweepLine);
        
        EventQueue eventQueue = new EventQueue(new EventComparator());
        problemVisualization.addSubArea(eventQueue);
        // Populate the event queue with the site events.
        for (int i = 0; i < this.input.length; i++) {
            eventQueue.insert(new SiteEvent(this.input[i]));
        }
        Iterator iterator = eventQueue.iterator();
        SiteEvent siteEvent;
        int k = 1;
        while (iterator.hasNext()) {
            siteEvent = (SiteEvent)iterator.next();
            siteEvent.createVisualization(k++);
        }
        
        BeachLine beachLine = new BeachLine(sweepLineX, eventQueue, modelTriangulation);
        problemVisualization.addSubArea(beachLine);
        
        Slot currentEvent = new Slot();
        problemVisualization.addSubArea(currentEvent);

        Animator a = Animator.getActiveAnimator();
        Event event;
//        SiteEvent siteEvent;
        CircleEvent circleEvent;
        while (!eventQueue.isEmpty()) {
            a.startOperation();
            beachLine.clearSelections();
            eventQueue.clearSelections();
            event = (Event)eventQueue.removeFirst();
            currentEvent.setElement(event);
            sweepLineX.assign(event.getX());
            sweepLine.setElement(
                    new LineSegment((int)sweepLineX.eval(), 0, (int)sweepLineX.eval(), HEIGHT, sweeplineDecorator));
            if (event instanceof SiteEvent) {
                siteEvent = (SiteEvent)event;
                beachLine.insert(siteEvent);
                modelTriangulation.setVertexPaintingStyledecorator(siteEvent.getSite(), vertexDecorator);
            } else /*if (events instanceof CircleEvent)*/ {
                circleEvent = (CircleEvent)event;
                modelTriangulation.addEdge(circleEvent.getSite(0), circleEvent.getSite(2));
                beachLine.delete(circleEvent);
            }
            a.endOperation();
        }
        
        return new FDT[]{ modelStructures[0], eventQueue, beachLine };
    }
    
    public FDT[] solve() {
        return new FDT[]{this.makeModelAnswer()[2]};
    }
    
    public String[] getModelAnswerVisualisations() {
        return new String[]{"2D", "array", "array"};
    }
    
    /* (non-Javadoc)
     * @see content.interfaces.SimulationExercise#getSeed()
     */
    public long getSeed() {
        // The seed might be set to a constant value for debugging purposes.
        if (STATIC_SEED) {
            return Fortune.STATIC_SEED_VALUE;
        } else {
            return this.seed;
        }
    }

    /* (non-Javadoc)
     * @see content.interfaces.SimulationExercise#setSeed(long)
     */
    public void setSeed(long seed) {
        this.seed = seed;
    }


    // ** Implementation of interface ConfigureVisualType **
    
    public VisualTypeConf[] conf() {
        VisualTypeConf sweep = new VisualTypeConf();
        VisualTypeConf eventQueue = new VisualTypeConf();
        VisualTypeConf beachLine = new VisualTypeConf();
        VisualTypeConf currentEvent = new VisualTypeConf();
        beachLine.setVisualConfValue("matrix.visual.VisualArray", VisualTypeConf.INDEXED, "false");
        beachLine.setVisualConfValue("matrix.visual.VisualArray", VisualTypeConf.ROTATED, "true");
//        beachLine.enable("matrix.visual.VisualStruct", VisualTypeConf.DRAG_OPERATION);
        beachLine.enable("matrix.visual.VisualKey", VisualTypeConf.HIGHLIGHT_OPERATION);
        beachLine.enable("matrix.visual.VisualStruct", VisualTypeConf.HIGHLIGHT_OPERATION);
        beachLine.enable("matrix.visual.VisualStruct-matrix.structures.FDT.FDT", 
                         VisualTypeConf.MINIMIZED);
        beachLine.enable("matrix.visual.VisualArrayComponent", VisualTypeConf.DROP_OPERATION);
        beachLine.enable("matrix.visual.VisualArrayComponent", VisualTypeConf.HIGHLIGHT_OPERATION);
        eventQueue.setVisualConfValue("matrix.visual.VisualArray", VisualTypeConf.INDEXED, "false");
        eventQueue.enable("matrix.visual.VisualStruct-matrix.structures.FDT.FDT", 
                          VisualTypeConf.MINIMIZED);
        currentEvent.setVisualConfValue("matrix.visual.VisualArray", VisualTypeConf.INDEXED, "false");
        currentEvent.enable("matrix.visual.VisualStruct", VisualTypeConf.DRAG_OPERATION);
        currentEvent.enable("matrix.visual.VisualStruct", VisualTypeConf.HIGHLIGHT_OPERATION);
        currentEvent.enable("matrix.visual.VisualStruct-matrix.structures.FDT.FDT", 
                             VisualTypeConf.MINIMIZED);
        currentEvent.enable("matrix.visual.VisualArrayComponent", VisualTypeConf.HIGHLIGHT_OPERATION);
//        stackTop.enable("matrix.visual.VisualArrayComponent", VisualTypeConf.DRAG_OPERATION);
//        queue.enable("matrix.visual.VisualList", VisualTypeConf.DROP_OPERATION);
//        queue.enable("matrix.visual.VisualList", VisualTypeConf.HIGHLIGHT_OPERATION);
//        queue.enable("matrix.visual.VisualStruct-matrix.structures.spatial.FDT.probe.Point", VisualTypeConf.MINIMIZED);
        return new VisualTypeConf[]{ sweep, eventQueue, beachLine, currentEvent };
    }
    
    public String[] getModelAnswerNames() {
    	ExerciseProperties lang = ExerciseProperties.getInstance();
    	String[] s = new String[] { lang.get("FORTUNE_MODEL_AREA"), lang.get("FORTUNE_EVENT_QUEUE_MODEL_TABLE"),
    			lang.get("FORTUNE_BEACHLINE_MODEL_TABLE")
    	};
    	return s;
        //return new String[]{"", "Event queue", "Beach line"};
    }
    
    /* ComparableExercise interface */
    
	public int[] getCompareIndices(){
        return new int[] {0};	
	}

	public int[] getGradeIndices(){
        return null; // Used only in recovery mode
	}

	public boolean canRecover(){
		return false;
	}
	
	public String[] getCompareStructureVisualisations(){
		return null;
	}
	
	public String[] getCompareStructureNames(){
		return null;
	}
	
	public VisualTypeConf[] getCompareVisualTypeConf(){
		return null;
	}

    

    // ** Implementation of interface AWTComponentUtilizer **

    private transient Application application;
    
    public void setApplication(Application a) {
        this.application = a; 
    }

    public Application getApplication() {
        return this.application; 
    }

    public Component[] getAWTComponents() {
    	ExerciseProperties lang = ExerciseProperties.getInstance();
        JButton nextEventButton = new JButton(lang.get("FORTUNE_EVENT_BUTTON"));
        nextEventButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                if (!Fortune.this.eventQueue.isEmpty()) {
                    Animator a = Animator.getActiveAnimator();
                    a.startOperation();
                    Fortune.this.eventQueue.clearSelections();
                    Fortune.this.beachLine.clearCircles();
                    Fortune.this.beachLine.clearSelections();
                    Event event = (Event)Fortune.this.eventQueue.removeFirst();
                    Fortune.this.currentEvent.setElement(event);
                    Fortune.this.sweepLineX.assign(event.getX());
                    Fortune.this.sweepLine.setElement(
                            new LineSegment((int)Fortune.this.sweepLineX.eval(), 0, 
                                            (int)Fortune.this.sweepLineX.eval(), HEIGHT, 
                                            sweeplineDecorator));
                    if (event instanceof SiteEvent) {
                        Fortune.this.beachLine.findArc(((SiteEvent)event).getSite());
                        Fortune.this.triangulation.setVertexPaintingStyledecorator(((SiteEvent)event).getSite(), vertexDecorator);
                    } else {
                        Fortune.this.beachLine.findArc(null);
                    }
                    a.endOperation();
                    Fortune.this.getApplication().validateAnimator();
                    Fortune.this.getApplication().validate();
                }
            }
        });
        return new Component[]{ nextEventButton };
    }

    public GridBagConstraints[] getAWTComponentConstraints() {
        GridBagConstraints nextEventButton = new GridBagConstraints();
        nextEventButton.insets = new Insets(3, 3, 3, 3);
        nextEventButton.gridy = 2;
        nextEventButton.gridx = 0;
        return new GridBagConstraints[]{ nextEventButton };
    }

    
    // ** Implementation of interface LayoutExercise **
    
    public GridBagConstraints[] getConstraints() {
        GridBagConstraints[] gbc = new GridBagConstraints[4];
        for (int i = 0; i < 4; i++) {
            gbc[i] = new GridBagConstraints();
        }
        gbc[0].insets = new Insets(3, 0, 3, 3);
        gbc[1].insets = new Insets(0, 0, 3, 3);
        gbc[2].insets = new Insets(3, 3, 3, 3);
        gbc[3].insets = new Insets(3, 3, 3, 3);
        gbc[2].gridy = 0;
        gbc[2].gridx = 0;
        gbc[0].gridy = 0;
        gbc[0].gridx = 1;
        gbc[0].gridwidth = 1;
        gbc[1].gridy = 1;
        gbc[1].gridx = 1;
        gbc[1].fill = GridBagConstraints.HORIZONTAL;
        gbc[1].gridwidth = 2;
        gbc[1].gridheight = 2;
        gbc[3].gridy = 1;
        gbc[3].gridx = 0;
//        gbc[0].gridwidth = 3;
//        gbc[3].fill = GridBagConstraints.HORIZONTAL;
        // problemVis, this.eventQueue, this.beachLine, this.currentEvent
        return new GridBagConstraints[]{ gbc[0], gbc[1], gbc[2], gbc[3] };
    }

    public GridBagConstraints[] getModelAnswerConstraints() {
        // TODO Auto-generated method stub
        // modelStructures[0], eventQueue, beachLine
        // return null;
        return getConstraints();
    }
    
	public GridBagConstraints[] getComparisonConstraints() {
		return null; // use default
	}
    
    
    private class EventComparator implements Comparator {
        public int compare(Object eventA, Object eventB) {
            Event a = (Event)eventA;
            Event b = (Event)eventB;
            if (a.getX() < b.getX()) {
                return -1;
            } else if (a.getX() > b.getX()) {
                return 1;
            } else {
                if (a.getY() < b.getY()) {
                    return -1;
                } else if ((a.getY() > b.getY())) {
                    return 1;
                } else {
                    return 0;
                }
            }
        }
    }
    
    private abstract class Event implements Struct {
        private final String visualization;
        
        public Event(String stringVisualization) {
            this.visualization = stringVisualization;
        }
        public abstract double getX();
        public abstract double getY();
        public abstract SpatialElement[] getSpatialElements();
        
        
        // ** Implementation of interface Struct **
        
        public Object getField(int location) {
            switch (location) {
                case 0:  return this.visualization;
                case 1:  return Integer.toString((int)Math.round(this.getX()));
                case 2:  return Integer.toString((int)Math.round(this.getY()));
                default: return null;
            }
        }
        
        public Object[] getFields() {
            return new Object[]{ this.getField(0), 
                                 this.getField(1),
                                 this.getField(2)};
        }
        
        public String getFieldName(int location) {
            switch (location) {
                case 0:  return "";
                case 1:  return "x";
                case 2:  return "y";
                default: return "";
            }
        }
        
        public String[] getFieldNames() {
            return new String[]{ this.getFieldName(0),
                                 this.getFieldName(1),
                                 this.getFieldName(2)};
        }

        public void setField(Object value, int location) {
            Note.warning(this, "Operation not implemented.");
        }
    }
    
    private class SiteEvent extends Event {

        private int id;
        private final java.awt.Point site;
        private Point siteVisualization;
        
        public SiteEvent(java.awt.Point site) {
            // Set the visualization string in the event queue.
            super(site.x + "," + site.y);
            this.site = site;
        }
        
        public void createVisualization(int id) {
            this.id = id;
            this.siteVisualization = new Point(site.x, site.y, Integer.toString(id), vertexDecorator);
        }
        
        public double getX() {
            return this.site.x;
        }
        
        public double getY() {
            return this.site.y;
        }

        public java.awt.Point getSite() {
            return this.site;
        }
        
        public int getID() {
            return this.id;
        }

        // ** Implementation of interface Area **
        
        public SpatialElement[] getSpatialElements() {
            return new SpatialElement[]{ this.siteVisualization };
        }

        public Rectangle getBoundingPolygon() {
            return BOUNDING_BOX_NONE;
        }
        
        public void insert(SpatialComparable e) {
            Note.warning(this, "Operation not implemented.");
        }

        public void delete(SpatialComparable e) {
            Note.warning(this, "Operation not implemented.");
        }
        
        public PaintingStyleDecorator getPaintingStyleDecorator() {
            Note.warning(this, "Operation not implemented.");
            return null;
        }
        
        public Area[] getSubAreas() {
            return new Area[0];
        }
        
        public boolean hasPaintingStyleDecorator() {
            return false;
        }
        
        public void setPaintingStyleDecorator(PaintingStyleDecorator psd) {
            Note.warning(this, "Operation not implemented.");
        }
        
        public void addSubArea(Area a) {
            Note.warning(this, "Operation not implemented.");
        }
        
        public void removeSubArea(Area a) {
            Note.warning(this, "Operation not implemented.");
        }

        public Object getElement() {
            Note.warning(this, "Operation not implemented.");
            return null;
        }
        
        public void setElement(Object o) {
            Note.warning(this, "Operation not implemented.");
        }
        
        private static final long serialVersionUID = 8522453768305713474L;
    }
    
    
    private class CircleEvent extends Event implements Selectable {
        
        private final SiteEvent site0;
        private final SiteEvent site1;
        private final SiteEvent site2;
        private final double centerX;
        private final double centerY;
        private final double radius;
        private final Point voronoiVertexEventVis;
        private VirtualBoolean selected;
        
        public CircleEvent(SiteEvent site0,
                           SiteEvent site1,
                           SiteEvent site2,
                           Ellipse2D circumcircle) {
//            super("(" + circumcircle.getMaxX() + "," + (int)circumcircle.getCenterY() + ") <- {" + 
//                  "(" + site0.x + "," + site0.y + ")," +
//                  "(" + site1.x + "," + site1.y + ")," +
//                  "(" + site2.x + "," + site2.y + ")}");
            super(site0.getID() + "-" + site1.getID() + "-" + site2.getID());
            this.site0 = site0;
            this.site1 = site1;
            this.site2 = site2;
            this.centerX  = circumcircle.getCenterX();
            this.centerY = circumcircle.getCenterY();
            this.radius = circumcircle.getMaxX() - circumcircle.getCenterX();
            this.voronoiVertexEventVis = new Point(this.getX(), this.getY(), circleEventDecorator);
            this.selected = new VirtualBoolean(true, this, "selected");
        }
        
        public double getX() {
            return (this.centerX + this.radius);
        }
        
        public double getY() {
            return this.centerY;
        }

        public double getCenterX() {
            return this.centerX;
        }
        
        public double getCenterY() {
            return this.centerY;
        }

        public double getRadius() {
            return this.radius;
        }
        
        public java.awt.Point getSite(int i) {
            switch (i) {
                case 0 : return this.site0.getSite();
                case 1 : return this.site1.getSite();
                case 2 : return this.site2.getSite();
                default : return null;
            }
        }

        // ** Implementation of interface Selectable **
        
        public long getSelectionTime() {
            if (this.selected.eval()) {
                return Selectable.clock.getNextTimeStamp();
            } else {
                return Selectable.NOT_SELECTED;
            }
        }
        
        public void setSelect(long globalClockTimestamp) {
            if (globalClockTimestamp == Selectable.NOT_SELECTED) {
                this.selected.assign(false);
            } else {
                Animator a = Animator.getActiveAnimator();
                a.startOperation();
                this.selected.toggle();
                a.endOperation();
            }
        }

        
        // ** Implementation of interface Area **
        
        public SpatialElement[] getSpatialElements() {
            return new SpatialElement[]{ this.voronoiVertexEventVis };
        }

        public Rectangle getBoundingPolygon() {
            return BOUNDING_BOX_NONE;
        }
        
        public void insert(SpatialComparable e) {
            Note.warning(this, "Operation not implemented.");
        }

        public void delete(SpatialComparable e) {
            Note.warning(this, "Operation not implemented.");
        }
        
        public PaintingStyleDecorator getPaintingStyleDecorator() {
            Note.warning(this, "Operation not implemented.");
            return null;
        }
        
        public Area[] getSubAreas() {
            return new Area[0];
        }
        
        public boolean hasPaintingStyleDecorator() {
            return false;
        }
        
        public void setPaintingStyleDecorator(PaintingStyleDecorator psd) {
            Note.warning(this, "Operation not implemented.");
        }
        
        public void addSubArea(Area a) {
            Note.warning(this, "Operation not implemented.");
        }
        
        public void removeSubArea(Area a) {
            Note.warning(this, "Operation not implemented.");
        }

        public Object getElement() {
            Note.warning(this, "Operation not implemented.");
            return null;
        }
        
        public void setElement(Object o) {
            Note.warning(this, "Operation not implemented.");
        }
        
        private static final long serialVersionUID = -1235692318800383564L;
    }
    
    
    // area visualization wrapper for a single element stored in a VirtualObject
    private class Slot implements Area, Array, DeleteEnabled, CDT {
        private final VirtualObject value;
        
        public Slot() {
            this(null);
        }
        
        public Slot(SpatialElement value) {
            this.value = new VirtualObject(value, this, "value");
        }

        // ** Implementation of the interface Area (visualization interface) **
        
        public void insert(SpatialComparable e) {
            Note.warning(this, "Operation not implemented.");
        }
        
        public void delete(SpatialComparable e) {
            if (this.getElement() == e) {
                this.clearElement();
            }
        }
        
        public Rectangle getBoundingPolygon() {
            return BOUNDING_BOX_NONE;
        }
        
        public Object getElement() {
            return this.value.getObject();
        }
        
        public void setElement(Object o) {
            if ((o instanceof SpatialElement) ||
                (o instanceof Area) ||
                (o instanceof Struct)) {
                this.value.setObject(o);
            }
        }
        
        public void clearElement() {
            if (this.getElement() instanceof SiteEvent) {
                Fortune.this.beachLine.findArc(null);
                Fortune.this.beachLine.clearSelections();
            }
            this.value.setObject(null);
        }

        public SpatialElement[] getSpatialElements() {
            if (this.value.getObject() instanceof SiteEvent) {
                SiteEvent siteEvent = (SiteEvent)this.value.getObject(); 
                return new SpatialElement[] {
                    new LineSegment(0, (int)siteEvent.getY(), 
                                       (int)siteEvent.getX(), (int)siteEvent.getY(),
                                       testLineDecorator) };
            } else if (this.value.getObject() instanceof CircleEvent) {
                CircleEvent circleEvent = (CircleEvent)this.value.getObject(); 
                return new SpatialElement[] {
                        circleEvent.getSpatialElements()[0],
                        new Circle(circleEvent.getCenterX(),
                                   circleEvent.getCenterY(),
                                   circleEvent.getRadius(),
                                   currentCircleDecorator)};
            } else if (this.value.getObject() instanceof SpatialElement) {
                return new SpatialElement[]{(SpatialElement)this.value.getObject()};
            } else if (this.value.getObject() instanceof Area) {
                return ((Area)this.value.getObject()).getSpatialElements();
            } else {
                return new SpatialElement[0];
            }
        }
        
        public Area[] getSubAreas() {
            return new Area[0];
        }

        public void addSubArea(Area a) {
            Note.warning(this, "Operation not implemented.");
        }
        
        public void removeSubArea(Area a) {
            Note.warning(this, "Operation not implemented.");
        }

        public boolean hasPaintingStyleDecorator() {
            return false;
        }
        
        public PaintingStyleDecorator getPaintingStyleDecorator() {
            Note.warning(this, "Operation not implemented.");
            return null;
        }

        public void setPaintingStyleDecorator(PaintingStyleDecorator psd) {
            Note.warning(this, "Operation not implemented.");
        }
        
        
        // ** Implementation of the interface Array (visualization interface) **
        
        public void setFirst(int i) {
            Note.warning(this, "Operation not implemented.");
        }
        
        public void setLast(int i) {
            Note.warning(this, "Operation not implemented.");
        }

        public int getFirst() {
            return 0;
        }
        
        public int getLast() {
            return 0;
        }
        
        public Object getObject(int address) {
            if (address == 0) {
                return this.getElement();
            } else {
                return null;
            }
        }
        
        public void setObject(Object o, int address) {
            if (address == 0) {
                this.setElement(o);
            }
        }
        
        
        // ** Implementation of interface DeleteEnabled **
        
        public boolean shouldDelete() {
            if ((this.getElement() != null) && (this.getElement() instanceof CircleEvent)) {
                return true;
            } else {
                return false;
            }
        }
        
        
        // ** Implementation of interface CDT **
        
        public CDT delete(Object o) {
            if (this.getElement() == o) {
                Animator a = Animator.getActiveAnimator();
                a.startOperation();
                if (o instanceof CircleEvent) {
                    CircleEvent circleEvent = (CircleEvent)o;
                    Fortune.this.triangulation.addEdge(circleEvent.getSite(0), 
                                                       circleEvent.getSite(2));
                    Fortune.this.beachLine.delete(circleEvent);
                }
                this.clearElement();
                a.endOperation();
            }
            return this;
        }
        
        public CDT getNewInstance() {
            Note.warning(this, "Operation not implemented.");
            return null;
        }
        
        public CDT insert(Object o) {
            Note.warning(this, "Operation not implemented.");
            return null;
        }
        
        public Object search(Object o) {
            Note.warning(this, "Operation not implemented.");
            return null;
        }

        private static final long serialVersionUID = -6854262903478445175L;
    }
    
    
    // A generator point - a single entry in the beach line that generates an arc.
    private class Generator implements Struct, Selectable {
        private SiteEvent site;
        private String visualization;
        private VirtualBoolean selected;
        
        public Generator(SiteEvent site) {
            this.site = site;
            this.visualization = site.getID() + "   (" + site.getX() + "," + site.getY() + ")";
            this.selected = new VirtualBoolean(true, this, "selected");
        }
        
        public java.awt.Point getSite() {
            return this.site.getSite();
        }

        public SiteEvent getSiteEvent() {
            return this.site;
        }

        //@Override
        protected Object clone() {
            return new Generator(this.site);
        }
        
        // ** Implementation of interface Struct **
        
        public Object getField(int location) {
            switch (location) {
                case 0:  return this.visualization;
                case 1:  return Integer.toString((int)Math.round(this.site.getX()));
                case 2:  return Integer.toString((int)Math.round(this.site.getY()));
                default: return null;
            }
        }
        
        public Object[] getFields() {
            return new Object[]{this.getField(0), 
                                this.getField(1),
                                this.getField(2)};
        }
        
        public String getFieldName(int location) {
            switch (location) {
                case 0:  return "";
                case 1:  return "x";
                case 2:  return "y";
                default: return "";
            }
        }
        
        public String[] getFieldNames() {
            return new String[]{ this.getFieldName(0),
                                 this.getFieldName(1),
                                 this.getFieldName(2)};
        }

        public void setField(Object value, int location) {
            Note.warning(this, "Operation not implemented.");
        }

        public Object getElement() {
            Note.warning(this, "Operation not implemented.");
            return null;
        }
        
        public void setElement(Object o) {
            Note.warning(this, "Operation not implemented.");
        }


        // ** Implementation of interface Selectable **
        
        public long getSelectionTime() {
            if (this.selected.eval()) {
                return Selectable.clock.getNextTimeStamp();
            } else {
                return Selectable.NOT_SELECTED;
            }
        }
        
        public void setSelect(long globalClockTimestamp) {
            if (globalClockTimestamp == Selectable.NOT_SELECTED) {
                this.selected.assign(false);
            } else {
                Animator a = Animator.getActiveAnimator();
                a.startOperation();
                this.selected.toggle();
                a.endOperation();
            }
        }
    }

    
    private class BeachLine extends SinglyLinkedList implements Area, Array, DeleteEnabled {
        // The current location of the sweepline.
        private final VirtualDouble sweepLineX;
        // Needed for adding and removing events when the beach line changes.
        private final EventQueue eventQueue;
        // List of circles to visualize new circle events
        private final SinglyLinkedList circleVis;
        // The triangulation created so far.
        private final Triangulation triangulation;
        // The current site being added to the beach line.
        private final VirtualObject findArc;
        
        public BeachLine(VirtualDouble sweepLineX, 
                         EventQueue eventQueue, 
                         Triangulation triangulation) {
            this.sweepLineX = sweepLineX;
            this.eventQueue = eventQueue;
            this.triangulation = triangulation;
            this.circleVis = new SinglyLinkedList();
            this.findArc = new VirtualObject(null, this, "findArc");
        }
        
        // Remove the generator points related to the given circle event.
        public void delete(CircleEvent circleEvent) {
            this.circleVis.clear();
            Node firstNode = null;
            Node secondNode = null;
            Node thirdNode = null;
            if (this.size.eval() > 2) {
                firstNode = this.getHeadNode();
                secondNode = firstNode.getNextNode();
                thirdNode = secondNode.getNextNode();
                if ((((Generator)firstNode.getKey()).getSite().equals(circleEvent.getSite(0))) &&
                    (((Generator)secondNode.getKey()).getSite().equals(circleEvent.getSite(1))) &&
                    (((Generator)thirdNode.getKey()).getSite().equals(circleEvent.getSite(2)))) {
                    firstNode.setNext(thirdNode);
                    //secondNode.clear();
                    this.size.dec();
                    if (thirdNode.hasNext()) {
                        // Remove associated events.
                        this.eventQueue.deleteEvent(circleEvent.getSite(1),
                                                    circleEvent.getSite(2),
                                                    ((Generator)thirdNode.getNextNode().getKey()).getSite());
                        if (!((Generator)firstNode.getKey()).getSite().equals(((Generator)thirdNode.getNextNode().getKey()).getSite())) {
                            checkCircleEvent(((Generator)firstNode.getKey()).getSiteEvent(), 
                                             ((Generator)thirdNode.getKey()).getSiteEvent(), 
                                             ((Generator)thirdNode.getNextNode().getKey()).getSiteEvent(), 
                                             this.sweepLineX.eval());
                        }
                    }
                    return;
                } else {
                    while (thirdNode.hasNext()) {
                        if ((((Generator)secondNode.getKey()).getSite().equals(circleEvent.getSite(0))) &&
                            (((Generator)thirdNode.getKey()).getSite().equals(circleEvent.getSite(1))) &&
                            (((Generator)thirdNode.getNextNode().getKey()).getSite().equals(circleEvent.getSite(2)))) {
                            secondNode.setNext(thirdNode.getNextNode());
                            //thirdNode.clear();
                            this.size.dec();

                            // Remove associated events.
                            this.eventQueue.deleteEvent(((Generator)firstNode.getKey()).getSite(),
                                                        circleEvent.getSite(0),
                                                        circleEvent.getSite(1));
                            if (!((Generator)firstNode.getKey()).getSite().equals(((Generator)secondNode.getNextNode().getKey()).getSite())) {
                                checkCircleEvent(((Generator)firstNode.getKey()).getSiteEvent(), 
                                                 ((Generator)secondNode.getKey()).getSiteEvent(), 
                                                 ((Generator)secondNode.getNextNode().getKey()).getSiteEvent(), 
                                                 this.sweepLineX.eval());
                            }
                            if (secondNode.getNextNode().hasNext()) {
                                // Remove associated events.
                                this.eventQueue.deleteEvent(circleEvent.getSite(1),
                                                            circleEvent.getSite(2),
                                                            ((Generator)secondNode.getNextNode().getNextNode().getKey()).getSite());
                                
                                if (!((Generator)secondNode.getKey()).getSite().equals(((Generator)secondNode.getNextNode().getNextNode().getKey()).getSite())) {
                                    checkCircleEvent(((Generator)secondNode.getKey()).getSiteEvent(), 
                                                     ((Generator)secondNode.getNextNode().getKey()).getSiteEvent(), 
                                                     ((Generator)secondNode.getNextNode().getNextNode().getKey()).getSiteEvent(), 
                                                     this.sweepLineX.eval());
                                }
                            }
                            return;
                        }
                        firstNode = secondNode;
                        secondNode = thirdNode;
                        thirdNode = secondNode.getNextNode();
                    }
                }
            }
        }
        
        public CDT delete(Object key) {
            return this;
        }
        
        private void checkCircleEvent(SiteEvent site0, 
                                      SiteEvent site1, 
                                      SiteEvent site2, 
                                      double sweepLineX) {
            CircleEvent circleEvent = validateCircleEvent(site0, site1, site2, sweepLineX);
            if (circleEvent != null) {
                this.eventQueue.insert(circleEvent);
                this.circleVis.insert(new Circle(circleEvent.getCenterX(),
                                                 circleEvent.getCenterY(),
                                                 circleEvent.getRadius(),
                                                 circleDecorator));
                this.circleVis.insert(new Point(circleEvent.getX(), circleEvent.getY(), circleDecorator));
                //this.circleVis.insert(new Point(circleEvent.getCenterX(), circleEvent.getCenterY(), circleCenterDecorator));
            }
        }
        
        public void clearCircles() {
            this.circleVis.clear();
        }
        
        public void clearSelections() {
            Iterator iterator = this.iterator();
            while (iterator.hasNext()) {
                ((Generator)iterator.next()).setSelect(Selectable.NOT_SELECTED);
            }
        }

        public void findArc(java.awt.Point newSite) {
            this.findArc.setObject(newSite);
        }
        
        public CDT insert(Object key) {
            this.circleVis.clear();
            if (key != null) {
                if (this.isEmpty()) {
                    this.setHeadNode(new Node(new Generator((SiteEvent)key), null));
                    this.size.inc();
                } else {
                    if (this.size.eval() == 1) {
                        // There is only one site currently in the beachline.
                        ((Generator)this.getHeadNode().getKey()).setSelect(Fortune.SELECTED);
                        this.getHeadNode().setNext(new Node(new Generator((SiteEvent)key), new Node(((Generator)this.getHead()).clone())));
                        this.size.inc(); 
                        this.size.inc();
                        if (!this.triangulation.addEdge(((Generator)this.getHead()).getSite(),
                                                        ((SiteEvent)key).getSite())) {
                            System.err.println("failed");
                        }
                    } else {
                        // Calculate the breakpoint for the pair of successive sites in 
                        // the beach line and test on which side the new site falls in.
                        Node firstNode = null;
                        Node secondNode = this.getHeadNode();
                        Node thirdNode = null;
                        Point2D[] breakpoints;
                        while (secondNode.hasNext()) {
                            thirdNode = secondNode.getNextNode();
                            breakpoints = this.breakpoints(((Generator)secondNode.getKey()).getSite(), 
                                                           ((Generator)thirdNode.getKey()).getSite());
                            
                            if ((((((Generator)thirdNode.getKey()).getSite().x) <= ((Generator)secondNode.getKey()).getSite().x) && (((SiteEvent)key).getSite().y < breakpoints[1].getY())) ||
                                (((((Generator)thirdNode.getKey()).getSite().x) >= ((Generator)secondNode.getKey()).getSite().x) && (((SiteEvent)key).getSite().y < breakpoints[0].getY()))) {
                                
                                if (firstNode != null) {
                                    this.eventQueue.deleteEvent(((Generator)firstNode.getKey()).getSite(), 
                                                                ((Generator)secondNode.getKey()).getSite(), 
                                                                ((Generator)thirdNode.getKey()).getSite());
                                }
                                
                                // Insert the node to the beachline.
                                ((Generator)secondNode.getKey()).setSelect(Fortune.SELECTED);
                                secondNode.setNext(new Node(new Generator((SiteEvent)key), new Node(((Generator)secondNode.getKey()).clone(), thirdNode)));
                                this.size.inc();
                                this.size.inc();
                                // Insert the found delaunay edge to the triangulation.
                                if (!this.triangulation.addEdge(((Generator)secondNode.getKey()).getSite(),
                                                                ((SiteEvent)key).getSite())) {
                                    System.err.println("failed");
                                }
                                if (firstNode == null) {
                                    // The site was added -> add the circle event corresponding to 
                                    // the new site triple to the event queue.
                                    checkCircleEvent((SiteEvent)key, 
                                                     ((Generator)secondNode.getKey()).getSiteEvent(), 
                                                     ((Generator)thirdNode.getKey()).getSiteEvent(), 
                                                     this.sweepLineX.eval());
                                    return this;
                                } else {
                                    // The site was added -> add the circle events corresponding to 
                                    // the two new site triples to the event queue.
                                    checkCircleEvent((SiteEvent)key, 
                                                     ((Generator)secondNode.getKey()).getSiteEvent(), 
                                                     ((Generator)thirdNode.getKey()).getSiteEvent(), 
                                                     this.sweepLineX.eval());
                                    checkCircleEvent(((Generator)firstNode.getKey()).getSiteEvent(), 
                                                     ((Generator)secondNode.getKey()).getSiteEvent(), 
                                                     (SiteEvent)key,
                                                     this.sweepLineX.eval());
                                    return this;
                                }
                            }
                            firstNode = secondNode;
                            secondNode = thirdNode;
                        }

                        this.eventQueue.deleteEvent(((Generator)firstNode.getKey()).getSite(), 
                                                    ((Generator)secondNode.getKey()).getSite(), 
                                                    ((Generator)thirdNode.getKey()).getSite());

                        // Insert the node at the end of the beachline.
                        ((Generator)secondNode.getKey()).setSelect(Fortune.SELECTED);
                        secondNode.setNext(new Node(new Generator((SiteEvent)key), new Node(((Generator)secondNode.getKey()).clone())));
                        this.size.inc();
                        this.size.inc();
                        if (!this.triangulation.addEdge(((Generator)secondNode.getKey()).getSite(),
                                                        ((SiteEvent)key).getSite())) {
                            System.err.println("failed");
                        }
                        if (firstNode != null) {
                            // The site was added -> add the circle event corresponding to 
                            // the new site triple to the event queue.
                            checkCircleEvent(((Generator)firstNode.getKey()).getSiteEvent(), 
                                             ((Generator)secondNode.getKey()).getSiteEvent(), 
                                             (SiteEvent)key, 
                                             this.sweepLineX.eval());
                        }
                    }
                }
            }
            return this;
        }
        
        private Point2D[] breakpoints(java.awt.Point siteA, java.awt.Point siteB) {
            double ax = siteA.x;
            double ay = siteA.y;
            double bx = siteB.x;
            double by = siteB.y;
            double x = this.sweepLineX.eval();
            double y1, y2;
            double r1, r2;
            double d1 = sqr(x) - 2*ax*x + sqr(ax) + sqr(ay); 
            double e1 = sqr(x) - 2*bx*x + sqr(bx) + sqr(by);           
            double[] args = new double[]{(2*x - 2*bx) * d1 - (2*x - 2*ax) * e1,
                                         4*ay*bx - 4*ay*x - 4*by*ax + 4*by*x,
                                         2*ax - 2*bx};
            double[] result = new double[2];
            QuadCurve2D.solveQuadratic(args, result);
            if (result[0] < result[1]) {
                y1 = result[0];
                y2 = result[1];
            } else {
                y1 = result[1];
                y2 = result[0];
            }
            r1 = (sqr(y1) - 2*ay*y1 + sqr(x) - 2*ax*x + sqr(ax) + sqr(ay)) / (2*x - 2*ax);
            r2 = (sqr(y2) - 2*ay*y2 + sqr(x) - 2*ax*x + sqr(ax) + sqr(ay)) / (2*x - 2*ax);
            return new Point2D[]{new Point2D.Double(x - r1, y1),
                                 new Point2D.Double(x - r2, y2)};
        }
        
        // ** Implementation of interface Area (visualization interface) **
        
        public void insert(SpatialComparable e) {
            Note.warning(this, "Operation not implemented.");
        }

        public void delete(SpatialComparable e) {
            Note.warning(this, "Operation not implemented.");
        }

        public SpatialElement[] getSpatialElements() {
            Vector vector  = new Vector();

            Iterator iterator = this.iterator();
            java.awt.Point currentSite = null;
            java.awt.Point nextSite = null;
            Generator currentGenerator;
            Generator nextGenerator;
            Point2D[] breakpoints = null;
            double breakpointX = 0;
            double breakpointY = 0;
            double previousY = 0;
            double w1, w2;
            double x1, y1, x2, y2, x3, y3, a, b, x21, y21, y1b, y3b;
            double[] result = new double[2];
            double[] args;
            java.awt.Point p = (java.awt.Point)this.findArc.getObject();
            Line2D sweepline = new Line2D.Double(this.sweepLineX.eval(), 0, this.sweepLineX.eval(), HEIGHT);
            if (iterator.hasNext()) {
                currentGenerator = ((Generator)iterator.next());
                vector.add(currentGenerator.getSiteEvent().getSpatialElements()[0]);
                currentSite = currentGenerator.getSite();
                if (iterator.hasNext()) {
                    while (iterator.hasNext()) {
                        nextGenerator = ((Generator)iterator.next());
                        vector.add(nextGenerator.getSiteEvent().getSpatialElements()[0]);
                        nextSite = nextGenerator.getSite();
                        if (currentSite.x == this.sweepLineX.eval()) {
                            x1 = this.sweepLineX.eval();
                            y1 = 0;
                            x2 = this.sweepLineX.eval();
                            y2 = HEIGHT;
                            x3 = nextSite.x; // ?
                            y3 = nextSite.y; // ?
                            b = currentSite.y;
                            x21 = x2 - x1;
                            y21 = y2 - y1;
                            y1b = y1 - b;
                            y3b = y3 - b;
                            args = new double[]{sqr(x3)*(sqr(x21) + sqr(y21)) + sqr(y3b)*(sqr(x21) + sqr(y21)) - x21*y1b + x1*y21,
                                                sqr(x21) + sqr(y21) + - y21 - 2*x3,
                                                sqr(x21) + sqr(y21)};
                            QuadCurve2D.solveQuadratic(args, result);
//                            System.err.println(result[0]);
//                            System.err.println(result[1]);
                            
                            if (currentGenerator.getSelectionTime() == Selectable.NOT_SELECTED) {
                                vector.add(new LineSegment((int)result[0] /* XXX */, currentSite.y, 
                                                           (int)this.sweepLineX.eval(), currentSite.y, beachLineDecorator));
                            } else {
                                vector.add(new LineSegment((int)result[0] /* XXX */, currentSite.y, 
                                                           (int)this.sweepLineX.eval(), currentSite.y, selectedArcDecorator));
                            }
                            previousY = currentSite.y;
                        } else {
                            if (nextSite.x == this.sweepLineX.eval()) {
                                breakpointY = nextSite.y;
                            } else {
                                breakpoints = this.breakpoints(currentSite, nextSite);
                                if (((currentSite.x > nextSite.x) && (breakpoints[0].getY() > breakpoints[1].getY())) ||
                                    ((currentSite.x < nextSite.x) && (breakpoints[0].getY() < breakpoints[1].getY()))) {
                                    breakpointY = breakpoints[0].getY();
                                    breakpointX = breakpoints[0].getX();
                                } else {
                                    breakpointY = breakpoints[1].getY();
                                    breakpointX = breakpoints[1].getX();
                                }
                            }

                            w1 = currentSite.y - previousY;
                            w2 = breakpointY - currentSite.y;
                            if (currentSite.y + w2 > 0) {
                                if (currentSite.y - w1 < HEIGHT) {
                                    // XXX
//                                    if (Math.abs(w1 + w2) < 6) {
//                                        vector.add(new LineSegment(0, currentSite.y, (int)this.sweepLineX.eval(), 
//                                                                   currentSite.y, beachLineDecorator));
//                                    } else {
                                    if (currentGenerator.getSelectionTime() == Selectable.NOT_SELECTED) {
                                        if ((p != null) &&
                                            ((p.y > currentSite.y - w1) && (p.y <= currentSite.y + w2))) {
                                            // Color the correct arc.
                                            vector.add(new Parabola(currentSite, sweepline, w1, w2, arcDecorator));
                                        } else {
                                            vector.add(new Parabola(currentSite, sweepline, w1, w2, beachLineDecorator));
                                        }
                                    } else {
                                        vector.add(new Parabola(currentSite, sweepline, w1, w2, selectedArcDecorator));
                                    }
//                                    }
                                } else {
                                    previousY = HEIGHT;
                                }
                            }
                            previousY = Math.max(0, currentSite.y + breakpointY - currentSite.y);
                        }
                        currentGenerator = nextGenerator;
                        currentSite = nextSite;
                    }

                    // The last site in the beach line.
                    w1 = currentSite.y - previousY;
                    w2 = HEIGHT - currentSite.y;
                    if ((currentSite.y + w2 > 0) &&
                        (currentSite.y - w1 < HEIGHT)) {
                        if (currentGenerator.getSelectionTime() == Selectable.NOT_SELECTED) {
                            if ((p != null) &&
                                ((p.y > currentSite.y - w1) && (p.y <= currentSite.y + w2))) {
                                vector.add(new Parabola(currentSite, sweepline, w1, w2, arcDecorator));
                            } else {
                                vector.add(new Parabola(currentSite, sweepline, w1, w2, beachLineDecorator));
                            }
                        } else {
                            vector.add(new Parabola(currentSite, sweepline, w1, w2, selectedArcDecorator));
                        }
                    }
                } else {
                    // The beach line has only a single site.
                    if (currentSite.x == this.sweepLineX.eval()) {
//                        System.err.println("(int)breakpointX = " + (int)breakpointX);
                        if (currentGenerator.getSelectionTime() == Selectable.NOT_SELECTED) {
                            vector.add(new LineSegment((int)0, currentSite.y, (int)this.sweepLineX.eval(), currentSite.y, beachLineDecorator));
                        } else {
                            vector.add(new LineSegment((int)0, currentSite.y, (int)this.sweepLineX.eval(), currentSite.y, selectedArcDecorator));
                        }
                    } else {
                        if (currentGenerator.getSelectionTime() == Selectable.NOT_SELECTED) {
                            if (p != null) {
                                vector.add(new Parabola(currentSite, sweepline, currentSite.y, HEIGHT - currentSite.y, arcDecorator));
                            } else {
                                vector.add(new Parabola(currentSite, sweepline, currentSite.y, HEIGHT - currentSite.y, beachLineDecorator));
                            }
                        } else {
                            vector.add(new Parabola(currentSite, sweepline, currentSite.y, HEIGHT - currentSite.y, selectedArcDecorator));
                        }
                    }
                }
            }

            iterator = this.circleVis.iterator();
            while (iterator.hasNext()) {
                vector.add(iterator.next());
            }
            
            SpatialElement[] spatialElements = new SpatialElement[vector.size()];
            vector.toArray(spatialElements);
            return spatialElements;
        }
        
        private double sqr(double number) {
            return (number * number);
        }

        public Rectangle getBoundingPolygon() {
            return BOUNDING_BOX_NONE;
        }

        public Area[] getSubAreas() {
            return new Area[0];
        }
        
        public void addSubArea(Area a) {
            Note.warning(this, "Operation not implemented.");
        }

        public void removeSubArea(Area a) {
            Note.warning(this, "Operation not implemented.");
        }

        public boolean hasPaintingStyleDecorator() {
            return false;
        }
        
        public void setPaintingStyleDecorator(PaintingStyleDecorator psd) {
            Note.warning(this, "Operation not implemented.");
        }
        
        public PaintingStyleDecorator getPaintingStyleDecorator() {
            return null;
        }
        
        
        // ** Implementation of interface Array **
        
        public Object getObject(int address) {
            Iterator iterator = this.iterator();
            for (int i = 0; i < address; i++) {
                iterator.next();
            }
            if (iterator.hasNext()) {
                return iterator.next();
            } else {
                return null;
            }
        }
        
        // Simulation functionality for inserting new sites/arc generators in the beach line.
        public void setObject(Object o, int address) {
            if ((o != null) && (o instanceof SiteEvent) && (Fortune.this.currentEvent.getElement() == o)) {
                Fortune.this.currentEvent.clearElement();
                // XXX
                //o = ((SiteEvent)o).getSite();
                if (this.isEmpty()) {
                    this.insert(o);
                } else {
                    Node firstNode = null, secondNode = null, thirdNode = this.getHeadNode();
                    address--;
                    while (address >= -1) {
                        address--;
                        firstNode = secondNode;
                        secondNode = thirdNode;
                        thirdNode = thirdNode.getNextNode();
                    }
                    
                    if ((firstNode != null) && (thirdNode != null)) {
                        this.eventQueue.deleteEvent(((Generator)firstNode.getKey()).getSite(), 
                                                    ((Generator)secondNode.getKey()).getSite(), 
                                                    ((Generator)thirdNode.getKey()).getSite());
                    }

                    ((Generator)secondNode.getKey()).setSelect(Fortune.SELECTED);
                    secondNode.setNext(new Node(new Generator((SiteEvent)o), new Node(((Generator)secondNode.getKey()).clone(), thirdNode)));
                    this.size.inc();
                    this.size.inc();
                    this.triangulation.addEdge(((Generator)secondNode.getKey()).getSite(),
                                               ((SiteEvent)o).getSite());
                    if (firstNode != null) {
                        // The site was added -> add the circle event corresponding to 
                        // the new site triple to the event queue.
                        checkCircleEvent(((Generator)firstNode.getKey()).getSiteEvent(), 
                                         ((Generator)secondNode.getKey()).getSiteEvent(), 
                                         (SiteEvent)o, 
                                         this.sweepLineX.eval());
                    }
                    if (thirdNode != null) {
                        // The site was added -> add the circle event corresponding to 
                        // the new site triple to the event queue.
                        checkCircleEvent((SiteEvent)o, 
                                         ((Generator)secondNode.getKey()).getSiteEvent(), 
                                         ((Generator)thirdNode.getKey()).getSiteEvent(), 
                                         this.sweepLineX.eval());
                    }
                }
            }
        }

        public Object getElement() {
            Note.warning(this, "Operation not implemented.");
            return null;
        }
        
        public void setElement(Object o) {
            Note.warning(this, "Operation not implemented.");
        }

        public int getFirst() {
            return 0;
        }
        
        public int getLast() {
            if (this.isEmpty()) {
                return 0;
            } else {
                return this.size() - 1;
            }
        }
        
        public void setFirst(int i) {
            Note.warning(this, "Operation not implemented.");
        }
        
        public void setLast(int i) {
            Note.warning(this, "Operation not implemented.");
        }


        // ** Implementation of interface DeleteEnabled **
        
        public boolean shouldDelete() {
            return true;
        }
        
        //@Override
        public boolean equals(Object obj) {
            if (obj instanceof BeachLine) {
                BeachLine beachLine = (BeachLine)obj;
                Iterator i1 = this.iterator();
                Iterator i2 = beachLine.iterator();
                Generator g1, g2;
                while (i1.hasNext()) {
                    g1 = (Generator)i1.next();
                    if (i2.hasNext()) {
                        g2 = (Generator)i2.next();
                    } else {
                        return false;
                    }
                    if (!g1.getSite().equals(g2.getSite())) {
                        return false;
                    }
                }
                if (i2.hasNext()) {
                    return false;
                } else {
                    return true;
                }
            } else {
                return false;
            }
        }
        
        private static final long serialVersionUID = 5453527052820949761L;
    }
    
    
    private CircleEvent validateCircleEvent(SiteEvent site0, 
                                            SiteEvent site1, 
                                            SiteEvent site2, 
                                            double sweepLineX) {
        Ellipse2D circle = 
            GeometryFunctions.circumcircle(new java.awt.Point[]{site0.getSite(), site1.getSite(), site2.getSite()});
        
        if (circle.getMaxX() > sweepLineX &&
            /*circle.getMaxX() < (MAX_X) &&*/
            converges(site0.getSite(), site1.getSite(), site2.getSite())) {
            return new CircleEvent(site0, site1, site2, circle);
        } else {
            return null;
        }
    }
    
    private static boolean converges(java.awt.Point site0, 
                                     java.awt.Point site1, 
                                     java.awt.Point site2) {
        double[] veca = new double[2];
        double[] vecb = new double[2];
        veca[0] = site1.x - site0.x;
        veca[1] = site1.y - site0.y;
        vecb[0] = site2.x - site1.x;
        vecb[1] = site2.y - site1.y;
        if (vecb[1] * veca[0] > vecb[0] * veca[1]) {
            return false;
        } else {
            return true;
        }
    }
    
//    private static double sqr(double number) {
//        return (number * number);
//    }

    
    private class EventQueue extends SortedList implements Array, Area {
        public EventQueue(Comparator comparator) {
            super(comparator);
        }
     
        public void deleteEvent(java.awt.Point site0, java.awt.Point site1, java.awt.Point site2) {
            if (!this.isEmpty()) {
                CircleEvent event;
                Node node = this.getHeadNode();
                if (node.getKey() instanceof CircleEvent) {
                    event = (CircleEvent)node.getKey();
                    if (event.getSite(0).equals(site0) &&
                        event.getSite(1).equals(site1) &&
                        event.getSite(2).equals(site2)) {
                        this.removeHead();
                        return;
                    }
                }
                Node previousNode;
                while (node.hasNext()) {
                    previousNode = node;
                    node = node.getNextNode();
                    if (node.getKey() instanceof CircleEvent) {
                        event = (CircleEvent)node.getKey();
                        if (event.getSite(0).equals(site0) &&
                            event.getSite(1).equals(site1) &&
                            event.getSite(2).equals(site2)) {
                            this.removeNode(node, previousNode);
                            return;
                        }
                    }
                }
            }
        }
        
        
        // ** Implementation of interface Array **
        
        public Object getObject(int address) {
            Iterator iterator = this.iterator();
            for (int i = 0; i < address; i++) {
                iterator.next();
            }
            if (iterator.hasNext()) {
                Event event = (Event)iterator.next();
                // XXX change the event queue vis here
                // return "(" + event.getX() + "," + event.getY() + ")";
                return event;
            } else {
                return null;
            }
        }
        
        public void setObject(Object o, int address) {
            // TODO Auto-generated method stub
            //System.out.println("BeachLine.setObject()");
        }

        public Object getElement() {
            Note.warning(this, "Operation not implemented.");
            return null;
        }
        
        public void setElement(Object o) {
            Note.warning(this, "Operation not implemented.");
        }

        public int getFirst() {
            return 0;
        }
        
        public int getLast() {
            if (this.isEmpty()) {
                return 0;
            } else {
                return this.size() - 1;
            }
        }
        
        public void setFirst(int i) {
            Note.warning(this, "Operation not implemented.");
        }
        
        public void setLast(int i) {
            Note.warning(this, "Operation not implemented.");
        }


        // ** Implementation of interface Area **
        
        public Rectangle getBoundingPolygon() {
            return BOUNDING_BOX_NONE;
        }

        public void addSubArea(Area a) {
            Note.warning(this, "Operation not implemented.");
        }

        public void removeSubArea(Area a) {
            Note.warning(this, "Operation not implemented.");
        }

        public Area[] getSubAreas() {
            return new Area[0];
        }
        
        public void delete(SpatialComparable e) {
            Note.warning(this, "Operation not implemented.");
        }
        
        public void insert(SpatialComparable e) {
            Note.warning(this, "Operation not implemented.");
        }
        
        public SpatialElement[] getSpatialElements() {
            Vector vector  = new Vector();
            Iterator iterator = this.iterator();
            SpatialElement[] subElements;
            Event event;
            while (iterator.hasNext()) {
                event = (Event)iterator.next();
                if (event instanceof CircleEvent) {
                    subElements = event.getSpatialElements();
                    for (int i = 0; i < subElements.length; i++) {
                        vector.add(subElements[i]);
                    }
                }
            }
            
            SpatialElement[] spatialElements = new SpatialElement[vector.size()];
            vector.toArray(spatialElements);
            return spatialElements;
        }
        
        public boolean hasPaintingStyleDecorator() {
            return false;
        }

        public PaintingStyleDecorator getPaintingStyleDecorator() {
            Note.warning(this, "Operation not implemented.");
            return null;
        }
        
        public void setPaintingStyleDecorator(PaintingStyleDecorator psd) {
            Note.warning(this, "Operation not implemented.");
        }
        
        public void clearSelections() {
            Iterator iterator = this.iterator();
            Object event;
            while (iterator.hasNext()) {
                event = iterator.next();
                if (event instanceof CircleEvent) {
                    ((CircleEvent)event).setSelect(Selectable.NOT_SELECTED);
                }
            }
        }
    }
    
    private static final long serialVersionUID = -8392619360512305537L;
}
