// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
package content.exercises.sda;

import java.awt.Point;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import java.util.Arrays;
import java.util.Comparator;
import java.util.LinkedList;
import java.util.Random;
import java.util.TreeSet;

import matrix.structures.FDT.FDT;
import matrix.structures.spatial.CDT.probe.QuadEdge;
import matrix.util.Note;
import content.exercises.sda.structures.PointGenerator;
import content.interfaces.SimulationExercise;

public class QuadEdgeDelete implements SimulationExercise {

    // For debugging
    private static final boolean STATIC_SEED = false;
    private static final long STATIC_SEED_VALUE = 1165329245160L;

    private long seed = System.currentTimeMillis();

    private QuadEdge quadEdge;
    

    // ** Implementation of interface SimulationExercise **

    public String getDescription() {
        // TODO
        return "";
    }

    public FDT[] init() {
        return this.getInitialStructures();
    }

    public FDT[] getInitialStructures() {
        Point2D[] polygonVertices;
        System.err.println("The random number generator seed for this exercise is: " + this.getSeed());
        Random r = new Random(this.getSeed());
        try {
            polygonVertices = PointGenerator.generatePoints(0,
                                                            0,
                                                            300,
                                                            300,
                                                            20,
                                                            10,
                                                            r);
        } catch (Exception e) {
            Note.err(this, "Failed to generate points for the polygon.");
            return null;
        }
        generatePolygon2Peasants(polygonVertices);
        this.quadEdge = new QuadEdge(polygonVertices);
        
        
        
        int[][] adjacencyList = new int[][]{{3, 2, 1, 5, 4, 6},
                                            {2, 4},
                                            {3},
                                            {},
                                            {},
                                            {4},
                                            {5, 3}};
//        int[][] adjacencyList = new int[][]{{1},
//                {2},
//                {0},
//                {},
//                {},
//                {},
//                {}};
        Point2D[] vertices = new Point2D[]{new Point(100, 100),
                                           new Point(120, 10),
                                           new Point(180, 80),
                                           new Point(120, 190),
                                           new Point(80, 10),
                                           new Point(50, 65),
                                           new Point(80, 180)};

//        int[][] adjacencyList = new int[][]{{1, 2, 3, 4, 5, 6},
//                                            {},
//                                            {},
//                                            {},
//                                            {},
//                                            {},
//                                            {}};
//        Point2D[] vertices = new Point2D[]{new Point(100, 100),
//                                           new Point(120, 10),
//                                           new Point(180, 80),
//                                           new Point(120, 190),
//                                           new Point(80, 10),
//                                           new Point(50, 65),
//                                           new Point(80, 180)};
//        // XXX testing
//        Point2D temp;
//        int index;
//        int index2;
//        for (int i = 0; i < vertices.length; i++) {
//            index = r.nextInt(vertices.length);
//            temp = vertices[index];
//            index2 = r.nextInt(vertices.length);
//            vertices[index] = vertices[index2];
//            vertices[index2] = temp;
//        }
//        //
        
        this.quadEdge = new QuadEdge(adjacencyList, vertices);
        
        return new FDT[]{ this.quadEdge };
    }

    public String[] getStructureNames() {
        // TODO
        return new String[0];
    }

    public long getSeed() {
        // Seed might be set to a constant value for debugging purposes.
        if (STATIC_SEED) {
            return QuadEdgeDelete.STATIC_SEED_VALUE;
        } else {
            return this.seed;
        }
    }

    public void setSeed(long seed) {
        this.seed = seed;
    }

    public FDT[] getAnswer() {
        return new FDT[0];
    }
    
    
    
    
    // XXX remove this, move to its own class
    private void generatePolygon2Peasants(Point2D[] points) {
        // Sort the points in ascending order according to the x-coordinate.
        Arrays.sort(points, new Comparator() {
            public int compare(Object arg0, Object arg1) {
                Point2D p0 = (Point2D)arg0;
                Point2D p1 = (Point2D)arg1;
                return (new Integer((int)p0.getX())).compareTo(new Integer((int)p1.getX()));
            }
        });
        // Draw a line from the point with the smallest x-coordinate to the point with the largest.
        Line2D l = new Line2D.Double(points[0], 
                                     points[points.length - 1]);
        
        // Divide the points to those above and those below the line and form the contour.
        LinkedList above = new LinkedList();
        TreeSet below = new TreeSet(new Comparator() {
            public int compare(Object arg0, Object arg1) {
                Point2D p0 = (Point2D)arg0;
                Point2D p1 = (Point2D)arg1;
                if ((new Integer((int)p0.getX())).compareTo(new Integer((int)p1.getX())) == 0) {
                    return -1;
                } else {
                    return -(new Integer((int)p0.getX())).compareTo(new Integer((int)p1.getX()));
                }
            }
        });
        for (int i = 0; i < points.length; i++) {
            if (l.relativeCCW(points[i]) < 0) {
                // TODO integer conversion does not belong here
                above.add(new java.awt.Point((int)points[i].getX(), 
                                             (int)points[i].getY()));
            } else {
                // TODO integer conversion does not belong here
                below.add(new java.awt.Point((int)points[i].getX(), 
                                             (int)points[i].getY()));
            }
        }
        above.addAll(below);
        above.toArray(points);
    }

    
}
