// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/* $Id: ExerBin.java,v 1.7 2007/09/10 08:09:08 vkaravir Exp $ */

package content.exercises.structures;

import matrix.structures.FDT.BinaryTree;
import matrix.structures.FDT.Tree;
import matrix.structures.FDT.probe.*;
import matrix.structures.util.MatrixComparable;

/**
 * This is an implementation of a binary tree. A binary tree is a
 * tree in which each node has two children (either of which can be null).
 */
public class ExerBin extends BinTree {
    
    private ExerBin parent=null;
    public static final boolean DEBUG = false;
    
    public ExerBin(Object o) {
        super(o);
    }
    
    public ExerBin(){
        super();
    }
    
    /**
     * Return the binary tree that contains the element in root node
     */
    public BinaryTree find(MatrixComparable k, BinaryTree tree) {
        if (DEBUG)
            System.out.println(k+":"+tree);
        // On leaf and no element
        if (tree == null)
            return null;
        // Key is invalid
        if (k == null)
            return null;
        // Recurse into right node
        MatrixComparable key = (Key)tree.getElement();
        if(k.eq(key))
            return tree;
        else if(k.leq(key))
            return(find(k,tree.getLeft()));
        else
            return(find(k, tree.getRight()));
        
    }
    
    public Tree getNewNode(Object key) {
        return new ExerBin(key);
    }
    
    
    public void setRight(BinaryTree p) {
        if (DEBUG)
            System.out.println("Right:"+p);
        super.setRight(p);
        
    }
    
    public void setElement(Object o) {
        if (o == null) {
            if (parent.getLeft() == this)
                parent.setLeft(null);
            else
                parent.setRight(null);
        }
        super.setElement(o);
    }
    /*
     public FDT delete(Object o) {
     System.out.println("Delete:"+o);
     BinaryTree tree = find((Element)o, (BinaryTree)getElement());
     System.out.println(o+""+tree.getRight()+""+tree.getLeft());
     
     
     if (tree.getRight() == null && tree.getLeft() == null)
     return super.delete(o);
     else {
     tree.setElement(""+tree.getElement().toString()+"(del)");
     return (BinaryTree)getElement();
     }
     }
     */
    
    private void setParent(ExerBin parent) {
        if (parent != null)
            this.parent = parent;
    }
    
    
    
    // Methods for interface CDT
    
    public BinaryTree realInsert(MatrixComparable k) {
        
        
        if(k == null ) return this;
        
        if (k.geq((MatrixComparable)getElement())) {
            setRight(insertHelp(k, (BinaryTree)getRight()));
            ((ExerBin)getRight()).setParent(this);
        } else {
            setLeft(insertHelp(k, (BinaryTree)getLeft()));
            ((ExerBin)getLeft()).setParent(this);
        }
        return this;
        
    }
    
    
    /**
     * Helper function for insert.
     * @param k the new key to be inserted
     * @param tree the subtree currently being examined
     */
    private BinaryTree insertHelp(MatrixComparable k, BinaryTree tree) {
        if(tree == null)
            return new ExerBin(k);
        
        if(tree.getElement() == null) {
            tree.setElement(k);
            return tree;
        }
        
        MatrixComparable key = (MatrixComparable)tree.getElement();
        if(k.leq(key)) {
            tree.setLeft(insertHelp(k, tree.getLeft()));
            if (tree.getLeft() != null)
                ((ExerBin)tree.getLeft()).setParent((ExerBin)tree);
        } else {
            tree.setRight(insertHelp(k, tree.getRight()));
            if (tree.getRight() != null)
                ((ExerBin)tree.getRight()).setParent((ExerBin)tree);
        }
        
        return tree;
    }
    
    public int getLargestHeight() {
        return getLargestHeight(this);
    }
    
    public int getMinimumHeight() {
        return getMinimumHeight(this);
    }
    
    public void delete() {
        if (parent != null) {
            if (DEBUG)
                System.out.println("Delete! With parent:"+parent.getElement());
            if (getLeft() == null && getRight() == null) {
                if (parent.getLeft() == this)
                    parent.setLeft(null);
                else
                    parent.setRight(null);
            } else this.setElement(new Key(" "));
            
        } else {
            if (DEBUG)
                System.out.println(parent);
            this.setElement(new Key(" "));
        }
    }
    
    
    /**
     * Returns the height of a subtree
     * @param tree the subtree currently being examined
     * @return the min height
     */
    private int getMinimumHeight(BinaryTree tree) {
        int right=0;
        int left=0;
        
        if (tree == null) return 0;
        
        // Parse left subtree
        if(tree.getRight() != null)
            right = getMinimumHeight(tree.getRight());
        
        // Parse right subtree
        if(tree.getLeft() != null)
            left = getMinimumHeight(tree.getLeft());
        
        // Return smallest
        return (right<left)?(right+1):(left+1);
    }
    
    /**
     * Returns the height of a subtree
     * @param tree the subtree currently being examined
     * @return the min height
     */
    private int getLargestHeight(BinaryTree tree) {
        int right=0;
        int left=0;
        
        if (tree == null) return 0;
        
        // Parse left subtree
        if(tree.getRight() != null)
            right = getLargestHeight(tree.getRight());
        
        // Parse right subtree
        if(tree.getLeft() != null)
            left = getLargestHeight(tree.getLeft());
        
        // Return largest
        return (right>left)?(right+1):(left+1);
    }
    
    static final long serialVersionUID = -4064810144734786900L;
}
