// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/*$Id: ExerPriorityElement_DH.java,v 1.5 2007/09/10 08:09:08 vkaravir Exp $*/
package content.exercises.structures;

import java.io.Serializable;

import matrix.structures.util.MatrixComparable;

/**
 * This class is a priority element. The priority value is an int variable inside this 
 * element and it is used for comparison of priority elements.
 * 
 * @author kstalnac
 *
 */
public class ExerPriorityElement_DH implements MatrixComparable, Serializable {

    /** Actual priority value. */
    private int priority = Integer.MIN_VALUE;
    
    /**
     * Constructor.
     *
     */
    public ExerPriorityElement_DH() {
    }

    /**
     * Constructor
     * @param priority priority value for this priority element
     */
    public ExerPriorityElement_DH(int priority) {
        this.priority = priority;
    }
    
    /**
     * Returns the priority as a comparison string for this priority element. 
     */
    public String getComparisonString() {
        return priority + "";
    }

    /**
     * Returns the priority value
     * @return priority value
     */
    public int getPriority() {
        return priority;
    }
    
    /**
     * Equals method.
     * @return true, if this object and the parameter object have the same priority, otherwise false
     */
    public boolean eq(MatrixComparable k) {
        if (!(k instanceof ExerPriorityElement_DH))
            return false;
        else {
            return getPriority() == ((ExerPriorityElement_DH)k).getPriority();
        }
    }

    /**
     * Less than method.
     * @return true, if the priority value of this object is less than the priority value of the 
     * parameter, otherwise false
     */
    public boolean lt(MatrixComparable k) {
        if (k == null)
            return false;
        if (!(k instanceof ExerPriorityElement_DH))
            return false;
        
        return getPriority() < ((ExerPriorityElement_DH)k).getPriority();
    }

    /**
     * Greater than method.
     * @return true, if the priority value of this object is greater than the priority value of the 
     * parameter, otherwise false
     */
    public boolean gt(MatrixComparable k) {
        if (k == null)
            return false;
        if (!(k instanceof ExerPriorityElement_DH))
            return false;
        return getPriority() > ((ExerPriorityElement_DH)k).getPriority();
    }

    /**
     * Less or equal than method.
     * @return true, if the priority value of this object is less or equal than the priority value of the 
     * parameter, otherwise false
     */
    public boolean leq(MatrixComparable k) {
        if (k == null)
            return false;
        if (!(k instanceof ExerPriorityElement_DH))
            return false;
        return getPriority() <= ((ExerPriorityElement_DH)k).getPriority();
    }

    /**
     * Greater or equals than method.
     * @return true, if the priority value of this object is greater of equal than the priority value of the 
     * parameter, otherwise false
     */
    public boolean geq(MatrixComparable k) {
        if (k == null)
            return false;
        if (!(k instanceof ExerPriorityElement_DH))
            return false;
        return getPriority() >= ((ExerPriorityElement_DH)k).getPriority();
    }
    
    private static final long serialVersionUID = -1557796654276033767L;
}
