// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package content.exercises.structures;

import java.awt.Color;

import matrix.structures.CDT.probe.*;
import matrix.structures.FDT.*;
import matrix.structures.other.Rotations;
import matrix.structures.util.MatrixComparable;
import matrix.util.Note;
import matrix.structures.CDT.CDT;

/**
 * @see RBTree
 *
 * @author Petri Tenhunen
 */

public class ExerRBTree extends RBTree {
	
	private boolean manualColoring;
	
	public ExerRBTree(){
		
	}
	
	public ExerRBTree(boolean manualColoring){
		this.manualColoring = manualColoring;
	}
	

  public class ExerRBNode extends RedBlackNode {

    private int i = 0;

    /** default constructor for empty tree */
    public ExerRBNode() {
      super();
    }

    /** Constructor for tree with one element */
    public ExerRBNode(MatrixComparable e) {
      super(e);
    }

    public Object getElement() {
        //System.out.println("Refresh:"+i++);
        //toggleColor();
        return super.getElement();
    }
      
      public void setElement(Object o) {
          //System.out.println("Set:"+i++);
          //toggleColor();
          super.setElement(o);
      }

    /** If the color of the node is red - set it black and vice versa */
    public void toggleColor() {
      setRed(!isRed());
    }

    public void rotateRight() {    
      Rotations.rotateWithLeft_(this, manualColoring);
    }

    public void rotateLeft() {
      Rotations.rotateWithRight_(this, manualColoring);
    }

    public void doubleRotateLeft() {
      Rotations.rotateDoubleWithLeft_(this, manualColoring);
    }

    public void doubleRotateRight() {
      Rotations.rotateDoubleWithRight_(this, manualColoring);
    }


    public Tree getNewNode(Object key) {
	//Note.out(this, "create new ExerRBNode " + key);
      ExerRBNode tmp =  new ExerRBNode((MatrixComparable)key);
      tmp.setRed(true);
      return tmp;
    }

      static final long serialVersionUID = -4893948488594514086L;
  }

  /** This method is invoked from RBTree coloring exercise to determine whether
   * the tree has a valid coloring.
   */
  public boolean[] isValidRBT() {
      return ((RedBlackNode)getElement()).isValid();
  }

  /** This method is invoked from RBTree coloring exercise to color all nodes with the given parameter
   * color.
   */
  public void color(Color c) {
      ((RedBlackNode)getElement()).colorTree(c);
  }

  public FDT superInsert(MatrixComparable o) {
      return super.insert(o);
  }

  public FDT superInsert(Object o) {
      return super.insert(o);
  }

  /** Overriden insertion does nothing */
  public CDT insert(Object o) {
    Note.show(this,"Insert nodes by dragging them on the leaf.");
    return this;
  }

  public Tree getNewNode(Object key) {
      //Note.out(this, "create new ExerRBNode " + key);
    return new ExerRBNode((MatrixComparable)key);
  }


  static final long serialVersionUID = -8279204555769956022L;
}
