// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
package content.interfaces;
import java.awt.Component;
import java.awt.GridBagConstraints;

import matrix.util.Application;

/**
 * Interface for specifying a set of AWT components
 * that are to be added in an <code>ExercisePanel</code>.
 *
 * For example, can be used to add buttons and checkboxes 
 * to an exercise for additional simulation purposes.
 * 
 * @see ExercisePanel
 */
public interface AWTComponentUtilizer {

    /**
     * @return  The components that are to be added in the panel.
     */
    public Component[] getAWTComponents();
    
    
    /**
     * This is used to provide the application reference to the 
     * exercise implementing the interface, where it might be 
     * needed for validation. The implementation can be blank 
     * if the reference isn't needed.
     * 
     * @param a The application reference.
     */
    public void setApplication(Application a);
    
    /**
     * Specify <code>GridBagConstraints</code> to be used with the 
     * components. The constraints are applied in order to the 
     * corresponding components and therefore there must be as many 
     * constraint objects as there are components.
     * 
     * The method should return null if no constraints are specfied.
     * 
     * @return  The constraints to be used when adding the components
     *          given with <code>getAWTComponents()</code> in the panel
     *          or null if none specified.
     */
    public GridBagConstraints[] getAWTComponentConstraints();

}
