// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
package content.interfaces;
import matrix.animation.AnimatedStructures;

/**
 * @author Otto Seppl
 *
 * This interface defines the additional methods an exercise class can implement to
 * create simulations from different algorithm variants. These simulation sequences can then be used
 * to assess whether the student solution is consistent with any of them.
 */
public interface AlgorithmVariants{

    /**
     * This method is the equivalent of the solve method in SimulationExerciseModel
     * interface. It creates and returns FDTs for different algoritm variants.
     * 
     * @return an array of solutions for the different algorithm variants
     */
    public AnimatedStructures[] createVariantSequences();    
    
    /**
     * Each variant is described by a nickname that can describes it in a short form.
     * This method returns a nickname for each sequence returned by 
     * createVariantSequences.
     *
     * @return nicknames for the variants
     */
    public String[] getVariantNames();
        
}
