// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/* $Id: AnimatedOperation.java,v 1.13 2007/09/07 13:32:45 vkaravir Exp $ */

package matrix.animation;

import java.io.Serializable;

/**
 * Operetion that can be undone and stored to a <code>Animator</code>.
 * All operations that will be stored in <code>Animator</code> should
 * implement this interface. </p><p>
 *
 * Animations can be saved and that is why all <code>AnimatedOperations</code>
 * should be serializable. <code>AnimatedOperation</code> interface extend Serializable
 * interfacce so this is no problem untill you create a new class implementing
 * <code>AnimatedOperation</code> and from there define a referenco to nonserializable
 * class.
 *
 * @see Animator
 */

public interface AnimatedOperation extends Serializable {
    /** reset previous state */
    public void undo();
    /** Redo last (undone) operation */
    public void redo();

    static final long serialVersionUID = -2783119947151781544L;
}
