// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
/*
* Matrix -- algorithm simulation system
* Copyright (C) 2000-2004 Matrix group
* (See the AUTHORS file and the author tag below for more information
* on the authors.)
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package matrix.decoration;

/**
 * Information decorator for static structures. Should only be used on
 * classes that implement static data structures. However, since Matrix
 * does not directly visualize static structures, but shows them through
 * adapters from package matrix.adaptation, the adapters must implement
 * InformationDecorator, AND be able to pass information from the adapted
 * structure to the visualization for information of static structures
 * to be shown. The aim of this interface is therefore to establish a
 * common way for the adapters to gain the required information.
 * 
 * @author Jussi Nikander
 *
 */

public interface StaticInformationDecorator {

	 /** 
	   * Tells if information message can be shown in a separate box or frame
	   * @return true if label is shown
	   */
	  public boolean isInfoBoxEnabled();
	  
	  /**
	   * Tells if information message can be shown in tooltips. 
	   * @return true if label is shown
	   */
	  public boolean isToolTipInfoEnabled();

	  /**
	   * Sets the info message for the i'th index in the current object
	   * @param i the index
	   * @param s the new label
	   */
	    public void setInfo(int i, String s);
	  
	  /**
	   * Returns the information associated with an index of this object
	   * @param i the index for which info should be retrieved
	   * @return the information
	   */
	  public String getInfo(int i);
	  
	  /**
	   * Returns the wait time in milliseconds. Tooltip in separate info box is displayed
	   * after this time is passed.
	   * @return time to wait
	   */
	  public int getWaitTime();

	  /** Set time to wait before Tooltip in separate info box is displayed */
	  public void setWaitTime(int t);

	  
		/** 
		 * Tells if information message can be shown in a separate box or frame
		 * @return true if label is shown
		 */
		public boolean isReferenceInfoBoxEnabled();
		
		/**
		 * Tells if information message can be shown in tooltips. 
		 * @return true if label is shown
		 */
		public boolean isReferenceToolTipInfoEnabled();
		
		/**
	     * Gets the information for the i:th reference from this node
	     * @param from the index of the node from which the reference originates
	     * @param refNro the index of the reference
	     * @return the tooltip text
	     */
	    public String getReferenceInformation(int from, int refNro);

	    /**
	     * Sets the information for the i:th reference
	     * @param i the index of the node from which the reference originates
	     * @param refNro the index of the referece
	     * @param s new tooltip text
	     */
	    public void setReferenceInformation(int from, int refNro, String s);

}
