// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
package matrix.structures.ADT.probe;

import matrix.structures.CDT.CDT;
import matrix.structures.CDT.interfaces.PrioritySupplied;
import matrix.structures.CDT.probe.BinHeap;
import matrix.structures.CDT.probe.OrderedList;

import java.util.*;

public class PriorityQueue implements    matrix.structures.ADT.ADT,
                                            matrix.structures.ADT.PriorityQueue{
    
    private matrix.structures.ADT.PriorityQueue implementation;
    private static CDT currentFactory  = new BinHeap();    
    private static Map implementations = new HashMap();
    
    static{
    	implementations.put("Heap",        new BinHeap());
        implementations.put("OrderedList", new OrderedList());
    }
    
    public CDT getCDT(){
        return (CDT)implementation;
    }
    
    public PriorityQueue getNewInstance(String implementingClassName){
    	CDT factory = (CDT)implementations.get(implementingClassName);
        CDT newInstance = factory.getNewInstance();
        return new PriorityQueue(newInstance);
    }    

    public PriorityQueue(CDT implementation){
    	this.implementation = (matrix.structures.ADT.PriorityQueue)implementation;
    }
    
    public PriorityQueue(){
        implementation = (matrix.structures.ADT.PriorityQueue)currentFactory.getNewInstance();
    }
    
    public PrioritySupplied ADTdeleteMin(){
    	return implementation.ADTdeleteMin();
    }
    
    public void ADTinsert(PrioritySupplied inserted){
    	implementation.ADTinsert(inserted);
    }

    public boolean ADTIsEmpty(){
        return implementation.ADTIsEmpty();
    }
}
