// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/* $Id: Stack.java,v 1.19 2007/09/07 13:36:07 vkaravir Exp $ */

   package matrix.structures.ADT.probe;

   import java.util.HashMap;
   import java.util.Map;

   import matrix.structures.ADT.ADT;

   import matrix.structures.CDT.probe.QueueImpl;
   import matrix.structures.CDT.probe.StackImpl;
   import matrix.structures.CDT.CDT;

/**
 * Stack is an abstract data type of stack. This can be used as an ordinary 
 * stack when implementing, for example, exercises.
 * This inherit StackImpl CDT structure so this can also be directly 
 * visualized in the Matrix.
 *  
 * @author kstalnac
 */
    public class Stack implements ADT, matrix.structures.ADT.Stack{
   
        //--ADT Factory---------------------------------------------//
        private matrix.structures.ADT.Stack implementation;
        private static CDT currentFactory  = new StackImpl();    
        private static Map implementations = new HashMap();
        
        static{
            implementations.put("Queue",       new QueueImpl());
        }
        
        /**
         * This method returns the underlying CDT structure which can be used
         * to visualize this ADT structure.
         */
        public CDT getCDT(){
            return (CDT)implementation;
        }
        
        public Stack getNewInstance(String implementingClassName){
            CDT factory = (CDT)implementations.get(implementingClassName);
            CDT newInstance = factory.getNewInstance();
            return new Stack(newInstance);
        }    

        public Stack(CDT implementation){
            this.implementation = (matrix.structures.ADT.Stack)implementation;
        }
        
        public Stack(){
            implementation = (matrix.structures.ADT.Stack)currentFactory.getNewInstance();
        }
         
        
        public Object ADTPeek(){
            return implementation.ADTPeek();
        }
        
        public Object ADTPop(){
            return implementation.ADTPop();
        }

        public void ADTPush(Object item){
            implementation.ADTPush(item);
        }

        public boolean ADTIsEmpty(){
            return implementation.ADTIsEmpty();
        }
        
        static final long serialVersionUID = 2832025079373813026L;
   }
