// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
package matrix.structures.CDT.probe;

import matrix.structures.CDT.CDT;
import matrix.structures.CDT.interfaces.PrioritySupplied;
import matrix.structures.FDT.*;
import matrix.structures.FDT.probe.Key;
import matrix.structures.FDT.probe.LinkedListImpl;

public class OrderedList extends LinkedListImpl implements CDT, matrix.structures.ADT.PriorityQueue {
    
    public OrderedList() {
        super();
    }
    
    public CDT getNewInstance(){
        return new OrderedList();
    }
    
    
    public OrderedList(Object key) {
        super();
        setElement(key);
    }
    
    /**
     * Returns new (List) node with a given key. 
     *
     * @param key key of new node
     * @return new (List) Node
     */    
    public LinkedList getNewNode(Object key) {
        return new OrderedList(key);
    }

    //------ADT PriorityQueue Intf ------//
    public boolean ADTIsEmpty(){
        return this.getNext() == null;
    }
    
    public void ADTinsert(PrioritySupplied o){
        insert(o); 
    }

    public PrioritySupplied ADTdeleteMin(){
        
        if (this.getNext() != null){
            PrioritySupplied returned = (PrioritySupplied)this.getNext().getElement();
            this.setNext(this.getNext().getNext());            
            return returned;
        }        
        return null;
    }

    
    /**
     * Inserts a new key to the correct position of the queue.
     *
     * @param key new key
     * @return queue after insertion
     */
    public CDT insert(Object key) {
        
        LinkedList cursor = this;
        
        // Elements with null priority elements, 
        // elements without priority elements
        // and null elements are discarded  
        if ((key instanceof PrioritySupplied && ((PrioritySupplied)key).getPriorityElement()==null)
                ||!(key instanceof PrioritySupplied) 
                || key == null)
            return this;
        
        while(cursor.getNext() != null 
                && ((PrioritySupplied)cursor.getNext().getElement()).getPriorityElement().leq(((PrioritySupplied)key).getPriorityElement()))
            cursor = cursor.getNext();
        
        LinkedList newNode = new OrderedList(key);
        newNode.setNext(cursor.getNext());
        cursor.setNext(newNode);        
        
        return this;
    }
    
    /**
     * This method deletes the node containing the given key
     */
    public CDT delete(Object o) {
        
        
        if (o == null || ((Key)o).eq(Key.Empty)){ //ADT DeleteMin
            if (this.getNext() != null)
                this.setNext(this.getNext().getNext());
        }
        else{ // Delete by key value until the proper key is found
            LinkedList cursor = this;
            while(cursor.getNext() != null 
                    && ((Key)cursor.getNext().getElement()).lt((Key)o))
                cursor = cursor.getNext();
            
            if (cursor.getNext() != null &&
                    ((Key)cursor.getNext().getElement()).eq((Key)o)){
                this.setNext(cursor.getNext().getNext());
            }
            
        }
            
        return this;
    }
    
    /**
     * Searching is not implemented for Ordered Queues
     */        
    public Object search(Object o) {
        return null;
    }
    
        
    static final long serialVersionUID = 6794728716870858032L;
}
