// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/* $Id: StackImpl.java,v 1.14 2007/09/07 13:37:03 vkaravir Exp $ */

package matrix.structures.CDT.probe;

import matrix.structures.CDT.CDT;
import matrix.structures.FDT.FDT;
import matrix.structures.FDT.probe.*;
import matrix.structures.memory.*;
import matrix.structures.util.MatrixComparable;
import matrix.structures.FDT.LinkedList;

/**
 * Implementation of a stack. This stack is implemented by using LinkedlistImpl.
 * This stack contains always a reference to the LinkedListImpl which is on the
 * top of the stack.
 * 
 * @author kstalnac
 */

public class StackImpl extends LinkedListImpl implements CDT, matrix.structures.ADT.Stack{

	/**First node of the stack. This has to be VirtualObject because otherwise 
	 * animation doesn't work. */
	protected VirtualObject firstNode;

	/**
	 * Constructor. Constructs StackImpl.
	 *
	 */
	public StackImpl() {
		super();
		firstNode = new VirtualObject(new LinkedListImpl(), this, "first node of the stack");
	}
	
    public CDT getNewInstance(){
        return new StackImpl();
    }

	/**
	 * Constructor
	 * @param key element to be saved into a stack
	 */
	public StackImpl(Object key) {
		super();
		firstNode = new VirtualObject(new LinkedListImpl(), this, "first node of the stack");
		setElement(key);
	}

	/**
	 * Sets the element of the top LinkedListImpl of the stack
	 * @see LinkedListImpl
	 * @param o element to be set on the top of the stack.
	 */
	public void setElement(Object o) {
		((FDT) firstNode.getObject()).setElement(o);
	}

	/**
	 * Returns the element from the top LinkedListImpl of the stack.
	 * @see LinkedListImpl
	 * @return element from the top LinkedListImpl of the stack
	 */
	public Object getElement() {
		return ((FDT) firstNode.getObject()).getElement();
	}

	/**
	 * Returns the next LinkedListImpl from top LinkedListImpl.
	 * @see LinkedListImpl.
	 * @return next LinkedListImpl
	 */
	public LinkedList getNext() {
		return ((LinkedList) firstNode.getObject()).getNext();
	}

	/**
	 * Returns true if there is a next LinkedListImpl.
	 * @see LinkedListImpl
	 * @return true if next LinkedListImpl exists
	 */
	public boolean hasNext() {
		return ((LinkedList) firstNode.getObject()).hasNext();
	}

	/**
     * Inserts a new key to the top of the stack and returns the structure
     * after insertion.
     * 
     * @param key new key
     * @return stack after insertion
     */
	public CDT insert(Object key) {
		if (key == null)
			return this;
		if (key instanceof Key && ((Key) key).toString() == "")
			return this;

		//stack is empty
		if (((FDT) firstNode.getObject()).getElement() == null) {
			((FDT) firstNode.getObject()).setElement(key);
			return this;
		}

		if (key instanceof MatrixComparable) {
			LinkedListImpl newNode = new LinkedListImpl();
			newNode.setElement(key);
			newNode.setNext((LinkedListImpl) firstNode.getObject());
			firstNode.setObject(newNode);
			return this;
		}
		if (key instanceof Table) {
			Table t = (Table) key;
			for (int i = 0; i < t.size(); i++)
				insert(t.getObject(i));
			return this;
		}
		Key k = new Key(key);
		LinkedListImpl newNode = new LinkedListImpl();
		newNode.setElement(key);
		newNode.setNext((LinkedListImpl) firstNode.getObject());
		firstNode.setObject(newNode);
		return this;
	}

   /**
    * This method deletes the node containing the given key
    * and all the nodes before it from the stack and returns 
    * the stack after deletion.
    *
    * @param o key to be deleted
    * @return the stack after deletion
    */
	public CDT delete(Object o) {
		boolean isInStack = false;
		LinkedListImpl temp = (LinkedListImpl) firstNode.getObject();
		while (temp != null) {
			if (o instanceof Key) {
				if (temp.getElement() == ((Key) o).getOrig())
					isInStack = true;
				else if (temp.getElement() == o)
					isInStack = true;
			} else {
				if (temp.getElement() == o)
					isInStack = true;
			}
			temp = (LinkedListImpl) temp.getNext();
		}
		if (isInStack) { //key to be removed is in the stack
			LinkedListImpl t = (LinkedListImpl) firstNode.getObject();
			if (t.getNext() == null) { //only one node in the stack
				t.setElement(null);
				return this;
			} else {
				LinkedListImpl tmp = (LinkedListImpl) t.getNext();
				t.setNext(null);
				if (o instanceof Key) {
					if (t.getElement() == ((Key) o).getOrig())
						firstNode.setObject(tmp);
					else if (getElement() == o)
						firstNode.setObject(tmp);
					else {
						firstNode.setObject(tmp);
						return delete(o);
					}
					return this;
				}
			}
		}
		return this;
	}

	
	public Object search(Object o) {
		return this;
	}

   /**
    * Returns the top node of the stack
    *
    * @return node from the top of the stack
    */
	public LinkedListImpl getTopOfStack() {
		return (LinkedListImpl) firstNode.getObject();
	}

   /**
    * Tests whether the stack is empty or not.
    * 
    * @return true, if the stack is empty
    */
 	public boolean isEmpty() {
		if (((LinkedListImpl) firstNode.getObject()).getNext() == null
				&& ((LinkedListImpl) firstNode.getObject()).getElement() == null) {
			return true;
		} else {
			return false;
		}
	}

    
    
    /**
     * This method pushes the given element to the top of the stack.
     * @param element element to be pushed to the top of the stack
     */
        public void ADTPush(Object element) {       
            insert(element);
       }
    
    /**
     * This method pops (removes) the element from the top of the stack and
     * returns it.
     * @return object removed from the top of the stack, or null if stack
     * is already empty
     */
        public Object ADTPop() {
          if (!ADTIsEmpty()) {
             Object top = getTopOfStack().getElement();
             delete(top);
             
             //Keys must be unwrapped - ask Ari about this
             if (top instanceof Key)
                 return ((Key)top).getObject();
             return top;
          }
          return null;
       }
    
    /**
     * Returns true if the stack is empty.
     * @return true if the stack is empty
     */
        public boolean ADTIsEmpty() {
          return isEmpty();
       }
    
    /**
     * This method returns the top element of the stack (without removing it).
     * @return top element of the stack, or null if stack is empty
     */
        public Object ADTPeek() {
          if (!isEmpty()) {
          /* First the Key object is fetched from the stack and then
           * the actual stored object is returned from the Key object. */
             Object top = getTopOfStack().getElement();
             
             //Keys must be unwrapped - ask Ari about this
             if (top instanceof Key)
                 return ((Key)top).getObject();
             return top;
          }
          return null;
       }
    

   /*
    Code from Stack
           
         public Stack(int[] key) {
      
         super();
         
         for (int i=0;i<key.length;i++){
         
            push(new Key(key[i]));
         
         }
         
      }

    */
        
        
 	static final long serialVersionUID = -1950783367597328219L;
}
