// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/* $Id: Graph.java,v 1.14 2007/09/07 13:41:47 vkaravir Exp $ */

package matrix.structures.FDT;

import matrix.structures.FDT.substructures.Vertex;


/** Common interface for all graphs.
 * Let V (vertices) be a finite set, and
 * let E (edges) be the subset of V x V. Then
 * G = (V,E) is called a (directed) graph.
 * In addition, G = (V,E,W) is a weighted graph in which
 * each edge e in E has an integer valued weight.
 * An ordinary graph has weights w in {0,1}.
 * @author Ari Korhonen, Riku Saikkonen, Jussi Nikander
 * @version $Revision: 1.14 $
 */
public interface Graph extends FDT {

    /**
     * Returns the nodes in the graph.
     * Must return at least the nodes from which all of the graph can
     * be reached; can return all nodes.
     * @return a Vertex array containg the nodes of the graph
     */
    public Vertex[] getVertices();

    /**
     * Set all the vertices of this graph.
     * @param v the new set of vertices
     */
    public void setVertices(Vertex[] v);

    /**
     * Inserts a new Vertex into the graph.
     * @param o the key of the new Vertex to be inserted
     */
    public void addVertex(Object o);


    /**
     * deletes a Vertex from the graph
     * @param v the vertex to be deleted
     */
    public void removeVertex(Vertex v);

    static final long serialVersionUID = 3068127734456542043L;
}
