// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/* $Id: LinkedList.java,v 1.16 2007/09/07 13:41:47 vkaravir Exp $ */

package matrix.structures.FDT;

/**
 * Interface for dynamic linked lists (lists consisting of
 * several LinkedList -type objects) for matrix.
 * This interface represents a normal linked list, where
 * each instance of LinkedList is a single node in the list.
 * Each list node contains a key and a pointer to the next node
 */

public interface LinkedList extends FDT {

    /**
     * Returns the next node in the list.
     * @return the successor of this list node
     */
    public LinkedList getNext();

    /**
     * Sets the next node in the list.
     * @param l the new successor node
     */
    public void setNext(LinkedList l);

    /**
     * Tells if this node has a successor node.
     * @return true if there is a successor node, false otherwise
     */
    public boolean hasNext();

    /**
     * Creates a new list node.
     * @return the new node
     */
    public LinkedList getNewNode(Object key);


    static final long serialVersionUID = -4824055583985702201L;
}
