// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/* $Id: StaticGraph.java,v 1.3 2007/09/07 13:42:39 vkaravir Exp $ */
/* 24/05/2006 moved from package matrix.structures.FDT (vkaravir) */ 

package matrix.structures.FDT.indexed;


/**
 * Interface for static graphs (i.e. adjacency matrix).
 * The interface extends the StaticStructure interface that enables
 * the primitive simulation.
 * The nodes of the graph are numbered from 0 to N-1, and
 * - the method getKey(i) returns the key value of the node i;
 * - the method getSuccessors(i) returns the corresponding successors of i.
 */
public interface StaticGraph extends StaticFDT {

    /** Determines the number of nodes within this tree.
     * The nodes should be numbered from 0 to N-1.
     * @return N
     */
    public int getNrOfNodes();

    /**
     * Retuns the indices of successing nodes of the node i.
     *@return the indices of successing nodes of the node i.
     */
    public int[] getSuccessors(int index);

    /**
     * Adds a new node to this graph.
     * @return the number of the new node or -1 if failed
     */
    public int addVertex();
    
    /**
     * Removes a node from this graph.
     */
    public void removeVertex(int vertex); 

    /**
     * Adds a new successor in [0,getNrOfNodes()-1] for a given node.
     */
    public void addSuccessor(int vertex, int successor);

    /**
     * Removes a successor (if exists) from this graph.
     */
    public void removeSuccessor(int vertex, int successor); 

    long serialVersionUID = -3939817460584842953L;
}
