// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/* $Id: BinTrei.java,v 1.22 2007/09/10 08:13:16 vkaravir Exp $ */

package matrix.structures.FDT.probe;

import matrix.structures.FDT.*;
import matrix.structures.memory.*;
import matrix.structures.other.Comparer;
import matrix.util.*;

public class BinTrei implements BinaryTree, Array {
    protected VirtualArray bt;
    protected int curr = 0;
    public static final String EMPTY=" ";
    public static final boolean DEBUG = false;
    
    public BinTrei() {
        bt = new VirtualArray(this, "Array representation of the tree");
    }
    
    public BinTrei(Object obj) {
        bt = new VirtualArray(this, "Array representation of the tree");
        setElement(obj);
    }
    
    public BinTrei(Object[] objs) {
        bt = new VirtualArray(this, "Array representation of the tree");
        for (int i=0; i<objs.length; i++)
            bt.setObject(objs[i], i);
    }

    public BinTrei(int i) {
        this();
        for(int j = 0; j < i; j++) {
            bt.setObject(new Key(EMPTY), j);
        }
    }

    // Private constructor for compatibility with BinaryTree
    private BinTrei(VirtualArray bt, int curr) {
        this.bt = bt;
        this.curr = curr;
    }

    // FDT interface Methods
    public FDT getNewStructure() { return new BinTrei(); }

    // CDT (not actually defined as an interface)

    // Swap node i with node j
    public void swap(int i, int j) {
        if (DEBUG)
            Note.out(this, "Swapping: " + i + " & " + j);
        bt.swap(i,j);
    }

    // Return current size of the tree
    public int size() { return bt.size(); }

    // True if pos is a leaf position
    public boolean isLeaf(int pos) {
        return (pos >= size()/2);
    }

    // set object into tree at pos
    public void setObject(Object obj, int pos) {
        Assert.notFalse(pos >= 0, "Illegal position");
        bt.setObject(obj, pos);
    }

    // get object from tree at pos
    public Object getObject(int pos) {
        Assert.notFalse(pos >= 0, "Illegal position");
        return bt.getObject(pos);
    }

    // Return position for left child of pos
    public int leftChild(int pos) {
        /* Assert.notFalse(pos < size()/2, "Position has no left child"); */
        return 2*pos + 1;
    }

    // Return position for right child of pos
    public int rightChild(int pos) {
        /* Assert.notFalse(pos < (size()-1)/2, "Position has no right child"); */
        return 2*pos + 2;
    }

    //Return position for parent
    public int parent(int pos) {
        Assert.notFalse(pos > 0, "Position has no parent");
        return (pos-1)/2;
    }

    // Compatibility with interface for BinaryTree

    public Object getElement() { return getObject(curr); }

    public void setElement(Object v) { setObject(v, curr); }
  /*
    public boolean equals(Object o) {
        if (o == null) return (getElement() == null);
        if (o instanceof BinTrei) {
            if (this == o) return true;
            BinTrei b = (BinTrei)o;
            if (bt.equals(b.getArray()))
                return (curr == b.getPos());
        }
        return false;
    }
  */
    public boolean equals(Object o) {
      if(o instanceof BinaryTree)
        return Comparer.compareIdentical(this, (BinaryTree)o);
      else
        return false;
    }

    BinaryTree left = null;
    public BinaryTree getLeft() {
	if (leftChild(curr) < size())
	    if (left == null) left = new BinTrei(bt, leftChild(curr));
	return left;
    }

    BinaryTree right = null;
    public BinaryTree getRight() {
	if (rightChild(curr) < size()) 
	    if (right == null) right = new BinTrei(bt, rightChild(curr));
	return right;
    }
    public boolean isLeaf() {
        return isLeaf(curr);
        //return (isLeaf(curr) || ((getLeft().getElement() == null) &&
        //                       (getRight().getElement() == null)));
    }

    public int getPos() { return curr; }

    // Methods for interface Array

    // public Object getObject(int i) {} /* allready implemented */
    public int getFirst() { return bt.getFirst(); }
    public int getLast() { return bt.getLast(); }
    public void setFirst(int i) { bt.setFirst(i); }
    public void setLast(int i) { bt.setLast(i); }

    // Methods for interface Tree
    public int getSubTreeCount() {
	if ((getLeft() == null) && (getRight() == null))
	    return 0;
	if ((getLeft() != null) && (getRight() == null))
	    return 1; 
	return 2;
    }
    public Tree[] getSubTrees() {
        Tree[] t = new Tree[getSubTreeCount()];
	for (int i=0; i<getSubTreeCount(); i++) 
	    t[i] = (i == 0) ? getLeft() : getRight();
        return t;
    }
    public int getSize() { return size(); }
    public int getSiblingCount() { return 0; }

    // Extra methods

    public void setLeft(BinaryTree p) {
        /* Assert.notFalse(p.isLeaf(), "Cannot insert subtree"); */
        bt.setObject(p != null ? p.getElement() : null, leftChild(curr));
        /* getLeft().setLeft(p.getLeft());
        getLeft().setRight(p.getRight()); */
    }

    public void setRight(BinaryTree p) {
        /* Assert.notFalse(p.isLeaf(), "Cannot insert subtree"); */
        bt.setObject(p != null ? p.getElement() : null, rightChild(curr));
        /* getRight().setLeft(p.getLeft());
        getRight().setRight(p.getRight()); */
    }

    public String toString() {
        return bt.toString();
    }

    // return n first elements in level order
    public String toString(int n) {
        return bt.toString(n);
    }

    static final long serialVersionUID = 5255951090283142099L;

    public void setSubTree(Tree t, int child) {
        // TODO Auto-generated method stub
        
    }
    public Tree getNewNode(Object key) {
        return new BinTrei(key);
    }
}
