// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/* $Id: CommonLabelTreeImpl.java,v 1.12 2008/07/17 09:39:33 lhakulin Exp $ */

package matrix.structures.FDT.probe;

import java.awt.Color;

import matrix.decoration.LabelDecorator;
import matrix.structures.memory.*;
import matrix.util.Note;

public class CommonLabelTreeImpl extends SimulationTree2Impl implements LabelDecorator {
    private String label;
    private VirtualArray referenceLabel;
    private boolean isLabelEnabled = true;
    private boolean isReferenceLabelEnabled = true;

    // construct an empty tree
    public CommonLabelTreeImpl(String label) {
        this(null, label);
    }

    // construct a new root node
    public CommonLabelTreeImpl(Object obj, String label) {
        super(obj);
        this.label = label;
        this.referenceLabel = new VirtualArray(this, "labels for references");
    }

    // operational interface

    public void setLabelVisible(boolean b) { isLabelEnabled = b; }
    public void setReferenceLabelVisible(boolean b) {
        isReferenceLabelEnabled = b;
    }

    // LabelDecorator

    public String getLabel() { return label; }
    public void setLabel(String s) { label = s; }
    public boolean isLabelEnabled() { return isLabelEnabled; }
    
    public void setLabelEnabled(boolean b) {
	Note.warning(this, "setLabelEnabled(boolean) not implemented.");
    }
    
    public void setReferenceLabel(int i, String s) {
        referenceLabel.setObject(s, i);
    }
    public String getReferenceLabel(int i) {
        return (String)referenceLabel.getObject(i);
    }
    public boolean isReferenceLabelEnabled() {
        return isReferenceLabelEnabled;
    }
    
    public void setReferenceLabelEnabled(boolean b) {
    	Note.warning(this, "setReferenceLabelEnabled(boolean) not implemented.");
    }
    
    public Color getLabelColor() {
    	// Return null if the color of the label is not defined
    	return null;
    }

    // toString

    public String toString() {
        if (getElement() instanceof Key)
            return getElement().toString();
        else {
            String s = "null";
            if (getElement() != null) s = getElement().getClass().toString();
            return super.toString() + "(" + s + ")";
        }
    }


    static final long serialVersionUID = -2538874300900565615L;
}
