// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/* $Id: CommonTreeImpl.java,v 1.11 2007/09/10 08:13:16 vkaravir Exp $ */

package matrix.structures.FDT.probe;

import matrix.structures.FDT.*;
import matrix.structures.memory.*;

/**
 *  A common tree implementation in which we have only two references: firstChild and nextSibling
 */

public class CommonTreeImpl implements FDT, CommonTree, Tree {
 
    private static final long serialVersionUID = -5125166318522040303L;
    
    private VirtualObject data;
    private VirtualObject child;
    private VirtualObject sibling;

//  private VirtualArray tree;

    /** The constructor for an empty tree. */
    public CommonTreeImpl() {
        data    = new VirtualObject(this, "data of the node");
        child   = new VirtualObject(this, "first child of the node");
        sibling = new VirtualObject(this, "next sibling of the node");
    }

    /** The constructor for a new root node. */
    public CommonTreeImpl(Object obj) {
        this();
        setElement(obj);
    }

    // CDT (not actually defined as an interface)

    /** Return the element value. */
    public Object getElement() {
        return data.getObject();
    }

    /** Set the element value. */
    public void setElement(Object v) {
        data.setObject(v);
    }

    // Common Tree

    public CommonTreeImpl getFirstChild() {
        return (CommonTreeImpl)child.getObject();
    }

    public void setFirstChild(CommonTreeImpl t) {
        child.setObject(t);
    }

    public CommonTreeImpl getNextSibling() {
        return (CommonTreeImpl)sibling.getObject();
    }

    public void setNextSibling(CommonTreeImpl t) {
        sibling.setObject(t);
    }


    /* Count the number of elements in the siblings list */
    private int countElements(CommonTreeImpl t) {
        if (t == null)
            return 0;
        return 1 + countElements(t.getNextSibling());
    }

    // Methods for interface Tree
    
    public int getSubTreeCount() {
        if (getElement() == null)
            return 0;
        return countElements(getFirstChild());
    }

    public Tree[] getSubTrees() {
        CommonTreeImpl[] t = new CommonTreeImpl[getSubTreeCount()];
        CommonTreeImpl nextSibling = getFirstChild();

        for (int i=0; i<t.length; i++) {
            t[i] = nextSibling;
            nextSibling = nextSibling.getNextSibling();
        }

        return t;
    }

    public void setSubTree(Tree t, int child) {
        if (child == 0) {
            CommonTreeImpl firstChild = getFirstChild();
            setFirstChild((CommonTreeImpl)t);
            ((CommonTreeImpl)t).setNextSibling(firstChild);
        } else { // child > 0
            CommonTreeImpl next = getFirstChild();
            int i=0;

            while (i < child-1) {
                next = next.getNextSibling();
                i++;
            }
            
            if (next.getNextSibling() != null) // delete next before set
                next.setNextSibling(next.getNextSibling().getNextSibling());

            ((CommonTreeImpl)t).setNextSibling(next.getNextSibling());
            next.setNextSibling((CommonTreeImpl)t);
        }
    }

    public Tree getNewNode(Object key) {
        return new CommonTreeImpl(key);
    }

    public String toString() {
        if (getElement() instanceof Key)
            return getElement().toString();
        else {
            String s = "null";
            if (getElement() != null)
                s = getElement().getClass().toString();
            return super.toString() + "(" + s + ")";
        }
    }

    public void insert(FDT s) {
        setSubTree(getNewNode(s), getSubTreeCount());
    }

}
