// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/* $Id: DirectedGraphImpl.java,v 1.18 2008/07/17 09:39:33 lhakulin Exp $ */

package matrix.structures.FDT.probe;

import java.awt.Color;

import matrix.structures.FDT.*;
import matrix.structures.FDT.substructures.Vertex;
import matrix.structures.memory.*;
import matrix.structures.other.Comparer;
import matrix.structures.util.MatrixComparable;
import matrix.util.Note;
import matrix.decoration.LabelDecorator;

/**
 * A very simple implementation of a graph. Cannot be visualised other than as a graph.
 * Supporst graph traversing algorithms.
 *
 * @author Jussi Nikander
 */

public class DirectedGraphImpl implements FDT, Graph, LabelDecorator {

    /** graph vertices are stored here */
    private VirtualArray vertices;

    public DirectedGraphImpl() {
        super();
        vertices = new VirtualArray(this, "vertices of the graph");
    }

    public VertexImpl getNewVertex(Object o) {
        VertexImpl v = new VertexImpl(o);
        v.setReferenceLabelEnabled(isReferenceLabelEnabled());
        return v;
        //return new VertexImpl(o);
    }

    // interface Traverse

    public void traverse() {
        //Depth_First_Search.dfs();
    }

    public boolean isVisited() {
        return false;
    }

    public void setVisited(boolean b) {
    }

    public String[] getTraversingMethods() {
        return new String[] { "DFS" };
    }

    public void setTraversingMethod(String s) {
    }

    // interface FDT

    public Object getElement() {
        return this;
    }

    public void setElement(Object o) {
        if (o instanceof Array){
            Array a = (Array)o;
            
            for(int i = a.getFirst(); i < a.getLast(); i++) {
                vertices.setObject(a.getObject(i), i-a.getFirst());                
            }
            
            vertices.setLast(a.getLast() - a.getFirst());
        }else{
            if (vertices.size() == 0)
                addVertex(o);
            else
                ((Vertex)vertices.getObject(0)).setElement(o);
        }
    }

    // interface SimulationGraph

    public Vertex[] getVertices() {
        Vertex[] tmp = new Vertex[vertices.size()];
        for(int i = 0; i < vertices.size(); i++)
            tmp[i] = (Vertex)vertices.getObject(i);
        return tmp;
    }

    public void setVertices(Vertex[] v) {
    	int i=0;
        
        // put the input in the first positions
        for(; i < v.length; i++)
            vertices.setObject(v[i], i);
        
        // null out the rest of the array positions
        for(; i < vertices.size(); i++)
            vertices.setObject(null, i);
    }

    public void addVertex(Object o) {
        VertexImpl vi = null;
        if(o instanceof VertexImpl)
          vi = (VertexImpl)o;
        else if(o instanceof MatrixComparable){
            vi = getNewVertex(o);
        }
        else if(o instanceof FDT) {
            vi = getNewVertex(new Key(((FDT)o).getElement()));
        }
        else
            vi = getNewVertex(new Key(o));
        for(int i =  0; i < vertices.size(); i++) {
          if(vertices.getObject(i) == null) {
            vertices.setObject(vi, i);
            return;
          }
        }
        vertices.setObject(vi, vertices.size());
    }

    public void removeVertex(Vertex v) {
        int i;
        Note.out(this, "deleting, size " + vertices.size());
        for(i = 0; i < vertices.size(); i++) {
          if(((Vertex)vertices.getObject(i)).equals(v))
                break;
        }
        for(;i < vertices.size(); i++)
            vertices.setObject(vertices.getObject(i + 1), i);
    }


    // interface LabelDecorator
    private boolean isLabelEnabled = false;
    private boolean isReferenceLabelEnabled = false;
    private String label = "";
    private String referenceLabel = "";

    public boolean isLabeled() { return isLabelEnabled();}

    public boolean isLabelEnabled() { return isLabelEnabled;}
    
    public void setLabelEnabled(boolean b) {
    	isLabelEnabled = b;
    }
    
    public void setLabel(String s) {label = s;}

    public String getLabel() { return label;}

    public boolean isReferenceLabelEnabled() {
        return isReferenceLabelEnabled;
    }
    
    public void setReferenceLabelEnabled(boolean b) {
        isReferenceLabelEnabled = b;
        for (int i = 0; i < vertices.size(); i++) {
            ((LabelDecorator)vertices.getObject(i)).setReferenceLabelEnabled(isReferenceLabelEnabled);
        }
    }
    
    public void setReferenceLabel(int i, String s) {
        referenceLabel = s;
    }
    
    public String getReferenceLabel(int i) {
        return referenceLabel;
    }
        
    public Color getLabelColor() {
    	// Return null if the color of the label is not defined
    	return null;
    }

    
    public boolean equals(Object o) {       
        if(o instanceof Graph) {
            return equals((Graph)o);
        }
        return false;
    }
    
    public boolean equals(Graph g) {
        return Comparer.compareGraph(this, g);
    }


    static final long serialVersionUID = -7993064623641268504L;
}
