/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package matrix.structures.FDT.probe;

import matrix.decoration.MatrixDecorator;
import matrix.structures.FDT.Array;
import matrix.structures.FDT.Matrix;
import matrix.structures.FDT.advanced.StyledArray;
import matrix.util.Note;

public class MatrixImpl extends VanillaTable implements Matrix, StyledArray {
    private int rows, cols;
    private String rowIndexes[]; // rows
    private String colIndexes[]; // cols 
    
    public MatrixImpl(int r, int c) {
        this.rows = r;
        this.cols = c;
        rowIndexes = new String[rows];
        colIndexes = new String[cols];
        for (int i=0; i<r; i++) {
            super.setObject(new MatrixComponent(c), i);
            rowIndexes[i] = ""+i;
        }
        for (int i=0; i<c; i++)
            colIndexes[i] = ""+i;
    }

    public MatrixImpl() {
        this(5,5);
    }
    
    // public interface

    public Object getObject(int row, int col) {
        VanillaTable vt = (VanillaTable) getObject(row);
        return vt.getObject(col);
    }

    public void setObject(Object o, int row, int col) {
        Array va = (Array)getObject(row);
        va.setObject(o, col);
    }

    public int getColumns() {
        return this.cols;
    }

    public int getRows() {
        return this.rows;
    }

    public void setColumns(int i) {
        this.cols = i; 
    }

    public void setRows(int r) {
        if (r > rows) { // enlarge indexes as well
            String[] oldIndexes = new String[rows];
            for (int i=0; i<rows; i++)
                oldIndexes[i] = rowIndexes[i];
            rowIndexes = new String[r];
            for (int i=0; i<r; i++)
                if (i<rows)
                    rowIndexes[i] = oldIndexes[i];
                else
                    rowIndexes[i] = ""+i;
        }
        this.rows = r;
    }
    
    // interface FDT

    public Object getElement() {
        return this;
    }
    public void setElement(Object o) {
        Note.err(this, "setElement not defined for FDT Matrix " + this);
    }

    // interface Array
    /*
    public Array getArray() {
        return (Array) this;
    }
    public Object getObject(int address) {
        return getObject(address);
    }
    */
    public void setObject(Object o, int address) {
        Note.show(this, "Cannot change a whole row/column of Matrix");
    }
    /*
    public int getFirst() {
        return getFirst();
    }
    public int getLast() {
        return getLast();
    }
    */
    public void setFirst(int i) {
        Note.err(this, "setFirst not defined for FDT Matrix " + this);
    }
    public void setLast(int i) {
        Note.err(this, "setFirst not defined for FDT Matrix " + this);
    }
    
    public String getIndexName(int address) {
        return rowIndexes[address];
    }
    
    public void setIndexName(String name, int address) {
        rowIndexes[address] = name;
    }
  
    public String getColumnIndexName(int address) {
        return colIndexes[address];
    }
    
    public void setColumnIndexName(String name, int address) {
        colIndexes[address] = name;
    }
    /*
    public String[] getXAxisLabels() {
        String[] s = new String[cols];
        for (int i=0; i<cols; i++)
            s[i] = "index "+i;
        return s;
    }

    public String[] getYAxisLabels() {
        String[] s = new String[cols];
        for (int i=0; i<rows; i++)
            s[i] = "index "+i;
        return s;
    }
    */
    
    class MatrixComponent extends VanillaTable implements StyledArray {
  
        public MatrixComponent(int c) {
            super(c);
        }
        
        public String getIndexName(int address) {
            return "//?//";
        }
        
        public boolean showIndexes() {
            return false;
        }

        /**
         * 
         */
        private static final long serialVersionUID = -4856584258937954176L;
    }
    
    /**
     * 
     */
    static final long serialVersionUID = -8996895218423328167L;
}
