// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/* $Id: GraphAdapter.java,v 1.24 2008/07/17 09:39:34 lhakulin Exp $ */

package matrix.structures.adaptation;

import java.awt.Color;

import matrix.structures.FDT.*;
import matrix.structures.coordinates.*;
import matrix.structures.FDT.advanced.GeometricGraph;
import matrix.structures.FDT.indexed.StaticGraph;
import matrix.structures.FDT.probe.VertexImpl;
import matrix.structures.FDT.substructures.Vertex;
import matrix.util.Note;
import matrix.decoration.LabelDecorator;
import matrix.decoration.ReferenceInformationDecorator;
import matrix.decoration.StaticInformationDecorator;


/** This adapter blasts static graphs into dynamic graphs in order to
 * visualize them through Graph interface.
 */
public class GraphAdapter implements FDT, LabelDecorator, Graph, GeometricGraph {
    StaticGraph sgraph;
    StaticXYCoordinates ggraph;

    Vertex[] vertices = null;

    public GraphAdapter () {
	/*
	sgraph = new matrix.structures.FDT.probe.GeometricGraph(0);
	*/
	vertices = new Vertex[3];
	vertices[0] = new VertexImpl("1");
	vertices[1] = new VertexImpl("2");
	vertices[2] = new VertexImpl("3");
    }
    
    public GraphAdapter(StaticGraph a) {
	super();
	this.sgraph = a;
	if (a != null)
	    update();
	if (a instanceof StaticXYCoordinates)
	    ggraph = (StaticXYCoordinates)a;
	else
	    ggraph = new XYCoordinatesAdapter(a.getNrOfNodes());

    }
    
    public GraphAdapter(matrix.structures.FDT.probe.GeometricGraph a) {
	this.sgraph = a;
	if (a != null)
	    update();
	if (a instanceof StaticXYCoordinates)
	    ggraph = (StaticXYCoordinates)a;
	else
	    ggraph = new XYCoordinatesAdapter(a.getNrOfNodes());
    }

    class MyVertexImpl extends VertexImpl implements ReferenceInformationDecorator {
	int index;
	GraphAdapter adapter;
	boolean info;
	boolean toolTip;
	boolean refInfo;
	boolean refToolTip;
	
	public MyVertexImpl(int index) {
	    super();
	    this.index=index;
	    setLabelEnabled(false);
	    setReferenceLabelEnabled(false);
	    if(sgraph instanceof StaticInformationDecorator) {
	    	StaticInformationDecorator sid = (StaticInformationDecorator)sgraph;
	    	info = sid.isInfoBoxEnabled();
	    	toolTip = sid.isToolTipInfoEnabled();
	    	refInfo = sid.isReferenceInfoBoxEnabled();
	    	refToolTip = sid.isReferenceToolTipInfoEnabled();
	    }
	}

/**
 * Called by update to add new successors to the adapted structure.
 * @param v
 */
	public void createSuccessor(Vertex v) {
		super.addSuccessor(v);
	}
	
/**
 * Called by visualization to add new edge to the structure.
 */
	public void addSuccessor(Vertex v) {
	    super.addSuccessor(v);
	    GraphAdapter.this.addSuccessor(this, v);
	}

/**
 *  Called by visualization to remove an edge from the structure.
 */
	public void removeSuccessor(Vertex v) {
	    //Note.out(this, "CALL removeSuccessor(Vertex v)");
	    super.removeSuccessor(v);
	    GraphAdapter.this.removeSuccessor(this, v);
	}

	public Object getElement() {
	    return getKey(index);
	}

	public void setElement(Object o) {
	    super.setElement(o);
	    setKey(index, o);
	}

	public boolean isReferenceInfoBoxEnabled() {
		return refInfo;
	}

	public boolean isReferenceToolTipInfoEnabled() {
		return refToolTip;
	}

	public String getReferenceInformation(int i) {
		if(refInfo || refToolTip) {
			return ((StaticInformationDecorator)ggraph).getReferenceInformation(index,i);
		}
		return null;
	}

	public void setReferenceInformation(int i, String s) {
		if(refInfo || refToolTip) {
			((StaticInformationDecorator)ggraph).setReferenceInformation(index,i,s);
		}
	}

	public boolean isInfoBoxEnabled() {
		return info;
	}

	public boolean isToolTipInfoEnabled() {
		return toolTip;
	}

	public void setInfo(String s) {
		if(info || toolTip) {
			((StaticInformationDecorator)ggraph).setInfo(index, s);
		}
	}

	public String getInfo() {
		if(info || toolTip) {
			return ((StaticInformationDecorator)ggraph).getInfo(index);
		}
		return null;
	}

	public int getWaitTime() {
		if(info) {
			return ((StaticInformationDecorator)ggraph).getWaitTime();
		}
		return 0;
	}

	public void setWaitTime(int t) {
		if(info) {
			((StaticInformationDecorator)ggraph).setWaitTime(t);
		}
	}
    }
    
    /* Check whether the current graph equals the underlying data structure. */
    private boolean areEquals() {
	int n=sgraph.getNrOfNodes();
	if ((vertices == null) || (n != vertices.length)) // chech that the number of nodes match
	    return false;
	for (int i=0; i<n; i++) {
	    if (sgraph.getKey(i) != vertices[i].getElement()) // check nodes and keys
		return false;
	    int[] successors = sgraph.getSuccessors(i);
	    Vertex[] vsuccessors = vertices[i].getSuccessors(); // for each node 
	    if (successors.length != vsuccessors.length) // check that the number of edges match
		return false;
	    for (int j=0; j<successors.length; j++) 
		if (vertices[successors[j]] != vsuccessors[j]) // check successors
		    return false;
	}
	//Note.out(this, "EQUAL - NO ACTION");
	return true;
    }

    /**
     * Public method for forcing the adapter to update itself according
     * to the underlying data structure.
     *
     */
    public void updateNotify() { update(); }
    
    /* Create a dynamic graph by "blasting" the given static graph. */
    private void update() {
	if (areEquals())
	    return;

	//Note.out(this, "UPDATE");

	this.vertices = new Vertex[sgraph.getNrOfNodes()];
	for (int i=0; i<sgraph.getNrOfNodes(); i++) {
	    vertices[i] = new MyVertexImpl(i);
	    vertices[i].setElement(sgraph.getKey(i));
	}
	for (int i=0; i<sgraph.getNrOfNodes(); i++) {
	    int[] successors = sgraph.getSuccessors(i);
	    //Note.out(this, i+" has "+successors.length+" successors");
	    for (int j=0; j<successors.length; j++)
		((MyVertexImpl)vertices[i]).createSuccessor(vertices[successors[j]]);
	}
    }
    
    // interface Traverse

    public void traverse() {
        //Depth_First_Search.dfs();
    }

    public boolean isVisited() {
        return false;
    }

    public void setVisited(boolean b) {
    }

    public String[] getTraversingMethods() {
        return new String[] { "DFS" };
    }

    public void setTraversingMethod(String s) {
    }

    // interface FDT

    public Object getElement() {
        return null;
    }

    public void setElement(Object o) {
	Note.err(this, "Not implemented, yet");
    }

    // interface SimulationGraph

    public Vertex[] getVertices() {
	return vertices;
    }

    public void setVertices(Vertex[] v) {
	for (int i=0; i<sgraph.getNrOfNodes(); i++)
	    removeVertex(0);
	for (int i=0; i<v.length; i++)
	    addVertex(v[i].getElement());
	update();
    }

    public void addVertex(Object o) {
	//Note.out(this, "ADD VERTEX " + o);
	if (sgraph != null) {
	    int i = addVertex();
	    setKey(i,o);
	    update();
	} else
	    Note.warning(this, "Static graph not initialized.");
    }

    public void removeVertex(Vertex v) {
	//Note.out(this, "removeVertex");
	if (sgraph != null) {
	    int x = getVertexNum(v);
	    if (x >= 0) {
		removeAllSuccessors(x);
		update();
		/*removeAllPredecessors(x);
		  update();*/
		removeVertex(x);
		update();
	    }
	    else
		Note.warning(this, "Cannot find vertex "+v);
	} else
	    Note.warning(this, "Static graph not initialized.");
    }

    private void removeAllSuccessors(int x) {
	//Note.out(this, "remove all successors for " + x);
	int[] succ = sgraph.getSuccessors(x);
	for (int i=succ.length; i>0; i--)
	    removeSuccessor(x, succ[i-1]);
    }

    private void removeAllPredecessors(int node) {
	//Note.out(this, "remove all predecessors for " + node);
	for (int x=0; x<sgraph.getNrOfNodes(); x++) {
	    int[] succ = sgraph.getSuccessors(x);
	    for (int i=0; i<succ.length; i++)
		if (succ[i] == node)
		    removeSuccessor(x, succ[i]);
	}
    }

    private int getVertexNum(Vertex v) {
	for (int i=0; i<vertices.length; i++) {
	    if (vertices[i] == v)
		return i;
	}
	return -1;
    }

    public void addSuccessor(Vertex u, Vertex v) {
	if (sgraph != null) {
	    int x = getVertexNum(u);
	    int y = getVertexNum(v);
	    if (x >= 0) {
		if (y >= 0) { 
		    addSuccessor(x,y);
		} else
		    Note.warning(this, "Cannot find vertex "+v);
	    } else
		Note.warning(this, "Cannot find vertex "+u);
	} else
	    Note.warning(this, "Static graph not initialized.");
    }
    
    void removeSuccessor(Vertex u, Vertex v) {
	if (sgraph != null) {
	    int x = getVertexNum(u);
	    int y = getVertexNum(v);
	    if (x >= 0) {
		if (y >= 0) {
		    removeSuccessor(x,y);
		} else
		    Note.warning(this, "Cannot find vertex "+v);
	    } else
		Note.warning(this, "Cannot find vertex "+u);
	} else
	    Note.warning(this, "Static graph not initialized.");
    }

    // interface LabelDecorator
    private boolean isLabelEnabled = false;
    private boolean isReferenceLabelEnabled = false;
    private String label = "";
    private String referenceLabel = "";

    public boolean isLabeled() { return isLabelEnabled();}

    public boolean isLabelEnabled() { return isLabelEnabled;}
    
    public void setLabelEnabled(boolean b) {
    	isLabelEnabled = b;
    }
    
    public void setLabel(String s) {label = s;}
    public String getLabel() { return label;}
    public boolean isReferenceLabelEnabled() {
	return isReferenceLabelEnabled;
    }
    
    public void setReferenceLabelEnabled(boolean b) {
	isReferenceLabelEnabled = b;
    }
    
    public void setReferenceLabel(int i, String s) {
	referenceLabel = s;
    }

    public String getReferenceLabel(int i) {
	return referenceLabel;
    }

    // interface GeometricGraph, ie, XYCoorinates
    public int getX(Vertex vertex) { return ggraph.getX(getVertexNum(vertex)); }
    public int getY(Vertex vertex) { return ggraph.getY(getVertexNum(vertex)); }
    public void setCoordinates(Vertex vertex, int x, int y) {
	ggraph.setCoordinates(getVertexNum(vertex), x, y);
    }

    // interface StaticGraph (if ever somebody implements a visualization directly for this)
    // Just let all the information go through this adapter
    public Object getKey(int index) { return sgraph.getKey(index); }
    public void setKey(int index, Object o) { sgraph.setKey(index,o); }
    public int getNrOfNodes() { return sgraph.getNrOfNodes(); }
    public int[] getSuccessors(int index) { return sgraph.getSuccessors(index); }
    public int addVertex() { return sgraph.addVertex(); }
    public void removeVertex(int vertex) { 
	//Note.out(this, "CALL removeVertex for " + vertex);
	int oldNrOfNodes = sgraph.getNrOfNodes();
	sgraph.removeVertex(vertex); 
	/* The following code checks whether the underlying structure is behaving soundly */
	if ((vertex < oldNrOfNodes) && (oldNrOfNodes == sgraph.getNrOfNodes() - 1)) {
	    //Note.out(this, "removeVertex succesful for " + sgraph);
	    return;
	}
	Note.warning(this, "removeVertex failed for " + sgraph);
    }
    public void addSuccessor(int vertex, int successor) { 
	//Note.out(this, "CALL addSuccessor for " + vertex + "("+successor+")");
	sgraph.addSuccessor(vertex, successor); 
	/* The following code checks whether the underlying structure is behaving soundly */
	int[] succ = sgraph.getSuccessors(vertex);
	for (int i=0; i<succ.length; i++)
	    if (succ[i] == successor) {
		//Note.out(this, "addSuccessor succesful for " + sgraph);
		return;
	    }
	Note.warning(this, "addSuccessor failed for " + sgraph);
    }
    public void removeSuccessor(int vertex, int successor) { 
	//Note.out(this, "CALL removeSuccessor for " + vertex + "("+successor+")");
	sgraph.removeSuccessor(vertex, successor); 
	/* The following code checks whether the underlying structure is behaving soundly */
	int[] succ = sgraph.getSuccessors(vertex);
	for (int i=0; i<succ.length; i++)
	    if (succ[i] == successor) {
		Note.warning(this, "removeSuccessor failed for " + sgraph);
		return;
	    }
	//Note.out(this, "removeSuccessor successful for " + sgraph);
    }

    public Color getLabelColor() {
    	// Return null if the color of the label is not defined
    	return null;
    }

}
