// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package matrix.structures.adaptation;

import matrix.structures.FDT.Tree;

public class TreeNodeAdapter extends Adapter implements Tree {
    private TreeNodeAdapter[] children;
    private Object key;

    public TreeNodeAdapter(Object key) {
        matrix.util.Note.out(this, "new TreeNodeAdapter " + this);
        setElement(key);
    }

    // Adapter methods

    private int childCounter = 0;

    /** This method replaces the children table with new possibly empty table.
     * The number of children for this node is children.length.
     * The addChild method is used to fill this table one new child node at a time.
     */
    public void setChildren(TreeNodeAdapter[] children) {
        childCounter = 0;
        this.children = children;
    }

    /** This methos is used to fill the table of children for this node.
     * This method should be invoked N times after setChildren method invokation where
     * N is the number of children for this node.
     */
    public void addChild(TreeNodeAdapter newChild) {
        if (newChild == this) return;
        if (childCounter < children.length)
            children[childCounter++] = newChild;
        else
            matrix.util.Note.err(this, "Too meny children ("+childCounter+") for " + this);
    }

    public String getName() {
        return getElement().toString();
    }

    // Interface FDT

    /**
     * @return the key of this Node.
     */
    public Object getElement() {
        return key;
    }

    /**
     * Sets the key of this Node.
     * @param o the new value of this Element
     */
    public void setElement(Object o) {
        key = o;
    }

    // Interface Tree

    /**
     * Returns the number of child nodes this node has.
     * @return the number of subtrees
     */
    public int getSubTreeCount() {
        return childCounter;
    }

    /**
     * Return a Tree table containing the subtrees of this
     * tree node. Empty subtrees return null.
     * @return a table of Trees
     */
    public Tree[] getSubTrees() {
        return children;
    }

    static final long serialVersionUID = 8373620501859825791L;

    public void setSubTree(Tree t, int child) {
        // TODO Auto-generated method stub
        
    }

    public Tree getNewNode(Object key) {
        // TODO Auto-generated method stub
        return null;
    }
}
