// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
package matrix.structures.memory;

import matrix.animation.Animator;
import matrix.structures.FDT.FDT;

/***
 * Memory Structure for storing floating point numbers in double precision.
 * @author Jussi Nikander
 *
 */
public class VirtualDouble extends VirtualPrimitive {

	/**
	 * Default constructor, creates a VirtualFloat with value 0.
	 *
	 */
    protected VirtualDouble() {
		this(0);
	}

	protected VirtualDouble(double value) {
		super();
		primitiveValue = new Double(value);
	}

	public VirtualDouble(FDT fdt, String name) {
		this();
		setFDT(fdt);
		setName(name);
	}

	public VirtualDouble(double value, FDT fdt, String name) {
		this(value);
		setFDT(fdt);
		setName(name);
	}

    public double assign(double value) {
        Double newValue = new Double(value);
        Animator.getActiveAnimator().putAssignment(this, newValue, 0);
        primitiveValue = newValue;
        return value;
    }

    /**
	 * This is the same as the == operator for primitives. In case of double
	 * precision arithmetic, it is not terribly useful.
	 * @param b
	 * @return true if the values are exactly equal
	 */
	public boolean equals(VirtualDouble b) { return this.eval() == b.eval(); }

	public boolean equals(Object o) {
		if(o instanceof VirtualDouble) {
			return equals((VirtualDouble)o);
		}
		return false;
	}

	/**
	 *  Numeric equivalence with a given double value as threshold. Returns true
	 *  if the difference between the two values is less than or equals to the given threshold.
	 * @param b
	 * @param threshold
	 * @return true if the difference between this and the parameter VirtualDouble is less than the given threshold
	 */
	public boolean numericEquals(VirtualDouble b, double threshold) {
		return (this.eval() - b.eval()) <= threshold;
	}

	/**
	 *  Numeric equivalence with the given parameter VirtualDouble. The two are
	 *  considered equal if the difference between them is no more than 0.001.
	 * @param b
	 * @return
	 */
	public boolean numericEquals(VirtualDouble b) {
		return (this.eval() - b.eval()) <= 0.001;
	}

	public double eval() {
		return ((Double)primitiveValue).doubleValue();
	}
	
}
