// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/* $Id: VirtualInteger.java,v 1.20 2007/09/07 13:42:38 vkaravir Exp $ */

/**
 * Virtual integer to be used with matrix's animator. Substitute primitive ints with
 * this when dealing with <code>Animator</code>.
 * @see VirtualPrimitive
 *
 * @author Petri Tenhunen
 */


package matrix.structures.memory;

import matrix.animation.Animator;
import matrix.structures.FDT.FDT;

public final class VirtualInteger extends VirtualPrimitive {

    /** Default constructor to animated <code>VirtualInteger</code> with initial
     * value 0
     */
    protected VirtualInteger() {
        this(0);
    }


    /** Constructor with initial value and boolean information if object is animated
     * or not.
     * @param value  value to be assigned into a new object
     * @param animated true if and only if object is animated
     */
    protected VirtualInteger(int value) {
        super();
        primitiveValue = new Integer(value);
    }
    
    public VirtualInteger(FDT fdt, String name) {
        this();
        setFDT(fdt);
        setName(name);
    }
    
    public VirtualInteger(int value, FDT fdt, String name) {
        this(value);
        setFDT(fdt);
        setName(name);
    }

    /* FDT */
    /*
    public Object getElement() { return this; }
    public void setElement(Object o) { matrix.util.Note.err(this, "Cannot set element for primitive"); }
    */

    public int assign(int value) {
        Integer newValue = new Integer(value);
        Animator.getActiveAnimator().putAssignment(this, newValue, 0);
        //Animator.getAnimator().put(new Assignment(this, newValue, 0));
        primitiveValue = newValue;
        return value;
    }

    /** same as == operator with primitives
     * @return true if and only if this.eval() == b.eval() returns true
     * @param b Object to be compared with this object
     */
    public boolean equals(VirtualInteger b) { return this.eval() == b.eval(); }

    /**
     * Equals method. Tests the equality of this VirtualInteger and the object given as
     * parameter. Object is equal to this, if it is VirtualInteger and eval method of both
     * VirtualIntegers return same value.
     * 
     * @param o object which equality with this VirtualInteger is tested
     * @return True if object given as parameter is equal to this VirtualInteger. False
     * if they are not equal.
     */
    public boolean equals(Object o) {
        if (o instanceof VirtualInteger)
            return this.eval() == ((VirtualInteger)o).eval();
        else
            return false;
    }
    
    /** Greater than relation
     * @return true if and only if integer value of this object is greater
     * than integer value of argument
     * @param b Object to be compared with this object
     */
    public boolean gt(VirtualInteger b) { return (this.eval() > b.eval()); }

    /** Less than relation
     * @return true if and only if integer value of this object is less
     * than integer value of argument
     * @param b Object to be compared with this object
     */
    public boolean lt(VirtualInteger b) { return (this.eval() < b.eval()); }

    /** same as ++ operation with primitives. Add one to this object. */
    public int inc() {
        assign(eval() + 1);
        return eval();
    }

    /** same as -- operation with primitives.  */
    public int dec() {
        assign(eval() - 1);
        return eval();
    }

    /** Get primitive value of this object
     * @return primiteva value assigned to this object
     */
    public int eval() {
        return ((Integer)primitiveValue).intValue();
    }


    static final long serialVersionUID = 266412969556197248L;
}
