// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/* $Id: VirtualPrimitive.java,v 1.13 2007/09/07 13:42:38 vkaravir Exp $ */


package matrix.structures.memory;

import matrix.structures.FDT.FDT;


/**
 * <p>
 * Overclass to all virtual primitives. Virtual primitives are <code>Matrix's</code>
 * corresponding classes to primitive types. Vitual primitives can be used with
 * <code>Animator</code> to store object's state history.
 * </p><p>
 * In every VirtualPrimitive class extended from this <code>VirtualPrimitive</code>
 * there are <code>assign(primitive)</code> and <code>eval()</code> methods. Assign
 * stores new value into a object and eval returns stored value. Argument and
 * return types depend on used class. There is one class to each animated primitive
 * type
 * <p>
 *
 * @see Animator
 *
 * @author Petri Tenhunen
 */
public abstract class VirtualPrimitive implements MemoryStructure {

    /** Object presenting used primitive. */
    protected Object primitiveValue;
    protected String name;
    protected FDT fdt;

  public VirtualPrimitive() {
  }

    /**
     * Returns object presentation of stored value.
     *
     * @param index redundant because only one primitive value is presented
     *              in VirtualPrimitive
     * @return      Instance of java class connected to used primitive
     *              (<code>Integer, Boolean, ...</code>).
     */
    public Object getMem(int index) {
        return primitiveValue;
    }

    /**
     * Set Object to memory.
     *
     * @see MemoryStructure#setMem(int, Object) MemoryStructure.setMem
     * @see #getMem(int) getMem
     */
    public void setMem(int index, Object obj) {

        primitiveValue = obj;

    }


    public String toString() { return primitiveValue.toString(); }


    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public FDT getFDT() {
        return fdt;
    }

    public void setFDT(FDT fdt) {
        this.fdt = fdt;
    }

    static final long serialVersionUID = 4541530603219707355L;
}
