// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package matrix.structures.simulationextensions;

/**
 * DroppableMethods interface is a simulation extension 
 * that could be used to describe methods that could be
 * invoked by the user during the simulation process.
 *
 * This interface could also be used to hide methods.
 * For example, by implementing this interface one can
 * allow only invokations for the insert method and not
 * for the delete and search methods for CDTs by not
 * declaring those methods in the list of droppable method 
 * names.
 */
public interface DroppableMethods {

    /** Returns an array of strings describing the names of methods
     * that could be invoked by drag & dropping elements during the
     * simulation. The user can select the method invoked after drop
     * operation. The dropped element is passed as an argument to the
     * selected method. The type of the argument should be Object.
     * @return array of string of method names 
     */
    public String[] getDroppableMethodNames();
}

