// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package matrix.structures.simulationextensions;

/**
 * Selectable interface is a simulation extension 
 * that could be used to identify entities that could be
 * selected during the simulation process.
 */
public interface Selectable {

    public static final long NOT_SELECTED = 0;
    
	/**
	 * The GlobalClock class creates timestamps used to mark the order of selection events
	 * in the matrix framework.
	 */
		
	public static final class GlobalClock{
		private long globalTime; 
		
		public GlobalClock(){
			globalTime = 0;
		}
		
		public final long peekLastGivenStamp(){
			return globalTime;
		}
		
		public final long getNextTimeStamp(){
			globalTime++;
			return globalTime;
		}
	}
	
	public static final GlobalClock clock = new GlobalClock();
	
	/**
	 * Selects the entity. The parameter value should be acquired from
	 * the global clock using the getNextTimeStamp method. The timestamps
	 * are used to figure out the order of selections and other operation 
	 * where such order is required.
	 *
	 * Entities can be unselected by setting the timestamp to Selectable.NOT_SELECTED.
	 * 
	 * @param globalClockTimestamp the timestamp of the selection event or Selectable.NOT_SELECTED if the entity is unselected
	 */
    public void setSelect(long globalClockTimestamp);
        
    /**
     * Tells when the entity was selected. A timestamp of Selectable.NOT_SELECTED
     * means thet the entity is not selected.
     * 
     * @return the selection timestamp
     */    
    public long getSelectionTime();
    
    
}

