// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
package matrix.structures.spatial.FDT.probe;

import matrix.structures.spatial.PaintingStyleDecorator;

/**
 * A data structure representing a circle of radius r centered at point (x,y). This class is essentially
 * a shorthand for an <code>Ellipse</code> where width and height are equal. See javadoc for
 * <code>matrix.structures.spatial.FDT.probe.Ellipse</code> for further information.
 * 
 * @author Jussi Nikander
 *
 */
public class Circle extends Ellipse {

    public Circle() {
		super();
	}
	
    /**
     *  Constructor for a circle.
     * @param x The x coordinate of the center of the circle.
     * @param y The y coordinate of the center of the circle.
     * @param r The radius of the circle.
     * @param key The key (string identifier) of the circle.
     */
	public Circle(double x, double y, double r, String key) {
		super(x, y, 2 * r, 2 * r, key);
	}
	
    public Circle(double x, double y, double r) {
        this(x, y, r, "");
    }

    public Circle(double x, double y, double r, PaintingStyleDecorator psd) {
        this(x, y, r, "");
        this.setPaintingStyleDecorator(psd);
    }
    
    private static final long serialVersionUID = 1151090030094939185L;
}
