// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
package matrix.structures.spatial.FDT.probe;

import java.awt.Shape;
import java.awt.geom.CubicCurve2D;

import matrix.structures.spatial.PaintingStyleDecorator;
import matrix.structures.spatial.SpatialComparable;
import matrix.util.Note;

/**
 * A class for storing and visualizing cubic bezier curves 
 * in the Area visualization.
 * 
 * @author Juha Helminen
 * @see matrix.structures.spatial.Area VisualArea 
 */
public class CubicBezier extends SpatialElement {

    // CubicCurve2D is not serializable so its properties are stored instead and
    // the object is created when needed if it does not exist.
    private transient CubicCurve2D cubicBezier;
    private final double x1, y1;
    private final double ctrl1x, ctrl1y;
    private final double ctrl2x, ctrl2y;
    private final double x2, y2;

    public CubicBezier(double x1, double y1, 
                       double ctrl1x, double ctrl1y,
                       double ctrl2x, double ctrl2y,
                       double x2, double y2) {
        super("");
        this.x1 = x1;
        this.y1 = y1;
        this.ctrl1x = ctrl1x;
        this.ctrl1y = ctrl1y;
        this.ctrl2x = ctrl2x;
        this.ctrl2y = ctrl2y;
        this.x2 = x2;
        this.y2 = y2;
    }

    public CubicBezier(double x1, double y1, 
            double ctrl1x, double ctrl1y,
            double ctrl2x, double ctrl2y,
            double x2, double y2,
            PaintingStyleDecorator psd) {
        this(x1, y1, ctrl1x, ctrl1y, ctrl2x, ctrl2y, x2, y2);
        this.setPaintingStyleDecorator(psd);
    }

    protected void createShape() {
        if (this.cubicBezier == null) {
            this.cubicBezier = new CubicCurve2D.Double(this.x1, this.y1, 
                                                       this.ctrl1x, this.ctrl1y, 
                                                       this.ctrl2x, this.ctrl2y, 
                                                       this.x2, this.y2);
        }
    }
    
    public Shape getShape() {
        this.createShape();
        return this.cubicBezier;
    }

    public boolean contains(SpatialComparable sc) {
        Note.warning(this, "Operation not implemented.");
        return false;
    }

    public boolean intersects(SpatialComparable sc) {
        Note.warning(this, "Operation not implemented.");
        return false;
    }

    private static final long serialVersionUID = -9012964738521299494L;
}
