package matrix.structures.spatial.FDT.probe;

import java.awt.Shape;
import java.awt.geom.Point2D;

import matrix.structures.memory.VirtualObject;
import matrix.structures.spatial.PaintingStyleDecorator;
import matrix.structures.spatial.SpatialComparable;

/**
 * A class for representing a label in the Area visualization
 * 
 * @author lhakulin
 *
 */
public class Label extends SpatialElement {

	private static final long serialVersionUID = 350649822531126231L;

	private VirtualObject x, y;
	
	public Label() {
		this(0,0,"");
	}

    public Label(double x, double y) {
        this(x, y, "");
    }

    public Label(double x, double y, PaintingStyleDecorator psd) {
        this(x, y, "", psd);
    }

    public Label(double x, double y, String key, PaintingStyleDecorator psd) {
        this(x, y, key);
        if (psd != null) {
            this.setPaintingStyleDecorator(psd);
        }
    }

    public Label(double x, double y, String key) {
		super(key);
		this.x = new VirtualObject(new Double(x), this, "x");
		this.y = new VirtualObject(new Double(y), this, "y");
	}

	public double getX() {
		return ((Double)x.getObject()).doubleValue();
	}
	
	public void setX(double x) {
		this.x.setObject(new Double(x));
	}
	
	public double getY() {
		return ((Double)y.getObject()).doubleValue();
	}
	
	public void setY(double y) {
		this.y.setObject(new Double(y));
	}
	
	public Point2D getPoint2D() {
		return new Point2D.Double(((Double)x.getObject()).doubleValue(),((Double)y.getObject()).doubleValue());
	}
	
	public boolean equals(SpatialElement o) {
		if(o instanceof Label) {
			Label comp = (Label)o;
			return spatialEquals(comp) && 
				getComparisonString().equals(comp.getComparisonString());
		}
		return false;
	}
	
	public boolean intersects(SpatialComparable sc) {
		if(sc instanceof Label) {
			return spatialEquals(sc);
		} else  {
			return sc.contains(this);
		}
	}

	public boolean contains(SpatialComparable sc) {
		if(sc instanceof Label) {
			return spatialEquals(sc);
		} else {
			return false;
		}
	}

	public Shape getShape() {
		return null;
	}

}
