// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
package matrix.structures.spatial.FDT.probe;

import java.awt.Shape;
import java.awt.geom.GeneralPath;
import java.awt.geom.QuadCurve2D;

import matrix.structures.spatial.PaintingStyleDecorator;
import matrix.structures.spatial.SpatialComparable;
import matrix.util.Note;

/**
 * A class for storing and visualizing quadratic bezier curves 
 * in the Area visualization.
 * 
 * @author Juha Helminen
 * @see matrix.structures.spatial.Area VisualArea 
 */

public class QuadBezier extends SpatialElement {

    private transient Shape quadBezier;
    private final double x1, y1;
    private final double ctrlx, ctrly;
    private final double x2, y2;

	public QuadBezier(double x1, double y1, 
					 double ctrlx, double ctrly, 
					 double x2, double y2) {
		super("");
		this.x1 = x1;
		this.y1 = y1;
		this.ctrlx = ctrlx;
		this.ctrly = ctrly;
		this.x2 = x2;
		this.y2 = y2;
	}

	public QuadBezier(double x1, double y1, 
                     double ctrlx, double ctrly, 
                     double x2, double y2, PaintingStyleDecorator psd) {
        this(x1, y1, ctrlx, ctrly, x2, y2);
        this.setPaintingStyleDecorator(psd);
    }

	protected void createShape() {
	    if (this.quadBezier == null) {
	        this.quadBezier = new QuadCurve2D.Double(this.x1, this.y1, 
	                                                 this.ctrlx, this.ctrly, 
	                                                 this.x2, this.y2);
	        // XXX debug
//	        this.quadBezier = new GeneralPath();
//	        ((GeneralPath)this.quadBezier).moveTo((float)this.x1, (float)this.y1);
//	        ((GeneralPath)this.quadBezier).lineTo((float)this.ctrlx, (float)this.ctrly);
//	        ((GeneralPath)this.quadBezier).lineTo((float)this.x2, (float)this.y2);
	    }
	}
	
    public Shape getShape() {
        this.createShape();
        return this.quadBezier;
    }

    public boolean contains(SpatialComparable sc) {
        Note.warning(this, "Operation not implemented.");
        return false;
	}

	public boolean intersects(SpatialComparable sc) {
        Note.warning(this, "Operation not implemented.");
        return false;
	}

    private static final long serialVersionUID = -3783231279141466376L;
}
