// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
package matrix.structures.spatial;

import java.awt.geom.Rectangle2D;
import java.awt.Shape;

public interface SpatialComparable {

	/**
	 * Tests whether this object and the parameter object are spatially equal. That is, if they
	 * contain exactly the same area. Other aspects of the objects are not taken into account.
	 * @param o the object to be tested 
	 * @return true if the objects are spatially equal, false otherwise
	 */
	public boolean spatialEquals(Object o);

	/**
	 *  Tests whether this object and the parameter SpatialComparable intersect. 
	 *  
	 *  @param sc the object to be tested for intersection
	 *  @return true if the two components overlap, false otherwise.
	 */
	public boolean intersects(SpatialComparable sc);

	/**
	 * Tests whether this object contains the parameter SpatialComparable. Returns true if
	 * the parameter object is completely surrounded by this object, false otherwise.
	 * 
	 * @param sc the object to be tested for containment
	 * @return true if this object contains the parameter object, false otherwise
	 */
	public boolean contains(SpatialComparable sc);
	
	/**
	 * Fetches a <code>java.awt.Shape</code> that represents the shape of this object. 
	 * The returned shape may be any object that implements the Shape interface and accurately
	 * represents the object.
	 * @return a Shape
	 */
	public Shape getShape();

	/**
	 * Fetches a matrix.structures.spatial.FDT.probe.Rectangle which is the
	 * smallest rectangle to completely contain this element.
	 * @return bounding rectangle
	 */
	public Rectangle2D getBoundingBox();
	
}
