// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/* 24/05/2006 moved from package matrix.structures.FDT (vkaravir) */ 

package matrix.structures.util;

/**
 * This is an interface for all Matrix objects that can be iterated through.
 * Iterators that iterate through FDT structures should return the keys of
 * the FDT, not the nodes (if the FDT has something that can be described as
 * nodes). If an <code>Iterator</code> is inserted into another structure,
 * the <code>Iterator</code> is iterated through, and each element 
 * is inserted separately.
 *
 * @author Jussi Nikander
 */

public interface Iterator {

    /**
     * Tells if this iterator has more elements to
     * be iterated through.
     * @true if there are more elements to be iterated through.
     */
    public boolean hasNext();

    /**
     * Returns the next element in this Iterator.
     * FDT should return the next key in iteration.
     * @return the next element in iteration
     */
    public Object next();

    /**
     * Resets this iterator to the first element to be iterated.
     */
    public void reset();

}
