// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/* 24/05/2006 moved from package matrix.structures.FDT (vkaravir) */ 

package matrix.structures.util;

import matrix.structures.FDT.FDT;

/**
 * The purpose of this interface is that every structure that
 * implements this can be loaded back to original structure from the
 * ASCII file. Through methods of this interface, nodes and edges
 * (that are parsed from ASCII file) can be added to such a structure.
 *
 */
public interface PolyMorph extends FDT {

    /**
     * Creates a new node to a PolyMorph structure and returns that
     * node.
     *
     * @param fdt FDT that is to be inserted in the new
     * node. (e.g. key or another structure)
     * @return a new created node that have parameter fdt in it
     */
    public FDT createNewNode(FDT fdt);

    /**
     * Sets the successors of a certain node (father). Parameters have to be
     * same nodes that were returned by createNewNode method.
     *
     * @param father a node whose successor is to be set
     * @param child a node that is to be set as a successor of a father node
     */
    public void setSuccessor(FDT father, FDT child);

}
