// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package matrix.uitools;

import java.lang.reflect.Method;
import java.util.HashMap;

import javax.swing.Action;

import matrix.uitools.actions.AbstractMatrixAction;
import matrix.util.Note;
import matrix.util.configuration.CAction;

public class ActionFactory {
    private static ActionFactory theInstance;
    private HashMap actions;
    private HashMap createdActions;
    private HashMap globalActions;
    
    public static ActionFactory getInstance() {
    	if (theInstance == null) 
    		theInstance = new ActionFactory();
    	return theInstance;
    }
    
    private ActionFactory() {
    	actions = new HashMap();
    	globalActions = new HashMap();
    	createdActions = new HashMap();
    }

	public void addAction(CAction action) {
		actions.put(action.getId(), action);
	}

	public Action createAction(String action, Object obj) {
	    CAction ca = (CAction) actions.get(action);
		if (ca == null) {
			Note.out(this, "Action " + action + " not defined.");
			return null;
		}
		try {
			Class c = Class.forName(ca.getImplClass());
			AbstractMatrixAction a = (AbstractMatrixAction) c.newInstance();			
			if (obj != null && obj instanceof MenuFrame) {
				Method m = c.getMethod("setFrame", new Class[] {Class.forName("matrix.uitools.MenuFrame")});
				if (m != null)
					m.invoke(a, new Object[] { (MenuFrame)obj });
				((MenuFrame)obj).getStructurePanel().addActionListener(a);
			}
			createdActions.put(obj.hashCode()+action, a);
			if (a.isGlobal())
				globalActions.put(action, a);
			return a;
		} catch (Exception e) {
			e.printStackTrace();
		}
        return null;
	}
}
