// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
package matrix.uitools.actions;

import java.awt.event.ActionEvent;
import java.io.*;
import java.net.URL;
import java.util.jar.JarFile;

import javax.swing.AbstractAction;
import javax.swing.AbstractButton;
import javax.swing.Action;
import javax.swing.ImageIcon;

import matrix.uitools.MenuFrame;
import matrix.uitools.StructurePanel;
import matrix.util.ApplicationAdapter;
import matrix.util.configuration.MatrixConfiguration;

public abstract class AbstractMatrixAction extends AbstractAction {
	private static final String DEFAULT_JAR_FILE = "matrix.jar";
    private MenuFrame mf;
	
	public AbstractMatrixAction() {
		super();
	}

	public AbstractMatrixAction(String name) {
		super(name);
	}

	/**
	 * @return Returns the mf.
	 */
	public MenuFrame getFrame() {
		if (mf == null)
			mf = (MenuFrame)((StructurePanel) ApplicationAdapter.getApplication()).getFrame();
		return mf;
	}

	/**
	 * @param mf The mf to set.
	 */
	public void setFrame(MenuFrame mf) {
		this.mf = mf;
		updateState();
	}
	
	public boolean isGlobal() {
		return false;
	}
    
    public ImageIcon getImageIcon(String url) {
        URL imUrl = getClass().getResource(url);
        if (imUrl != null) 
            return new ImageIcon(imUrl);
        if (!url.startsWith("/")) {
            imUrl = getClass().getResource("/" + url);
            if (imUrl != null)
                return new ImageIcon(imUrl);
        }
        File f = new File(url);
        try {
            if (f.exists()) 
            return new ImageIcon(url);
            else {
            String jarFile = MatrixConfiguration.getJarFile();
            if (jarFile == null)
                jarFile = System.getProperty("java.class.path", DEFAULT_JAR_FILE);
            JarFile jar = new JarFile(MatrixConfiguration.getJarFile());
            InputStream in = jar.getInputStream(jar.getEntry(url));
            byte[] b = readFromStream(in);        
           return new ImageIcon(b);
            }      
        } catch (Exception e) {
            //e.printStackTrace();
        }
           return null;
    }

    public void actionPerformed(ActionEvent event) {
        if (event != null && event.getSource() != null && event.getSource() instanceof AbstractButton) {
            Action a = ((AbstractButton)event.getSource()).getAction();
            if (a != null && a.equals(this)) {
                actionPerformedOn(event);
            }
        }
        updateState();
    }
    
    
    private static byte[] readFromStream(InputStream in) throws IOException {
        int cursor = 0;
        int readBytes = 0;
        byte[] b = new byte[65536]; 
        while((readBytes=in.read(b,cursor,10000))!=-1)
            cursor+=readBytes;
        in.close();
        
        byte[] c=new byte[cursor];
        System.arraycopy(b,0,c,0,cursor);
        return c;
    }
    
    public abstract void actionPerformedOn(ActionEvent e);
    
    public void updateState() {}
}
