// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package matrix.uitools.actions;

import java.awt.EventQueue;
import java.awt.event.ActionEvent;
import java.lang.reflect.InvocationTargetException;

import javax.swing.Action;
import javax.swing.ImageIcon;

import matrix.animation.Animator;

public class AnimatorPlayAction extends AbstractMatrixAction {
    private static final long serialVersionUID = 2819504584328078849L;
    private boolean playing;
    private ImageIcon playIcon, stopIcon;
    public static final String DEFAULT_NAME = "Play";

    public AnimatorPlayAction() {
        this(DEFAULT_NAME);
    }

    public AnimatorPlayAction(String name) {
        super(name);
        init();
    }

    private void init() {
        playIcon = getImageIcon("pics/Play16.gif");
        stopIcon = getImageIcon("pics/Stop.gif");
        setIcon(playIcon);
        setEnabled(false);
    }
    
    private void setIcon(ImageIcon icon) {
        putValue(Action.SMALL_ICON, icon);
    }
    
    private void setToolTipText(String text) {
        putValue(Action.SHORT_DESCRIPTION, text);
    }
    
    public void actionPerformedOn(final ActionEvent e) {
        final Animator animator = getFrame().getStructurePanel().getAnimator();
        if (playing) {
            playing = false;
            setIcon(playIcon);
            setToolTipText("Play");
            if (animator.hasPreviousOperation())
                animator.undo();
            return;
        }
        playing = true;
        setIcon(stopIcon);
        setToolTipText("Stop");
        new Thread() {
            public void run() {
                while (animator.hasNextOperation() && playing) {
                    try {
                        Thread.sleep(getFrame().getVisualAnimator().getWaitTime());
                        try {
                            EventQueue.invokeAndWait(new Runnable() {
                                public void run() {
                                    animator.redo();
                                    getFrame().getStructurePanel().getVisualAnimator().fireActionEvent(new ActionEvent(this, e.getID(), null));
                                }
                            });
                        } catch (InvocationTargetException ite) {
                            ite.getTargetException().printStackTrace();
                        }
                    } catch (InterruptedException ie) {
                    }
                }
                if (!animator.hasNextOperation()) {
                    playing = false;
                    setIcon(playIcon);
                    setToolTipText("Play");
                }
            }
        }.start();
    }
    
    public void updateState() {
        setEnabled(getFrame().getStructurePanel().getAnimator().hasNextOperation());
    }

}
