// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/* $Id: Note.java,v 1.29 2008/01/21 15:57:02 jtn Exp $ */

package matrix.util;

import java.awt.event.*;
import java.io.PrintStream;

import javax.swing.*;

import matrix.visual.VisualType;

public class Note /*implements ActionListener*/ {
    private static int counter = 0;
    private static boolean outVisible = true;
    private static PrintStream out = System.out;
    public JDialog dialog;
    

    public static void setPrintStream(PrintStream s) {
        out = s;
    }

    public static PrintStream getPrintStream() {
        return out;
    }

    private static void sysOut(String s) {
        out.println(++counter + ". " + s);
    }
    
    public static void setOut(boolean value) {
        outVisible = value;
    }

    public static void toggleOut() {
	outVisible = !outVisible;
    }

    public static boolean getOut() {
        return outVisible;
    }

    public static void out(Object o, String s) {
        if (outVisible) {
            if (o != null)
                sysOut(o.getClass()+": "+s);
            else 
                sysOut(s);
        }
    }

    public static void err(Object o, String s) {
        if (o != null)
            sysOut("Error in " + o.getClass()+": "+s);
        else 
            sysOut(s);
        (new Exception()).printStackTrace(out);
    }
    
    public static void warning(Object o, String s) {
        if (o != null)
            sysOut("Warning in " + o.getClass()+": "+s);
        else 
            sysOut(s);
    }
    
    public static void printStackTrace() {
        (new RuntimeException()).printStackTrace(out);
    }
    

    public synchronized static void show(Object o, String s) {
	Note.out(null, "o: " + o + "s: " + s);
        Note n = new Note();
        if (o instanceof VisualType)
            n.show((VisualType)o, s);
        else if (o instanceof JFrame) 
            n.show((JFrame)o, o.getClass().getName(), s);
	else if(o instanceof Application)
	    ((Application)o).showMessage("", s);
        else 
            n.show((Object)o, s, true);
    }


    /*
    public static void assertCont(boolean condition, String msg) {
        if (!condition)
            (new RuntimeException("Assertion failed: " + msg)).
                printStackTrace();
    }

    public static void assertCont(boolean condition) {
        assertCont(condition, "");
    }

    public static void assert(boolean condition, String msg) {
        if (!condition)
            throw new RuntimeException("Assertion failed: " + msg);
    }

    public static void assert(boolean condition) {
        assert(condition, "");
    }
    */
    private void show(VisualType vt, String s) {
	Application app = vt.getApplication();
	if(app != null)
	    app.showMessage("Title", s);
	else 
	    show(vt,s,true);
	/*
        Note.out(null, "VisualShow invoked");
        Button dismiss = new Button("Dismiss");
	Frame f;
	if(vt.getFrame() instanceof Frame)
	    f = (Frame)vt.getFrame();
	else
	    f = new Frame(); // XXX -> change
        dialog = new Dialog(f, vt.getClass().toString(), false);
        dialog.add("North", new Label(" "+s+" "));
        dialog.add("South", dismiss);
        dismiss.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) { 
                    Note.out(null, "dismiss!"); dispose(); 
                }
        });
        dialog.pack();
        dialog.show();
	*/
    }
    
    JFrame errorFrame = new JFrame("Error");

    private void show(Object o, String s, boolean b) {
        Note.out(null, "Note.Show invoked");
        JButton dismiss = new JButton("Dismiss");
        String name = null;
	if(o != null) {
	    if(o instanceof String)
		name = (String)o;
	    else name = o.getClass().toString(); 
	} 
	else 
	    name = "Alert";
        dialog = new JDialog(errorFrame, name, false);
        dialog.add("North", new JLabel(" "+s+" "));
        dialog.add("South", dismiss);
        dismiss.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) { 
                dispose(); 
                errorFrame.dispose(); 
            }
        });
        dialog.pack();
        dialog.setVisible(true);
    }

  private void show(JFrame mf, String title, String s) {
        Note.out(null, "VisualShow invoked");
        JButton dismiss = new JButton("Dismiss");
        //dialog = new Dialog(mf, mf.getClass().toString(), false);
	dialog = new JDialog(mf, title, false);
	JComponent text = null;

	// In case of a long message, show text area. Short messages
	// can be shown in a label.
	if (s.indexOf("\n") >= 0) {
	  JTextArea ta = new JTextArea(s);
	  ta.setColumns(40);
	  ta.setRows(20);
	  text = ta;
      ta.setEditable(false);
	}
	else {
	  text = new JLabel(" "+ s + " ");
	}
	dialog.getContentPane().add("Center", text);
        dialog.getContentPane().add("South", dismiss);
        dismiss.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) { dispose(); }
        });

	dialog.addWindowListener(new WindowAdapter() {
	    public void windowClosing(WindowEvent e) { dispose(); }
	  });
        
        dialog.pack();
        dialog.setVisible(true);
    }

    private void dispose() { dialog.dispose(); }
}
