// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/* $Id: RandomKey.java,v 1.21 2007/09/07 13:39:38 vkaravir Exp $ */

package matrix.util;

import java.util.Random;

/**
 * The <code>RandomKey</code> class creates random <code>String</code>s, which can be
 * used, for example, as random keys in testing or demonstrating the workings of different
 * data structures or algorithms.
 *
 * @author Jussi Nikander
 */ 

public class RandomKey {

    //public static Random random = new Random();

    /**
     * Changes an int value to a String. Legal values are 1-62
     * @return A single character <code>String</code>
     */
    public static String intToKey(int n) {
	boolean b = false;
	String tmp = "";
	if(n > 0 && n < 63) {
	    if(n < 11) {
		tmp = Integer.toString(n - 1);
		return tmp;
	    } else {
		if(n > 36) {
		    b = true;
		    n -= 26;
		}
		n -= 8;
		tmp = keyIt(n);
		if(b) {
		    tmp = tmp.toUpperCase();
		}
		return tmp;
	    }
	}
	return "";
    }


    /**
     * returns a one character long <code>String</code> depending on the input.
     * Legal values of the parameter are from 3 to 28 and legal returnvalues are
     * a to z. 
     * @param n an int.
     * @return a single character <code>String</code> corresponding to the param int.
     */
    public static String keyIt(int n) {
	if(n > 2 && n < 29) {
	  char[] str = new char[1]; 
	  str[0]= (char)('a'+(n-3));
	  return new String(str);
	} else
	    return "";
    }


    /**
     * Creates a random three-letter key. Legal values of letters are a-z, A-Z, 0-9
     * @return <code>String</code> containing the key
     */
    public static String createRandomKey(Random r) {
	return createRandomKey(r,3);
    }


    /**
     * Creates a random key.
     * @param n the length of the key.
     * @return <code>String</code> containing the key
     */
    public static String createRandomKey(Random r, int n) {
	String key;
	key = intToKey((int) (r.nextDouble() * 61) + 1);
	n--;
	while(n > 0) {
	    key += intToKey((int) (r.nextDouble() * 61) + 1);	    
	    n--;
	}
	return key;
    }


    public static String createUppercaseRandomKey(Random r, int n) {
	String key = "";
	while(n > 0) {
	    key += keyIt((int) (r.nextDouble() * 26) + 3);
	    n--;
	}
	key = key.toUpperCase();
	return key;
    }
	
    
    /**
     * Creates a random key with no duplicate characters. Maximum length
     * of the key is 20 characters.
     *
     * @param n the length of the key
     */
    public static String createNoDuplicateRandomKey(Random r, int n) {
	if(n > 20)
	    n = 20;
	String key;
	Character a;
	key = intToKey((int) (r.nextDouble() * 61) + 1);
	n--;
	while(n > 0) {
	    a = new Character(intToKey((int) (r.nextDouble() * 61) + 1).charAt(0));
	    if(key.indexOf(a.charValue()) == -1) {
		key += a.toString();
		n--;
	    }
	}
	return key;
    }


    public static String createNoDuplicateUppercaseRandomKey(Random r, int n) {
       if(n > 20)
	    n = 20;
	String key;
	String a;

	// Ascii codes of letters A-Z are from 65 to 90.

	key = keyIt((int) (r.nextDouble() * 25) + 3);
	n--;
	while(n > 0) {
	    a = keyIt((int) (r.nextDouble() * 25) + 3);
	    if(key.indexOf(a) == -1) {
		key += a;
		n--;
	    }
	}
	key = key.toUpperCase();
	return key;

//  	key = new Character((char)(random.nextInt(26) + 65)).toString();
//  	n--;
//  	while(n > 0) {
//  	    a = new Character((char)(random.nextInt(26) + 65));
//  	    if(key.indexOf(a.charValue()) == -1) {
//  		key += a.toString();
//  		n--;
//  	    }
//  	}
//  	return key;
    }


    public static String createNoDuplicateLowercaseRandomKey(Random r, int n) {
      return createNoDuplicateUppercaseRandomKey(r,n).toLowerCase();
    }



  /**
     Creates sorted random keys that contain alphabet
     sequences of 3 letters.
     <p>
     XXX not tested

     @param n number of keys to create
     @return keys in a String array
   */
  public static String[] createSortedAlphabetKeys(Random r, int n) {
    String[] str = new String[n];
    for (int i = 0; i < n; i++) {
      str[i] = createAlphabetKey(r, 3);
    }
    return str;
  }



  /**
     Creates upper case alphabetic key that has 
     <code>size</code> characters.
     <p>
     XXX not tested

     @param size number of characters of the key to create
   */
  public static String createAlphabetKey(Random r,int size) {
    StringBuffer buf = new StringBuffer();
    for (int i = 0; i < size; i++) {
      // Ascii codes of letters A-Z are from 65 to 90.
      int rand = ((int)(r.nextDouble() * 25))+26;
      buf.append(rand+"");
    }
    return buf.toString();
  }

  /**
  Creates an array of unsorted integers that are between the
  given bounds. The integers are greater than or equal to min and 
  less than max.

  @param min the lower bound
  @param max the upper bound
  @param num number of keys to generate
  */
  public static int[] createIntKeys(Random r, int min, int max, int num) {
      int[] keys = new int[num];
      for (int i = 0; i < num; i++) {
        // Math.random returns number greater than or equal to 0.0 
        // and less than 1.0
        keys[i] = createIntKey(r,min,max);
      }
      return keys;
    }


  /**
     Creates an array of sorted integers that are between the
     given bounds. The integers are greater than or equal to min and 
     less than max.

     @param min the lower bound
     @param max the upper bound
     @param num number of keys to generate
   */
  public static int[] createSortedIntKeys(Random r, int min, int max, int num) {
    int[] keys = createIntKeys(r,min,max,num);
    java.util.Arrays.sort(keys);
    return keys;
  }


 /**
     Creates an array of distinct sorted integers that are between the
     given bounds. The integers are greater than or equal to min and 
     less than max.

     @param min the lower bound
     @param max the upper bound
     @param num number of keys to generate
   */

  public static int[] createNoDuplicateSortedIntKeys(Random r,int min, int max, int num) {
    if (min >= max) throw new IllegalArgumentException("min >= max");

    java.util.Hashtable keyHash = new java.util.Hashtable();
    int[] keys = new int[num];
    for (int i = 0; i < num; i++) {
      // Math.random returns number greater than or equal to 0.0 
      // and less than 1.0
      int tmp = 0;
      do {
	tmp = createIntKey(r,min,max);
      } while (keyHash.get(new Integer(tmp)) != null);
      keyHash.put(new Integer(tmp), new Integer(tmp));
      keys[i] = tmp;
    }
    java.util.Arrays.sort(keys);
    return keys;
  }
  


  /**
     Creates random integer that is between the
     given bounds. The integer is greater than or equal to min and 
     less than max.
     
     @param min the lower bound
     @param max the upper bound
  */
  public static int createIntKey(Random r,int min, int max) {
    int key = ((int)(r.nextDouble()*(max-min)))+min;
    return key;
    
  }

  /**
   * Creates a random key with two characters.
   * First character is uppercase letter and the second one is a number (1-9).
   * @return <code>String</code> containing the key
   */
  public static String createRandomKeyWithLetterAndNumber(Random r) {
  	String key;
  	key = createUppercaseRandomKey(r, 1);
  	key += createIntKey(r, 1, 10);	    
  	return key;
  }


}

