// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package matrix.util.configuration;

/**
 * This class corresponds to a MenuItem in matrix configuration.
 * Since the actual MenuItems are not created until needed, this is
 * not a subclass of java.awt.MenuItem.
 */

public class CMenuItem extends ConfItem {
    private String name;
    private boolean disabled;
    private String type;
    private boolean checked;
    // tells if this menu item changes global settings
    private boolean global;
    private int accesskey;
    private String group;
    private char mnemonic;
    private String action;

    public CMenuItem() {
	super(ConfItem.MENUITEM);
	accesskey = -1;
    }

    public void setName(String n) {
	name = n;
    }

    public String getName() {
	return name;
    }

    public void setDisabled(boolean b) {
	disabled = b;
    }

    public boolean getDisabled() {
	return disabled;
    }

    public void setType(String t) {
	t = t.toLowerCase();
	if(!(t.equals("radio") || t.equals("checkbox"))) {
	    return;
	}
	type = t;
    }

    public String getType() {
	return type;
    }

    public void setChecked(boolean b) {
	checked = b;
    }

    public boolean getChecked() {
	return checked;
    }

    public void setAccessKey(int key) {
	accesskey = key;
    }

    public int getAccessKey() {
	return accesskey;
    }

    public void setGroup(String gr) {
	group = gr;
    }

    public String getGroup() {
	return group;
    }

    public void setGlobal(boolean b) {
	global = b;
    }

    public boolean isGlobal() {
	return global;
    }

    public String toString() {
	return "MenuItem: " + name + " type: " + type + " dsbl: " + disabled + " chk: " + checked + " acc: " + accesskey + 
	    " grp: " + group;
    }

    public void setMnemonic(char c) {
        this.mnemonic = c;        
    }
    
    public char getMnemonic() {
        return mnemonic;
    }

	/**
	 * @return Returns the action.
	 */
	public String getAction() {
		return action;
	}

	/**
	 * @param action The action to set.
	 */
	public void setAction(String action) {
		this.action = action;
	}
    
    
}
