// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package matrix.util.configuration;

import java.lang.reflect.Method;
import java.util.HashMap;

public class CMethod extends ConfItem {
    // the class where this method can be found. 
    // non-null only if the method is static.
    private Class mClass;
    private String mName;
    private Method method;
    private boolean isStatic;

    private HashMap candidates = new HashMap();

    public CMethod(Class c, String name) {
        super(ConfItem.METHOD);
        mName = name;
        mClass = c;
        isStatic = false;
    }

    public CMethod(Class c, String name, boolean sta) {
        this(c,name);
        isStatic = sta;
    }

    public Class getWrappedClass() {
        return mClass;
    }

    public String getMethodName() {
        return mName;
    }

    public void setMethod(Method m) {
        method = m;
    }

    public void addCandidate(Method m, Class objectClass, Class[] formalParams){
        StringBuffer key = new StringBuffer(objectClass.getName());
        for (int i=0; i<formalParams.length; i++)
            key.append(formalParams[i].getName());

        candidates.put(key.toString(), m);        
    }

    public Method getCandidate(Class objectClass, Class[] formalParams){
        StringBuffer key = new StringBuffer(objectClass.getName());
        for (int i=0; i<formalParams.length; i++)
            key.append(formalParams[i].getName());

        return (Method)candidates.get(key.toString());        

    }

    public Method getMethod() {
        return method;
    }

    public boolean isStatic() {
        return isStatic;
    }

    public String toString() {
        if(method != null) 
            return "Method " + method.toString();
        else
            return "Method(dummy): " + mClass.toString() + "." + mName;
    }

}
