// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/* $Id: MatrixConstructor.java,v 1.6 2007/09/10 08:12:38 vkaravir Exp $ */

package matrix.util.configuration;

import java.lang.reflect.*;

/**
 * This class wraps a class constructor and arguments given to it. The constructor may have
 * arbitary number of arguments. The newInstance-method can be used to create new instances
 * of the wrapped class.
 */
public class MatrixConstructor {
	
    private Constructor type;
    private Object[] initargs;
	
    public MatrixConstructor(Class c, Object[] o) throws NoSuchMethodException {
	Class[] args = new Class[o.length];
	for(int i = 0; i < o.length; i++) {
	    if(o[i] instanceof Primitive)
		args[i] = ((Primitive)o[i]).getPrimitiveClass();
	    else if(o[i] instanceof MatrixConstructor) 
		args[i] = ((MatrixConstructor)o[i]).type.getDeclaringClass();
	    else
		args[i] = o[i].getClass();
	}
	/*
	System.out.println("Class: " + c.toString() + " with args:");
	for(int i = 0 ; i < args.length; i++) {
	    System.out.println("  " + args[i]);
	}
	*/
	type = c.getConstructor(args);
	initargs = o;
    }

    /**
     * Creates a new instance of the class represented by
     * the constructor. If any of the class' arguments
     * is a class itself, it must be wrapped into an instance of
     * MatrixConstructor. All classes held in MatrixConstructor are
     * instantiated recursively.
     */
    public Object newInstance() 
	throws InstantiationException,
	       IllegalAccessException,
	       IllegalArgumentException,
	       InvocationTargetException {
	Object[] args = new Object[initargs.length];
	for(int i = 0; i < args.length; i++) {
	    if(initargs[i] instanceof MatrixConstructor) 
		args[i] = ((MatrixConstructor)initargs[i]).newInstance();
	    else if(initargs[i] instanceof Primitive)
		args[i] = ((Primitive)initargs[i]).getValue();
	    else 
		args[i] = initargs[i];
	}
	return type.newInstance(args);
    }

}
