// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/* $Id: MG_Line.java,v 1.15 2007/09/07 13:34:43 vkaravir Exp $ */

package matrix.util.export;

import java.awt.Color;
import java.io.*;

/**
 * Represents line in matrix graphics. 
 */
public class MG_Line extends MG_Element {
    private int x1, y1, x2, y2;

    public MG_Line(int x1, int y1, int x2, int y2, Color c) {
	super();
	this.x1 = x1;
	this.y1 = y1;
	this.x2 = x2;
	this.y2 = y2;
	color = c;
    }

    public MG_Line(int[] co, Color c) {
	this(co[0], co[1], co[2], co[3], c);
    }

    public MG_Line(int[] coord, Color c, int step) {
	this(coord[0], coord[1], coord[2], coord[3], c);
	int i = 0;
	setOpacity(0);
	while (i <= step) {
	    addStep(this, i);
	    i++;
	}
    }

    public int[] getCoord() {
	return new int[] {x1, y1, x2, y2};
    }

    public String toString() {
	return new String("MG_Line x1:" + x1 + " y1:" + y1 + " x2:" +
			  x2 + " y2:" + y2 + " color:" + color + 
			  " opacity:" + getOpacity());
    }
    
    public MG_Element getStep(int step) {
	MG_Line temp = (MG_Line)steps.get(new Integer(step));
	if (temp == null) return null;
	return new MG_Line(temp.getCoord(), temp.getColor());
    }

    public String toLatex() {
	int y1l = -y1;
	int y2l = -y2;
	String tex = "\\move (" + x1 + " " + y1l + ")\n" +
	    "\\lvec (" + x2 + " " + y2l + ")\n";
	float gLevel = ColorConverter.toGrayLevel(color);
	if (gLevel != 0 && gLevel == 1)
	    tex = "\\setgray 1\n" + tex + "\\setgray 0\n";
	return tex;
    }

    public void toSVG(Writer out, SVGUtil util)
	        throws IOException {
	int[] colors = ColorConverter.toRGB(color);
	out.write("<line");
	out.write(" x1=\"" + x1 + "\"");
	out.write(" y1=\"" + y1 + "\"");
	out.write(" x2=\"" + x2 + "\"");
	out.write(" y2=\"" + y2 + "\"");
	out.write(" fill=\"none\"");
	if (getOpacity() != 1)
	    out.write(" opacity=\"" + getOpacity() + "\"");
	out.write(" stroke=\"rgb(" + colors[0] + "," + colors[1] + "," +
	    colors[2] + ")\"");
	out.write(">");
	animate(out, util);
	out.write("</line>\n");
    }

    public int getType() {
	return MG_Element.MG_LINE;
    }

    /**
     * Creates the SVG elements needed to animate this element.
     *
     * @param out Output Writer
     * @param util SVGUtil used exporting this animation
     * @see SVGUtil
     */
    private void animate(Writer out, SVGUtil util) throws IOException {
	int i = 0;
	MG_Line curr = this;
	while (hasStep(i+1)) {
	    MG_Line next = (MG_Line)steps.get(new Integer(i+1));
	    if (curr.x1 != next.x1) 
		out.write(util.animate("x1", i, "" + curr.x1, "" + next.x1));
	    if (curr.y1 != next.y1)
		out.write(util.animate("y1", i, "" + curr.y1, "" + next.y1));
	    if (curr.x2 != next.x2) 
		out.write(util.animate("x2", i, "" + curr.x2, "" + next.x2));
	    if (curr.y2 != next.y2)
		out.write(util.animate("y2", i, "" + curr.y2, "" + next.y2));
	    if (!curr.color.equals(next.color)) {
		int[] c1 = ColorConverter.toRGB(curr.color);
		int[] c2 = ColorConverter.toRGB(next.color);
		String from = "rgb(" + c1[0] + "," + c1[1] + "," + c1[2] + ")";
		String to = "rgb(" + c2[0] + "," + c2[1] + "," + c2[2] + ")";
		out.write(util.animate("animateColor", "stroke", i, 
				       from, to));
	    }
	    if (curr.getOpacity() != next.getOpacity())         // opacity
		out.write(util.animate("opacity", i, "" + curr.getOpacity(), 
				       "" + next.getOpacity()));
	    i++;
	    curr = next;
	}
	float pause = util.getPause();
	float stepLength = util.getStepLength();
	float endTime = (i)*(pause + stepLength) + pause;
	if (!util.addPanel()) {
	    String opac = "0";
	    if (i == util.getMaxStep())
		opac = "0.5";
	    out.write(SVGUtil.animateAdditive("opacity", endTime, stepLength, 
					      opac, "sum"));
	}
	else if (i < util.getMaxStep() && util.addPanel())
	    out.write(util.animate("opacity", i, ""+curr.getOpacity(), "0"));
    }
}
	
	
