// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/* $Id: MatrixClassLoader.java,v 1.7 2007/09/07 13:34:42 vkaravir Exp $ */

package matrix.util.io;

import java.io.*;
import java.util.Hashtable;

import matrix.util.Note;

public class MatrixClassLoader extends ClassLoader {

    private Hashtable classNames = new Hashtable();

    private Hashtable fileNames = new Hashtable();

    private Hashtable classes = new Hashtable();


    public synchronized Class loadClass(String className) 
	throws ClassNotFoundException {
        return (loadClass(className, true));
    }


    public synchronized Class loadClass(String className, boolean resolveIt)
        throws ClassNotFoundException {
        Class result;
	if (className == null) {
	    throw new NullPointerException("className == null");
	}
        result = (Class)classes.get(className);
        if (result != null) {
            return result;
        }

        try {
            result = super.findSystemClass(className);
            return result;
        } catch (ClassNotFoundException e) {
	    // do nothing...
        }

	String fileName = (String)fileNames.get(className);
	if (fileName == null) {
	    throw new ClassNotFoundException("can't find file for class: " 
					     + className);
        }

	byte[]  classData = readFile(fileName);
        if (classData == null) {
            throw new ClassNotFoundException("can't read class file: " 
					     + fileName);
        }
	
	result = defineClass(className, classData, 0, classData.length);
        if (result == null) {
            throw new ClassFormatError();
        }

        if (resolveIt) {
            resolveClass(result);
        }

        classes.put(className, result);
	return result;
    }



    private byte[] readFile(String fileName)  {
	try {
	    FileInputStream fi = new FileInputStream(fileName);
	    byte[] result = new byte[fi.available()];
	    fi.read(result);
	    fi.close();
	    return result;
	} catch (IOException e) {
	    Note.err(this, e.getMessage());
	    return null;
	}
    }


    
    public synchronized boolean registerClassFile(String fileName) {
	try {
	    byte[] bytes = readFile(fileName);
	    if (bytes == null) {
		return false;
	    }
	    Class clazz =  defineClass(null, bytes, 0, bytes.length);
	    classNames.put(fileName, clazz.getName());
	    fileNames.put(clazz.getName(), fileName);
	    return true;
	} catch (ClassFormatError e) {
	    //Note.err(this, e.getMessage());
	    return false;
	}
    }


    public synchronized String className(String fileName) {
	return (String)classNames.get(fileName);
    }


    public synchronized String fileName(String className) {
	return (String)fileNames.get(className);
    }


    public Class loadClassFromFile(String fileName) 
	throws ClassNotFoundException, 
               InstantiationException, 
               IllegalAccessException {
	if (!registerClassFile(fileName)) {
	    throw new ClassNotFoundException("can't load class file: " 
                                             + fileName);
	}
	String clazzName = className(fileName);
	Class clazz = null;
	clazz = loadClass(clazzName);
	return clazz;
    }
    
    
    public static boolean implementsInterface(Class clazz, Class iface) {
	
	Class[] ifaces = clazz.getInterfaces();
	
	if (ifaces.length != 0) {
	    for (int i = 0; i < ifaces.length; i++) {
		if (iface.equals(ifaces[i])) {
		    return true;
		} else if (implementsInterface(ifaces[i], iface)) {
		    return true;
		}
	    }
	    
	} 

	if (clazz.getSuperclass() != null) {
	    if (implementsInterface(clazz.getSuperclass(), iface)) {
		return true;
	    }
	}

	return false;
    }





    
}



