// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/* $Id: LayoutBounds.java,v 1.5 2007/09/10 08:11:11 vkaravir Exp $ */

package matrix.visual;

/**
 * A (x,y,width,height) tuple for specifying bounds (position+size) of
 * nodes and layouts in layout coordinates.
 *
 * @author Riku Saikkonen
 */
public class LayoutBounds {
    public int x, y, width, height;

    public LayoutBounds(int x, int y, int width, int height) {
        this.x = x; this.y = y;
        this.width = width; this.height = height;
    }

    public LayoutBounds(LayoutBounds lb) {
        this.x = lb.x; this.y = lb.y;
        this.width = lb.width; this.height = lb.height;
    }

    /** Return the union of this with other.
     * @param other the other LayoutBounds object to union with
     * @return a new LayoutBounds Object
     */
    public LayoutBounds union(LayoutBounds other) {
        int nx = Math.min(x, other.x), ny = Math.min(y, other.y);
        return new LayoutBounds(nx,
                                ny,
                                Math.max(x+width, other.x+other.width) - nx,
                                Math.max(y+height, other.y+other.height) - ny);
    }
}
