// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package matrix.visual;

import java.io.StringReader;



/**
 * Static class representation for the ini-file. This is needed when
 * there is an applet or inifile is not readable.
 *
 * @author Petri Tenhunen
 */
public final class StaticConfigure extends StringReader {

  public StaticConfigure() { super(iniText); }

  private static final String iniText =
"# This is the configuration file for Matrix RepresentationFactory. This file \n"+
"# MUST be placed in the Matrix root directory, or the system WILL NOT work.\n"+
"# All lines beginning with an # are considered comments. \n"+
"# The file is divided to different blocks. Each block starts with a \n"+
"# line reading !BLOCKNAME, where BLOCKNAME is the name of the specific\n"+
"# block. Do not change block names.\n"+
"#\n"+
"# There are a few reserved words that are used as special commands.\n"+
"# all of these are marked with a ! and written in CAPITAL LETTERS, so they shouldn't\n"+
"# be mixed up with class names, etc.\n"+
"#\n"+
"# UNDERLYINGTYPE is a special command used in creating new visualization. It\n"+
"# tells the program that this particular VisualType is a wrapper type, and\n"+
"# that it should call the getVisualType() -method to retrieve the correct\n"+
"# type.\n"+
"#\n"+
"# author: Jussi Nikander\n"+
"\n"+
"# Object to VisualType mapping. This tells what kinds of visualizations are\n"+
"# available to objects, depending on the interfaces they implement. There are\n"+
"# also a few classes included. The priority of the visualizations is from\n"+
"# top to bottom. So, if a class implements more than one interface, the \n"+
"# order in which the interfaces are mentioned here tells how it is visualized\n"+
"# by default.\n"+
"!VISUALIZATIONS\n"+
"\n"+
"matrix.structures.FDT.Tree		matrix.visual.VisualLayeredTree\n"+
"matrix.structures.FDT.Array		matrix.visual.VisualArray\n"+
"matrix.structures.FDT.LinkedList	matrix.visual.VisualList\n"+
"#matrix.structures.FDT.StaticLinkedList	matrix.visual.VisualStaticList\n"+
"matrix.structures.memory.Key		matrix.visual.VisualKey\n"+
"matrix.structures.FDT.Trash		matrix.visual.VisualTrash\n"+
"matrix.structures.FDT.Vertex		matrix.visual.VisualLayeredGraph\n"+
"matrix.structures.FDT.Graph		matrix.visual.VisualLayeredGraph\n"+
"java.lang.reflect.Method		matrix.visual.VisualClassMethod\n"+
"\n"+
"# String to VisualType mappings. These mappings are used in some special\n"+
"# cases to force an object to be visualized in certain way. There should be\n"+
"# no need to edit any of these.\n"+
"!NAMEDVISUALIZATIONS\n"+
"\n"+
"\"code\"			matrix.visual.VisualCode\n"+
"\"leaf tree\"		matrix.visual.VisualLeafTree\n"+
"\"layered tree\"		matrix.visual.VisualLayeredTree\n"+
"\"array\"			matrix.visual.VisualArray\n"+
"#\"adj list\"		matrix.visual.VisualAdjList #not in use\n"+
"\"key\"			matrix.visual.VisualKey\n"+
"\"trash\"			matrix.visual.VisualTrash\n"+
"\"dummy graph\"		matrix.visual.VisualDummyGraph\n"+
"\"layered graph\"		matrix.visual.VisualLayeredGraph\n"+
"\"list\"			matrix.visual.VisualList\n"+
"\"append list\"		matrix.visual.VisualList2\n"+
"#\"static list\"		matrix.visual.StaticLinkedList\n"+
"\n"+
"# VisualiType to VisualType mappings for Picker. This mapping tells what\n"+
"# kind of new VisualContainer to create from a VisualType\n"+
"# These are used when objects are dragged with the mouse. There should\n"+
"# be no reason to edit any of these, unless you create a new visualization.\n"+
"!NEWVISUALIZATIONS\n"+
"\n"+
"matrix.visual.VisualLayeredTree			matrix.visual.VisualLayeredTree\n"+
"matrix.visual.VisualLayeredTreeComponent	matrix.visual.VisualLayeredTree\n"+
"matrix.visual.VisualLeafTree			matrix.visual.VisualLeafTree\n"+
"matrix.visual.VisualTreeComponent		matrix.visual.VisualLeafTree\n"+
"matrix.visual.VisualArray			matrix.visual.VisualArray\n"+
"matrix.visual.VisualArrayComponent		!UNDERLYINGTYPE\n"+
"matrix.visual.VisualList			matrix.visual.VisualList\n"+
"matrix.visual.VisualListComponent		matrix.visual.VisualList\n"+
"#matrix.visual.VisualStaticList			matrix.visual.VisualStaticList\n"+
"#VisualAdjList					VisualAdjList #not in use\n"+
"matrix.visual.VisualKey				matrix.visual.VisualKey\n"+
"matrix.visual.VisualTrash			matrix.visual.VisualTrash\n"+
"matrix.visual.VisualLayeredGraph		matrix.visual.VisualLayeredGraph\n"+
"matrix.visual.VisualLayeredGraphComponent	matrix.visual.VisualLayeredGraph\n"+
"matrix.visual.VisualDummyGraph			matrix.visual.VisualDummyGraph\n"+
"matrix.visual.VisualGraphComponent		matrix.visual.VisualDummyGraph\n"+
"matrix.visual.VisualClass			matrix.visual.VisualClass\n"+
"matrix.visual.VisualClassMethod			matrix.visual.VisualClass\n"+
"\n"+
"# FDT to String mappings. This mapping is used to create the representations\n"+
"# -submenu of the popup menu. \n"+
"!FDTREPRESENTATIONS\n"+
"\n"+
"matrix.structures.FDT.Tree		\"leaf tree\"    # these two lines to one line?\n"+
"matrix.structures.FDT.Tree		\"layered tree\" # these two lines to one line?\n"+
"matrix.structures.FDT.Array		\"array\"\n"+
"matrix.structures.FDT.LinkedList	\"list\"\n"+
"matrix.structures.FDT.LinkedList	\"append list\"\n"+
"#matrix.structures.FDT.StaticLinkedList	\"static list\"\n"+
"matrix.structures.FDT.Vertex		\"dummy graph\"\n"+
"matrix.structures.FDT.Vertex		\"layered graph\"\n"+
"matrix.structures.FDT.Graph		\"dummy graph\"\n"+
"matrix.structures.FDT.Graph		\"layered graph\"\n"+
"#SimulationGraph			\"adjency list\"\n"+
"#Graph					\"adj list\"\n"+
"matrix.structures.code.Code		\"code\"\n"+
"#CDT					???? # how this should be handled?\n"+
"\n"+
"#End marker. Everything under this will be ignored by the parser.\n"+
"!END\n"+
"\n"+
"# String to Object mappings in add -menu\n"+
"!STRUCTURENAMES\n"+
"\n"+
"\"-=- FDTs -=-\";			null \n"+
"\"Virtual Array\";		Table(\"        \") # Table w. 8 empty pos. == VA How to handle empty pos. in parser?\n"+
"\"Linked List\";			LinkedListImpl\n"+
"\"Dynamic Binary Tree\";		BinTree\n"+
"\"Static Binary Tree(8)\";	BinTrei(8) #create a new constructor\n"+
"\"Common Tree\";			SimulationTree2Impl\n"+
"\"Directed Graph\";		DirectedGraphImpl\n"+
"\"Undirected Graph\";		UndirectedGraphImpl\n"+
"\"-=- CDTs -=-\";			null\n"+
"\"Binary Search Tree\";		BinSearchTree\n"+
"\"B-Tree\";			BTree #how to handle minimum degree? Can't be hard-coded to B-Tree...\n"+
"\"Red-Black-Tree\";		RBTree\n"+
"\"Digital Search Tree\";		DigitalSearchTree\n"+
"\"Radix Search Tree\";		RadixSearchTrie\n"+
"\"Binary Heap\";			BinHeap\n"+
"\"AVL Tree\";			AVLTree\n"+
"\"String Searching\";		SearchTrable\n"+
"\"-=- Utils -=-\";		null\n"+
"\"Trash\";			Trash\n"+
"\"Table of Keys\";		Table(\"ABCDEFGHIJKLMNOPQRSTUVWXYZ\")\n"+
"\"Table of Random Keys\";		Table(10)\n"+
"\n"+
"\n"+
"\n"+
"#!VISUALIZATIONNAMES\n"+
"\n"+
"matrix.visual.VisualLayeredTree			\"layered tree\"\n"+
"matrix.visual.VisualLayeredTreeComponent	\"layered tree\"\n"+
"matrix.visual.VisualLeafTree			\"leaf tree\"\n"+
"matrix.visual.VisualLeafTreeComponent		\"leaf tree\"\n"+
"matrix.visual.VisualArray			\"array\"\n"+
"#matrix.visual.VisualArrayComponent		??? #should return the representation of the component\n"+
"matrix.visual.VisualList			\"list\"\n"+
"matrix.visual.VisualListComponent		\"list\"\n"+
"matrix.visual.VisualAdjList			\"adj list\"\n"+
"matrix.visual.VisualKey				\"key\"\n"+
"matrix.visual.VisualTrash			\"trash\"\n"+
"matrix.visual.VisualLayeredGraph		\"layered graph\"\n"+
"matrix.visual.VisualLayeredGraphComponent	\"layered graph\"\n"+
"matrix.visual.VisualDummyGraph			\"dummy graph\"\n"+
"matrix.visual.VisualDummyGraphComponent		\"dummy graph\"\n"+
"matrix.visual.VisualClass			\"class\"\n"+
"matrix.visual.VisualClassMethod			\"class\"\n"+
"#default				\"No representation found\" #should be hardcoded to parser\n"+
"\n"+
"\n";
}

