// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/* $Id: VisualClass.java,v 1.19 2007/09/10 08:11:09 vkaravir Exp $ */

package matrix.visual;

import java.awt.Graphics;
import java.lang.reflect.Method;

import matrix.structures.FDT.*;
import matrix.structures.FDT.probe.VanillaTable;
import matrix.util.Note;
//import matrix.animation.*;

/**
 * This is the container class for class representation.
 *
 * @author Ari Korhonen
 * @author Panu Silvasti
 * @version $Revision: 1.19 $
 */
public class VisualClass extends VisualArray {
    Class clazz;
    boolean allMethodsEnabled = false;
    Array localMethods = null;
    Array allMethods = null;

    public VisualClass(Object o) {
	super(makeArray(o));
	if(o instanceof FDT)
	    setStructure((FDT)o);
	Note.out(this, "Structure is: " + getStructure());
        clazz = o.getClass();
        //addPopUpCheckboxMenuItem("AllMethods", allMethodsEnabled);
    }

    /** Toggle allMethods flag. */
    public void AllMethods() {
        allMethodsEnabled = !allMethodsEnabled;
        if (allMethodsEnabled)
            setStructure(getAllMethods());
        else 
            setStructure(getLocalMethods());
        super.validateComponents();
        validate();
    }


    public static Array makeArray(Object o) {
	Note.out(null, "Start makeArray");
        return filterLocalMethods(o.getClass());
    }


    private static Array filterLocalMethods(Class clazz) {
        Method[] methods = clazz.getMethods();
        int count = 0;
        for (int i = 0;i<methods.length;i++) {
            if (methods[i].getDeclaringClass().equals(clazz))
                count++;
        }
        Method[] ret = new Method[count];
        count = 0;
        for (int i = 0;i<methods.length;i++) {
            if (methods[i].getDeclaringClass().equals(clazz))
                ret[count++] = methods[i];
        }
        return new VanillaTable(ret);
    }


    private Array getLocalMethods() {
        if (localMethods == null) {
            localMethods = filterLocalMethods(clazz);
        }
        return localMethods;
    }
    

    private Array getAllMethods() {
        if (allMethods == null) {
            allMethods = new VanillaTable(clazz.getMethods());
        }
        return allMethods;
    }


    public void validateComponents() {
        
        super.validateComponents();
    }

    public FDT getStructure() {
	return fdt;
    }

    // --- Dynamic Graph Behaviour ---

    public void write() {       
        getApplication().showMessage("Sorry", "write ["+read+"] invoked: not impemented yet");
    }
    
    public void insert(Object o) {
        getApplication().showMessage("Sorry", "insert ["+read+"] invoked: not impemented yet");
    }
    
    public void paintComponent(Graphics g) {
        super.paintComponent(g);
        // if (isMinimized()) { paint_minimized(g); return; }
        if (isPicked()) return; // no frames
        paintFrame(g);
    }
}


