// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/* $Id: VisualDummyGraph.java,v 1.18 2007/09/10 08:11:12 vkaravir Exp $ */

package matrix.visual;

import matrix.structures.FDT.*;
import matrix.structures.FDT.substructures.Vertex;

/**
 * A dummy graph layout algorithm. Lays the graph nodes in a row.
 * XXX should use a vertical row (because tree layouts grow down)?
 * (XXX) should use a circle instead?
 *
 * @author Riku Saikkonen
 * @version $Revision: 1.18 $
 */
public class VisualDummyGraph extends VisualGraph {
    public VisualDummyGraph(Vertex v) { super(v); }
    public VisualDummyGraph(Graph g) { super(g); }

    protected LayoutSize doActualLayout() {
        int count = getItemCount();
        int x = 0;
        int h = 0;

        for (int i = 0; i < count; i++) {
            VisualNode vn = (VisualNode) getItem(i);
            LayoutSize vnSize = vn.lGetPreferredSize();

            vn.lSetBounds(x, 0, vnSize.width, vnSize.height);

            x += vnSize.width + getGapWidth();

            if (h < vnSize.height)
                h = vnSize.height;
        }

        return new LayoutSize(x, h);
    }
}
