// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package matrix.visual;

import java.awt.Component;
import java.awt.Insets;
import java.awt.Rectangle;

import matrix.structures.FDT.advanced.GeometricGraph;
import matrix.structures.FDT.substructures.Vertex;

public class VisualGeometricGraph extends VisualGraph {
	
	private static int dist = 10;
	
	public VisualGeometricGraph(GeometricGraph g) {
        super(g);
    }

    protected LayoutSize doActualLayout() {
        Rectangle bounds = new Rectangle(0,0,1,1);//null;
        GeometricGraph geomStr = (GeometricGraph) getStructure(); 
        Component[] nodes = getItems();
        for (int i=0; i < nodes.length; i++) {
            VisualGraphComponent node = (VisualGraphComponent) nodes[i];
            Vertex v = (Vertex) node.getStructure();
            LayoutSize l = node.lGetPreferredSize();
            node.lSetBounds(geomStr.getX(v), geomStr.getY(v), l.width, l.height);
            if (bounds != null)
                bounds.add(new Rectangle(geomStr.getX(v), geomStr.getY(v), l.width, l.height));
            else
                bounds = new Rectangle(geomStr.getX(v), geomStr.getY(v), l.width, l.height);
            // *** Start of Reference hack ***
            // If a node is both predecessor and successor of this node in directed graph,
            // references are drawn in a way that shows both references separately.
            if(isDirected()) {
            	VisualReference[] pred = node.getPredecessors();
            	VisualReference[] succ = node.getSuccessors();
            	for(int j = 0; j < pred.length; j++) {
            		for(int k = 0; k < succ.length; k++) {
            			if(pred[j].getSource() == succ[k].getTarget()) {
            				VisualReference predecessor = pred[j];
            				VisualReference successor = succ[k];
            				predecessor.forgetMidPoints();
            				successor.forgetMidPoints();
            				VisualComponent otherComp = predecessor.getSource();
            				Vertex vi = (Vertex) otherComp.getStructure();
            				double x1, x2, y1, y2, xmid, ymid, xp1, yp1, xp2, yp2;
            				LayoutSize li = otherComp.lGetPreferredSize();
            				x1 = geomStr.getX(v) + l.width/2;
            				x2 = geomStr.getX(vi) + li.width/2;
            				y1 = geomStr.getY(v) + l.height/2;
            				y2 = geomStr.getY(vi) + li.height/2;
            				xmid = (x1+x2)/2;
            				ymid = (y1+y2)/2;
            				// vector parts placed to x1 and y1
            				x1 = x2-x1;
            				y1 = y2-y1;
            				double length = Math.sqrt(x1*x1 + y1*y1);
            				// size of the orthogonal vector parts placed to x2 and y2
            				x2 = (dist*y1/length);
            				y2 = -(dist*x1/length);
            				xp1 = xmid + x2;
            				xp2 = xmid - x2;
            				yp1 = ymid + y2;
            				yp2 = ymid - y2;
            				VisualGraphComponent dummy = new VisualGraphDummyComponent(this);
            				dummy.lSetBounds((int)xp1, (int)yp1, 1, 1);
            				predecessor.addMidPoint(dummy);
            				dummy = new VisualGraphDummyComponent(this);
            				dummy.lSetBounds((int)xp2, (int)yp2, 1, 1);
            				successor.addMidPoint(dummy);
            			}
            		}
            	}
            	
            } else {
            	VisualReference[] succ = node.getSuccessors();
            	for(int j = 0; j < succ.length; j++) {
            		succ[j].forgetMidPoints();
            	}
            }
            // *** end of Reference hack ***
        }
        return new LayoutSize(bounds.width, bounds.height);
    }

    /**
     * Inserts the given structure. If the structure is already a vertex of this
     * graph, the position of the vertex is changed. Otherwise, the insert is
     * done as usual.
     * 
     */
    public void insert(Object structure) {        
        GeometricGraph geomStr = (GeometricGraph) getStructure(); 
        if (structure instanceof Vertex && hasVertex((Vertex)structure)) {
            Insets i = getInsets();
            geomStr.setCoordinates((Vertex) structure, picker.getPick().getX() - this.getX() - i.left,
                                   picker.getPick().getY() - this.getY() - i.top);
        } else
            super.insert(structure);
    }
}
