// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package matrix.visual;

import matrix.structures.FDT.Graph;
import matrix.structures.FDT.substructures.Vertex;

/**
 * A version of the Kamada-Kawai graph drawing algorithm from
 * paper "An Algorithm for Drawing General Undirected Graphs" in
 * Information Processing Letters 31 (1989) by Kamada and Kawai.
 * This implementation handles the nodes as points.
 *
 * @author Ville Karavirta
 */
public class VisualKKPointGraph extends VisualKKGraph {

    public VisualKKPointGraph(Graph g) {
        super(g);
    }

    public VisualKKPointGraph(Vertex v) {
        super(v);
    }


    protected double calculateEdgeLength() {
        double len;
         if (isWeighted) //little bit longer edge length for weighted graphs
            len = 30*Math.sqrt(nodes)*1.7;//*1.15; // by changing this coefficient it is possible to adjust the length of the edges
        else
            len = 30*Math.sqrt(nodes)*1.15;
        len = Math.min(len, 200);
        return len;
    }

    protected Matrix2D calculateW() {
        Matrix2D w = new Matrix2D(nodes, nodes);
        w.assign(len);
        return w;
    }
}
