// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/* $Id: VisualKey.java,v 1.65 2008/08/28 12:47:48 lhakulin Exp $ */

package matrix.visual;

import java.awt.*;

import content.exercises.InsertionSortPartlyOrdered.InsertionBarTableKey;

import matrix.decoration.*;
import matrix.structures.FDT.*;
import matrix.structures.FDT.probe.Key;

public class VisualKey extends VisualPrimitive {

    private static final long serialVersionUID = 1252144691970372442L;
    protected Key key;
    private java.awt.Component awtComponent = null;
    
    public static int MIN_SIZE = 8;
    
    public VisualKey(Key k) {
        super(k);
        setName(k.getVisualizationString());
        key = k;
    }
    
    public void rename() {
        if (getParent() instanceof VisualReference) {
            ((VisualReference)getParent()).rename();
            setInvalid();
            return;
        }
        rename(getApplication().getInput("Rename: ", getName()));
    }
    
    public void rename(String newName) {
        if (newName.equals(getName()))
            return;
        getAnimator().startOperation();
        key = new Key(newName);
        setName(getKey());
        read = this;
        write();
        setInvalid();
        getAnimator().endOperation();
    }
    
    public void doLayout() {
        Font font;
        Rectangle r = getBounds();
        if ((isVisible()) && ((font = getFont()) != null)) {
            if (isEmpty()) {
                width = r.width;
                height = r.height;
            } else {
                height = getFontMetrics(font).getHeight();
                
                if (getKey().length() == 1) {
                    if (getKey().charAt(0) >= '\u0100') { 
                        /* If the key is a single character which is unicode character greater 
                         * or equal than 0100 (thus, something else than basic latin
                         * or latin-1 supplement character), the width is the actual width of 
                         * the key. Also the height is changed little smaller so that VisualKey 
                         * doesn't take so much space.
                         */ 
                        width = getFontMetrics(font).stringWidth(getKey());
                        height = getFontMetrics(font).getAscent();
                    } else {
                        width = getFontMetrics(font).stringWidth("WW");
                    }
                    if (this instanceof VisualLabel) //this is weight
                        width = getFontMetrics(font).stringWidth(getKey()) + 6;
                }
                else {
                    width = getFontMetrics(font).stringWidth(getKey()) + 4;
                    if (width < getFontMetrics(font).stringWidth("WW"))
                        width = getFontMetrics(font).stringWidth("WW");
                }
            }
        }
    }
    
    public Dimension getPreferredSize() { 
        if (width < MIN_SIZE) 
            width = MIN_SIZE;
        if (height < MIN_SIZE)
            height = MIN_SIZE;
        return new Dimension(width, height);
    }
    
    public String getKey() {
        // if this is label of a reference (weight) then ask key from the underlying structure
        if (getParent() instanceof VisualReference && 
                !(((VisualReference)getParent()).getTarget() instanceof VisualArrow)) {
            VisualComponent vc = ((VisualReference)getParent()).getSource();
            if (vc.getStructure() instanceof LabelDecorator) {
                LabelDecorator ld = (LabelDecorator)vc.getStructure();
                int index = vc.getReferenceNumber(((VisualReference)getParent()).getTarget());
                String label = ld.getReferenceLabel(index);
                key.changeKeyValue(label); 
                
            }
        }
        return key != null ? 
                ((key.getVisualizationString() != null) ? key.getVisualizationString() : "") : 
                    "<E>";
    }
    
    public void setKey(String s) {
        if (getKey() != s)
            key.changeKeyValue(s);
    }
    
    /* We cannot assign anything to keys, thus we call parent's write method. */
    public void assign() {
        if (getParent() instanceof VisualType) {
            ((VisualType)getParent()).write();
        }
    }
    
    public void selectEntity(boolean isShiftDown) {
        getAnimator().startOperation();
        key.select();
        getAnimator().endOperation();
    }
    
    public void pressed() {
    	if (key instanceof InsertionBarTableKey){
    		((InsertionBarTableKey)key).pressed(); // For index highlighting in exercise InsertionSort
    	}
    	super.pressed();
    }
    
    public void delete() {
        if (getParent() instanceof VisualType)
            ((VisualType)getParent()).removeKey(key);
    }
    
    public void write() {
        ((VisualType)getParent()).write();
    }
    
    private final static int ROUND = 15;
    
    public FDT getStructure() {
        /** Changes:
         *   Since 20.5.2005 normal keys are no longer regenerated over and over again.
         *   This drops the number of keys generated by a factor of 100.
         *  
         *   TODO: Whether this should be done to the other key types should be investigated
         *         further. 
         */
        
        Key k = key;
        k.setOrig(key);
        return k;
        
    }
    
    /**
     * Determines whether this component has been visited by looking first at the FDT and if it is not
     * Visitable then by returning isVisited flag. 
     * @return true if this component has been visited; false otherwise.
     */
    public boolean isVisited() {
        if ((key != null) && (key instanceof Visitable)) {
            return ((Visitable)key).isVisited();
        }
        return super.isVisited() || key.isSelected(); 
        
    }
    /*
     public boolean isVisited() {
     return (super.isVisited() || key.isSelected());
     }
     */
    public boolean isEmpty() {
        return getKey().trim().equals(Key.EMPTY);
    }
    protected Font lastFont = null;
    protected String lastKey = null;
    protected int textX = 0, textY = 0;
    
    public void paintComponent(Graphics g) {
        super.paintComponent(g);
        if (this instanceof VisualLabel)
            return;
        
        if (g == null) {
            return;
            //Note.warning(this, "Graphics not defined");
        }
        int x = 1;
        int y = 1;
		int w = width-2;
		int h = height-2;
        
        setBackgroundColor(g);
        g.fillRect(x, y, w, h);
        
        Font font = getFont();
        String keyLabel = getKey();
        if (font != null) {
            if ((font != lastFont) || (keyLabel != lastKey)) {
                FontMetrics fm = getFontMetrics(font);
                lastFont = font;
                textX = x + (w - fm.stringWidth(keyLabel))/2;
                textY = y + h - (h - fm.getHeight())/2 - fm.getDescent();
            }
            //setColor(g);
            setPenColor(g);
            g.drawString(keyLabel, textX, textY);
            //resetColor(g);
        }
    }
    
    public static void fontSizeChanged(int newFontSize) {
        if (newFontSize > 0) {
            MIN_SIZE = newFontSize/2+1;
        }
    }
    
    /* This method is called from configuration during system initialization */
    public static void setMinSize(int size) {
        if (size > 0) {
            MIN_SIZE = size;
        }
    }
    
}
