// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/* $Id: VisualLeafTree.java,v 1.17 2007/09/10 08:11:12 vkaravir Exp $ */

package matrix.visual;

import matrix.structures.FDT.Tree;

/**
 * A tree-drawing algorithm that keeps the left-right order of the leaves.
 *
 * @author Ari Korhonen, Riku Saikkonen
 * @version $Revision: 1.17 $
 */
public class VisualLeafTree extends VisualTree {
    public VisualLeafTree(Tree t) {
        super(t);
    }

    public String getRepresentationName() {
        return "Leaf Tree";
    }
    
    protected LayoutSize doActualLayout() {
        return getTreeSize(getRootNode(),
                           // XXX hack to fix extra space given by getTreeSize
                           new LayoutBounds(-getGapWidth(), -getGapHeight(),
                                            0, 0));
    }

    /* This is the actual method to recursively count the size of the tree and
       lay out its representation */
    private LayoutSize getTreeSize(VisualTreeComponent vtc,
                                  LayoutBounds bounds) {
        LayoutSize vtcSize = vtc.lGetPreferredSize();
        VisualTreeComponent children[] = vtc.getChildren();
        
        int x = bounds.x;
        int y = bounds.y;
        int w = 0, h = 0;
        bounds.y = bounds.y + vtcSize.height + getGapHeight();
        //bounds.x += getGapWidth();
        
        // getTreeSize of all subtrees of vtc
        for (int i = 0; i < children.length; i++) {
            bounds.x = x + w + getGapWidth();
            LayoutSize d = getTreeSize(children[i], new LayoutBounds(bounds));
            w += d.width + getGapWidth();
            h = h > d.height ? h : d.height;
        }
        
        if ((w < vtcSize.width) && (children.length > 0)) {
            int extra = (vtcSize.width - w) / (children.length + 1);
            //Note.out(this, " move childs of " + index + " right " + extra);
            for (int i = 0; i < children.length; i++) {
                    VisualTreeComponent c = children[i];
                    LayoutBounds cb = c.lGetBounds();
                    // move component to right a little bit
                    cb.x += (i+1) * extra;                    

                    // c.lSetBounds(cr); doesn't move children...
                    getTreeSize(c, cb);
            }
        }
        
        w = w > vtcSize.width ? w : vtcSize.width;
        h = h + vtcSize.height + getGapHeight();
        vtc.lSetBounds(x + (w - vtcSize.width) / 2,
                      y + getGapHeight(),
                      vtcSize.width,
                      vtcSize.height);
        //Note.out(this, "vtc = " + vtc + " bounds = " + vtc.lGetBounds());
        return new LayoutSize(w,h);
    }
}
