// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/* $Id: VisualListComponent.java,v 1.32 2007/09/10 08:11:08 vkaravir Exp $ */

package matrix.visual;

import java.awt.Graphics;
import java.io.PrintStream;

import matrix.structures.FDT.*;
import matrix.structures.CDT.*;
import matrix.util.*;

public class VisualListComponent extends VisualNode {
    protected LinkedList list;
    protected VisualType vt;
    protected VisualListComponent prev, next;
    private int index = -1;

    public VisualListComponent(LinkedList list) {
        super((FDT)list);
        this.list = list;

        //setName("VisualListComponent");
	/*
        addPopUpMenuSeparator(); // separator
        addPopUpMenuItem(new java.awt.MenuItem("invalidater"));
        addPopUpMenuItem(new java.awt.MenuItem("setInvalid"));
        addPopUpMenuItem(new java.awt.MenuItem("TataEiOle"));
	*/
    }
    
    public void invalidater() {
        invalidate();
        repaint();
    }

    public void addComponents() {
        add(vt = getRepresentation(list != null ? list.getElement() : null));
    }
    /*
    public VisualType getVisualType() {
        return vt;
    }
    */
    public FDT getStructure() {
        return list;
    }
    /*
    public FDT getKey() {
	if(getStructure() == null) return null;
	return turnFDT(getStructure().getElement());
    }
    */
    public void assign() {
        //((VisualList)getVisualContainer()).insertAt(getIndex());
	if (getStructure().getElement() == null || getSemantics() == SWAP_SEMANTICS) {
	    getStructure().setElement(read.getStructure().getElement());
	} else if (list instanceof DoublyLinkedList) {
	    LinkedList tmp = list.getNewNode(read.getStructure());
	    tmp.setNext(list.getNext());
	    list.setNext(tmp);
	    ((DoublyLinkedList)tmp).setPrev((DoublyLinkedList)list);
	    if (tmp.getNext() != null)
		((DoublyLinkedList)tmp.getNext()).setPrev((DoublyLinkedList)tmp);
	} else {
	    LinkedList tmp = list.getNewNode(read.getStructure());
	    tmp.setNext(list.getNext());
	    list.setNext(tmp);
	}
    }

    public void moveReference(VisualReference r, 
                              VisualListComponent oldTarget, 
                              VisualListComponent newTarget) {
	//Note.out(this, "moveListReference2 " + r);
	Validator.add(new Command(list,
				  newTarget.getStructure()) {
		public void execute() {
		    LinkedList l = (LinkedList)parameter;
		    LinkedList t = (LinkedList)parameter2;
		    if (l == t)
			l.setNext(null);
		    else
			l.setNext(t);
		}
	    });
	Validator.add(new Command(r) {
		public void execute() {
		    ((VisualType)parameter).setVisited(false);
		}
	    });
	r.setVisited(true);
    }

    public void moveReference(VisualReference r, 
                              VisualType oldTarget, 
                              VisualType newTarget) {
	//Note.out(this, "moveListReference1 " + r + " to " + oldTarget);
        
        while ((newTarget instanceof VisualType) && 
	       (!(newTarget instanceof VisualListComponent))) {
	    if (newTarget.getParent() instanceof VisualType)
		newTarget = (VisualType)newTarget.getParent();
	    else
		break;
	}
        
	//Note.out(this, "resolved " + newTarget);
	
        if (newTarget instanceof VisualListComponent) 
	    moveReference(r, (VisualListComponent)oldTarget, (VisualListComponent)newTarget);
	else if (newTarget == null) { //removing the end of the list
	    moveReference(r, (VisualListComponent)oldTarget, (VisualListComponent)oldTarget);
	}
	else
            getApplication().showMessage("Sorry", "Cannot move to " + 
					 ((newTarget != null) ? newTarget.getName() : "null"));
    }
    
    public void erase() {
	if (!(getStructure() instanceof CDT)) {
	    getStructure().setElement(null);
	    setInvalid();
	} else {
	    //getApplication().showMessage("Sorry", "Delete not implemented.");
	    ((VisualList)getParent()).delete(getStructure().getElement());
	    setInvalid();
	}
    }
    
    protected void removeKeyHelp(Object o) {
	if (!(getStructure() instanceof CDT)) {
	    remove();
	}
	else {
	    super.removeKeyHelp(o);
	}
    }   
    
    public void remove() {
	erase();		
    }
    
    /*
      public Dimension getPreferredSize() {
        if (getVisualType() != null) {
            Insets in = getInsets();
            Dimension d = getVisualType().getPreferredSize();
            width = (int)Math.sqrt(d.width*d.width+d.height*d.height);
            height = in.top + width*d.height/d.width + in.bottom;
            width += in.left + in.right;
        }
        return new Dimension(width,height);
    }
    */
    public int getIndex() { 
        if (index < 0) 
            Note.err(this, "getIndex(): not set");
        return index;
    }

    public void setIndex(int index) {
        this.index = index;
    }

    public VisualListComponent getNext() {
        return next;
    }

    public void setNext(VisualListComponent vlc) {
        next = vlc;
        if (vlc != null)
            if (!this.equals(vlc.getPrevious()))
                vlc.setPrevious(this);
    }

    public VisualListComponent getPrevious() {
        return prev;
    }

    public void setPrevious(VisualListComponent vlc) {
        prev = vlc;
        if (vlc != null)
            if (!this.equals(vlc.getNext()))
                vlc.setNext(this);
    }

    private void paintLabel(Graphics g, int x, int y, int w, int h) {
        if (labeled) g.drawString(getName(), x + w + 1, y + h);
    }
    
    public void dump(PrintStream out) {
	super.dump(out);
	out.println("list: " + list.toString());
    }
    /*
    public void paint(Graphics g) {
        // super.paint(g) is invoked at the end of this method 
        Rectangle r = getBounds();
        Insets in = getInsets();
        int x = in.left; //in.left 
        int y = in.top; //in.top;
        int w = width - in.left - in.right;
        int h = height - in.top - in.bottom;
        
        //if (hidden()) { return; }
        
        /*
        if (isEmpty()) {
            setClearColor(g);
            g.fillRect(x,y,w,h);
            setColor(g);
            if (hasFocus()) g.fillRect(x,y,w,h);
            else g.drawRect(x,y,w,h);   
            paintLabel(g,x,y,w,h);      
        } else {
            if (hasFocus()) setColor(g); else setClearColor(g);
            g.fillOval(x,y,w,h);
            setColor(g);    
            g.drawOval(x,y,w,h);
            paintLabel(g,x,y,w,h);
        }
        resetColor(g);

        setBackgroundColor(g);
        g.fillOval(x,y,w,h);
        setBorderColor(g);
        g.drawOval(x,y,w,h);
        setPenColor(g);
        paintLabel(g,x,y,w,h);
        super.paint(g);
    }

    public Point getBorderAtAngle(double angle) {
        Rectangle r = getBounds();
        Insets in = getInsets();
        int x = in.left;
        int y = in.top;
        int w = width - in.left - in.right;
        int h = height - in.top - in.bottom;
        Point c = getCenter();

        if (isEmpty()) { // square
            return c; // XXX
        } else if (isMinimized() || isDuplicate()) { // triangle
            return c; // XXX
        } else { // ellipse
            int ellA = w / 2, ellB = h / 2;
            return new Point((int) (c.x + Math.cos(angle) * ellA),
                             (int) (c.y + Math.sin(angle) * ellB));
        }
    }
    */
}
