// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package matrix.visual;

import java.awt.*;

import matrix.structures.FDT.*;
import matrix.util.export.ConfigurableGraphics;
import matrix.util.export.MatrixGraphics;

/** VisualReziseHotSpot is an abstraction for special active resize components within containers.
 * @see VisualContainer
 *
 * @author Ari Korhonen
 * @version $Revision: 1.8 $
 */
public class VisualSearchHotSpot extends VisualHotSpot {

    // --- constructors ---

    public VisualSearchHotSpot(FDT fdt) {
        super(fdt);
        setLabel("(?)");
    }

    /** This method is invoked when a new key is dropped into this HotSpot. 
	Search method is invoked with the read key as the parameter.
     */
    public void assign() {
	if (getVisualContainer() != null) {
	    getVisualContainer().search(read.getStructure());
	}
        setInvalid();
    }

    public void setInvalid() {
	
    }

    public void paintComponent(Graphics g) {
    	if (g instanceof MatrixGraphics)
    		return;
       	if (g instanceof ConfigurableGraphics) {
            if (((ConfigurableGraphics)g).isDisabled("StructureBorder")) {
                //if exporting and structure borders are not wanted then stop here
                return;
            }
        }
        Rectangle r = getBounds();
        setBorderColor(g);
        g.fillRect(0, 0, r.width, r.height);
        if (g instanceof ConfigurableGraphics) {
        // If g is ConfigurableGraphics, we have to check whether the hotspots are 
        // wanted to be painted or not.
        //
            if (((ConfigurableGraphics)g).isDisabled("VisualHotSpot")) {
                super.paintComponent(g);
                return;            
            }
        }
        ((Graphics2D)g).scale(getFont().getSize()/16.0, getFont().getSize()/16.0);
        if (hasFocus()) {
            g.setColor(opaqueWhite);
            g.fillRoundRect(1, 2, 15, 14, 1, 1);
            g.setColor(opaqueWhite.darker().darker());
            g.drawRoundRect(1, 2, 14, 13, 1, 1);
            g.setColor(g.getColor().darker().darker());
            g.drawRoundRect(1, 2, 15, 14, 1, 1);
            
        }
        g.setColor(Color.white);
        g.fillOval(5,5,7,7);
       //super.paintComponent(g);
        //((Graphics2D)g).scale(1, 1);
    }
}
