// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package prototype.ui;

import java.awt.*;

import javax.swing.*;
import java.awt.event.*;
import java.lang.reflect.*;

import content.interfaces.AWTComponentUtilizer;
import content.interfaces.ButtonExercise;
import content.interfaces.ButtonGroups;
import content.interfaces.ConfigureVisualType;
import content.interfaces.Exercise;
import content.interfaces.HasRotatedContainers;
import content.interfaces.SimulationExercise;
import content.interfaces.StyledExercise;
import content.interfaces.SwapBehaviour;
import content.interfaces.LayoutExercise;

import matrix.animation.Animator;
import matrix.simulation.VisualTypeConf;
import matrix.uitools.StructurePanel;
import matrix.visual.*;

public class ExercisePanel extends StructurePanel {
    
    /**
     * 
     */
    private static final long serialVersionUID = -4955491229396826152L;

    public static final boolean DEBUG = true;
    
    SimulationExercise exercise;
    
    private boolean swapEnabled = false;
    
    // constructors
    
    
    /**
     */
    public ExercisePanel(Animator animator, SimulationExercise e) {
        super(animator);
        this.exercise = e;
        initExercise(e);
        
        if (e instanceof SwapBehaviour){
            setSwapEnabled(((SwapBehaviour)e).getSwapBehaviour());
        }
    }
    
    
    
    public void setExercise(SimulationExercise e) {
        exercise = e;
    }
    
    public SimulationExercise getExercise() {
        return exercise;
    }
    
    
    protected void initExercise(SimulationExercise e) {
        
        // animator is changed in case the active animator is some
        // other animator (than the animator of this frame)
        Animator.setActiveAnimator(getAnimator());
        
        VisualType[] vt = null;
        // This if-else clause will create the data structures used in the exercise.
        if(e instanceof LayoutExercise) {
            if(e instanceof StyledExercise) {
                vt = addStructures(e.init(), ((StyledExercise)e).getStructureVisualisations(), ((LayoutExercise)e).getConstraints());    		   
            } else {
                vt = addStructures(e.init(), ((LayoutExercise)e).getConstraints());    		  
            }
        } else if(e instanceof StyledExercise) {
            vt = addStructures(e.init(), ((StyledExercise)e).getStructureVisualisations());
        } else {
            vt = addStructures(e.init());
        }
        
        //getAnimator().endOperation();
        //getAnimator().disposeUndo();
        
        String[] names = e.getStructureNames();
        for (int i = 0; i < names.length; i++) {
            vt[i].setName(names[i]);
        }
        
        // rotates containers when needed
        if(e instanceof HasRotatedContainers){
            boolean[] rotateInfo = ((HasRotatedContainers)e).getContainerRotations();
            for (int i = 0; i < vt.length; i++) {
                if (vt[i] instanceof VisualContainer)
                    ((VisualContainer)vt[i]).setRotated(rotateInfo[i]);
            }	  
        }
        
        
        // disable hotSpot
        for (int i = 0; i < vt.length; i++) {
            if (vt[i] instanceof VisualContainer) { // should be
                ((VisualContainer)vt[i]).setHotSpotEnabled(false);
                ((VisualContainer)vt[i]).setSearchHotSpotEnabled(false);
                ((VisualContainer)vt[i]).setNullHotSpotEnabled(false);
                if (vt[i] instanceof VisualArray) {
                    ((VisualArray)vt[i]).setResizeHotSpotEnabled(false);
                }
            }
        }
        
        
        if (e instanceof ConfigureVisualType) {
            VisualTypeConf[] conf = ((ConfigureVisualType)e).conf();
            for (int i = 0; i < vt.length; i++) {
                vt[i].configure(conf[i]);
            }
        }
        
        
        if (e instanceof ButtonExercise) {
            JButton[] buttons = makeButtons((ButtonExercise)e);
            layoutButtons(e, buttons);
        }
        
        if (e instanceof AWTComponentUtilizer) {
            // The exercise contains additional components -> add them.
            Component[] components = ((AWTComponentUtilizer)e).getAWTComponents();
            GridBagConstraints[] gbc = ((AWTComponentUtilizer)e).getAWTComponentConstraints();
            if (gbc == null) {
                for (int i = 0; i < components.length; i++) {
                    add(components[i]);
                }
            } else {
                for (int i = 0; i < components.length; i++) {
                    add(components[i], gbc[i]);
                }
            }
            // Give the application reference to the exercise object.
            // Might, e.g., be needed for validating the application. 
            ((AWTComponentUtilizer)e).setApplication(this);
        }
        
    }
    
    
    
    
    
    
    
    protected void layoutButtons(Exercise e, JButton[] buttons) {
        
        if (e instanceof ButtonGroups) {
            // top and botton insets=5, left and right insets = 5
            Insets buttonInsets = new Insets(2,20,2,20);
            int[] groups = ((ButtonGroups)e).buttonGroups();
            int k = 0;
            for (int i = 0; i < groups.length; i++) {
                JPanel panel = new JPanel();
                while (k < groups[i]) {
                    panel.add(buttons[k++]);
                }
                addComponent(panel, buttonInsets);
                panel.setBackground(getBackground());
                k = groups[i];
            }
            JPanel panel = new JPanel();
            for (int i = k; i < buttons.length; i++) {
                panel.add(buttons[i]);
            }
            addComponent(panel,buttonInsets);
            panel.setBackground(getBackground());
        }
        
        else {
            for (int i = 0; i < buttons.length; i++) {
                addComponent(buttons[i]);
            }
        }
    }
    
    
    
    protected JButton[] makeButtons(ButtonExercise bex) {
        final String[] buttons = bex.buttonNames();
        final String[] commands = bex.buttonCommands();
        JButton[] but = new JButton[buttons.length];
        for (int i = 0; i < buttons.length; i++) {
            final JButton b = new JButton(buttons[i]);
            
            b.setActionCommand(commands[i]);
            b.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent evt) {
                    reflection(b.getActionCommand());    
                    updateGradeButton();	// disable/enable grade button if necessary
                }
            });
            but[i] = b;
        }
        return but;
    }
    
    
    /**
     * This method allows reflection calls to be invoked from exercises.
     * See, for example, content.interfaces.ButtonExercise that can be applied to
     * include push buttons that invoke methods for objects. 
     * @param cmd - the method to be invoked. Use "/method-name" to call a method
     * in the corresponding Exercise. Use, e.g., "reflectSelectedVisualType(exerciseReflectionCommand(method-name))"
     * to invoke method-name for selected object through VisualType reflection.
     */
    public void reflection(String cmd) {
        String param = null;
        int p1 = cmd.indexOf('(');
        int p2 = cmd.lastIndexOf(')');
        Class cl;
        Object callObj;
        if (p1 < p2) {
            param = cmd.substring(p1+1, p2);
            cmd = cmd.substring(0, p1);
        }
        if (p2 - p1 <= 1) {
            param = null;
        }
        if (cmd.charAt(0)=='/') {
            /* Prefix with / to call exercise's methods instead of
             exercise panel's. */
            cmd = cmd.substring(1);
            cl = exercise.getClass();
            callObj = exercise;
        } else {
            cl = getClass();
            callObj = this;
        }
        do {
            Method[] m = cl.getDeclaredMethods();
            for (int i=0; i<m.length; i++) {
                if (m[i].getName().equals(cmd)) {
                    Class[] c = m[i].getParameterTypes();
                    if ((c.length == 0) && (param == null)) {
                        // methods with no parameters
                        try { m[i].invoke(callObj, null); }
                        catch (Exception e) {
                            if(e instanceof InvocationTargetException) {
                                Throwable invoked = ((InvocationTargetException)e).getTargetException();
                                invoked.printStackTrace();
                                showMessage("InvocationTargeException", invoked.toString());
                            } else {
                                e.printStackTrace();
                                showMessage("Exception", e.toString());
                            }
                        }
                        validate();
                        return;
                    } else if ((c.length == 1) && (param != null)) {
                        // methods with single parameter
                        Object[] parameter = new Object[] { param };
                        try { m[i].invoke(callObj, parameter); }
                        catch (Exception e) {
                            if(e instanceof InvocationTargetException) {
                                Throwable invoked = ((InvocationTargetException)e).getTargetException();
                                invoked.printStackTrace();
                                showMessage("InvocationTargetException", invoked.toString());
                            } else {
                                e.printStackTrace();
                                showMessage("Exception", e.toString());
                            }
                        }
                        validate();
                        return;
                    }
                }
            }
            cl = cl.getSuperclass();
        } while (cl != null);
        showMessage("Error", "Method " + cmd + " not found in " + getClass());
    }
    
    /**
     * This method allows reflection calls to be invoked from exercises.
     * See, for example, content.interfaces.ButtonExercise that can be applied to
     * include push buttons that invoke methods for objects. 
     * @param cmd - the method to be invoked. Use "/method-name" to call a method
     * in the corresponding Exercise. Use, e.g., "reflectSelectedVisualType(exerciseReflectionCommand(method-name))"
     * to invoke method-name for selected object through VisualType reflection.
     */
    public void reflectSelectedVisualType(String cmd) {
        if (VisualType.read != null) {
            VisualType.read.reflection(cmd);
        }
    }
    
    
    public void reset() {
        // The same animator should not be used again to avoid problems with a corrupted animator affecting exercise grading
        // Therefore, we create a new one.
        Animator a = new Animator();
        Animator.setActiveAnimator(a);
        setAnimator(a);
        
        removeComponents();
        removeAll(); // Otherwise AWT-components are not removed
        addPicker();
        repaint();
        initExercise(exercise);
        addEmptyComponentToTheBottom();
        validate(); //ADDED 3.2.2004
        //(otherwise panels have incorrect lengths after reset)
    }
    
    
    public void setSwapEnabled(boolean value) {
        swapEnabled = value;
        if (swapEnabled) {
            turnSwapOn();
        } else {
            turnSwapOff();
        }
    }
    
    
    
    public boolean getSwapEnabled() {
        return swapEnabled;
    }
    
    
    private void turnSwapOn() {
        setSwapSemantics(VisualType.SWAP_SEMANTICS);
    }
    
    
    private void turnSwapOff() {
        setSwapSemantics(VisualType.INSERT_SEMANTICS);
    }
    
}


