// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package prototype.ui;

import javax.swing.JMenuBar;

import content.interfaces.ConfigureVisualType;
import content.interfaces.HasRotatedContainers;
import content.interfaces.ModelAnswerNames;
import content.interfaces.SimulationExerciseModel;
import content.interfaces.StyledExercise;
import content.interfaces.LayoutExercise;

import matrix.animation.Animator;
import matrix.simulation.VisualTypeConf;
import matrix.structures.FDT.FDT;
import matrix.uitools.*;
import matrix.visual.VisualContainer;
import matrix.visual.VisualType;
import matrix.visual.VisualArray;

public class ModelAnswerFrame extends MenuFrame {

  public ModelAnswerFrame(SimulationExerciseModel mod) {
    super(new Animator());
    setTitle("Model answer");
    getVisualAnimator().setResetButtonVisible(false);
    getVisualAnimator().setPlayButtonVisible(false);
    setSize(715, 715);
    StructurePanel p = getStructurePanel();

    Animator.setActiveAnimator(getAnimator());
    FDT[] answer = mod.makeModelAnswer();
    getAnimator().rewind();

    VisualType[] vt = null;
    if(mod instanceof LayoutExercise) {
  	  if(mod instanceof StyledExercise) {
	        vt = p.addStructures(answer, ((StyledExercise)mod).getModelAnswerVisualisations(), ((LayoutExercise)mod).getModelAnswerConstraints());    		   
	  } else {
	        vt = p.addStructures(answer, ((LayoutExercise)mod).getModelAnswerConstraints());    		  
	  }
    } else if(mod instanceof StyledExercise) {
      vt = p.addStructures(answer, ((StyledExercise)mod).getModelAnswerVisualisations());
    } else {
      vt = p.addStructures(answer);
    }

    if (mod instanceof ModelAnswerNames) {
	String[] names = ((ModelAnswerNames)mod).getModelAnswerNames();
	for (int i = 0; i < vt.length; i++) {
	    vt[i].setName(names[i]);
	}
    }

    //Rotates containers if requested
    if(mod instanceof HasRotatedContainers){
	boolean[] rotateInfo = ((HasRotatedContainers)mod).getModelAnswerRotations();
	for (int i = 0; i < vt.length; i++) {
	    if (vt[i] instanceof matrix.visual.VisualContainer)
		((matrix.visual.VisualContainer)vt[i]).setRotated(rotateInfo[i]);
	}	  
    }

    for (int i = 0; i < vt.length; i++) {
        if (vt[i] instanceof VisualContainer) {
            ((VisualContainer)vt[i]).setHotSpotEnabled(false);
	    ((VisualContainer)vt[i]).setNullHotSpotEnabled(false);
	    ((VisualContainer)vt[i]).setSearchHotSpotEnabled(false);
        }
        if (vt[i] instanceof VisualArray) {
            ((VisualArray)vt[i]).setResizeHotSpotEnabled(false);
        }
    }
    
    if (mod instanceof ConfigureVisualType) {
        VisualTypeConf[] conf = ((ConfigureVisualType)mod).conf();
        for (int i = 0; i < vt.length; i++) {
            vt[i].configure(conf[i]);
        }
    }

    getStructurePanel().disableVisualComponents();
    getVisualAnimator().validate();
  }


  protected JMenuBar createMenuBar() {
      return MenuFactory.getInstance().getMenuBar(this);
  }


}
