// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/* $Id: QueueTest.java,v 1.7 2007/09/07 13:33:27 vkaravir Exp $ */

package tests.matrix.structures.ADT.probe;

import junit.framework.TestCase;
import matrix.structures.ADT.probe.Queue;

/**
 * This class tests methods in matrixt.structures.ADT.probe.Queue class.
 * 
 * @author kstalnac
 */
public class QueueTest extends TestCase {

	private Queue queue1;
	
	public static void main(String[] args) {
		junit.textui.TestRunner.run(QueueTest.class);
	}

	/**
	 * Sets up the objects for the test methods.
	 * @see TestCase#setUp()
	 */
	protected void setUp() throws Exception {
		queue1 = new Queue();
	}

	/**
	 * Removes the objects after every test method is finished.
	 * @see TestCase#tearDown()
	 */
	protected void tearDown() throws Exception {
		queue1 = null;
	}

	/**
	 * Constructor for TestQueueADT.
	 * @param name 
	 */
	public QueueTest(String name) {
		super(name);
	}

	/**
	 * Tests put method.
	 *
	 */	
	public final void testPut() {
		queue1.ADTEnqueue(new Integer(5));
		queue1.ADTEnqueue(new Integer(17));
		assertEquals("Queue should not be empty!", false, queue1.ADTIsEmpty());
	}

	/**
	 * Tests get method. 
	 *
	 */	
	public final void testGet() {
		queue1.ADTEnqueue(new Integer(1));
		queue1.ADTEnqueue(new Integer(2));
		queue1.ADTEnqueue(new Integer(3));
		assertEquals("Removed element should be 1", 1, ((Integer)queue1.ADTDequeue()).intValue());
		queue1.ADTDequeue();
		assertEquals("First element in the queue should be 3", 3, ((Integer)queue1.ADTPeek()).intValue());
		assertEquals("Removed element should be 3", 3, ((Integer)queue1.ADTDequeue()).intValue());
		assertEquals("Queue should be empty!", true, queue1.ADTIsEmpty());
		assertEquals("If queue is empty, ADTDequeue() should return null.", null, queue1.ADTDequeue());
		
		queue1.ADTEnqueue(new Integer(4));
		queue1.ADTEnqueue(new Integer(5));
		queue1.ADTEnqueue(new Integer(6));
		queue1.ADTEnqueue(new Integer(7));
		assertEquals("First element in the queue should be 4.", 4, ((Integer)queue1.ADTPeek()).intValue());
		queue1.ADTDequeue();
		queue1.ADTDequeue();
		queue1.ADTDequeue();
		assertEquals("Removed element should be 7", 7, ((Integer)queue1.ADTDequeue()).intValue());
		assertEquals("Queue should be empty!", true, queue1.ADTIsEmpty());
		
	}

	/**
	 * Tests "first" method.
	 *
	 */
	public final void testFirst() {
		assertEquals("If queue is empty, ADTPeek() should return null.", null, queue1.ADTPeek());	
		queue1.ADTEnqueue(new Integer(5));
		assertEquals("first element in queue should be 5", 5, ((Integer)queue1.ADTPeek()).intValue());

		queue1 = new Queue();
		queue1.ADTEnqueue(new Integer(1));
		queue1.ADTEnqueue(new Integer(2));
		queue1.ADTEnqueue(new Integer(3));
		queue1.ADTEnqueue(new Integer(4));
		queue1.ADTEnqueue(new Integer(5));
		queue1.ADTEnqueue(new Integer(6));
		assertEquals("first element in queue should be 1", 1, ((Integer)queue1.ADTPeek()).intValue());
	}

	/**
	 * Tests ADTIsEmpty method.
	 *
	 */
	public final void testIsEmpty() {
		assertEquals("Queue should be empty!", true, queue1.ADTIsEmpty());
		queue1.ADTEnqueue(new Integer(1));
		assertEquals("Queue should not be empty!", false, queue1.ADTIsEmpty());
	}

	
}
