// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/* $Id: VirtualArrayTest.java,v 1.4 2007/09/07 13:33:26 vkaravir Exp $ */

package tests.matrix.structures.memory;

import junit.framework.TestCase;
import matrix.structures.memory.*;

/**
 * This class is a Junit TestCase for testing the methods
 * of class matrix.structures.memory.VirtualArray.
 * 
 * @author kstalnac
 */
public class VirtualArrayTest extends TestCase {

    VirtualArray va;
    
    public static void main(String[] args) {
        junit.textui.TestRunner.run(VirtualArrayTest.class);
    }

    /**
     * Sets up the objects for the test methods.
     * @see TestCase#setUp()
     */
    protected void setUp() throws Exception {
        VirtualInteger[] viArray = new VirtualInteger[8];
        for (int i = 0; i < 8; i++) {
            viArray[i] = new VirtualInteger(i, null, "Virtual array test - integer no. "+i);
        }
        va = new VirtualArray(viArray, null, "Virtual array test - the array being tested");
    }

    /**
     * Removes the objects after every test method.
     * @see TestCase#tearDown()
     */
    protected void tearDown() throws Exception {
        va = null;
    }

    /**
     * Constructor for VirtualArrayTest.
     * @param name name of the test
     */
    public VirtualArrayTest(String name) {
        super(name);
    }

    /**
     * Tests swap method.
     *
     */
    public final void testSwap() {
        va.swap(0,7);
        assertEquals("The first element in the array should be VirtualInteger with value 7", 
                     7, ((VirtualInteger)va.getObject(0)).eval());
        assertEquals("The last element in the array should be VirtualInteger with value 0", 
                     0, ((VirtualInteger)va.getObject(7)).eval());
    }

    /**
     * Tests setObject(Object, int) method
     */
    public final void testSetObjectObjectint() {
        VirtualInteger viTemp = new VirtualInteger(10, null, "VirtualArray test test data");
        va.setObject(viTemp, 5);
        assertEquals("The sixth element in the array should be VirtualInteger with value 10", 
                10, ((VirtualInteger)va.getObject(5)).eval());
        viTemp = new VirtualInteger(-14, null, "VirtualArray test test data");
        va.setObject(viTemp, -1);
        assertEquals("The first element in the array should now be in index -1 and it should " +
                     "be VirtualInteger with value -14", 
                     -14, ((VirtualInteger)va.getObject(-1)).eval());
        assertEquals("The last element in the array should be VirtualInteger with value 7", 
                     7, ((VirtualInteger)va.getObject(7)).eval());
        viTemp = new VirtualInteger(1234, null, "VirtualArray test test data");
        va.setObject(viTemp, 123);
        assertEquals("The last element in the array should be in index 123 and it should be " +
                     "VirtualInteger with value 1234", 
                     1234, ((VirtualInteger)va.getObject(123)).eval());
        assertEquals("The element in the index 50 should be null.", null, va.getObject(50));
    }

    /**
     * Tests size() method.
     *
     */
    public final void testSize() {
        VirtualInteger viTemp = new VirtualInteger(10, null, "VirtualArray test test data");
        assertEquals("The size of the array should be 8.", 8, va.size());
        va.setObject(viTemp, -1);
        assertEquals("The size of the array should be 9.", 9, va.size());
        va.setObject(viTemp, -5);
        assertEquals("The size of the array should be 13.", 13, va.size());
        va.setObject(viTemp, 120);
        assertEquals("The size of the array should be 126", 126, va.size());
    }

    /**
     * Tests equals(Object) and equals(VirtualArray) methods.
     */
    public final void testBothEquals() {
        VirtualBoolean[] array1 = new VirtualBoolean[2];
        VirtualArray vaTemp = new VirtualArray(array1, null, "VirtualArray test array which is never used");
        assertFalse("Given arrays should not be equal.", va.equals(vaTemp));//uses equals(VirtualArray)
        assertFalse("Given arrays should not be equal.", va.equals(array1));//uses equals(Object)
        
        VirtualInteger[] array2 = new VirtualInteger[8];
        for (int i = 0; i < 8 ; i++) {
            array2[i] = new VirtualInteger(i, null, "VirtualArray test test data");
        }
        vaTemp = new VirtualArray(array2, null, "VirtualArray test - array holding VirtualIntegers");
        
        //these tests failed earlier because VirtualInteger didn't override the equals(Object o) method
        //call sequence VirtualArray.equals(Array) -> Comparer.compareArrays(Array,Array)
        //-> VirtualInteger.equals(Object)
        assertTrue("Arrays should be equal.", va.equals(vaTemp)); //uses equals(VirtualArray)
        assertTrue("Arrays should be equal.", va.equals((Object)vaTemp)); //uses equals(Object)
        
        assertTrue("Arrays should be equal to itself.", va.equals(va));//uses equals(VirtualArray)
    }

    /**
     * Tests getFirst() method.
     *
     */
    public final void testGetFirst() {
        assertEquals("First element should be at index 0.", 0, va.getFirst());
        VirtualInteger viTemp = new VirtualInteger(5, null, "VirtualArray test test data");
        va.setObject(viTemp, -2);
        assertEquals("First element should be at index -2.", -2, va.getFirst());
    }

    /**
     * Tests getLast method.
     *
     */
    public final void testGetLast() {
        assertEquals("Last element should be at index 7.", 7, va.getLast());
        VirtualInteger viTemp = new VirtualInteger(34, null, "VirtualArray test test data");
        va.setObject(viTemp, 123);
        assertEquals("Last element should be at index 123.", 123, va.getLast());
    }

    /**
     * Tests setFirst method.
     *
     */
    public final void testSetFirst() {
        va.setFirst(-4);
        assertEquals("Element at index -2 should be null.", null, va.getObject(-2));
        assertEquals("Size should be 12.", 12, va.size());
        va.setFirst(3);
        assertEquals("Element at first index should be VirtualInteger 3.", 3, ((VirtualInteger)va.getObject(3)).eval());
        va.setFirst(va.getLast());
        assertEquals("Size should be 1.", 1, va.size());
    }

    /**
     * Tests setLast method.
     *
     */
    public final void testSetLast() {
        va.setLast(15);
        assertEquals("Element at index 14 should be null.", null, va.getObject(14));
        assertEquals("Size should be 16.", 16, va.size());
        va.setLast(5);
        assertEquals("Size should be 6.", 6, va.size());
    }


}
