// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/* $Id: VirtualBooleanTest.java,v 1.3 2007/09/07 13:33:26 vkaravir Exp $ */

package tests.matrix.structures.memory;

import junit.framework.TestCase;
import matrix.structures.memory.VirtualBoolean;

/**
 * This class is a Junit TestCase to test methods of 
 * matrix.structures.memory.VirtualBoolean.
 * 
 * @author kstalnac
 */
public class VirtualBooleanTest extends TestCase {

    private VirtualBoolean vb;
    
    public static void main(String[] args) {
        junit.textui.TestRunner.run(VirtualBooleanTest.class);
    }

    /**
     * Sets up the objects for the test methods.
     * @see TestCase#setUp()
     */
    protected void setUp() throws Exception {
        vb = new VirtualBoolean(null, "VirtualBoolean test - object being tested");
    }

    /**
     * Removes the objects after every test method is finished.
     * @see TestCase#tearDown()
     */
    protected void tearDown() throws Exception {
        vb = null;
    }

    /**
     * Constructor for VirtualBooleanTest.
     * @param name name of the test
     */
    public VirtualBooleanTest(String name) {
        super(name);
    }

    /**
     * Tests eval method when the object is first false and then object is set to true
     * and eval is called also for this. 
     *
     */
    public final void testEval() {
        assertFalse("vb should have been false", vb.eval());
        vb = new VirtualBoolean(true, null, "VirtualBoolean test - object being tested");
        assertTrue("vb should have been true", vb.eval());
    }

    /**
     * Tests assign method. Assigns true and false values to VirtualBoolean object and
     * then checks that eval method produces correct answer.
     *
     */
    public final void testAssign() {
        vb.assign(true);
        assertTrue("vb should have been true", vb.eval());
        vb.assign(false);
        assertFalse("vb should habe been false", vb.eval());
    }

    /**
     * Tests toggle method. Toggles object couple of times and check that eval method
     * returns correct answer everytime.
     *
     */
    public final void testToggle() {
        vb.toggle();
        assertTrue("vb should have been toggled to true", vb.eval());
        vb.toggle();
        assertFalse("vb should have been toggled to false", vb.eval());
    }
    
}
