// Matrix - algorithm simulation system
// Copyright (C) 2000- 2007 Matrix group
// (See the AUTHORS file and the author tag below for more information
// on the authors.)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
package tests.matrix.structures.spatial.CDT.probe;

import matrix.structures.spatial.CDT.probe.PRQuadTree;
import matrix.structures.spatial.CDT.probe.PRQuadTreeNode;
import matrix.structures.spatial.FDT.probe.Point;
import junit.framework.TestCase;

public class PRQuadTreeTest extends TestCase {

	private PRQuadTree qtree;
	private int counter = 0;
	
	public PRQuadTreeTest(String s) {
		super(s);
	}
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		junit.textui.TestRunner.run(PRQuadTreeTest.class);
	}

	protected void setUp() throws Exception {
		System.out.println(counter++ + ": ** Creating an empty PR-Quadtree with dimensions [0-100,0-100] **");
		this.qtree = new PRQuadTree(0, 100, 0, 100);
		print();
	}

	protected void tearDown() throws Exception {
		System.out.println(counter++ + ": ** Destroying the tree. **");
		this.qtree = null;
	}
	
	private void print() {
		System.out.println(this.qtree + "\n");
		
	}
	
	private void insert(Point p) {
		System.out.println(counter++ + ": ** Inserting point (" + p.getX() + "," + p.getY() + ") **");
		this.qtree.insert(p);
		print();
	}
	
	private void delete(Point p) {
		System.out.println(counter++ + ": ** Deleting point (" + p.getX() + "," + p.getY() + ") **");
		this.qtree.delete(p);
		print();
	}
	
	private Point point(double x, double y) {
		return new Point(x, y, "(" + x + "," + y + ")");
	}
	
	public final void test() {
		assertEquals("The tree should be empty.", 
				 true,
				 (((PRQuadTreeNode)this.qtree.getElement()).getElement() == null));
		// *** INSERTIONS ***
		insert(point(80, 80));
		assertEquals("First insert should go in the root.", 
					 true,
					 ((PRQuadTreeNode)this.qtree.getElement()).getElement().equals(point(80, 80)));
		insert(point(10, 15));
		assertEquals("Root was split. It should be empty now.", 
				 true,
				 (((PRQuadTreeNode)this.qtree.getElement()).getElement() == null));
		assertEquals("Point [80,80] should go in the NE-quadrant.", 
					 true,
					 ((PRQuadTreeNode)this.qtree.getElement()).getNE().getElement().equals(point(80, 80)));
		assertEquals("Point [10,15] should go in the SW-quadrant.", 
				 true,
				 ((PRQuadTreeNode)this.qtree.getElement()).getSW().getElement().equals(point(10, 15)));
		insert(point(70, 15));
		assertEquals("Point [70,15] should go in the SE-quadrant.", 
				 true,
				 ((PRQuadTreeNode)this.qtree.getElement()).getSE().getElement().equals(point(70, 15)));
		insert(point(37, 79));
		assertEquals("Point [37,79] should go in the NW-quadrant.", 
				 true,
				 ((PRQuadTreeNode)this.qtree.getElement()).getNW().getElement().equals(point(37, 79)));
		insert(point(17, 23));
		assertEquals("SW-node was split. It should be empty now.", 
				 true,
				 (((PRQuadTreeNode)this.qtree.getElement()).getSW().getElement() == null));
		assertEquals("Point [17,23] should go in the SW->SW->NE-quadrant.", 
				 true,
				 (((PRQuadTreeNode)this.qtree.getElement()).getSW().getSW().getNE().getElement().equals(point(17, 23))));
		assertEquals("Point [10,15] should go in the SW->SW->NW-quadrant.", 
				 true,
				 (((PRQuadTreeNode)this.qtree.getElement()).getSW().getSW().getNW().getElement().equals(point(10, 15))));
		// *** DELETIONS ***
		delete(point(17, 23));
		assertEquals("Point [37,79] collapse into the NW-quadrant.", 
				 true,
				 ((PRQuadTreeNode)this.qtree.getElement()).getNW().getElement().equals(point(37, 79)));
		delete(point(37, 79));
		assertEquals("Point [37,79] should have been removed.", 
				 true,
				 (((PRQuadTreeNode)this.qtree.getElement()).getNW().isEmpty()));
		delete(point(80, 80));
		assertEquals("Point [80,80] should have been removed.", 
				 true,
				 (((PRQuadTreeNode)this.qtree.getElement()).getNE().isEmpty()));
		delete(point(10, 15));
		assertEquals("Point [10,15] should have been removed.", 
				 true,
				 (((PRQuadTreeNode)this.qtree.getElement()).getSW() == null));
		assertEquals("Point [70,15] should collapse into the root.", 
				 true,
				 ((PRQuadTreeNode)this.qtree.getElement()).getElement().equals(point(70, 15)));
		delete(point(70, 15));
		assertEquals("The tree should be empty.", 
				 true,
				 (((PRQuadTreeNode)this.qtree.getElement()).isEmpty()));
	}

}
