// MatrixPro - algorithm simulation system
// Copyright (C) 2000-2007 See the AUTHORS file and the author tag
// below for more information on the authors.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package matrixpro.toolbar;

import java.util.Comparator;

/**
 * A class that represents one Tool (ie a component in the toolbar). This class
 * merely holds the information about the classname of the component and the
 * position that it is in the toolbar.
 * 
 * @version $Id: Tool.java,v 1.5 2007/09/10 08:32:32 vkaravir Exp $
 */
public class Tool {
    /** Name of the class and the component */
    private String className, name;

    /** Position of the component. */
    private int pos;

    /** Comparator used to compare the positions of Tools. */
    private static Comparator comparator = null;

    /** Constructs a new Tool with the given className, name and position. */
    public Tool(String clas, String name, int pos) {
        className = clas;
        this.name = name;
        this.pos = pos;
    }

    /** Returns the name of the class this Tool is representing. */
    public String getClassName() {
        return className;
    }

    /** Returns the name of the component this Tool is representing. */
    public String getName() {
        return name;
    }

    /** Returns the position of this Tool in the Toolbar. */
    public int getPosition() {
        return pos;
    }

    /** Returns a String representation of this object. */
    public String toString() {
        if (className.endsWith("MethodComponent"))
            return className.substring(className.lastIndexOf(".") + 1) + " - "
                    + name;
        else
            return className.substring(className.lastIndexOf(".") + 1);
    }

    /** Returns a Comparator used to compare the positions of Tools. */
    public static Comparator getComparator() {
        if (comparator == null)
            comparator = new Comparator() {
                public int compare(Object o1, Object o2) {
                    return ((Tool) o1).getPosition()
                            - ((Tool) o2).getPosition();
                }

                public boolean equals(Object o1, Object o2) {
                    return (((Tool) o1).getPosition() == ((Tool) o2)
                            .getPosition());
                }
            };
        return comparator;
    }
}
