// MatrixPro - algorithm simulation system
// Copyright (C) 2000-2007 See the AUTHORS file and the author tag
// below for more information on the authors.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package matrixpro.toolbar;

import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.util.Enumeration;
import java.util.Hashtable;

import javax.swing.JPanel;
import javax.swing.JScrollPane;

import matrix.animation.VisualAnimator;
import matrix.structures.FDT.probe.Key;
import matrix.visual.VisualContainer;
import matrix.visual.VisualKey;
import matrix.visual.VisualType;

import matrixpro.ui.MainFrame;

/**
 * An implementation of <code>Toolbar</code> that is used in MatrixPro.
 * 
 * @version $Id: ToolbarPanel.java,v 1.13 2007/09/10 08:32:31 vkaravir Exp $
 * @author Ville Karavirta
 */
public class ToolbarPanel extends JPanel implements ActionListener, Toolbar {
    /** MatrixFrame that this toolbar will be added to. */
    private MainFrame mf;

    /** The VisualType that is currently selected in the frame. */
    private VisualType vt;

    /** The panel that all the ToolbarComponents will be added to. */
    private JPanel componentPanel;

    /** The VisualAnimator. */
    private VisualAnimator vanim;

    /** Hashtable comtaining all the components of this toolbar. */
    private Hashtable components = new Hashtable();

    /** Counter of the components in this toolbar. */
    // XXX Is it possible to get rid of this??
    private int count = 0;

    /** GridBagConstraints used when adding new ToolbarComponents. */
    private GridBagConstraints constr;

    /** The ScrollPane that contains the componentPanel. */
    // XXX A scrollpane that adds only vertical scrollbars would be pop!
    private JScrollPane sp;

    /**
     * Constructs a new object of this class.
     * 
     * @param mf
     *            The <code>MatrixFrame</code> that this ToolbarPanel will be
     *            added to.
     */
    public ToolbarPanel(MainFrame mf) {
        super();
        this.mf = mf;
        vt = new VisualKey(new Key("Unknown"));
        constr = new GridBagConstraints();
        constr.gridx = 0;
        constr.fill = GridBagConstraints.HORIZONTAL;
        constr.anchor = GridBagConstraints.NORTH;
        addComponents();
        setListeners();
        setVisible(true);
    }

    /** Adds the components to this toolbar. */
    private void addComponents() {
        JPanel animToolBar = new JPanel();
        vanim = mf.getVisualAnimator();
        vanim.addActionListener(this);
        animToolBar.add(vanim);
        componentPanel = new JPanel();
        componentPanel.setLayout(new GridBagLayout());
        sp = new JScrollPane();
        sp.setViewportView(componentPanel);
        setLayout(new GridBagLayout());
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.gridy = 0;
        gbc.weightx = 2;
        add(vanim, gbc);
        gbc.fill = GridBagConstraints.BOTH;
        gbc.weightx = 4;
        gbc.gridy = 1;
        gbc.weighty = 1;
        gbc.anchor = GridBagConstraints.NORTH;
        add(sp, gbc);
        addEmptyComponent();
    }

    /** Adds an empty component to the end of componentPanel. */
    // This way the other components will be aligned top.
    private void addEmptyComponent() {
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.weightx = 0.0;
        gbc.weighty = 1.0;
        gbc.gridx = 0;
        gbc.gridy = 100;
        gbc.anchor = GridBagConstraints.SOUTH;
        gbc.fill = GridBagConstraints.NONE;
        componentPanel.add(new JPanel(), gbc);
    }

    /** Adds the listeners to the components. */
    private void setListeners() {
        mf.getStructurePanel().addMouseMotionListener(
                new MouseMotionListener() {
                    public void mouseDragged(MouseEvent e) {
                    }

                    public void mouseMoved(MouseEvent e) {
                        if ((VisualType.read == null)
                                || (visualTypeSelected()
                                        && vt != VisualType.read && VisualType.read
                                        .getApplication() == mf
                                        .getStructurePanel())) {
                            invalidateComponents();
                            validate();
                            update();
                        }
                    }
                });
        mf.getStructurePanel().addMouseListener(new MouseListener() {
            public void mouseClicked(MouseEvent e) {
                resetSelectedVisualType();
                updateApplication();
            }

            public void mouseEntered(MouseEvent e) {
            }

            public void mouseExited(MouseEvent e) {
            }

            public void mouseReleased(MouseEvent e) {
            }

            public void mousePressed(MouseEvent e) {
            }
        });
    }

    /*
     * public void validate() { update(); super.validate(); }
     */

    /** Updates the components in this toolbar. */
    public void update() {
        boolean isSelected = visualTypeSelected();
        if ((isSelected || VisualType.read != vt)
                && (VisualType.read.getApplication() == mf.getStructurePanel() || VisualType.read
                        .getApplication() == null))
            vt = VisualType.read;
        Enumeration en = components.elements();
        while (en.hasMoreElements())
            ((ToolbarComponent) en.nextElement()).update();
        updateStepLabel();
        mf.setClipboardEnabled();
    }

    /** Invalidates all the components in this toolbar. */
    public void invalidateComponents() {
        Component[] components = getComponents();
        for (int i = 0; i < components.length; i++)
            components[i].invalidate();
        components = componentPanel.getComponents();
        for (int i = 0; i < components.length; i++)
            components[i].invalidate();
    }

    public void actionPerformed(ActionEvent e) {
        updateStepLabel();
    }

    /** Updates the StepLabelComponent. */
    public void updateStepLabel() {
        mf.firePropertyChangeEvent("step", null, "");
    }

    // ----- Methods for interface Toolbar ---- //

    public void addComponent(ToolbarComponent c) {
        addComponent(c, "component" + count);
    }

    public void addComponent(ToolbarComponent c, String name) {
        components.put(name, c);
        constr.gridy = count;
        componentPanel.add(c.getComponent(), constr);
        c.update();
        componentPanel.validate();
        count++;
    }

    public void addComponent(ToolbarComponent c, String name, int position) {
        components.put(name, c);
        constr.gridy = position;
        componentPanel.add(c.getComponent(), constr, position);
        c.update();
        componentPanel.validate();
        count++;
    }

    public MainFrame getFrame() {
        return mf;
    }

    public VisualType getSelectedVisualType() {
        return vt;
    }

    public Enumeration getToolbarComponents() {
        return components.elements();
    }

    public void resetSelectedVisualType() {
        VisualType.read = new VisualKey(new Key("Unknown"));
    }

    public void setSelectedVisualType(VisualType newVt) {
        vt = newVt;
        VisualType.read = newVt;
    }

    public void updateApplication() {
        // update();
        mf.update();
    }

    public void validateVisualAnimator() {
        if (vanim != null)
            vanim.validate();
    }

    public boolean visualTypeSelected() {
        if (VisualType.read == null) {
            resetSelectedVisualType();
            return false;
        }
        if (VisualType.read.getApplication() != mf.getStructurePanel())
            return !vt.getStructure().equals(new Key("Unknown"));
        if (VisualType.read.getParent() == null
                && !(VisualType.read instanceof VisualContainer)) {
            vt = new VisualKey(new Key("Unknown"));
            VisualType.read = new VisualKey(new Key("Unknown"));
            return false;
        }
        return (VisualType.read != null
                && VisualType.read.getStructure() != null && !VisualType.read
                .getStructure().equals(new Key("Unknown")));
    }
}
