// MatrixPro - algorithm simulation system
// Copyright (C) 2000-2007 See the AUTHORS file and the author tag
// below for more information on the authors.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package matrixpro.toolbar.components;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.ByteArrayOutputStream;
import java.io.PrintStream;

import javax.swing.JButton;
import javax.swing.JComponent;

import matrix.util.ApplicationAdapter;

import matrixpro.toolbar.Toolbar;
import matrixpro.toolbar.ToolbarComponent;

/**
 * A component for the <code>Toolbar</code> that shows debug information about
 * the animator.
 * 
 * @version $Id: AnimatorDumpComponent.java,v 1.6 2007/09/03 13:25:54 vkaravir
 *          Exp $
 */
public class AnimatorDumpComponent extends ToolbarComponent implements
        ActionListener {
    /** Toolbar that this component will be added to. */
    private Toolbar tb;

    private JButton button;

    /**
     * Constructs a new instance of this class. The text on the button will be
     * "Animator dump".
     * 
     * @param tb
     *            The Toolbar this component is added.
     */
    public AnimatorDumpComponent(Toolbar tb) {
        this(tb, "Animator dump");
    }

    /**
     * Constructs a new instance of this class.
     * 
     * @param tb
     *            The Toolbar this component is added.
     * @param name
     *            Text on the button.
     */
    public AnimatorDumpComponent(Toolbar tb, String name) {
        this.tb = tb;
        button = new JButton(name);
        button.addActionListener(this);
    }

    public void actionPerformed(ActionEvent e) {
        ByteArrayOutputStream buffer = new ByteArrayOutputStream();
        PrintStream out = new PrintStream(buffer);
        ApplicationAdapter.getApplication().getAnimator().dump(out);
        tb.getFrame().getStructurePanel()
                .showMessage("Dump", buffer.toString());
    }

    public void update() {
    }

    public JComponent getComponent() {
        return button;
    }

    public String getDescription() {
        return "Shows debug information of the animator";
    }

    public static ToolbarComponent getNewInstance(Toolbar tb) {
        return new AnimatorDumpComponent(tb);
    }
}
