// MatrixPro - algorithm simulation system
// Copyright (C) 2000-2007 See the AUTHORS file and the author tag
// below for more information on the authors.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package matrixpro.toolbar.components;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JComponent;

import matrixpro.toolbar.Toolbar;
import matrixpro.toolbar.ToolbarComponent;

/**
 * Component for <code>Toolbar</code> that deletes the selected structure.
 * 
 * @version $Id: DeleteComponent.java,v 1.8 2007/09/10 08:39:09 vkaravir Exp $
 */
public class DeleteComponent extends ToolbarComponent implements ActionListener {
    /** The Toolbar this component is added. */
    private Toolbar tb;

    private JButton button;

    /** Default text on the button. */
    private static final String DEFAULT_NAME = "Delete";

    /**
     * Constructs a new instance of this component.
     * 
     * @param tb
     *            The <code>Toolbar</code> this component is added
     */
    public DeleteComponent(Toolbar tb) {
        this(tb, DEFAULT_NAME);
    }

    /**
     * Constructs a new instance of this component.
     * 
     * @param tb
     *            The <code>Toolbar</code> this component is added
     * @param name
     *            Text on the button
     */
    public DeleteComponent(Toolbar tb, String name) {
        button = new JButton(name);
        this.tb = tb;
        button.addActionListener(this);
    }

    public void actionPerformed(ActionEvent e) {
        delete();
    }

    /**
     * Deletes the selected <code>VisualType</code>.
     */
    private void delete() {
        if (tb.visualTypeSelected())
            tb.getFrame().delete();
    }

    public void update() {
        button.setEnabled(tb.visualTypeSelected());
    }

    public JComponent getComponent() {
        return button;
    }

    public String getDescription() {
        return "Deletes the selected structure";
    }

    public static ToolbarComponent getNewInstance(Toolbar tb) {
        return new DeleteComponent(tb);
    }
}
