// MatrixPro - algorithm simulation system
// Copyright (C) 2000-2007 See the AUTHORS file and the author tag
// below for more information on the authors.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package matrixpro.toolbar.components;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Stack;

import javax.swing.JButton;
import javax.swing.JComponent;

import matrix.animation.AnimatedOperation;
import matrix.animation.Animator;
import matrix.animation.endOperation;
import matrix.animation.startOperation;
import matrix.util.ApplicationAdapter;

import matrixpro.toolbar.Toolbar;
import matrixpro.toolbar.ToolbarComponent;

/**
 * A component for the Toolbar that breaks an operation in the Animator to
 * several smaller operations.
 * 
 * @version $Id: DisjoinStepsComponent.java,v 1.7 2007/09/03 13:25:55 vkaravir
 *          Exp $
 */
public class DisjoinStepsComponent extends ToolbarComponent implements
        ActionListener {
    private Toolbar tb;

    private JButton button;

    private int beginPos = -1;

    private int endPos = -1;

    /**
     * Constructs a new instance of this class. The text on the button will be
     * "Disjoin steps".
     * 
     * @param tb
     *            The Toolbar this component is added.
     */
    public DisjoinStepsComponent(Toolbar tb) {
        button = new JButton("Disjoin steps (begin)");
        this.tb = tb;
        button.addActionListener(this);
    }

    public void actionPerformed(ActionEvent e) {
        maybeInsertBreaks();
    }

    /**
     * Called when an action is performed on this object. If the begin and end
     * positions are selected disjoins the steps.
     */
    private void maybeInsertBreaks() {
        Animator a = ApplicationAdapter.getApplication().getAnimator();
        if (beginPos == -1) {
            beginPos = a.getState();
            button.setText("Disjoin steps (end)");
        } else if (endPos == -1) {
            endPos = a.getState();
            insertBreaks();
            beginPos = -1;
            endPos = -1;
            button.setText("Disjoin steps (begin)");
        }
    }

    /** Disjoins the operations between start and end position. */
    private void insertBreaks() {
        if (beginPos == endPos || beginPos == -1 || endPos == -1)
            return;
        tb.getFrame().setUndoPoint("disjoin steps");
        int start = Math.min(beginPos, endPos);
        int end = Math.max(beginPos, endPos);
        Animator a = ApplicationAdapter.getApplication().getAnimator();
        a.rewind();
        Stack redo = a.disposeRedo();
        Stack tmp = new Stack();
        while (!redo.empty())
            tmp.push(redo.pop());
        int par = 0;
        for (int i = 0; i < start; i++) {
            AnimatedOperation ao = (AnimatedOperation) tmp.pop();
            a.put(ao);
            if (ao instanceof startOperation)
                par++;
            else if (ao instanceof endOperation)
                par--;
        }
        for (int i = 0; i < par; i++)
            a.endOperation();
        // par = 0;
        for (int i = 0; i < end - start; i++) {
            AnimatedOperation ao = (AnimatedOperation) tmp.pop();
            if (ao instanceof startOperation) {
                par++;
                if (par > 1)
                    a.put(ao);
            } else if (ao instanceof endOperation) {
                par--;
                if (par > 0)
                    a.put(ao);
            } else {
                a.startOperation();
                a.put(ao);
                a.endOperation();
            }
        }
        for (int i = 0; i < par; i++)
            a.startOperation();
        while (!tmp.empty())
            a.put((AnimatedOperation) tmp.pop());
        a.rewind();
        tb.updateApplication();
    }

    public void update() {
        button.setText("Disjoin steps (begin)");
        beginPos = endPos = -1;
    }

    public String getDescription() {
        return "Disjoins the steps between the selected begin and end steps";
    }

    public JComponent getComponent() {
        return button;
    }

    public static ToolbarComponent getNewInstance(Toolbar tb) {
        return new DisjoinStepsComponent(tb);
    }
}
