// MatrixPro - algorithm simulation system
// Copyright (C) 2000-2007 See the AUTHORS file and the author tag
// below for more information on the authors.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package matrixpro.toolbar.components;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Stack;

import javax.swing.JButton;
import javax.swing.JComponent;

import matrix.animation.AnimatedOperation;
import matrix.animation.Animator;
import matrix.animation.endOperation;
import matrix.animation.startOperation;
import matrix.util.ApplicationAdapter;

import matrixpro.toolbar.Toolbar;
import matrixpro.toolbar.ToolbarComponent;

/**
 * A component for the Toolbar that inserts a break in the Animator at wanted
 * position.
 * 
 * @version $Id: InsertBreakComponent.java,v 1.8 2007/09/03 13:46:11 vkaravir
 *          Exp $
 */
public class InsertBreakComponent extends ToolbarComponent {
    private Toolbar tb;

    private JButton button;

    private static final String DEFAULT_NAME = "Insert break";

    /**
     * Constructor. Creates a new instance of this class with the default text
     * "Insert break".
     */
    public InsertBreakComponent(Toolbar tb) {
        this(tb, DEFAULT_NAME);
    }

    /**
     * Constructor. Creates a new instance of this class with the given text.
     */
    public InsertBreakComponent(Toolbar tb, String name) {
        button = new JButton(name);
        this.tb = tb;
        button.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                insertBreak();
            }
        });
    }

    /** Inserts the break at the current position in the animator. */
    private void insertBreak() {
        Animator a = ApplicationAdapter.getApplication().getAnimator();
        if (a.getStartsBefore() == 0 || a.getEndsAfter() == 0)
            return;
        if (!a.hasPreviousOperation() || !a.hasNextOperation())
            return;
        int startsBefore = a.getStartsBefore();
        tb.getFrame().setUndoPoint("insert break");
        int start = a.getState();
        a.rewind();
        Stack s = a.disposeRedo();
        Stack tmp = new Stack();
        while (!s.empty())
            tmp.push(s.pop());
        int cnt = 0;
        while (!tmp.empty()) {
            AnimatedOperation ao = (AnimatedOperation) tmp.pop();
            if (cnt == start) {
                for (int i = 0; i < startsBefore; i++)
                    a.put(new endOperation());
                for (int i = 0; i < startsBefore; i++)
                    a.put(new startOperation());
            }
            a.put(ao);
            cnt++;
        }
        a.rewind();
        tb.updateApplication();
    }

    public String getDescription() {
        return "Inserts a break to the animation";
    }

    public JComponent getComponent() {
        return button;
    }

    public static ToolbarComponent getNewInstance(Toolbar tb) {
        return new InsertBreakComponent(tb);
    }
}
