// MatrixPro - algorithm simulation system
// Copyright (C) 2000-2007 See the AUTHORS file and the author tag
// below for more information on the authors.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package matrixpro.toolbar.components;

import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.JCheckBox;
import javax.swing.JComponent;

import matrix.visual.VisualContainer;
import matrix.visual.VisualType;

import matrixpro.toolbar.Toolbar;
import matrixpro.toolbar.ToolbarComponent;

/**
 * Component for <code>Toolbar</code> to toggle the automatic labeling of
 * nodes.
 * 
 * @version $Id: LabelNodesComponent.java,v 1.8 2007/09/10 08:39:10 vkaravir Exp $
 */
public class LabelNodesComponent extends ToolbarComponent implements
        ItemListener {
    private JCheckBox checkbox;

    private Toolbar tb;

    /**
     * Constructs a new instance of this class. The text on the button will be
     * "Node labeling".
     * 
     * @param tb
     *            The Toolbar this component is added.
     */
    public LabelNodesComponent(Toolbar tb) {
        checkbox = new JCheckBox("Node labeling");
        checkbox.setEnabled(false);
        this.tb = tb;
        checkbox.addItemListener(this);
    }

    /** Method invoked when the state of the checkbox is changed. */
    public void itemStateChanged(ItemEvent e) {
        label();
    }

    /** Turns node labeling on or off depending on its current value. */
    private void label() {
        VisualType vts = tb.getSelectedVisualType();
        if (!(vts instanceof VisualContainer))
            return;
        boolean label = ((VisualContainer) vts).isAutoLabeled();
        ((VisualContainer) vts).setAutoLabeling(!label);
        vts.invalidate();
        tb.getFrame().getStructurePanel().validate();
        vts.repaint();
        vts.validate();
    }

    public void update() {
        VisualType vt = tb.getSelectedVisualType();
        if (vt instanceof VisualContainer) {
            checkbox.setEnabled(true);
            checkbox.setSelected(((VisualContainer) vt).isAutoLabeled());
        } else
            checkbox.setEnabled(false);
    }

    public JComponent getComponent() {
        return checkbox;
    }

    public String getDescription() {
        return "Numbers the nodes of a tree or a graph";
    }

    public static ToolbarComponent getNewInstance(Toolbar tb) {
        return new LabelNodesComponent(tb);
    }
}
