// MatrixPro - algorithm simulation system
// Copyright (C) 2000-2007 See the AUTHORS file and the author tag
// below for more information on the authors.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package matrixpro.toolbar.components;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JComponent;

import matrixpro.toolbar.Toolbar;

/**
 * A component for the ContextualPanel that represents a method that can be
 * called for the selected structure.
 * 
 * @see matrixpro.toolbar.components.ContextualPanel
 * @version $Id: MethodComponent.java,v 1.9 2007/09/10 08:39:11 vkaravir Exp $
 */
public class MethodComponent {
    private Toolbar tb;

    private JButton button;

    private String method;

    /**
     * Constructs a new MethodComponent that will be added to the given Toolbar
     * and calls the given method on the selected structure. The text on the
     * Button will be the same as the method's name.
     */
    public MethodComponent(Toolbar tb, String method) {
        this(tb, method, method);
    }

    /**
     * Constructs a new MethodComponent that will be added to the given Toolbar
     * and calls the given method on the selected structure. The text on the
     * Button will be the given name.
     */
    public MethodComponent(Toolbar tb, String method, String name) {
        button = new JButton(name);
        this.tb = tb;
        this.method = method;
        button.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                callMethod();
            }
        });
    }

    /**
     * Calls the method this component represents for the selected structure.
     */
    private void callMethod() {
        tb.getSelectedVisualType().reflection(
                "simulatedFDTMethodInvokation(" + method + ")");
        tb.getFrame().firePropertyChangeEvent(method, null, "");
    }

    public JComponent getComponent() {
        return button;
    }

    public String getDescription() {
        return "Button for operation " + method;
    }
}
