// MatrixPro - algorithm simulation system
// Copyright (C) 2000-2007 See the AUTHORS file and the author tag
// below for more information on the authors.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package matrixpro.toolbar.components;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Stack;

import javax.swing.JButton;
import javax.swing.JComponent;

import matrix.animation.AnimatedOperation;
import matrix.animation.Animator;
import matrix.animation.endOperation;
import matrix.animation.startOperation;
import matrix.util.ApplicationAdapter;

import matrixpro.toolbar.Toolbar;
import matrixpro.toolbar.ToolbarComponent;

/**
 * A component for the Toolbar that removes a wanted break in the Animator.
 * 
 * @version $Id: RemoveBreakComponent.java,v 1.7 2007/09/03 13:46:12 vkaravir
 *          Exp $
 */
public class RemoveBreakComponent extends ToolbarComponent implements
        ActionListener {
    private static final String DEFAULT_NAME = "Remove break";

    private Toolbar tb;

    private JButton button;

    /**
     * Constructor. Creates a new instance of this class with the default text
     * "Remove break".
     */
    public RemoveBreakComponent(Toolbar tb) {
        this(tb, DEFAULT_NAME);
    }

    /**
     * Constructor. Creates a new instance of this class with the given text.
     */
    public RemoveBreakComponent(Toolbar tb, String name) {
        button = new JButton(name);
        this.tb = tb;
        button.addActionListener(this);
    }

    public void actionPerformed(ActionEvent e) {
        removeBreak();
    }

    /** Removes the break at the current position in the animator. */
    private void removeBreak() {
        tb.getFrame().setUndoPoint("remove break");
        Animator a = ApplicationAdapter.getApplication().getAnimator();
        // can't remove first or last break this way
        if (!a.hasNextOperation() || !a.hasPreviousOperation())
            return;
        int start = a.getState();
        a.rewind();
        Stack s = a.disposeRedo();
        Stack tmp = new Stack();
        while (!s.empty())
            tmp.push(s.pop());
        int cnt = 0;
        while (!tmp.empty()) {
            AnimatedOperation ao = (AnimatedOperation) tmp.pop();
            a.put(ao);
            // remove the end and start operations before the wanted state
            if (cnt == start - 2) {
                ao = (AnimatedOperation) tmp.pop();
                if (!(ao instanceof endOperation))
                    a.put(ao);
                ao = (AnimatedOperation) tmp.pop();
                if (!(ao instanceof startOperation))
                    a.put(ao);
            }
            cnt++;
        }
        a.rewind();
        tb.updateApplication();
    }

    public String getDescription() {
        return "Removes a break from the animation";
    }

    public JComponent getComponent() {
        return button;
    }

    public static ToolbarComponent getNewInstance(Toolbar tb) {
        return new RemoveBreakComponent(tb);
    }
}
