// MatrixPro - algorithm simulation system
// Copyright (C) 2000-2007 See the AUTHORS file and the author tag
// below for more information on the authors.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package matrixpro.toolbar.components;

import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.JComboBox;
import javax.swing.JComponent;

import matrix.uitools.StructurePanel;
import matrix.util.ApplicationAdapter;
import matrix.visual.RepresentationFactory;
import matrix.visual.VisualContainer;
import matrix.visual.VisualType;

import matrixpro.toolbar.Toolbar;
import matrixpro.toolbar.ToolbarComponent;

/**
 * A component for the Toolbar that can be used to change the representation of
 * the selected visualization.
 * 
 * @version $Id: RepresentationComponent.java,v 1.7 2007/09/03 13:25:54 vkaravir
 *          Exp $
 */
public class RepresentationComponent extends ToolbarComponent {
    private static final String DEFAULT_TEXT = "Layout";

    private JComboBox choice;

    private Toolbar tb;

    private boolean represReseted = false;

    private VisualType lastSelected = null;

    /**
     * Constructs a new instance of this class.
     * 
     * @param tb
     *            The Toolbar this component is added.
     */
    public RepresentationComponent(Toolbar tb) {
        choice = new JComboBox();
        this.tb = tb;
        choice.setEnabled(false);
        choice.addItem(DEFAULT_TEXT);
        choice.addItemListener(new ItemListener() {
            public void itemStateChanged(ItemEvent e) {
                JComboBox source = (JComboBox) e.getSource();
                if (!source.isEnabled()
                        || source.getSelectedItem().equals(DEFAULT_TEXT))
                    return;
                changeRepresentation();
            }
        });
    }

    /** Changes the representation of the selected visualization. */
    private void changeRepresentation() {
        VisualType vt = tb.getSelectedVisualType();
        if (!(vt instanceof VisualContainer))
            return;
        if (represReseted) {
            int index = -1;
            StructurePanel s = (StructurePanel) ApplicationAdapter
                    .getApplication();
            for (int i = 0; i < s.getComponentCount(); i++) {
                if (s.getComponent(i) == vt) {
                    index = i;
                    break;
                }
            }
            ((StructurePanel) ApplicationAdapter.getApplication())
                    .changeRepresentation((VisualContainer) vt, (String) choice
                            .getSelectedItem());
            tb.setSelectedVisualType((VisualType) s.getComponent(index));
            tb.updateApplication();
        }
    }

    public void update() {
        VisualType vt = tb.getSelectedVisualType();
        if (vt == lastSelected)
            return;
        choice.setEnabled(false);
        choice.removeAllItems();
        choice.addItem("Representation");
        lastSelected = vt;
        if (vt instanceof VisualContainer) {
            represReseted = false;
            String[] nam = RepresentationFactory.getRepresentationNames(vt
                    .getStructure());
            for (int i = 0; i < nam.length; i++)
                choice.addItem(nam[i]);
            if (choice.getItemCount() > 1)
                choice.setEnabled(true);
        }
        represReseted = true;
    }

    public JComponent getComponent() {
        return choice;
    }

    public String getDescription() {
        return "Change the layout of the visualization";
    }

    public static ToolbarComponent getNewInstance(Toolbar tb) {
        return new RepresentationComponent(tb);
    }
}
