// MatrixPro - algorithm simulation system
// Copyright (C) 2000-2007 See the AUTHORS file and the author tag
// below for more information on the authors.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package matrixpro.toolbar.components;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JComponent;

import matrixpro.toolbar.Toolbar;
import matrixpro.toolbar.ToolbarComponent;
import matrixpro.util.FileHelper;

/**
 * A component for the Toolbar that saves the structures in the current frame.
 * 
 * @version $Id: SaveComponent.java,v 1.8 2007/09/10 08:39:09 vkaravir Exp $
 */
public class SaveComponent extends ToolbarComponent {

    private Toolbar tb;

    private JButton save;

    private static final String DEFAULT_NAME = "Save";

    /**
     * Constructs a new instance of this class. The text on the button will be
     * "Save".
     * 
     * @param tb
     *            The Toolbar this component is added.
     */
    public SaveComponent(Toolbar tb) {
        this(tb, DEFAULT_NAME);
    }

    /**
     * Constructs a new instance of this class. The text on the button will be
     * the given text.
     * 
     * @param tb
     *            The Toolbar this component is added.
     * @param name
     *            The text that will be the Button's label.
     */
    public SaveComponent(Toolbar tb, String name) {
        save = new JButton(name);
        this.tb = tb;
        save.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                save();
            }
        });
    }

    /** Saves the structures of the current frame. */
    private void save() {
        FileHelper.save(tb.getFrame());
    }

    public JComponent getComponent() {
        return save;
    }

    public String getDescription() {
        return "Saves the animation";
    }

    public static ToolbarComponent getNewInstance(Toolbar tb) {
        return new SaveComponent(tb);
    }
}
