// MatrixPro - algorithm simulation system
// Copyright (C) 2000-2007 See the AUTHORS file and the author tag
// below for more information on the authors.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package matrixpro.toolbar.components;

import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JSlider;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import matrix.animation.VisualAnimator;
import matrix.util.Note;

import matrixpro.animation.SmallVisualAnimator;
import matrixpro.toolbar.Toolbar;
import matrixpro.toolbar.ToolbarComponent;

/**
 * Component that can be used to change the speed of an animation. Note: This
 * component must be used when SmallVisualAnimator is used.
 * 
 * @version $Id: SpeedAdjustComponent.java,v 1.6 2007/09/03 13:25:53 vkaravir
 *          Exp $
 */
public class SpeedAdjustComponent extends ToolbarComponent {
    /** Slider used to adjust the speed. */
    private JSlider slider;

    /** Toolbar that this component is added to. */
    private Toolbar tb;

    /** The default speed. */
    private static final int DEFAULT_SPEED = 20;

    /**
     * Creates a new SpeedAdjustmentComponent.
     */
    public SpeedAdjustComponent(Toolbar tb) {
        slider = new JSlider(JSlider.HORIZONTAL, 1, 50, 25);
        slider.addChangeListener(new ChangeListener() {
            public void stateChanged(ChangeEvent e) {
                speedChanged();
            }
        });
        slider.setBorder(BorderFactory.createTitledBorder(BorderFactory
                .createEmptyBorder(), "Animation speed"));
        this.tb = tb;
        speedChanged();
    }

    /**
     * Changes the speed of the animation.
     */
    private void speedChanged() {
        VisualAnimator va = tb.getFrame().getVisualAnimator();
        if (va instanceof SmallVisualAnimator)
            ((SmallVisualAnimator) va).setWaitTime(DEFAULT_SPEED
                    * (slider.getMaximum() - slider.getValue()));
        else
            Note.err(this, "Wrong kind of VisualAnimator!");
    }

    public String getDescription() {
        return "Adjust the speed of the animation";
    }

    public JComponent getComponent() {
        return slider;
    }

    public static ToolbarComponent getNewInstance(Toolbar tb) {
        return new SpeedAdjustComponent(tb);
    }
}
