// MatrixPro - algorithm simulation system
// Copyright (C) 2000-2007 See the AUTHORS file and the author tag
// below for more information on the authors.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/* $Id: ToggleBalanceComponent.java,v 1.9 2007/09/10 08:39:09 vkaravir Exp $ */
package matrixpro.toolbar.components;

import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.lang.reflect.Method;

import javax.swing.JCheckBox;
import javax.swing.JComponent;

import matrix.structures.FDT.FDT;
import matrix.structures.simulationextensions.CallableMethods;

import matrixpro.toolbar.Toolbar;
import matrixpro.toolbar.ToolbarComponent;

public class ToggleBalanceComponent extends ToolbarComponent implements
        PropertyChangeListener {
    private JCheckBox checkbox;

    private Toolbar tb;

    public ToggleBalanceComponent(Toolbar tb) {
        checkbox = new JCheckBox("Balance");
        this.tb = tb;
        checkbox.addItemListener(new ItemListener() {
            public void itemStateChanged(ItemEvent e) {
                toggle();
            }
        });
        tb.getFrame().addPropertyChangeListener(this);
    }

    private void toggle() {
        FDT vt = tb.getSelectedVisualType().getStructure();
        if (vt instanceof CallableMethods) {
            String[] methods = ((CallableMethods) vt).getCallableMethodNames();
            for (int i = 0; i < methods.length; i++) {
                if (methods[i].equals("toggleBalance")) {
                    try {
                        Method m = vt.getClass().getDeclaredMethod(
                                "toggleBalance", null);
                        m.invoke(vt, null);
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                    checkbox.setEnabled(true);
                    return;
                }
                // checkbox.setEnabled(true);
            }
        }
        checkbox.setEnabled(false);
    }

    public void update() {
        FDT vt = tb.getSelectedVisualType().getStructure();
        if (vt instanceof CallableMethods) {
            String[] methods = ((CallableMethods) vt).getCallableMethodNames();
            for (int i = 0; i < methods.length; i++) {
                if (methods[i].equals("toggleBalance")) {
                    try {
                        Method m = vt.getClass().getDeclaredMethod("balance",
                                null);
                        Object o = m.invoke(vt, null);
                        checkbox.setSelected(((Boolean) o).booleanValue());
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                    checkbox.setEnabled(true);
                    return;
                }
                // setEnabled(true);
            }
        }
        checkbox.setEnabled(false);
    }

    public String getDescription() {
        return "Toggle the autobalance of some CDT-structures";
    }

    public JComponent getComponent() {
        return checkbox;
    }

    public static ToolbarComponent getNewInstance(Toolbar tb) {
        return new ToggleBalanceComponent(tb);
    }

    public void propertyChange(PropertyChangeEvent e) {
        String property = e.getPropertyName();
        if (property.equals("toggleBalance"))
            update();
    }
}
