// MatrixPro - algorithm simulation system
// Copyright (C) 2000-2007 See the AUTHORS file and the author tag
// below for more information on the authors.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package matrixpro.ui;

import java.awt.Point;

import javax.swing.JEditorPane;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;

import matrixpro.util.FileHelper;

public class HelpFrame extends JFrame implements HyperlinkListener {

    private static final long serialVersionUID = 5808693095984402350L;

    public static final String HELP_LOCATION = "/docs/manual/html/";

    private static final String DEFAULT_TITLE = "Help";

    private JScrollPane instrScroll;

    private String currPage = "index.html";

    public HelpFrame() {
        this(DEFAULT_TITLE);
    }

    public HelpFrame(String title) {
        super(title);
        init();
        setSize(600, 400);
    }

    private void init() {
        JSplitPane split = new JSplitPane();
        split.setOneTouchExpandable(true);
        split.setDividerLocation(300);
        JEditorPane help = new JEditorPane();
        JEditorPane.registerEditorKitForContentType("text/lhtml",
                "matrixpro.util.html.ResourceHTMLEditorKit");
        help.setEditable(false);
        help.setContentType("text/lhtml");
        help.addHyperlinkListener(this);
        String html = FileHelper.getDocumentFile(HELP_LOCATION + "index.html");
        help.setText(html);
        instrScroll = new JScrollPane(help);

        getContentPane().add(instrScroll);
    }

    public void hyperlinkUpdate(HyperlinkEvent e) {
        if (e.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
            JEditorPane pane = (JEditorPane) e.getSource();
            String desc = e.getDescription();
            if (desc.startsWith("http")) {
                JOptionPane.showMessageDialog(this,
                        "Loading external pages not allowed. Tried to access "
                                + desc + ".", "Error",
                        JOptionPane.ERROR_MESSAGE);
                return;
            } else if (desc.lastIndexOf('#') != -1)
                desc = desc.substring(0, desc.lastIndexOf('#'));
            if (desc.equals(currPage))
                return;
            currPage = desc;
            String fileName = HelpFrame.HELP_LOCATION + desc;
            String text = FileHelper.getDocumentFile(fileName);
            pane.setText(text);
            instrScroll.getViewport().setViewPosition(new Point(0, 0));
        }
    }
}
