// MatrixPro - algorithm simulation system
// Copyright (C) 2000-2007 See the AUTHORS file and the author tag
// below for more information on the authors.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package matrixpro.ui;

import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.GridLayout;

import javax.swing.BorderFactory;
import javax.swing.JPanel;

import matrix.uitools.StructurePanel;
import matrix.util.ApplicationAdapter;
import matrix.visual.VisualType;

/**
 * A tab to be added to the OptionsDialog. With this tab the user can change the
 * parameters of the <code>GridBagConstraints</code> object that defines the
 * layout constraints.
 * 
 * @author <a href="mailto:vkaravir@cs.hut.fi">Ville Karavirta</a>
 * @see matrixpro.ui.OptionsDialog
 * @see matrixpro.ui.LayoutPanel
 * @see java.awt.GridBagConstraints
 */
public class LayoutTab extends JPanel {
    /**
     * 
     */
    private static final long serialVersionUID = -3744106652587530366L;

    /**
     * The panels that actually contain the ui components to set the values.
     * There is one panel for each of the visualizations.
     */
    private LayoutPanel[] panels;

    /**
     * Constructs a ne LayoutTab.
     * 
     * @param mf
     *            The <code>MainFrame</code> that these layout options are
     *            done for.
     */
    public LayoutTab(MainFrame mf) {
        // currently only one layout type is available
        /*
         * String[] lolist = new String[] {"GridBagLayout"}; setLayout(new
         * GridBagLayout()); layouts = new JComboBox(lolist);
         */
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 0;
        // add(layouts, gbc);
        VisualType[] vis = ((StructurePanel) ApplicationAdapter
                .getApplication()).getVisualComponents();
        JPanel lays = new JPanel();
        lays.setLayout(new GridLayout(vis.length, 1));
        lays.setBorder(BorderFactory.createTitledBorder(BorderFactory
                .createLineBorder(Color.black, 2), "Visualizations"));
        gbc.gridy = 1;
        add(lays, gbc);
        String[] vlist = new String[vis.length];
        panels = new LayoutPanel[vis.length];
        for (int i = 0; i < vis.length; i++) {
            vlist[i] = vis[i].getName();
            panels[i] = new LayoutPanel(vis[i], mf);
            lays.add(panels[i]);
        }
    }

    /**
     * Sets the currently selected values for the visualizations.
     */
    public void setValues() {
        for (int i = 0; i < panels.length; i++)
            panels[i].setValues();
    }
}
