// MatrixPro - algorithm simulation system
// Copyright (C) 2000-2007 See the AUTHORS file and the author tag
// below for more information on the authors.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package matrixpro.ui;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;

import matrix.util.ApplicationAdapter;

import matrixpro.toolbar.ToolbarFactory;
import matrixpro.toolbar.ToolbarPanel;
import matrixpro.util.PropertiesHandler;

/**
 * Dialog used to set the preferences.
 * 
 * @version $Id: OptionsDialog.java,v 1.8 2007/09/10 08:35:55 vkaravir Exp $
 */
public class OptionsDialog extends JDialog {
    private JTabbedPane tab;

    private MainFrame frame;

    private ToolbarTab tooltab;

    private LayoutTab layoutTab;

    /**
     * Constructs a new OptionsDialog with the given parameters.
     * 
     * @param f
     *            The <code>MainFrame</code> connected to this dialog
     * @param modal
     *            Make this dialog modal or not
     */
    public OptionsDialog(MainFrame f, boolean modal) {
        super(f, "Preferences", modal);
        frame = f;
        setSize(430, 400);
        // setResizable(false);
        getContentPane().setLayout(new BorderLayout());
        tab = new JTabbedPane();
        tooltab = new ToolbarTab(f);
        tab.add("Toolbar", tooltab);
        tab.add("Layout", new JScrollPane(layoutTab = new LayoutTab(f)));
        addButtonPanel();
        getContentPane().add(tab, "Center");
        setVisible(true);
    }

    /** Adds the ok and cancel buttons. */
    private void addButtonPanel() {
        JPanel buttons = new JPanel();
        JButton ok = new JButton("OK");
        ok.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                ok();
            }
        });
        buttons.add(ok);
        JButton cancel = new JButton("Cancel");
        cancel.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                dispose();
            }
        });
        buttons.add(cancel);
        JButton apply = new JButton("Apply");
        apply.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                apply();
            }
        });
        buttons.add(apply);
        getContentPane().add(buttons, "South");
    }

    /**
     * Invoked when the ok button is pressed. Saves the properties and closes
     * the frame.
     */
    private void ok() {
        apply();
        dispose();
    }

    private void apply() {
        if (tooltab.isChanged()) {
            tooltab.setProperties();
            PropertiesHandler.store();
            frame.setToolbar((ToolbarPanel) ToolbarFactory
                    .getToolbarFromProperties(frame));
        }
        layoutTab.setValues();
        ApplicationAdapter.getApplication().validate();
    }

}
