// MatrixPro - algorithm simulation system
// Copyright (C) 2000-2007 See the AUTHORS file and the author tag
// below for more information on the authors.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package matrixpro.util;

import java.io.BufferedReader;
import java.util.HashMap;
import java.util.LinkedList;

public class ExerciseDescriptionParser {
    public static final String INSTRUCTIONS_CLASS = "instructions";

    public static final String TASK_CLASS = "task";

    public static final String CODE_CLASS = "code";

    public static final String CODE_NAME_ATTRIBUTE = "name";

    private static final int TASK = 1;

    private static final int INSTRUCTIONS = 2;

    private static final int CODE = 3;

    private int parserState;

    private String instructions;

    private String task;

    private String currentCodeBlock;

    private String currentCodeBlockName;

    private LinkedList codeBlockNames;

    private HashMap codeBlocks;

    public ExerciseDescriptionParser(BufferedReader reader) {
        currentCodeBlock = "";
        currentCodeBlockName = "";
        task = "";
        instructions = "";
        codeBlocks = new HashMap();
        codeBlockNames = new LinkedList();
        parserState = 0;

        try {
            String temp = reader.readLine();
            while (temp != null) {
                if (temp.toLowerCase().indexOf("<span") != -1
                        && temp.toLowerCase().indexOf("/>") != -1) {
                    boolean statechanged = parse(temp.substring(temp
                            .toLowerCase().indexOf("<span") + 5, temp
                            .toLowerCase().indexOf("/>")));
                    if (statechanged)
                        temp = temp.substring(temp.toLowerCase().indexOf("/>",
                                temp.toLowerCase().indexOf("<span") + 5) + 2); // crop
                                                                                // the
                                                                                // <span/>-tag
                                                                                // from
                                                                                // input
                }
                switch (parserState) {
                case 0:
                    break;
                case TASK:
                    task = task + temp + "\n";
                    break;
                case INSTRUCTIONS:
                    instructions = instructions + temp + "\n";
                    break;
                case CODE:
                    currentCodeBlock = currentCodeBlock + temp + "\n";
                    break;
                }
                temp = reader.readLine();
            }
            if (currentCodeBlockName != "") { // Write last codeblock to
                                                // hashmap when parsing is
                                                // finished
                codeBlocks.put(currentCodeBlockName, currentCodeBlock);
                codeBlockNames.add(currentCodeBlockName);
                currentCodeBlockName = "";
                currentCodeBlock = "";
            }
        } catch (Exception e) {
        }

    }

    private boolean parse(String s) {
        String className = "";
        if (s.toLowerCase().indexOf("class=\"") != -1
                && s.substring(s.toLowerCase().indexOf("class=\"") + 7)
                        .indexOf("\"") != -1) {
            className = s.substring(s.toLowerCase().indexOf("class=\"") + 7, s
                    .indexOf("\"", s.toLowerCase().indexOf("class=\"") + 7));
        }

        if (className != "") {
            if (className.equals(INSTRUCTIONS_CLASS)) {
                parserState = INSTRUCTIONS;
                return true;
            }
            if (className.equals(TASK_CLASS)) {
                parserState = TASK;
                return true;
            }
            if (className.equals(CODE_CLASS)
                    && s.toLowerCase().indexOf("name=\"") != -1) {
                parserState = CODE;
                if (currentCodeBlockName != "") {
                    codeBlocks.put(currentCodeBlockName, currentCodeBlock);
                    codeBlockNames.add(currentCodeBlockName);
                    currentCodeBlockName = "";
                    currentCodeBlock = "";
                }
                currentCodeBlockName = s.substring(s.toLowerCase().indexOf(
                        "name=\"") + 6, s.toLowerCase().indexOf("\"",
                        s.toLowerCase().indexOf("name=\"") + 6));
                return true;
            }
            return false;
        }
        return false;
    }

    public String getInstructions() {
        return instructions;
    }

    public String getTask() {
        return task;
    }

    public String getCodeBlockName(int index) {
        return (String) codeBlockNames.get(index);
    }

    public int getNumberOfCodeBlocks() {
        return codeBlockNames.size();
    }

    public String getCodeBlock(String blockName) {
        return (String) codeBlocks.get(blockName);
    }

    public HashMap getCodeBlocks() {
        return codeBlocks;
    }

    public LinkedList getCodeBlockNames() {
        return codeBlockNames;
    }
}
