// MatrixPro - algorithm simulation system
// Copyright (C) 2000-2007 See the AUTHORS file and the author tag
// below for more information on the authors.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package matrixpro.util;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

import matrix.structures.FDT.FDT;
import matrix.util.Note;

import matrixpro.animation.CountingAnimator;

/**
 * Class that handles the undo-redo functionality of the application.
 * 
 * @version $Id: UndoRedoHandler.java,v 1.7 2007/09/10 08:35:54 vkaravir Exp $
 */
public class UndoRedoHandler {
    private UndoRedoWrapper wrap;

    public UndoRedoHandler(CountingAnimator oldA, FDT[] oldStructures) {
        deepCopy(oldA, oldStructures, oldA.getState());
    }

    public UndoRedoWrapper undo(CountingAnimator newA, FDT[] newStructures) {
        UndoRedoWrapper oldWrap = wrap;
        deepCopy(newA, newStructures, -1);
        return oldWrap;
    }

    public UndoRedoWrapper redo(CountingAnimator oldA, FDT[] oldStructures) {
        UndoRedoWrapper newWrap = wrap;
        deepCopy(oldA, oldStructures, -1);
        return newWrap;
    }

    private void deepCopy(CountingAnimator a, FDT[] s, int pos) {
        try {
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            ObjectOutputStream out = new ObjectOutputStream(baos);
            for (int i = 0; i < s.length; i++)
                out.writeObject(s[i]);
            out.writeObject(a);
            ByteArrayInputStream bais = new ByteArrayInputStream(baos
                    .toByteArray());
            ObjectInputStream in = new ObjectInputStream(bais);
            FDT[] structures = new FDT[s.length];
            for (int i = 0; i < s.length; i++)
                structures[i] = (FDT) in.readObject();
            CountingAnimator animator = (CountingAnimator) in.readObject();
            if (pos > 0)
                animator.GoTo(pos, animator.getMaximum());
            wrap = new UndoRedoWrapper(animator, structures);
        } catch (Exception e) {
            e.printStackTrace();
            Note.show("Oops!", "Undo/redo failed!");
        }
    }

    public class UndoRedoWrapper {
        private CountingAnimator animator;

        private FDT[] structures;

        public UndoRedoWrapper(CountingAnimator a, FDT[] s) {
            animator = a;
            structures = s;
        }

        public CountingAnimator getAnimator() {
            return animator;
        }

        public FDT[] getStructures() {
            return structures;
        }
    }
}
