// MatrixPro - algorithm simulation system
// Copyright (C) 2000-2007 See the AUTHORS file and the author tag
// below for more information on the authors.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package matrixpro.util.export;

import java.io.File;

import javax.swing.JComponent;
import javax.swing.filechooser.FileFilter;

import matrixpro.ui.MainFrame;

/**
 * An export format for MatrixPro. This class should be extended to add new
 * export formats.
 * 
 * @author vkaravir
 * @version $Id: MPExportFormat.java,v 1.3 2007/09/10 08:35:56 vkaravir Exp $
 */
public abstract class MPExportFormat {

    public MPExportFormat() {
    }

    /**
     * Method that returns the JComponent used as an accessory in the
     * JFileChooser. By default returns null, subclasses should override this
     * method if they want to use an accessory.
     * 
     * @return The JComponent used as an accessory in the JFileChooser.
     * @see javax.swing.JFileChooser#setAccessory(javax.swing.JComponent)
     */
    public JComponent getAccessory() {
        return null;
    }

    // ABSTRACT METHODS BEGIN
    /**
     * Returns the FileFilter used when exporting to the format of this class.
     * 
     * @see javax.swing.filechooser.FileFilter
     */
    public abstract FileFilter getFileFilter();

    /**
     * Returns the filename extension for the format of this class. That is, for
     * example, "jpg" for JPEG pictures.
     * 
     * @return Returns the filename extension for the format of this class.
     */
    public abstract String getFileExtension();

    /**
     * Exports to the given file using the format of this class. The format
     * class does not need to worry about existing files when used with
     * MPExporter.
     * 
     * @param mf
     *            The MatrixFrame the content of which is exported.
     * @param saveFile
     *            The file used when exporting.
     * @return Boolean representing whether the export succeeded.
     * @see matrixpro.util.export.MPExporter
     */
    public abstract boolean export(MainFrame mf, File saveFile);
    // ABSTRACT METHODS END
}