// MatrixPro - algorithm simulation system
// Copyright (C) 2000-2007 See the AUTHORS file and the author tag
// below for more information on the authors.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package matrixpro.util.export;

import java.awt.Graphics;
import java.awt.GridLayout;
import java.awt.Rectangle;
import java.awt.image.BufferedImage;
import java.io.File;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import javax.swing.BorderFactory;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.filechooser.FileFilter;

import matrix.util.Note;
import matrix.util.export.ConfigurableGraphics;

import matrixpro.ui.MainFrame;
import matrixpro.util.FileHelper;

/**
 * PNG picture export format for MatrixPro.
 * 
 * @version $Id: PNGExportFormat.java,v 1.7 2007/09/10 08:35:55 vkaravir Exp $
 */
public class PNGExportFormat extends MPExportFormat {
    /** The <code>JPanel</code> used when png is selected. */
    private JPanel png;

    /** <code>JCheckBox</code> for the png series selection. */
    private JCheckBox serie;

    /** <code>JCheckBox</code> for the structure border selection. */
    private JCheckBox border;

    public PNGExportFormat() {
        super();
        if (!imageioAvailable()) {
            Note.show(this, "Package javax.imageio not found!");
            throw new RuntimeException();
        }
    }

    public FileFilter getFileFilter() {
        return new PNGFileFilter();
    }

    public String getFileExtension() {
        return "png";
    }

    public boolean export(MainFrame mf, File saveFile) {
        if (!imageioAvailable()) {
            Note
                    .show(
                            new FileHelper(),
                            "Exporting PNG failed! Class javax.imageio.ImageIO not found. Your Java version might be too old.");
            return false;
        }
        try {
            if (!serie.isSelected()) {
                Rectangle bounds = mf.getStructurePanel().getBounds();
                BufferedImage i = new BufferedImage(bounds.width,
                        bounds.height, BufferedImage.TYPE_INT_RGB);
                Graphics g = i.createGraphics();
                g.setClip(bounds);

                boolean brdr = border.isSelected();
                ConfigurableGraphics cg = new ConfigurableGraphics(g);
                cg.addDisable("VisualHotSpot");
                if (!brdr) {
                    cg.addDisable("StructureBorder");
                }

                mf.getStructurePanel().paint(cg);

                Method m = getWriteMethod();
                Object[] o = new Object[3];
                o[0] = i;
                o[1] = "PNG";
                o[2] = saveFile;
                m.invoke(null, o);
            } else {
                int step = 0;
                Rectangle bounds = mf.getStructurePanel().getBounds();
                mf.getStructurePanel().getAnimator().rewind();
                mf.invalidateComponents();
                mf.validate();
                mf.getStructurePanel().validate();
                mf.validate();
                mf.getStructurePanel().repaint();
                Method m = getWriteMethod();
                Object[] o = new Object[3];
                String baseFileName = saveFile.getAbsoluteFile().toString()
                        .substring(
                                0,
                                saveFile.getAbsoluteFile().toString().indexOf(
                                        "." + getFileExtension()));
                boolean owAll = false;
                String[] options = new String[] { "YES TO ALL", "YES", "NO",
                        "CANCEL" };
                do {
                    boolean write = true;
                    if (step != 0) {
                        mf.getStructurePanel().getAnimator().redo();
                        mf.invalidateComponents();
                        mf.validate();
                        mf.getStructurePanel().validate();
                        mf.validate();
                        mf.getStructurePanel().repaint();
                    }
                    BufferedImage i = new BufferedImage(bounds.width,
                            bounds.height, BufferedImage.TYPE_INT_RGB);
                    Graphics g = i.createGraphics();
                    g.setClip(bounds);

                    boolean brdr = border.isSelected();
                    ConfigurableGraphics cg = new ConfigurableGraphics(g);
                    cg.addDisable("VisualHotSpot");
                    if (!brdr) {
                        cg.addDisable("StructureBorder");
                    }

                    mf.getStructurePanel().paint(cg);

                    File file = new File(baseFileName + "" + step + ".png");
                    if (!owAll && file.exists()) {
                        int overwrite = JOptionPane.showOptionDialog(mf,
                                "File " + file.getName()
                                        + " exists! Overwrite?", "File exists",
                                JOptionPane.DEFAULT_OPTION,
                                JOptionPane.QUESTION_MESSAGE, null, options,
                                options[1]);
                        if (overwrite == 2
                                || overwrite == JOptionPane.CLOSED_OPTION)
                            write = false;
                        else if (overwrite == 0)
                            owAll = true;
                        else if (overwrite == 3)
                            return false;
                    }
                    if (write) {
                        o[0] = i;
                        o[1] = "PNG";
                        o[2] = file;
                        m.invoke(null, o);
                    }
                    step++;
                } while (mf.getStructurePanel().getAnimator()
                        .hasNextOperation());
            }
        } catch (IllegalAccessException e) {
        } catch (ClassNotFoundException e) {
        } catch (NoSuchMethodException e) {
        } catch (InvocationTargetException e) {
        }
        return true;
    }

    public JComponent getAccessory() {
        if (png == null)
            initPngPanel();
        return png;
    }

    private static Method getWriteMethod() throws IllegalAccessException,
            ClassNotFoundException, NoSuchMethodException,
            InvocationTargetException {
        Class c = Class.forName("javax.imageio.ImageIO");
        Method m = c.getMethod("write", new Class[] {
                Class.forName("java.awt.image.RenderedImage"), "".getClass(),
                Class.forName("java.io.File") });
        return m;
    }

    private static boolean imageioAvailable() {
        try {
            Class.forName("javax.imageio.ImageIO");
        } catch (ClassNotFoundException e) {
            return false;
        }
        return true;
    }

    /**
     * Method that adds components to the panel used when tex is selected.
     */
    private void initPngPanel() {
        png = new JPanel();
        png.setLayout(new GridLayout(2, 1));
        png.setBorder(BorderFactory.createTitledBorder(BorderFactory
                .createBevelBorder(2), "PNG"));
        serie = new JCheckBox("Picture series", false);
        serie
                .setToolTipText("Check to export a picture for each animation step,"
                        + " uncheck to export only the current view.");
        png.add(serie);
        border = new JCheckBox("Structures with borders", true);
        border.setToolTipText("Check to export structures with border, "
                + "uncheck to export structures without borders");
        png.add(border);
    }

}
