// MatrixPro - algorithm simulation system
// Copyright (C) 2000-2007 See the AUTHORS file and the author tag
// below for more information on the authors.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package matrixpro.util.export;

import java.io.File;

/**
 * <code>FileFilter</code> used in export on a JFileChooser to keep other than
 * latex documents (.tex) from appearing in the directory listing.
 * 
 * @version $Id: TexFileFilter.java,v 1.4 2007/09/10 08:35:56 vkaravir Exp $
 */
public class TexFileFilter extends javax.swing.filechooser.FileFilter {

    /**
     * Returns true if the given file is accepted by this filter, false
     * otherwise.
     */
    public boolean accept(File f) {
        if (f.isDirectory())
            return true;
        String ext = getExtension(f);
        if (ext != null)
            if (ext.equals("tex"))
                return true;
            else
                return false;
        return false;
    }

    /** Returns the description of this filter. */
    public String getDescription() {
        return "LaTeX picture (*.tex)";
    }

    /**
     * Returns the extension (string after the last .) of the given file.
     */
    private static String getExtension(File f) {
        String ext = null;
        String s = f.getName();
        int i = s.lastIndexOf('.');

        if (i > 0)
            ext = s.substring(i + 1).toLowerCase();
        return ext;
    }
}
